@@ -32,6 +32,8 @@
 #include "gretl_func.h"
 #include "modelspec.h"
 #include "libset.h"
+#include "forecast.h"
+#include "cmd_private.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -42,6 +44,11 @@
 # include <unistd.h>
 #endif 
 
+enum {
+    ERRFATAL_AUTO,
+    ERRFATAL_FORCE
+};
+
 #ifdef HAVE_READLINE
 # include <readline/readline.h>
 /* readline functions from complete.c */
@@ -65,26 +72,26 @@ PRN *cmdprn;
 MODELSPEC *modelspec;
 MODEL tmpmod;
 FILE *dat, *fb;
-int i, j, dot, opt, err, errfatal, batch;
+int i, j, dot, opt, errfatal, batch;
 int runit, loopstack, looprun;
-int data_status, runfile_open;
-int echo_off;               /* suppress command echoing */
+int data_status;
 int plot_count;             /* graphs via gnuplot */
-int ignore;                 /* trap for comments */
 int order;                  /* VAR lag order */
 int lines[1];               /* for gnuplot command */
 char *line;                 /* non-Readline command line */
 char texfile[MAXLEN];
 char response[3];
-char linebak[MAXLEN];      /* for storing comments */
+char linebak[MAXLINE];      /* for storing comments */
 char *line_read;
 
 gretl_equation_system *sys;
 gretl_restriction_set *rset;
 
-static void exec_line (char *line, LOOPSET **ploop, PRN *prn); 
+static int exec_line (char *line, LOOPSET **ploop, PRN *prn); 
+static int push_input_file (FILE *fp);
+static FILE *pop_input_file (void);
 
-void usage(void)
+static void usage(void)
 {
     logo();
     printf(_("\nYou may supply the name of a data file on the command line.\n"
@@ -95,6 +102,7 @@ void usage(void)
 	     " -h or --help      Print this info and exit.\n"
 	     " -v or --version   Print version info and exit.\n"
 	     " -e or --english   Force use of English rather than translation.\n"
+	     " -q or --basque    Force use of Basque translation.\n"
 	     "Example of batch mode usage:\n"
 	     " gretlcli -b myfile.inp >myfile.out\n"
 	     "Example of run mode usage:\n"
@@ -186,38 +194,37 @@ void file_get_line (void)
 	strcpy(line, "quit");
     } else {
 	*linebak = 0;
-	strncat(linebak, line, MAXLEN-1);
+	strncat(linebak, line, MAXLINE - 1);
     }
 
     if (!strncmp(line, "noecho", 6)) {
-	echo_off = 1;
+	set_gretl_echo(0);
     }
-    if (!echo_off && cmd.ci == RUN && batch && *line == '(') {
+
+    if (gretl_echo_on() && cmd.ci == RUN && batch && *line == '(') {
 	printf("%s", line);
 	*linebak = 0;
     }
 }
 
-void fn_get_line (void)
+int fn_get_line (void)
 {
+    int err = 0;
+
     clear(line, MAXLINE);
-    
-    gretl_function_get_line(line, MAXLINE, &Z, datainfo);
+    gretl_function_get_line(line, MAXLINE, &Z, datainfo, &err);
 
-    if (*line == '\0') {
-	return;
-    }
-
-    *linebak = 0;
-    strncat(linebak, line, MAXLEN-1);
-
-    if (!strncmp(line, "noecho", 6)) {
-	echo_off = 1;
-    }
-    if (!echo_off && cmd.ci == RUN && batch && *line == '(') {
-	printf("%s", line);
+    if (*line != '\0') {
 	*linebak = 0;
+	strncat(linebak, line, MAXLINE - 1);
+
+	if (gretl_echo_on() && cmd.ci == RUN && batch && *line == '(') {
+	    printf("%s", line);
+	    *linebak = 0;
+	}
     }
+
+    return err;
 }
 
 unsigned char gp_flags (int batch, gretlopt opt)
@@ -256,13 +263,26 @@ void nls_init (void)
     reset_local_decpoint();
 }
 
-static void force_english (void)
+static void force_language (int f)
 {
-    setlocale (LC_ALL, "C");
+    if (f == ENGLISH) {
+	setlocale(LC_ALL, "C");
+    } else {
+# ifdef WIN32
+	setlocale(LC_ALL, "eu");
+# else
+	setlocale(LC_ALL, "eu_ES");
+# endif
+    }
 
 # ifdef WIN32
-    SetEnvironmentVariable("LC_ALL", "C");
-    putenv("LC_ALL=C");
+    if (f == ENGLISH) { 
+	SetEnvironmentVariable("LC_ALL", "C");
+	putenv("LC_ALL=C");
+    } else {
+	SetEnvironmentVariable("LC_ALL", "eu");
+	putenv("LC_ALL=eu");
+    }	
 # endif
 }
 
@@ -300,11 +320,13 @@ static int clear_data (void)
     return err;
 }
 
-static void get_an_input_line (void)
+static int get_an_input_line (void)
 {
+    int err = 0;
+
     if (gretl_executing_function()) {
 	/* reading from compiled function */
-	fn_get_line();
+	err = fn_get_line();
     } else if (runit || batch) {
 	/* reading from script file */
 	file_get_line();
@@ -325,6 +347,8 @@ static void get_an_input_line (void)
 	file_get_line(); /* "file" = stdin here */
 #endif
     }
+
+    return err;
 }
 
 static int maybe_get_input_line_continuation (char *tmp)
@@ -340,20 +364,20 @@ static int maybe_get_input_line_continuation (char *tmp)
 	tmp[0] = '\0';
 
 	if (gretl_executing_function()) {
-	    gretl_function_get_line(tmp, MAXLEN - 1, &Z, datainfo);
+	    gretl_function_get_line(tmp, MAXLINE - 1, &Z, datainfo, &err);
 	} else if (batch || runit) {
-	    fgets(tmp, MAXLEN - 1, fb);
+	    fgets(tmp, MAXLINE - 1, fb);
 	} else {
 #ifdef HAVE_READLINE
 	    rl_gets(&line_read, "> ");
 	    strcpy(tmp, line_read);
 #else
-	    fgets(tmp, MAXLEN - 1, stdin); 
+	    fgets(tmp, MAXLINE - 1, stdin); 
 #endif /* HAVE_READLINE */
 	}
 
 	if (*tmp != '\0') {
-	    if (strlen(line) + strlen(tmp) > MAXLEN - 1) {
+	    if (strlen(line) + strlen(tmp) > MAXLINE - 1) {
 		err = 1;
 		break;
 	    } else {
@@ -366,6 +390,27 @@ static int maybe_get_input_line_continuation (char *tmp)
     return err;
 }
 
+static void set_errfatal (int code)
+{
+    static int hoe = -1;
+
+    if (hoe < 0) {
+	hoe = get_halt_on_error();
+    }
+
+    if (code == ERRFATAL_FORCE) {
+	/* for contexts where contunuation on error is
+	   bound to make a big mess */
+	errfatal = 1;
+    } else if (hoe == 0) {
+	/* we've been explicitly told to keep going on error */
+	errfatal = 0;
+    } else {
+	/* default: errors are fatal in batch mode only */
+	errfatal = batch;
+    }
+}
+
 int main (int argc, char *argv[])
 {
     int cli_get_data = 0;
@@ -374,6 +419,7 @@ int main (int argc, char *argv[])
     char tmp[MAXLINE];
     LOOPSET *loop = NULL;
     PRN *prn;
+    int err = 0;
 
 #ifdef WIN32
     strcpy(tmp, argv[0]);
@@ -389,8 +435,8 @@ int main (int argc, char *argv[])
     }
     
     if (argc > 1) {
-	int english = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &english);
+	int force_lang = 0;
+	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
 
 	switch (opt) {
 	case OPT_BATCH:
@@ -414,7 +460,9 @@ int main (int argc, char *argv[])
 	    break;
 	case OPT_RUNIT:
 	    runit = 1;
-	    if (*filearg == '\0') usage();
+	    if (*filearg == '\0') {
+		usage();
+	    }
 	    strcpy(runfile, filearg); 
 	    cli_get_data = 1;
 	    break;
@@ -423,8 +471,8 @@ int main (int argc, char *argv[])
 	}
 
 #ifdef ENABLE_NLS
-	if (english) {
-	    force_english();
+	if (force_lang) {
+	    force_language(force_lang);
 	    if (argc == 2) {
 		cli_get_data = 1;
 	    }
@@ -440,11 +488,11 @@ int main (int argc, char *argv[])
     }
 #endif
 
+    libgretl_init();
     logo();     /* print version info */
-    session_time(stdout);
-    fb = stdin; /* may be reset later with "run" command */
+    session_time(NULL);
 
-    prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
+    prn = gretl_print_new(GRETL_PRINT_STDOUT);
 
     line = malloc(MAXLINE);
     if (line == NULL) {
@@ -462,7 +510,7 @@ int main (int argc, char *argv[])
     if (!batch) {
 	strcpy(cmdfile, gretl_user_dir());
 	strcat(cmdfile, "session.inp");
-	cmdprn = gretl_print_new(GRETL_PRINT_FILE, cmdfile);
+	cmdprn = gretl_print_new_with_filename(cmdfile);
 	if (cmdprn == NULL) {
 	    printf(_("Can't open file to save commands\n"));
 	    return EXIT_FAILURE;
@@ -489,7 +537,9 @@ int main (int argc, char *argv[])
 	    err = get_xmldata(&Z, &datainfo, paths.datfile, &paths, 
 			      DATA_NONE, prn, 0);
 	} else if (err == GRETL_CSV_DATA) {
-	    err = import_csv(&Z, &datainfo, paths.datfile, &paths, prn);
+	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
+	} else if (err == GRETL_OCTAVE) {
+	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
 	} else if (err == GRETL_BOX_DATA) {
 	    err = import_box(&Z, &datainfo, paths.datfile, prn);
 	} else if (err == GRETL_SCRIPT) { /* maybe it's a script file? */
@@ -523,7 +573,7 @@ int main (int argc, char *argv[])
 	noalloc("models"); 
     }
 
-    libgretl_init(&cmd);
+    gretl_cmd_init(&cmd);
 
     /* print list of variables */
     if (data_status) {
@@ -543,6 +593,11 @@ int main (int argc, char *argv[])
 	}
     } 
 
+    if (!batch) {
+	fb = stdin;
+	push_input_file(fb);
+    }
+
     if (!batch && !runit && !data_status) {
 	fprintf(stderr, _("Type \"open filename\" to open a data set\n"));
     }
@@ -552,43 +607,47 @@ int main (int argc, char *argv[])
 #endif
 
     if (batch || runit) {
-	sprintf(line, "run %s\n", runfile);
+	/* re-initialize: will be incremented by "run" cmd */
+	runit = 0;
+	sprintf(line, "run %s", runfile);
 	exec_line(line, &loop, prn);
     }
 
     /* should we stop immediately on error, in batch mode? */
-    if (get_halt_on_error()) {
-	errfatal = batch;  /* exit on error in batch mode */
-    }
+    set_errfatal(ERRFATAL_AUTO);
 
     /* main command loop */
-    while (strcmp(cmd.word, "quit")) {
-	char linecopy[MAXLEN];
+    while (strcmp(cmd.word, "quit") && fb != NULL) {
+	char linecopy[MAXLINE];
 
-	if (err && batch && errfatal) {
+	if (err && errfatal) {
 	    gretl_abort(linecopy);
 	}
 
 	if (looprun) { 
-	    if (loop_exec(loop, line, &Z, &datainfo,
-			  models, &echo_off, prn)) {
+	    if (loop_exec(loop, line, &Z, &datainfo, models, prn)) {
 		return 1;
 	    }
 	    gretl_loop_destroy(loop);
 	    loop = NULL;
-	    looprun = errfatal = 0;
+	    looprun = 0;
+	    set_errfatal(ERRFATAL_AUTO);
 	} else {
-	    get_an_input_line();
+	    err = get_an_input_line();
+	    if (err) {
+		errmsg(err, prn);
+		continue;
+	    }
 	}
 
 	cmd_overflow = maybe_get_input_line_continuation(tmp);
 	if (cmd_overflow) {
 	    fprintf(stderr, _("Maximum length of command line "
-			      "(%d bytes) exceeded\n"), MAXLEN);
+			      "(%d bytes) exceeded\n"), MAXLINE);
 	    break;
 	} else {
 	    strcpy(linecopy, line);
-	    exec_line(line, &loop, prn);
+	    err = exec_line(line, &loop, prn);
 	}
     } /* end of get commands loop */
 
@@ -604,7 +663,7 @@ int main (int argc, char *argv[])
 	free_datainfo(datainfo);
     }
 
-    if (runfile_open && fb != NULL) {
+    if (fb != stdin && fb != NULL) {
 	fclose(fb);
     }
 
@@ -615,8 +674,8 @@ int main (int argc, char *argv[])
     }
 
     gretl_print_destroy(prn);
-
-    libgretl_cleanup(&cmd);
+    gretl_cmd_free(&cmd);
+    libgretl_cleanup();
 
     return 0;
 }
@@ -641,95 +700,128 @@ static void printf_strip (char *s, int loopstack)
     }
 }
 
-static int handle_user_defined_function (char *line, int *fncall)
+static int handle_user_defined_function (char *line, int *fncall,
+					 PRN *prn)
 {
     int ufunc = gretl_is_user_function(line);
     int err = 0;
 
+    /* allow for nested function calls */
     if (ufunc && gretl_compiling_function()) {
-	/* allow for nested function calls */
-	;
-    } else if (ufunc) {
-	/* an actual function call */
-	err = gretl_function_start_exec(line);
+	return 0;
+    }
+
+    /* an actual function call */
+    else if (ufunc) {
+	echo_function_call(line, CMD_ECHO_TO_STDOUT, prn);
+	err = gretl_function_start_exec(line, &Z, datainfo);
 	*fncall = 1;
     } 
 
     return err;
 }
 
-static void exec_line (char *line, LOOPSET **ploop, PRN *prn) 
+static int do_autofit_plot (PRN *prn)
+{
+    int *plotlist;
+    int err = 0;
+
+    plotvar(&Z, datainfo, "time");
+
+    plotlist = gretl_list_new(3);
+    plotlist[1] = gretl_model_get_depvar(models[0]);
+    plotlist[2] = varindex(datainfo, "autofit");
+    plotlist[3] = varindex(datainfo, "time");
+    lines[0] = 1;
+
+    err = gnuplot(plotlist, lines, NULL, &Z, datainfo,
+		  &plot_count, gp_flags(batch, 0));
+
+    free(plotlist);
+
+    if (err) {
+	pputs(prn, _("gnuplot command failed\n"));
+    }
+
+    return err;
+}
+
+static int exec_line (char *line, LOOPSET **ploop, PRN *prn) 
 {
     LOOPSET *loop = *ploop;
     int chk, nulldata_n, renumber;
     int dbdata = 0, do_arch = 0, do_nls = 0;
+    unsigned char echo_flags;
     char s1[12], s2[12];
     int fncall = 0;
     double rho;
+    int err = 0;
 
-    if (*line == '\0') return;
+    if (string_is_blank(line)) {
+	return 0;
+    }
 
     /* catch any user-defined functions */
-    err = handle_user_defined_function(line, &fncall);
+    err = handle_user_defined_function(line, &fncall, prn);
     if (err) {
 	errmsg(err, prn);
-	return;
+	return err;
     } else if (fncall) {
-	return;
+	return 0;
     }
 
     if (gretl_compiling_function()) {
 	err = gretl_function_append_line(line);
-	if (err) errmsg(err, prn);
-	return;
+	if (err) {
+	    errmsg(err, prn);
+	}
+	return err;
     }  
     
     /* are we ready for this? */
-    if (!data_status && !ignore && 
+    if (!data_status && !cmd.ignore && 
 	!ready_for_command(line)) {
 	fprintf(stderr, _("You must open a data file first\n"));
-	err = 1;
-	return;
-    }
-
-    cmd.opt = get_gretl_options(line, &err);
-    if (err) {
-	errmsg(err, prn);
-	return;
+	return 1;
     }
 
     compress_spaces(line);
 
-    /* if we're stacking commands for a loop, parse "lightly" */
-    if (loopstack) {
-	get_cmd_ci(line, &cmd);
-    } else {
-	getcmd(line, datainfo, &cmd, &ignore, &Z, 
-	       (runit)? NULL : cmdprn);
-    }
-
     /* tell libgretl if we're in batch mode */
     gretl_set_batch_mode(batch);
 
+    /* if we're stacking commands for a loop, parse "lightly" */
+    if (loopstack) {
+	err = get_command_index(line, &cmd);
+    } else {
+	err = parse_command_line(line, &cmd, &Z, datainfo);
+	if (err && gretl_executing_function()) {
+	    gretl_function_stop_on_error(datainfo, prn);
+	}	
+    }
+
+    if (err) {
+	errmsg(err, prn);
+	return err;
+    }
+
     /* if in batch mode, echo comments from input */
-    if (batch && cmd.ci == CMD_COMMENT && !echo_off) {
+    if (batch && cmd.ci == CMD_COMMENT && gretl_echo_on()) {
 	printf_strip(linebak, loopstack);
     }
 
-    if ((err = cmd.errcode)) {
-	errmsg(err, prn);
-	return;
+    if (cmd.ci < 0) {
+	/* there's nothing there */ 	
+	return 0;
     }
 
-    if (cmd.ci < 0) return; /* there's nothing there */ 
-
     if (sys != NULL && cmd.ci != END && cmd.ci != EQUATION &&
 	cmd.ci != SYSTEM) {
 	printf(_("Command '%s' ignored; not valid within equation system\n"), 
 	       line);
 	gretl_equation_system_destroy(sys);
 	sys = NULL;
-	return;
+	return 1;
     }
 
     if (cmd.ci == LOOP && !batch && !runit) {
@@ -742,41 +834,56 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (!ok_in_loop(cmd.ci, loop)) {
 	    printf(_("Command '%s' ignored; not available in loop mode\n"), line);
 	} else {
-	    if (!echo_off) {
-		echo_cmd(&cmd, datainfo, line, (batch || runit)? 1 : 0, 
-			 0, loopstack, cmdprn);
+	    if (gretl_echo_on()) {
+		echo_flags = CMD_ECHO_TO_STDOUT;
+		if (loopstack) {
+		    echo_flags |= CMD_STACKING;
+		}
+		if (batch || runit) {
+		    echo_flags |= CMD_BATCH_MODE;
+		}
+		echo_cmd(&cmd, datainfo, line, echo_flags, cmdprn);
 	    }
 	    loop = add_to_loop(line, cmd.ci, cmd.opt,
 			       datainfo, &Z, loop, 
 			       &loopstack, &looprun);
 	    if (loop == NULL) {
+		loopstack = 0;
+		set_errfatal(ERRFATAL_FORCE);
 		print_gretl_errmsg(prn);
 		err = 1;
 	    } 
 	    *ploop = loop;
 	}
-	return;
+	return err;
     }
 
-    if (!echo_off) {
-	echo_cmd(&cmd, datainfo, line, (batch || runit)? 1 : 0, 0, 0,
-		 cmdprn);
+    if (gretl_echo_on()) {
+	echo_flags = CMD_ECHO_TO_STDOUT;
+	if (batch || runit) {
+	    echo_flags |= CMD_BATCH_MODE;
+	}
+	echo_cmd(&cmd, datainfo, line, echo_flags, cmdprn);
     }
 
     check_for_loop_only_options(cmd.ci, cmd.opt, prn);
 
     switch (cmd.ci) {
 
+    case ADDOBS:
     case ADF: 
-    case COINT2: 
     case COINT: 
+    case COINT2:
     case CORR: 
     case CRITERIA: 
     case CRITICAL: 
     case DATA:
     case DIFF: 
     case ESTIMATE:
-    case GRAPH: 
+    case FUNC:
+    case FUNCERR:
+    case GRAPH:
+    case PLOT: 
     case HURST:
     case INFO: 
     case KPSS:
@@ -790,8 +897,9 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case MULTIPLY: 
     case OUTFILE: 
     case PCA:
-    case PLOT: 
     case PRINT: 
+    case REMEMBER:
+    case RENAME:
     case RHODIFF:
     case RMPLOT: 
     case RUNS: 
@@ -803,7 +911,9 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case VARLIST:
     case VARTEST: 
 	err = simple_commands(&cmd, line, &Z, datainfo, prn);
-	if (err) errmsg(err, prn);
+	if (err) {
+	    errmsg(err, prn);
+	}
 	break;
 
     case ADD:
@@ -876,10 +986,11 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case ARCH:
 	order = atoi(cmd.param);
 	clear_model(models[1]);
-	*models[1] = arch(order, cmd.list, &Z, datainfo, 
-			  NULL, cmd.opt, prn);
-	if ((err = (models[1])->errcode)) 
+	*models[1] = arch_model(cmd.list, order, &Z, datainfo, 
+				cmd.opt, prn);
+	if ((err = (models[1])->errcode)) { 
 	    errmsg(err, prn);
+	}
 	if ((models[1])->ci == ARCH) {
 	    do_arch = 1;
 	    swap_models(&models[0], &models[1]); 
@@ -892,14 +1003,14 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 #ifdef HAVE_X12A
 	if (cmd.opt & OPT_X) {
 	    *models[0] = arma_x12(cmd.list, (const double **) Z, datainfo,
-				  ((cmd.opt & OPT_V) ? prn : NULL), &paths); 
+				  &paths, cmd.opt, prn);
 	} else {
 	    *models[0] = arma(cmd.list, (const double **) Z, datainfo, 
-			      (cmd.opt & OPT_V)? prn : NULL);
+			      cmd.opt, prn);
 	}
 #else
 	*models[0] = arma(cmd.list, (const double **) Z, datainfo, 
-			  (cmd.opt & OPT_V)? prn : NULL);
+			  cmd.opt, prn);
 #endif
 	if ((err = (models[0])->errcode)) { 
 	    errmsg(err, prn); 
@@ -917,11 +1028,11 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (cmd.ci == COEFFSUM) {
 	    err = sum_test(cmd.list, models[0], &Z, datainfo, prn);
 	} else if (cmd.ci == CUSUM) {
-	    err = cusum_test(models[0], &Z, datainfo, prn, NULL);
+	    err = cusum_test(models[0], &Z, datainfo, OPT_NONE, prn);
 	} else if (cmd.ci == RESET) {
-	    err = reset_test(models[0], &Z, datainfo, prn, NULL);
+	    err = reset_test(models[0], &Z, datainfo, OPT_NONE, prn);
 	} else if (cmd.ci == CHOW) {
-	    err = chow_test(line, models[0], &Z, datainfo, prn, NULL);
+	    err = chow_test(line, models[0], &Z, datainfo, OPT_NONE, prn);
 	} else {
 	    err = vif_test(models[0], &Z, datainfo, prn);
 	}
@@ -933,8 +1044,8 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case CORC:
     case HILU:
     case PWE:
-	rho = estimate_rho(cmd.list, &Z, datainfo, 1, cmd.ci,
-			   &err, prn);
+	rho = estimate_rho(cmd.list, &Z, datainfo, cmd.ci,
+			   &err, cmd.opt, prn);
 	if (err) {
 	    errmsg(err, prn);
 	    break;
@@ -962,8 +1073,8 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 			 " mode\n"));
 	    break;
 	}	
-	err = dataset_drop_listed_vars(cmd.list, &Z, datainfo, 
-				       &renumber);
+	err = dataset_drop_listed_variables(cmd.list, &Z, datainfo, 
+					    &renumber);
 	if (err) {
 	    pputs(prn, _("Failed to shrink the data set"));
 	} else {
@@ -971,16 +1082,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 		pputs(prn, _("Take note: variables have been renumbered"));
 		pputc(prn, '\n');
 	    }
-	    varlist(datainfo, prn);
-	}
-	break;
-
-    case RENAME:
-	err = rename_var_by_id(cmd.str, cmd.param, datainfo);
-	if (err) {
-	    errmsg(err, prn);
-	} else {
-	    varlist(datainfo, prn);
+	    maybe_list_vars(datainfo, prn);
 	}
 	break;
 
@@ -1027,16 +1129,15 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}
 	break;
 
-    case EQNPRINT:
     case TABPRINT:
+    case EQNPRINT:
 	strcpy(texfile, cmd.param);
-	if ((err = model_test_start(cmd.ci, 0, prn))) 
+	if ((err = model_test_start(cmd.ci, 0, prn))) {
 	    break;
-	if (cmd.ci == EQNPRINT) {
-	    err = eqnprint(models[0], datainfo, texfile, cmd.opt);
-	} else {
-	    err = tabprint(models[0], datainfo, texfile, cmd.opt);
 	}
+	err = texprint(models[0], datainfo, texfile, 
+		       (cmd.ci == EQNPRINT)? (cmd.opt | OPT_E) :
+		       cmd.opt);
 	if (err) {
 	    pputs(prn, _("Couldn't open tex file for writing\n"));
 	} else {
@@ -1045,72 +1146,41 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case FCAST:
+    case FIT:
 	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = fcast(line, models[0], datainfo, &Z);
-	if (err < 0) {
-	    err *= -1;
-	    pputs(prn, _("Error retrieving fitted values\n"));
+	if (cmd.ci == FIT) {
+	    err = add_forecast("fcast autofit", models[0], &Z, datainfo, cmd.opt);
+	} else {
+	    err = add_forecast(line, models[0], &Z, datainfo, cmd.opt);
+	}
+	if (err) {
 	    errmsg(err, prn);
 	} else {
-	    err = 0;
-	    varlist(datainfo, prn);
+	    if (cmd.ci == FIT) {
+		pprintf(prn, _("Retrieved fitted values as \"autofit\"\n"));
+	    }
+	    maybe_list_vars(datainfo, prn);
+	    if (cmd.ci == FIT && dataset_is_time_series(datainfo)) {
+		do_autofit_plot(prn);
+	    }
 	}
 	break;
 
     case FCASTERR:
 	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = fcast_with_errs(line, models[0], &Z, datainfo, prn, cmd.opt); 
+	err = display_forecast(line, models[0], &Z, datainfo, cmd.opt, prn);
 	if (err) {
 	    errmsg(err, prn);
 	}
 	break;
 
-    case FIT:
-	if ((err = model_test_start(cmd.ci, 0, prn))) break;
-	err = fcast("fcast autofit", models[0], datainfo, &Z);
-	if (err < 0) {
-	    err *= -1;
-	    pputs(prn, _("Error retrieving fitted values\n"));
-	    errmsg(err, prn);
-	    break;
-	}
-	err = 0;
-	pputs(prn, _("Retrieved fitted values as \"autofit\"\n"));
-	varlist(datainfo, prn);
-	if (dataset_is_time_series(datainfo)) {
-	    plotvar(&Z, datainfo, "time");
-	    cmd.list = realloc(cmd.list, 4 * sizeof(int));
-	    cmd.list[0] = 3; 
-	    if ((models[0])->ci == ARMA) {
-		cmd.list[1] = (models[0])->list[4];
-	    } else {
-		cmd.list[1] = (models[0])->list[1];
-	    }
-	    cmd.list[2] = varindex(datainfo, "autofit");
-	    cmd.list[3] = varindex(datainfo, "time");
-	    lines[0] = 1;
-	    err = gnuplot(cmd.list, lines, NULL, &Z, datainfo,
-			  &plot_count, gp_flags(batch, 0));
-	    if (err) {
-		pputs(prn, _("gnuplot command failed\n"));
-	    }
-	}
-	break;
-		
     case FREQ:
 	err = freqdist(cmd.list[1], (const double **) Z, 
 		       datainfo, !batch, prn, cmd.opt);
 	break;
 
-    case FUNC:
-	err = gretl_start_compiling_function(line);
-	if (err) {
-	    errmsg(err, prn);
-	}
-	break;
-
     case GENR:
-	err = generate(&Z, datainfo, line, models[0]);
+	err = generate(line, &Z, datainfo, models[0], cmd.opt);
 	if (err) { 
 	    errmsg(err, prn);
 	} else {
@@ -1121,7 +1191,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     case GNUPLOT:
 	if ((cmd.opt & OPT_Z) && 
 	    (cmd.list[0] != 3 || 
-	     !isdummy(Z[cmd.list[3]], datainfo->t1, datainfo->t2))) { 
+	     !gretl_isdummy(datainfo->t1, datainfo->t2, Z[cmd.list[3]]))) { 
 	    pputs(prn, _("You must supply three variables, the last of "
 			 "which is a dummy variable\n(with values 1 or 0)\n"));
 	    break;
@@ -1173,10 +1243,12 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (data_status) {
 	    clear_data();
 	}
-	if (cmd.opt) {
+	if (cmd.opt & OPT_B) {
 	    err = import_box(&Z, &datainfo, datfile, prn);
+	} else if (cmd.opt & OPT_O) {
+	    err = import_octave(&Z, &datainfo, datfile, prn);
 	} else {
-	    err = import_csv(&Z, &datainfo, datfile, &paths, prn);
+	    err = import_csv(&Z, &datainfo, datfile, prn);
 	}
 	if (!err) { 
 	    data_status = 1;
@@ -1217,7 +1289,9 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}
 
 	if (chk == GRETL_CSV_DATA) {
-	    err = import_csv(&Z, &datainfo, datfile, &paths, prn);
+	    err = import_csv(&Z, &datainfo, datfile, prn);
+	} else if (chk == GRETL_OCTAVE) {
+	    err = import_octave(&Z, &datainfo, datfile, prn);
 	} else if (chk == GRETL_BOX_DATA) {
 	    err = import_box(&Z, &datainfo, datfile, prn);
 	} else if (chk == GRETL_XML_DATA) {
@@ -1250,7 +1324,7 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if (err > 1) {
 	    errmsg(err, prn);
 	} else if (cmd.opt & OPT_S) {
-	    varlist(datainfo, prn);
+	    maybe_list_vars(datainfo, prn);
 	}
 	break;
 
@@ -1260,29 +1334,29 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	if ((cmd.opt & OPT_S) || (cmd.opt & OPT_O) || !cmd.opt) {
 	    clear_model(models[1]);
 	    err = nonlinearity_test(models[0], &Z, datainfo, 
-				    AUX_SQ, OPT_NONE, prn, NULL);
+				    AUX_SQ, OPT_NONE, prn);
 	    if (err) errmsg(err, prn);
 	    if (cmd.opt == OPT_S) break;
-	    if (!err && !batch && page_pause()) break; 
+	    if (!err && !batch && scroll_pause_or_quit()) break; 
 	}
 	/* non-linearity (logs) */
 	if ((cmd.opt & OPT_L) || (cmd.opt & OPT_O) || !cmd.opt) {
 	    err = nonlinearity_test(models[0], &Z, datainfo, 
-				    AUX_LOG, OPT_NONE, prn, NULL);
+				    AUX_LOG, OPT_NONE, prn);
 	    if (err) errmsg(err, prn);
 	    if (cmd.opt == OPT_L) break;
-	    if (!err && !batch && page_pause()) break;
+	    if (!err && !batch && scroll_pause_or_quit()) break;
 	}
 	/* autocorrelation */
 	if ((cmd.opt & OPT_M) || (cmd.opt & OPT_O)) {
 	    int order = atoi(cmd.param);
 
-	    err = autocorr_test(models[0], order, &Z, datainfo, prn, NULL);
+	    err = autocorr_test(models[0], order, &Z, datainfo, OPT_NONE, prn);
 	    if (err) errmsg(err, prn);
 	}
 	/* heteroskedasticity */
 	if ((cmd.opt & OPT_W) || !cmd.opt) {
-	    err = whites_test(models[0], &Z, datainfo, prn, NULL);
+	    err = whites_test(models[0], &Z, datainfo, OPT_NONE, prn);
 	    if (err) errmsg(err, prn);
 	}
 	/* groupwise heteroskedasticity */
@@ -1417,24 +1491,21 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case QUIT:
-	if (batch) {
-	    pputs(prn, _("Done\n"));
-	    break;
-	}
 	if (runit) {
-	    runit = 0;
-	    if (fb != NULL) {
-		fclose(fb);
+	    runit--;
+	    fclose(fb);
+	    fb = pop_input_file();
+	    if (fb == NULL) {
+		pputs(prn, _("Done\n"));
+	    } else {
+		strcpy(cmd.word, "endrun"); /* overwrite "quit" */
 	    }
-	    fb = stdin;
-	    runfile_open = 0;
-	    strcpy(cmd.word, "endrun"); /* overwrite "quit" */
 	    break;
 	}
 	printf(_("commands saved as %s\n"), cmdfile);
 	gretl_print_destroy(cmdprn);
 
-	if (cmd.param[0] == 'x') break;
+	if (cmd.opt & OPT_X) break;
 
 	printf(_("type a filename to store output (enter to quit): "));
 	*outfile = '\0';
@@ -1460,29 +1531,31 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	break;
 
     case RUN:
+    case INCLUDE:
 	err = getopenfile(line, runfile, &paths, 1, 1);
 	if (err) { 
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
 	}
+	if (fb != NULL) {
+	    push_input_file(fb);
+	}
 	if ((fb = fopen(runfile, "r")) == NULL) {
 	    fprintf(stderr, _("Couldn't open script \"%s\"\n"), runfile);
-	    if (runit) {
-		fb = stdin;
-		runit = 0;
-	    } else {
-		if (batch) exit(EXIT_FAILURE);
-	    }
+	    fb = pop_input_file();
 	} else {
 	    fprintf(stderr, _("%s opened OK\n"), runfile);
-	    pprintf(cmdprn, "run \"%s\"\n", runfile);
-	    runfile_open = 1;
-	    if (!batch) runit = 1;
+	    if (cmd.ci == INCLUDE) {
+		pprintf(cmdprn, "include \"%s\"\n", runfile);
+	    } else {
+		pprintf(cmdprn, "run \"%s\"\n", runfile);
+	    }
+	    runit++;
 	}
 	break;
 
     case SET:
-	err = parse_set_line(line, &echo_off, prn);
+	err = execute_set_line(line, prn);
 	if (err) {
 	    errmsg(err, prn);
 	}
@@ -1532,6 +1605,8 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 
 	if (err) {
 	    errmsg(err, prn);
+	} else if (get_gretl_msg()) {
+	    print_gretl_msg(prn);
 	} else {
 	    print_smpl(datainfo, get_full_length_n(), prn);
 	}
@@ -1594,10 +1669,6 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
 	}
 	break;
 
-    case VARDUP:
-	err = 1;
-	break;
-
     default:
 	pprintf(prn, _("Sorry, the %s command is not yet implemented "
 		       "in gretlcli\n"), cmd.word);
@@ -1606,26 +1677,50 @@ static void exec_line (char *line, LOOPSET **ploop, PRN *prn)
     }
 
     if (err && gretl_executing_function()) {
-	gretl_function_error();
+	gretl_function_stop_on_error(datainfo, prn);
     }
 
     if (!err && (is_model_cmd(cmd.word) || do_nls || do_arch)
 	&& !is_quiet_model_test(cmd.ci, cmd.opt)) { 
 
 	attach_subsample_to_model(models[0], datainfo);
-
 #ifdef MSPEC_DEBUG
 	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
 		(models[0])->ID, get_model_count());
 #endif
-
 	err = modelspec_save(models[0], &modelspec);
     }
 
+    return err;
 }
 
+/* apparatus for keeping track of input stream */
 
+#define N_STACKED_FILES 8
 
+static int nfiles;
+static FILE *fstack[N_STACKED_FILES];
 
+static int push_input_file (FILE *fp)
+{
+    int err = 0;
 
+    if (nfiles >= N_STACKED_FILES) {
+	err = 1;
+    } else {
+	fstack[nfiles++] = fp;
+    }
 
+    return err;
+}
+
+static FILE *pop_input_file (void)
+{
+    FILE *ret = NULL;
+
+    if (nfiles > 0) {
+	ret = fstack[--nfiles];
+    }
+
+    return ret;
+}
