@@ -37,8 +37,11 @@ int main (void)
     PRN *prn;                   /* pointer to struct for printing */
     int model_count = 0;        /* keep a tally of models estimated */
 
+    /* basic initialization of library */
+    libgretl_init();
+
     logo(); /* print version info and session time */
-    prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL); /* simple printing */
+    prn = gretl_print_new(GRETL_PRINT_STDOUT); /* simple printing */
 
     /* create the datainfo struct and data matrix -- pass in pointer
        to data array; specify the number of variables (allowing one
@@ -126,6 +129,8 @@ int main (void)
     free_datainfo(datainfo);
     gretl_print_destroy(prn);
 
+    libgretl_cleanup();
+
     return 0;
 
 }
