@@ -69,7 +69,7 @@ drag_data_received  (GtkWidget          *widget,
 
 #ifdef USE_GNOME
 static char *optrun, *optdb;
-static int opteng, optdump;
+static int opteng, optbasque, optdump;
 
 static const struct poptOption options[] = {
     { "run", 'r', POPT_ARG_STRING, &optrun, 0, 
@@ -80,6 +80,8 @@ static const struct poptOption options[] = {
       N_("open a remote (web) database on startup"), "REMOTE_DB" },
     { "english", 'e', POPT_ARG_NONE, &opteng, 0, 
       N_("force use of English"), NULL },
+    { "basque", 'q', POPT_ARG_NONE, &optbasque, 0, 
+      N_("force use of Basque"), NULL },
     { "dump", 'c', POPT_ARG_NONE, &optdump, 0, 
       N_("dump gretl configuration to file"), NULL },
     { NULL, '\0', 0, NULL, 0, NULL, NULL },
@@ -91,7 +93,6 @@ DATAINFO *datainfo;
 char *errtext;
 char cmdfile[MAXLEN], scriptfile[MAXLEN];
 char trydatfile[MAXLEN], tryscript[MAXLEN];
-char line[1024];
 PATHS paths;                /* useful paths */
 double **Z;                 /* data set */
 MODEL **models;             /* gretl models structs */
@@ -108,6 +109,7 @@ char dbproxy[21];
 
 char editor[MAXSTR] = "emacs";
 char calculator[MAXSTR] = "xcalc";
+char latex[MAXSTR] = "latex";
 char viewdvi[MAXSTR] = "xdvi";
 char viewps[MAXSTR] = "gv";
 
@@ -125,7 +127,7 @@ char tramodir[MAXSTR] = "";
 
 static void spreadsheet_edit (gpointer p, guint u, GtkWidget *w) 
 {
-    show_spreadsheet(NULL);
+    show_spreadsheet(SHEET_EDIT_DATASET);
 }
 
 static void manual_update_query (gpointer p, guint u, GtkWidget *w)
@@ -160,6 +162,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>" },    
     { N_("/File/Open data/import CSV..."), NULL, open_data, OPEN_CSV, NULL },
     { N_("/File/Open data/import ASCII..."), NULL, open_data, OPEN_ASCII, NULL },
+    { N_("/File/Open data/import Octave..."), NULL, open_data, OPEN_OCTAVE, NULL },
     { N_("/File/Open data/import BOX..."), NULL, open_data, OPEN_BOX, NULL },
     { N_("/File/Open data/import Gnumeric..."), NULL, open_data, 
       OPEN_GNUMERIC, NULL },
@@ -171,6 +174,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/standard format..."), NULL, open_data, APPEND_DATA, NULL },
     { N_("/File/Append data/from CSV..."), NULL, open_data, APPEND_CSV, NULL },
     { N_("/File/Append data/from ASCII..."), NULL, open_data, APPEND_ASCII, NULL },
+    { N_("/File/Append data/from Octave..."), NULL, open_data, APPEND_OCTAVE, NULL },
     { N_("/File/Append data/from Gnumeric..."), NULL, open_data, 
       APPEND_GNUMERIC, NULL },
     { N_("/File/Append data/from Excel..."), NULL, open_data, 
@@ -183,6 +187,8 @@ GtkItemFactoryEntry data_items[] = {
       SAVE_DATA_AS, NULL },
     { N_("/File/Save data as/_gzipped..."), NULL, 
       file_save, SAVE_GZDATA, NULL },
+    { N_("/File/Save data as/_database..."), NULL, 
+      file_save, SAVE_DBDATA, NULL },
 
     /* File, Export data */
     { N_("/File/_Export data"), NULL, NULL, 0, "<Branch>" },
@@ -328,10 +334,10 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>" },
     { N_("/Data/Difference of means/assuming equal variances..."), NULL, 
-      gretl_callback, MEANTEST, NULL },
+      selector_callback, MEANTEST, NULL },
     { N_("/Data/Difference of means/assuming unequal variances..."), NULL, 
-      gretl_callback, MEANTEST2, NULL },
-    { N_("/Data/Difference of variances..."), NULL, gretl_callback, VARTEST, NULL },
+      selector_callback, MEANTEST2, NULL },
+    { N_("/Data/Difference of variances..."), NULL, selector_callback, VARTEST, NULL },
     { N_("/Data/sep5"), NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Add variables"), NULL, NULL, 0, "<Branch>", },
     { N_("/Data/Add variables/time trend"), NULL, add_index, 1, NULL },
@@ -358,6 +364,8 @@ GtkItemFactoryEntry data_items[] = {
       SETSEED, NULL },
     { N_("/Data/Add variables/sep2"), NULL, NULL, 0, "<Separator>" },
     { N_("/Data/Add variables/Define _new variable..."), NULL, gretl_callback, GENR, NULL },
+    { N_("/Data/Add observations..."), NULL, do_add_obs, 0, NULL },
+    { N_("/Data/Remove extra observations"), NULL, do_remove_obs, 0, NULL },
     { N_("/Data/Refresh window"), NULL, refresh_data, 0, NULL },
 
     /* Sample menu */
@@ -380,7 +388,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/Set missing _value code..."), NULL, gretl_callback, 
       GSETMISS, NULL },
     { N_("/Sample/sep4"), NULL, NULL, 0, "<Separator>" },  
-    { N_("/Sample/_Add case markers..."), NULL, gretl_callback, MARKERS, NULL },
+    { N_("/Sample/_Add case markers..."), NULL, open_data, OPEN_MARKERS, NULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL },
     { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>" },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL },
@@ -397,14 +405,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Frequency plot"), NULL, NULL, 0, "<Branch>" },
     { N_("/Variable/Frequency plot/simple"), NULL, do_freqplot, 0, NULL },
     { N_("/Variable/Frequency plot/against Normal"), NULL, do_freqplot, 
-      NORMAL, NULL },
+      DIST_NORMAL, NULL },
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
-      GAMMA, NULL },
+      DIST_GAMMA, NULL },
     { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL }, 
     { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>" },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL },
-    { N_("/Variable/Correlogram"), NULL, gretl_callback, CORRGM, NULL },
+    { N_("/Variable/Correlogram"), NULL, do_corrgm, CORRGM, NULL },
     { N_("/Variable/Spectrum"), NULL, NULL, 0, "<Branch>" },
     { N_("/Variable/Spectrum/sample periodogram"), NULL, do_pergm, 0, NULL }, 
     { N_("/Variable/Spectrum/Bartlett lag window"), NULL, do_pergm, 1, NULL }, 
@@ -446,7 +454,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/Poi_sson..."), NULL, model_callback, POISSON, NULL },
     { N_("/Model/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL },
     { N_("/Model/Least _Absolute Deviation..."), NULL, model_callback, LAD, NULL },
-    { N_("/Model/_Rank correlation..."), NULL, gretl_callback, SPEARMAN, NULL },
+    { N_("/Model/_Rank correlation..."), NULL, selector_callback, SPEARMAN, NULL },
     { N_("/Model/_Pooled OLS (panel)..."), NULL, model_callback, POOLED, NULL },
     { N_("/Model/Nonlinear Least Squares..."), NULL, gretl_callback, NLS, NULL },
 #ifdef ENABLE_GMP
@@ -467,11 +475,12 @@ GtkItemFactoryEntry data_items[] = {
 
 static void gui_usage (void)
 {
-    gui_logo(stdout);
+    gui_logo(NULL);
     printf(I_("You may supply the name of a data file on the command line.\n"));
     printf(I_("Or you may do \"gretl -r script_file\" to open a script.\n"));
     printf(I_("Or you may do \"gretl -d database\" to open a gretl database.\n"));
     printf(I_("You may do \"gretl -e\" to force use of English.\n"));
+    printf(I_("You may do \"gretl -q\" to force use of Basque.\n"));
     exit(0);
 }
 
@@ -551,10 +560,10 @@ void nls_init (void)
 	    !g_strcasecmp(mylang, "C")) return;
     }
 
-    loc = setlocale (LC_ALL, "");
+    loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
-    bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
 }
 
 static void root_check (void)
@@ -571,11 +580,14 @@ static void root_check (void)
     }
 }
 
-static void force_english (void)
+static void force_language (int f)
 {
-    setlocale (LC_ALL, "C");
-
-    force_english_help();
+    if (f == ENGLISH) {
+	setlocale(LC_ALL, "C");
+	force_english_help();
+    } else {
+	setlocale(LC_ALL, "XXX");
+    }
 }
 
 #endif /* ENABLE_NLS */
@@ -605,21 +617,25 @@ int main (int argc, char *argv[])
     gtk_init(&argc, &argv);
 #endif
 
+    libgretl_init();
     set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
-    root_check();
+
     set_rcfile();
+    if (!expert) {
+	root_check();
+    }
     init_fileptrs();
 
     if (argc > 1) {
-	int english = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &english);
+	int force_lang = 0;
+	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
 
 	switch (opt) {
 	case OPT_HELP:
 	    gui_usage();
 	    break;
 	case OPT_VERSION:
-	    gui_logo(stdout);
+	    gui_logo(NULL);
 	    exit(EXIT_SUCCESS);
 	    break;
 	case OPT_RUNIT:
@@ -652,15 +668,16 @@ int main (int argc, char *argv[])
 	}
 
 #ifdef ENABLE_NLS
-	if (english) {
-	    force_english();
+	if (force_lang) {
+	    force_language(force_lang);
 	    if (argc == 2) {
 		gui_get_data = 1;
 	    }
 	}
 #endif
-    } else 
+    } else {
 	gui_get_data = 1;
+    }
 
     strcpy(cmdfile, paths.userdir);
     strcat(cmdfile, "session.inp");
@@ -691,9 +708,10 @@ int main (int argc, char *argv[])
 	int ftype;
 	PRN *prn; 
 
-	prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
-	if (prn == NULL) 
+	prn = gretl_print_new(GRETL_PRINT_STDERR);
+	if (prn == NULL) {
 	    exit(EXIT_FAILURE);
+	}
 
 	strcpy(paths.datfile, filearg);
 
@@ -712,7 +730,10 @@ int main (int argc, char *argv[])
 			      prn, 0);
 	    break;
 	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, &paths, prn);
+	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
+	    break;
+	case GRETL_OCTAVE:
+	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
 	    break;
 	case GRETL_BOX_DATA:
 	    err = import_box(&Z, &datainfo, paths.datfile, prn);
@@ -816,8 +837,11 @@ int main (int argc, char *argv[])
     free(models);
 
     library_command_free();
+    libgretl_cleanup();
 
-    if (data_status) free_datainfo(datainfo);
+    if (data_status) {
+	free_datainfo(datainfo);
+    }
 
     free_command_stack();
     exit_free_modelspec();
@@ -939,7 +963,7 @@ void populate_varlist (void)
 
     j = 0;
     for (i=0; i<datainfo->v; i++) {
-	if (hidden_var(i, datainfo)) continue;
+	if (is_hidden_variable(i, datainfo)) continue;
 	sprintf(id, "%d", i);
 	row[0] = id;
 	row[1] = datainfo->varname[i];
@@ -1200,14 +1224,13 @@ int restore_sample (gretlopt opt)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    restore_sample(OPT_NONE);
+    int err = restore_sample(OPT_NONE); /* FIXME OPT_C? */
 
-    if (verbose) {
+    if (verbose && !err) {
 	infobox(_("Full sample range restored"));
 	set_sample_label(datainfo);    
-	restore_sample_state(FALSE);
-	strcpy(line, "smpl full");
-	verify_and_record_command(line);
+	gretl_command_strcpy("smpl full");
+	check_and_record_command();
     }
 }
 
