@@ -22,6 +22,7 @@
 #include "gretl.h"
 #include "selector.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 static int default_var;
 static int *xlist;
@@ -39,6 +40,7 @@ struct _selector {
     GtkWidget *default_check;
     GtkWidget *extra;
     GtkWidget *extra2;
+    GtkWidget *add_button;
     int code;
     int error;
     gretlopt opts;
@@ -46,8 +48,14 @@ struct _selector {
     gpointer data;
 };
 
-#define WANT_TOGGLES(c) (c == OLS || c == TOBIT || c == ARMA || \
-                         c == GARCH || c == COINT2 || c == TSLS || \
+#define WANT_TOGGLES(c) (c == ARMA || \
+                         c == COINT || \
+                         c == COINT2 || \
+                         c == GARCH || \
+                         c == HILU || \
+                         c == OLS || \
+                         c == TOBIT || \
+                         c == TSLS || \
                          c == VAR)
 
 void clear_selector (void)
@@ -167,30 +175,52 @@ static void add_auxvar_callback (GtkWidget *w, selector *sr)
 	g_list_foreach(mylist, (GFunc) add_auxvar, sr);
 }
 
+static int selection_at_max (selector *sr, int nsel)
+{
+    int ret = 0;
+
+    if (TWO_VARS_CODE(sr->code) && nsel == 2) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 static void add_var_on_right (gint i, selector *sr)
 {
     gchar *row[2];
     gint j, rows;
     gint already_there = 0;
+    gint at_max = 0;
 
     if (!GTK_IS_CLIST(sr->rightvars)) return;
 
     rows = GTK_CLIST(sr->rightvars)->rows;
 
     gtk_clist_get_text(GTK_CLIST(sr->varlist), i, 0, &row[0]);
+
     for (j=0; j<rows; j++) {
 	gchar *test;
 
+	if (selection_at_max(sr, j + 1)) {
+	    at_max = 1; 
+	    break;
+	}	    
 	gtk_clist_get_text(GTK_CLIST(sr->rightvars), j, 0, &test);
 	if (!strcmp(test, row[0])) {
 	    already_there = 1; 
 	    break;
 	}
     }
-    if (!already_there) {
+
+    if (!already_there && !at_max) {
 	gtk_clist_get_text(GTK_CLIST(sr->varlist), i, 1, &row[1]);
 	gtk_clist_append(GTK_CLIST(sr->rightvars), row);
     }
+
+    if (sr->add_button != NULL && at_max) {
+	gtk_widget_set_sensitive(sr->add_button, FALSE);
+    }
 }
 
 static void add_all_to_right_callback (GtkWidget *w, selector *sr)
@@ -203,8 +233,9 @@ static void add_all_to_right_callback (GtkWidget *w, selector *sr)
     gtk_clist_select_all(GTK_CLIST(sr->varlist));
     mylist = GTK_CLIST(sr->varlist)->selection;
 
-    if (mylist != NULL) 
+    if (mylist != NULL) {
 	g_list_foreach(mylist, (GFunc) add_var_on_right, sr);
+    }
 }
 
 static void add_to_right_callback (GtkWidget *w, selector *sr)
@@ -216,8 +247,33 @@ static void add_to_right_callback (GtkWidget *w, selector *sr)
 
     mylist = GTK_CLIST(sr->varlist)->selection;
 
-    if (mylist != NULL) 
+    if (mylist != NULL) {
 	g_list_foreach(mylist, (GFunc) add_var_on_right, sr);
+    }
+}
+
+static void set_vars_from_main (selector *sr)
+{
+    GList *mylist = GTK_CLIST(mdata->listbox)->selection;
+
+    if (mylist != NULL) {
+	g_list_foreach(mylist, (GFunc) add_var_on_right, sr);
+    }
+}
+
+static void set_single_var (selector *sr, int v)
+{
+    gchar *row[2];
+    gchar *vstr;
+
+    vstr = g_strdup_printf("%d", v);
+
+    row[0] = vstr;
+    row[1] = datainfo->varname[v];
+
+    gtk_clist_append(GTK_CLIST(sr->rightvars), row);
+    
+    g_free(vstr);
 }
 
 static void remove_right_var (gint i, selector *sr)
@@ -231,6 +287,14 @@ static void remove_from_right_callback (GtkWidget *w, selector *sr)
     mylist = g_list_sort(mylist, list_sorter);
 
     g_list_foreach(mylist, (GFunc) remove_right_var, sr);
+
+    if (sr->add_button != NULL && !GTK_WIDGET_SENSITIVE(sr->add_button)) {
+	int nsel = GTK_CLIST(sr->rightvars)->rows;
+
+	if (!selection_at_max(sr, nsel)) {
+	    gtk_widget_set_sensitive(sr->add_button, TRUE);
+	}
+    }
 }
 
 static void remove_auxvar (gint i, selector *sr)
@@ -260,6 +324,9 @@ static void clear_vars (GtkWidget *w, selector *sr)
 	gtk_entry_set_text(GTK_ENTRY(sr->rightvars), "");
     } else {
 	gtk_clist_clear(GTK_CLIST(sr->rightvars));
+	if (sr->add_button != NULL) {
+	    gtk_widget_set_sensitive(sr->add_button, TRUE);
+	}
     }
 
     if (MODEL_CODE(sr->code)) {
@@ -952,6 +1019,7 @@ static void selector_init (selector *sr, guint code, const char *title)
     sr->auxvars = NULL;
     sr->default_check = NULL;
     sr->extra = NULL;
+    sr->add_button = NULL;
     sr->cmdlist = NULL;
     sr->data = NULL;
 
@@ -1007,6 +1075,24 @@ static void verbose_callback (GtkWidget *w,  selector *sr)
     }
 }
 
+static void corc_callback (GtkWidget *w,  selector *sr)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	sr->opts &= ~OPT_B;
+    } else {
+	sr->opts |= OPT_B;
+    }
+}
+
+static void engle_granger_callback (GtkWidget *w,  selector *sr)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	sr->opts &= ~OPT_N;
+    } else {
+	sr->opts |= OPT_N;
+    }
+}
+
 static void build_pq_spinners (selector *sr)
 {
     GtkWidget *hbox, *tmp;
@@ -1108,6 +1194,36 @@ build_selector_switches (selector *sr)
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
 	gtk_widget_show(tmp);
 
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sr->dlg)->vbox),
+			   hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+    } else if (sr->code == HILU) {
+	tmp = gtk_check_button_new_with_label
+	    (_("Fine-tune using Cochrane-Orcutt"));
+	gtk_signal_connect(GTK_OBJECT(tmp), "toggled",
+			   GTK_SIGNAL_FUNC(corc_callback), sr);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+	gtk_widget_show(tmp);
+	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sr->dlg)->vbox),
+			   hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+    } else if (sr->code == COINT) {
+	tmp = gtk_check_button_new_with_label
+	    (_("Cointegrating regression includes a constant"));
+	g_signal_connect(G_OBJECT(tmp), "toggled",
+			 G_CALLBACK(engle_granger_callback), sr);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+	gtk_widget_show(tmp);
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sr->dlg)->vbox),
 			   hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
@@ -1222,7 +1338,7 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
 	gchar id[5];
 
 	if (i == 0 && !MODEL_CODE(cmdcode)) continue;
-        if (hidden_var(i, datainfo)) continue;
+        if (is_hidden_variable(i, datainfo)) continue;
 	if (screen_scalar(i, cmdcode)) continue;
 	sprintf(id, "%d", i);
 	row[0] = id;
@@ -1397,6 +1513,11 @@ static char *get_topstr (int cmdnum)
 	return N_("Select variables to omit");
     case COEFFSUM:
 	return N_("Select coefficients to sum");
+    case SPEARMAN:
+    case MEANTEST:
+    case MEANTEST2:
+    case VARTEST:
+	return N_("Select two variables");
     case PRINT:
 	return N_("Select variables to display");
     case GR_PLOT: 
@@ -1419,7 +1540,12 @@ static void add_omit_list (gpointer p, selector *sr)
 	    gchar *row[2];
 	    gchar id[5];
 
-	    if (pmod->list[i] == 0) continue;
+	    if (pmod->list[i] == 0) {
+		continue;
+	    }
+	    if (pmod->list[i] == LISTSEP) {
+		break;
+	    }
 	    sprintf(id, "%d", pmod->list[i]);
 	    row[0] = id;
 	    row[1] = datainfo->varname[pmod->list[i]];
@@ -1465,16 +1591,19 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     GtkWidget *left_vbox, *mid_vbox, *right_vbox, *tmp;
     GtkWidget *top_hbox, *big_hbox, *scroller;
     selector *sr;
-    int i;
+    int i, vnum = 0;
 
     open_dialog = get_open_dialog();
+
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
     }
 
     sr = mymalloc(sizeof *sr);
-    if (sr == NULL) return;
+    if (sr == NULL) {
+	return;
+    }
 
     selector_init(sr, cmdcode, title);
 
@@ -1523,17 +1652,25 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     if (cmdcode == OMIT || cmdcode == ADD || cmdcode == COEFFSUM) {
         add_omit_list(p, sr);
     } else {
+	int nleft = 0;
+
 	for (i=1; i<datainfo->v; i++) {
 	    gchar *row[2];
 	    gchar id[5];
 
-	    if (hidden_var(i, datainfo)) continue;
-	    if (screen_scalar(i, cmdcode)) continue;
+	    if (is_hidden_variable(i, datainfo) || screen_scalar(i, cmdcode)) {
+		continue;
+	    }
 	    sprintf(id, "%d", i);
 	    row[0] = id;
 	    row[1] = datainfo->varname[i];
 	    gtk_clist_append(GTK_CLIST(sr->varlist), row);
+	    vnum = i;
+	    nleft++;
 	}
+	if (nleft != 1) {
+	    vnum = 0;
+	}	
     }
 
     gtk_clist_set_column_width (GTK_CLIST(sr->varlist), 1, 80 * gui_scale);
@@ -1553,13 +1690,13 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     /* middle: vertical holder for push/pull buttons */
     mid_vbox = gtk_vbox_new(FALSE, 5);
 
-    tmp = gtk_button_new_with_label (_("Select ->"));
-    gtk_box_pack_start(GTK_BOX(mid_vbox), tmp, TRUE, FALSE, 0);
-    gtk_signal_connect (GTK_OBJECT(tmp), "clicked", 
+    sr->add_button = gtk_button_new_with_label (_("Select ->"));
+    gtk_box_pack_start(GTK_BOX(mid_vbox), sr->add_button, TRUE, FALSE, 0);
+    gtk_signal_connect (GTK_OBJECT(sr->add_button), "clicked", 
                         GTK_SIGNAL_FUNC(add_to_right_callback), sr);
-    gtk_widget_show(tmp);
+    gtk_widget_show(sr->add_button);
 
-    if (p == NULL) {
+    if (p == NULL && !TWO_VARS_CODE(sr->code)) {
 	tmp = gtk_button_new_with_label (_("All ->"));
 	gtk_box_pack_start(GTK_BOX(mid_vbox), tmp, TRUE, FALSE, 0);
 	gtk_signal_connect (GTK_OBJECT(tmp), "clicked", 
@@ -1607,10 +1744,16 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     /* buttons: "OK", Clear, Cancel, Help */
     build_selector_buttons(sr, okfunc);
 
+    if (TWO_VARS_CODE(sr->code) && mdata_selection_count() == 2) {
+	set_vars_from_main(sr);
+    } else if (SAVE_DATA_ACTION(sr->code) && vnum > 0) {
+	set_single_var(sr, vnum);
+    }
+
     gtk_widget_show(sr->dlg);
 
     if (SAVE_DATA_ACTION(sr->code)) {
-	gtk_window_set_modal(GTK_WINDOW(sr->dlg), TRUE);
+	gretl_set_window_modal(sr->dlg);
     }
 }
 
@@ -1685,7 +1828,7 @@ static void selection_add_item (gint i, struct list_maker *lmkr)
     }
 }
 
-char *mdata_selection_to_string (int n_required)
+char *main_window_selection_as_string (void)
 {
     GList *mylist = GTK_CLIST(mdata->listbox)->selection;
     struct list_maker lmkr;    
@@ -1706,17 +1849,6 @@ char *mdata_selection_to_string (int n_required)
 	lmkr.liststr[0] = 0;
     }
 
-    if (n_required && lmkr.n_items != n_required) {
-	gchar *msg;
-
-	msg = g_strdup_printf(_("Please select %d variables first"),
-			      n_required);
-	errbox(msg);
-	g_free(msg);
-	free(lmkr.liststr);
-	lmkr.liststr = NULL;
-    }
-
     return lmkr.liststr;
 }
 
@@ -1729,7 +1861,13 @@ int selector_code (const selector *sr)
 
 const char *selector_list (const selector *sr)
 {
-    return sr->cmdlist;
+    const char *ret = NULL;
+
+    if (sr->cmdlist != NULL && *sr->cmdlist != '\0') {
+	ret = sr->cmdlist;
+    }
+
+    return ret;
 }
 
 int selector_list_hasconst (const selector *sr)
