@@ -27,6 +27,7 @@
 
 #include "ssheet.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 #define DEFAULT_PRECISION 6
 #define DEFAULT_SPACE 8
@@ -40,12 +41,15 @@ static GtkWidget *sheet_popup;
 static char newvarname[9], newobsmarker[9];
 static int numcolumns, numrows;
 static int sheet_modified;
+static int origvars;
+static int orig_main_v;
 
 static void sheet_add_var (void);
 static void sheet_add_obs (void);
 static void sheet_insert_obs (void);
 static void sheet_clear (gpointer *data, guint all, GtkWidget *w);
 static void get_data_from_sheet (void);
+static void set_dataset_locked (gboolean s);
 
 static GtkItemFactoryEntry sheet_items[] = {
     { N_("/_Observation"), NULL, NULL, 0, "<Branch>" },
@@ -58,6 +62,26 @@ static GtkItemFactoryEntry sheet_items[] = {
     { N_("/_Clear/_All data"), NULL, sheet_clear, 1, NULL }
 };
 
+static void disable_obs_menu (GtkItemFactory *ifac)
+{
+    GtkWidget *w = gtk_item_factory_get_item(ifac, "/Observation");
+
+    gtk_widget_set_sensitive(w, FALSE);
+}
+
+static int spreadsheet_hide (int i, const DATAINFO *pdinfo)
+{
+    int ret = 0;
+
+    if (pdinfo->vector[i] == 0) {
+	ret = 1;
+    } else if (is_hidden_variable(i, pdinfo)) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 /* ........................................................... */
 
 static int check_data_in_sheet (void)
@@ -88,9 +112,11 @@ static int check_data_in_sheet (void)
 
 static void name_new_var (GtkWidget *widget, dialog_t *ddata) 
 {
-    const char *buf = dialog_data_get_text(ddata);
+    const char *buf = edit_dialog_get_text(ddata);
 
-    if (buf == NULL || validate_varname(buf)) return;
+    if (buf == NULL || validate_varname(buf)) {
+	return;
+    }
 
     *newvarname = '\0';
     strncat(newvarname, buf, 8);
@@ -98,11 +124,9 @@ static void name_new_var (GtkWidget *widget, dialog_t *ddata)
     close_dialog(ddata);
 }
 
-/* ........................................................... */
-
 static void name_new_obs (GtkWidget *widget, dialog_t *ddata) 
 {
-    const char *buf = dialog_data_get_text(ddata);
+    const char *buf = edit_dialog_get_text(ddata);
 
     if (buf == NULL) return;
 
@@ -112,26 +136,22 @@ static void name_new_obs (GtkWidget *widget, dialog_t *ddata)
     close_dialog(ddata);
 }
 
-/* ........................................................... */
-
 static void name_var_dialog (void) 
 {
     edit_dialog (_("gretl: name variable"), 
 		 _("Enter name for new variable\n"
 		 "(max. 8 characters)"),
 		 NULL, name_new_var, mdata, 
-		 0, 0);
+		 0, 0, 1);
 }
 
-/* ........................................................... */
-
 static void new_case_dialog (void) 
 {
     edit_dialog (_("gretl: case marker"), 
 		 _("Enter case marker for new obs\n"
 		 "(max. 8 characters)"),
 		 NULL, name_new_obs, mdata, 
-		 0, 0);
+		 0, 0, 1);
 }
 
 /* ........................................................... */
@@ -141,45 +161,50 @@ static void sheet_add_var (void)
     GtkSheet *sheet = GTK_SHEET(gretlsheet); 
 
     *newvarname = '\0';
-
+    
     name_var_dialog();
 
     if (*newvarname != '\0') {
 	gtk_sheet_add_column(sheet, 1);
+	gtk_sheet_column_button_add_label(sheet, numcolumns, newvarname);
+	gtk_sheet_set_column_title(sheet, numcolumns, newvarname);
+	gtk_sheet_set_active_cell(sheet, 0, numcolumns);
+	gtk_sheet_moveto(sheet, 0, numcolumns, 0.0, 0.8);
 	numcolumns++;
-	gtk_sheet_column_button_add_label(sheet, numcolumns-1, newvarname);
-	gtk_sheet_set_column_title(sheet, numcolumns-1, newvarname);
-	gtk_sheet_set_active_cell(sheet, 0, numcolumns-1);
-	gtk_sheet_moveto(sheet, 0, numcolumns-1, 0.0, 0.8);
 	sheet_modified = 1;
     }
 }
 
-/* ........................................................... */
-
 static void sheet_add_obs (void)
 {
     GtkSheet *sheet = GTK_SHEET(gretlsheet);
     char rowlabel[OBSLEN];
+    int i, n = 1;
 
     *newobsmarker = '\0';
 
-    if (datainfo->markers) new_case_dialog();
+    if (datainfo->markers) {
+	new_case_dialog();
+    } else {
+	n = add_obs_dialog(NULL, 1);
+    }
+
+    if (n <= 0) return;
 
     if (!datainfo->markers || *newobsmarker != '\0') {
-	gtk_sheet_add_row(sheet, 1);
-	numrows++;
-	get_full_obs_string(rowlabel, numrows-1, datainfo);
-	gtk_sheet_row_button_add_label(sheet, numrows-1, rowlabel);
-	gtk_sheet_set_row_title(sheet, numrows-1, rowlabel);
-	gtk_sheet_set_active_cell(sheet, numrows-1, 0);
-	gtk_sheet_moveto(sheet, numrows-1, 0, 0.8, 0.0);
+	for (i=0; i<n; i++) {
+	    gtk_sheet_add_row(sheet, 1);
+	    get_full_obs_string(rowlabel, numrows + i, datainfo);
+	    gtk_sheet_row_button_add_label(sheet, numrows + i, rowlabel);
+	    gtk_sheet_set_row_title(sheet, numrows + i, rowlabel);
+	}
+	gtk_sheet_set_active_cell(sheet, numrows, 0);
+	gtk_sheet_moveto(sheet, numrows, 0, 0.8, 0.0);
+	numrows += n;
 	sheet_modified = 1;
     }
 }
 
-/* ........................................................... */
-
 static void sheet_insert_obs (void)
 {
     GtkSheet *sheet = GTK_SHEET(gretlsheet);
@@ -188,7 +213,9 @@ static void sheet_insert_obs (void)
 
     newobsmarker[0] = '\0';
 
-    if (datainfo->markers) new_case_dialog();
+    if (datainfo->markers) {
+	new_case_dialog();
+    }
 
     if (!datainfo->markers || *newobsmarker != '\0') {
 	gtk_sheet_insert_rows(sheet, sheet->active_cell.row, 1);
@@ -410,53 +437,73 @@ static gint activate_sheet_cell (GtkWidget *w, gint row, gint column,
 
 /* ........................................................... */
 
+static int var_added_since_ssheet_opened (int i, int main_v)
+{
+    return (i >= orig_main_v && i < main_v);
+}
+
 static void get_data_from_sheet (void)
 {
-    int i, t, n = datainfo->n, oldv = datainfo->v; 
-    int newvars = numcolumns - oldv + 1, newobs = numrows - n;
+    int oldv = datainfo->v;
+    int newvars = numcolumns - origvars;
+    int newobs = numrows - datainfo->n;
     int missobs = 0;
+
     gchar *celltext;
     GtkSheet *sheet = GTK_SHEET(gretlsheet);
 
-    if (check_data_in_sheet()) return;
+    int i, colnum, t;
+
+    if (check_data_in_sheet()) {
+	return;
+    }
+
+    if (!sheet_modified) {
+	infobox(_("No changes were made"));
+	return;
+    }
 
     if (newobs > 0) {
-	if (grow_nobs(newobs, &Z, datainfo)) {
+	if (dataset_add_observations(newobs, &Z, datainfo) ||
+	    dataset_destroy_hidden_variables(&Z, datainfo)) {
 	    errbox(_("Failed to allocate memory for new data"));
 	    return;
 	}
-	n = datainfo->n;
     }
 
     if (newvars > 0) {
-	if (dataset_add_vars(newvars, &Z, datainfo)) {
+	if (dataset_add_series(newvars, &Z, datainfo)) {
 	    errbox(_("Failed to allocate memory for new data"));
 	    return;
 	}
 	for (i=0; i<newvars; i++) { 
-	    strcpy(datainfo->varname[i+oldv], 
-		   sheet->column[i+oldv-1].button.label);
-	    strcpy(VARLABEL(datainfo, i + oldv), "");
+	    colnum = origvars + i; 
+	    strcpy(datainfo->varname[oldv + i], sheet->column[colnum].button.label);
 	}
     }
 
-    for (i=0; i<datainfo->v-1; i++) {
-	for (t=0; t<n; t++) {
-	    if (datainfo->vector[i+1] == 0 && t > 0) {
-		continue;
-	    }
-	    celltext = gtk_sheet_cell_get_text(sheet, t, i);
+    colnum = 0;
+    for (i=1; i<datainfo->v; i++) {
+	if (spreadsheet_hide(i, datainfo)) {
+	    continue;
+	}
+	if (var_added_since_ssheet_opened(i, oldv)) {
+	    continue;
+	}
+	for (t=0; t<datainfo->n; t++) {
+	    celltext = gtk_sheet_cell_get_text(sheet, t, colnum);
 	    if (celltext != NULL && *celltext != 0) {
-		Z[i+1][t] = atof(celltext);
+		Z[i][t] = atof(celltext);
 	    } else {
-		Z[i+1][t] = NADBL;
+		Z[i][t] = NADBL;
 		missobs = 1;
 	    }
 	}
+	colnum++;
     }
 
     if (datainfo->markers && datainfo->S != NULL) {
-	for (t=0; t<n; t++) {
+	for (t=0; t<datainfo->n; t++) {
 	    strcpy(datainfo->S[t], sheet->row[t].button.label); 
 	}
     }
@@ -478,66 +525,79 @@ static void get_data_from_sheet (void)
 static void add_data_to_sheet (GtkWidget *w)
 {
     gchar label[32], rowlabel[10];
-    gint i, t, n = datainfo->n;
     GtkSheet *sheet = GTK_SHEET(w);
-    double xx;
+    int i, j, t;
 
-    /* row and column buttons */
-    for (i=0; i<datainfo->v-1; i++) {
-	gtk_sheet_column_button_add_label(sheet, i, datainfo->varname[i+1]);
-	gtk_sheet_set_column_title(sheet, i, datainfo->varname[i+1]);
+    numrows = datainfo->n;
+    numcolumns = j = 0;
+
+    for (i=1; i<datainfo->v; i++) {
+	if (spreadsheet_hide(i, datainfo)) {
+	    continue;
+	}
+	gtk_sheet_column_button_add_label(sheet, j, datainfo->varname[i]);
+	gtk_sheet_set_column_title(sheet, j, datainfo->varname[i]);
+	j++;
     }
-    numcolumns = i;
 
-    for (t=0; t<n; t++) {
+    numcolumns = origvars = j;
+
+    for (t=0; t<datainfo->n; t++) {
 	get_full_obs_string(rowlabel, t, datainfo);
 	gtk_sheet_row_button_add_label(sheet, t, rowlabel);
 	gtk_sheet_set_row_title(sheet, t, rowlabel);
     }
-    numrows = t;
 
-    /* enter the data values */
-    for (t=0; t<n; t++) {
-	for (i=0; i<datainfo->v-1; i++) {
-	    if (datainfo->vector[i+1] == 0 && t > 0) continue;
-	    xx = Z[i+1][t];
-	    if (!na(xx)) {
-		sprintf(label, "%.*f", DEFAULT_PRECISION, Z[i+1][t]);
+    j = 0;
+    for (i=1; i<datainfo->v; i++) {
+	if (spreadsheet_hide(i, datainfo)) {
+	    continue;
+	}
+	for (t=0; t<datainfo->n; t++) {
+	    if (!na(Z[i][t])) {
+		sprintf(label, "%.*f", DEFAULT_PRECISION, Z[i][t]);
 	    } else {
 		*label = 0;
 	    }
-	    gtk_sheet_set_cell(sheet, t, i, GTK_JUSTIFY_RIGHT, label); 
+	    gtk_sheet_set_cell(sheet, t, j, GTK_JUSTIFY_RIGHT, label); 
 	}
+	j++;
     }
 
+    orig_main_v = datainfo->v;
+    
     sheet_modified = 0;
 }
 
-/* ........................................................... */
-
 static void add_skel_to_sheet (GtkWidget *w)
 {
     GtkSheet *sheet = GTK_SHEET(w);
     gchar rowlabel[10];
-    gint i, t, n = datainfo->n;
+    int i, j, t;
 
-    for (i=0; i<datainfo->v-1; i++) {
-	gtk_sheet_column_button_add_label(sheet, i, datainfo->varname[i+1]);
-	gtk_sheet_set_column_title(sheet, i, datainfo->varname[i+1]);
+    numrows = datainfo->n;
+    numcolumns = j = 0;
+
+    for (i=1; i<datainfo->v; i++) {
+	gtk_sheet_column_button_add_label(sheet, j, datainfo->varname[i]);
+	gtk_sheet_set_column_title(sheet, j, datainfo->varname[i]);
+	j++;
     }
-    numcolumns = i;
 
-    for (t=0; t<n; t++) {
+    numcolumns = origvars = j;
+
+    for (t=0; t<datainfo->n; t++) {
 	get_full_obs_string(rowlabel, t, datainfo);
 	gtk_sheet_row_button_add_label(sheet, t, rowlabel);
 	gtk_sheet_set_row_title(sheet, t, rowlabel);
     }
-    numrows = t;
 
-    for (i=0; i<datainfo->v-1; i++) {
-	for (t=0; t<n; t++) {
-	    gtk_sheet_set_cell(sheet, t, i, GTK_JUSTIFY_RIGHT, ""); 
+    j = 0;
+    for (i=1; i<datainfo->v; i++) {
+	for (t=0; t<datainfo->n; t++) {
+	    gtk_sheet_set_cell(sheet, t, j, GTK_JUSTIFY_RIGHT, ""); 
 	}
+	j++;
     }
 
     sheet_modified = 0;
@@ -548,6 +608,7 @@ static void add_skel_to_sheet (GtkWidget *w)
 static void free_spreadsheet (GtkWidget *widget, gpointer data) 
 {
     gretlsheet = NULL;
+    set_dataset_locked(FALSE);
 }
 
 /* ........................................................... */
@@ -571,7 +632,7 @@ static void maybe_exit_sheet (GtkWidget *w, GtkWidget *swin)
 
 /* ........................................................... */
 
-void show_spreadsheet (DATAINFO *pdinfo) 
+void show_spreadsheet (SheetCode code) 
 {
     GtkWidget *tmp, *button_box;
     GtkWidget *scroller, *main_vbox;
@@ -584,7 +645,7 @@ void show_spreadsheet (DATAINFO *pdinfo)
 	return;
     }
 
-    if (pdinfo == NULL && datainfo->v == 1) {
+    if (datainfo->v == 1) {
 	errbox(_("Please add a variable to the dataset first"));
 	return;
     }
@@ -608,6 +669,11 @@ void show_spreadsheet (DATAINFO *pdinfo)
     gtk_item_factory_create_items(ifac, 
 				  sizeof sheet_items / sizeof sheet_items[0],
 				  sheet_items, sheetwin);
+
+    if (complex_subsampled()) {
+	disable_obs_menu(ifac);
+    }
+
     mbar = gtk_item_factory_get_widget(ifac, "<main>");
     gtk_accel_group_attach(accel, GTK_OBJECT (sheetwin));
     gtk_box_pack_start(GTK_BOX(main_vbox), mbar, FALSE, TRUE, 0);
@@ -677,12 +743,31 @@ void show_spreadsheet (DATAINFO *pdinfo)
 
     GTK_SHEET_SET_FLAGS(gretlsheet, GTK_SHEET_AUTORESIZE);
 
-    if (pdinfo != NULL) {
-	add_skel_to_sheet(gretlsheet);
-    } else {
+    if (code == SHEET_EDIT_DATASET) {
 	add_data_to_sheet(gretlsheet);
+    } else {
+	add_skel_to_sheet(gretlsheet);
     }
 
     gtk_widget_show(sheetwin);
+    set_dataset_locked(TRUE);
 }
 
+/* mechanism for locking dataset against changes while editing */
+
+static int locked;
+
+static void set_dataset_locked (gboolean s)
+{
+    locked = s;
+    flip(mdata->ifac, "/Sample", !s);
+}
+
+int dataset_locked (void)
+{
+    if (locked) {
+	errbox(_("You can't do this while editing the dataset"));
+    }
+
+    return locked;
+}
