@@ -1058,7 +1058,7 @@ int boxplots (int *list, char **bools, double ***pZ, const DATAINFO *pdinfo,
 	    sprintf(errtext, _("Dropping %s: insufficient observations"),
 		    pdinfo->varname[list[i+1]]);
 	    errbox(errtext);
-	    list_exclude(i+1, list);
+	    gretl_list_delete_at_pos(list, i+1);
 	    if (list[0] == 0) {
 		free(plotgrp->plots);
 		free(plotgrp);
@@ -1777,7 +1777,7 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
 	    char formula[80];
 	    
 	    sprintf(formula, "bool_%d = %s", i-1, bools[i-1]);
-	    err = generate(pZ, pdinfo, formula, NULL);
+	    err = generate(formula, pZ, pdinfo, NULL, OPT_P);
 	    if (err) {
 		char errtxt[128];
 
@@ -1809,7 +1809,7 @@ int boolean_boxplots (const char *str, double ***pZ, DATAINFO *pdinfo,
     free(s);
 
     if (nbool) {
-	dataset_drop_vars(nbool, pZ, pdinfo);
+	dataset_drop_last_variables(nbool, pZ, pdinfo);
     }
     
     return err;
