@@ -214,6 +214,10 @@ void unselectrow (GtkCList *clist, gint row, gint column,
 
 void open_data (gpointer data, guint code, GtkWidget *widget)
 {
+    if (dataset_locked()) {
+	return;
+    }
+
     switch (code) {
     case OPEN_DATA:
     case APPEND_DATA:
@@ -226,6 +230,10 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
 	delimiter_dialog();
 	file_selector(_("Open CSV file"), code, NULL);
 	break;
+    case OPEN_OCTAVE:
+    case APPEND_OCTAVE:
+	file_selector(_("Open Octave file"), code, NULL);
+	break;
     case OPEN_BOX:
 	file_selector(_("Open BOX file"), code, NULL);
 	break;
@@ -237,6 +245,9 @@ void open_data (gpointer data, guint code, GtkWidget *widget)
     case APPEND_EXCEL:
 	file_selector(_("Open Excel file"), code, NULL);
 	break;
+    case OPEN_MARKERS:
+	file_selector(_("gretl: add markers"), code, NULL);
+	break;
     default:
 	errbox("Unrecognized data code");
 	break;
@@ -258,26 +269,27 @@ void open_script (gpointer data, guint action, GtkWidget *widget)
 
 void file_save (gpointer data, guint file_code, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
 
     switch (file_code) {
     case SAVE_OUTPUT:
-	file_selector(_("Save output file"), SAVE_OUTPUT, mydata);
+	file_selector(_("Save output file"), SAVE_OUTPUT, vwin);
 	break;
     case SAVE_CONSOLE:
-	file_selector(_("Save console output"), SAVE_CONSOLE, mydata);
+	file_selector(_("Save console output"), SAVE_CONSOLE, vwin);
 	break;
     case SAVE_CMDS: 
-	file_selector(_("Save command log"), SAVE_CMDS, mydata);
+	file_selector(_("Save command log"), SAVE_CMDS, vwin);
 	break;
     case SAVE_SCRIPT:
-	file_selector(_("Save command script"), SAVE_SCRIPT, mydata);
+	file_selector(_("Save command script"), SAVE_SCRIPT, vwin);
 	break;
     case SAVE_DATA:
     case SAVE_DATA_AS:
     case SAVE_GZDATA:
     case SAVE_BIN1:
     case SAVE_BIN2:
+    case SAVE_DBDATA:
 	data_save_selection_wrapper(file_code);
 	break;
     case EXPORT_CSV:
@@ -288,17 +300,14 @@ void file_save (gpointer data, guint file_code, GtkWidget *widget)
     case EXPORT_DAT:
 	data_save_selection_wrapper(file_code);
 	break;
-    case SAVE_TEX_TAB:
-    case SAVE_TEX_EQ:
-    case SAVE_TEX_TAB_FRAG:
-    case SAVE_TEX_EQ_FRAG:
-	file_selector(_("Save LaTeX file"), file_code, mydata->data);
+    case SAVE_TEX:
+	file_selector(_("Save LaTeX file"), file_code, vwin->data);
 	break;
     case SAVE_MODEL:
-	file_selector(_("Save model output"), file_code, mydata);
+	file_selector(_("Save model output"), file_code, vwin);
 	break;
     case SAVE_GP_CMDS:
-	file_selector(_("Save gnuplot commands"), file_code, mydata);
+	file_selector(_("Save gnuplot commands"), file_code, vwin);
 	break;
     default:
 	dummy_call();
@@ -320,7 +329,8 @@ void print_report (gpointer data, guint unused, GtkWidget *widget)
 
     if (bufopen(&prn)) return;
 
-    data_report (datainfo, &paths, prn);
+    data_report(datainfo, &paths, prn);
+
     view_buffer(prn, 77, 400, _("gretl: data summary"), 
 		DATA_REPORT, NULL);
 }
@@ -347,8 +357,6 @@ void fit_resid_callback (gpointer data, guint code, GtkWidget *widget)
     add_fit_resid(pmod, code, 0);
 }
 
-/* ........................................................... */
-
 void var_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
 {
     windata_t *mydata = (windata_t *) data; 
@@ -357,8 +365,6 @@ void var_resid_callback (gpointer data, guint eqnum, GtkWidget *widget)
     add_var_resid(var, eqnum);
 }
 
-/* ........................................................... */
-
 void model_stat_callback (gpointer data, guint which, GtkWidget *widget)
 {
     windata_t *mydata = (windata_t *) data; 
@@ -367,15 +373,11 @@ void model_stat_callback (gpointer data, guint which, GtkWidget *widget)
     add_model_stat(pmod, which);
 }
 
-/* ........................................................... */
-
 void model_callback (gpointer data, guint model_code, GtkWidget *widget) 
 {
-    selection_dialog (_("gretl: specify model"), do_model, model_code);
+    selection_dialog(_("gretl: specify model"), do_model, model_code);
 }
 
-/* ........................................................... */
-
 #ifdef ENABLE_GMP
 void mp_ols_callback (gpointer data, guint model_code, GtkWidget *widget)
 {
@@ -383,69 +385,25 @@ void mp_ols_callback (gpointer data, guint model_code, GtkWidget *widget)
 }
 #endif /* ENABLE_GMP */
 
-/* ........................................................... */
-
-void model_test_callback (gpointer data, guint action, GtkWidget *widget)
+void model_genr_callback (gpointer data, guint u, GtkWidget *widget)
 {
-    char title[36], query[MAXLABEL], defstr[MAXLEN];
-    char startdate[OBSLEN], enddate[OBSLEN];
-    void (*okfunc)() = NULL;
-    guint varclick = VARCLICK_NONE;
     windata_t *mydata = (windata_t *) data;
 
-    *defstr = '\0';
-
-    switch (action) {
-    case ARCH:
-	strcpy(title, _("gretl: ARCH test"));
-	strcpy(query, _("Lag order for ARCH test:"));
-	strcpy(defstr, "1");
-	okfunc = do_arch;
-	break;
-    case LMTEST: 
-	strcpy(title, _("gretl: autocorrelation"));
-	strcpy(query, _("Lag order for test:"));
-	if (dataset_is_panel(datainfo)) {
-	    strcpy(defstr, "1");
-	} else {
-	    sprintf(defstr, "%d", datainfo->pd);
-	}
-	okfunc = do_autocorr;
-	break;
-    case CHOW:
-	ntodate(startdate, datainfo->t1, datainfo);
-	ntodate(enddate, datainfo->t2, datainfo);
-	strcpy(title, _("gretl: Chow test"));
-	sprintf(query, _("Enter observation at which\n"
-		"to split the sample\n"
-		"(between %s and %s):"), startdate, enddate);
-	okfunc = do_chow;
-	break;
-    case FCASTERR: 
-	strcpy(title, _("gretl: forecast"));
-	sprintf(query, _("Starting obs (min = %s)\n"
-		"and ending obs (max = %s)?"), 
-		datainfo->stobs, datainfo->endobs);
-	sprintf(defstr, "%s %s", datainfo->stobs, datainfo->endobs);
-	okfunc = do_forecast;
-	break;
-    case MODEL_GENR:
-	strcpy(title, _("gretl: add var"));
-	strcpy(query, _("Enter formula for new variable:"));
-	okfunc = do_model_genr;
-	varclick = VARCLICK_INSERT_NAME;
-	break;
-    }
-
-    edit_dialog(title, query, defstr, okfunc, mydata, 
-		action, varclick);   
+    edit_dialog(_("gretl: add var"), _("Enter formula for new variable:"),
+		"", do_model_genr, mydata, 
+		MODEL_GENR, VARCLICK_INSERT_NAME, 0);   
 }
 
-/* ........................................................... */
-
 void selector_callback (gpointer data, guint action, GtkWidget *widget)
 {
     windata_t *vwin = (windata_t *) data;
+    char title[64];
+
+    if (action == ADD || action == OMIT || action == COEFFSUM) {
+	set_window_busy(vwin);
+    }
+
+    strcpy(title, "gretl: ");
 
     if (action == COINT) {
 	selection_dialog(_("gretl: cointegration test"), do_coint, action);
@@ -479,6 +437,15 @@ void selector_callback (gpointer data, guint action, GtkWidget *widget)
 	simple_selection(_("gretl: model tests"), do_coeff_sum, action, vwin);
     } else if (action == GR_PLOT) {
 	simple_selection(_("gretl: model tests"), do_graph_from_selector, action, vwin);
+    } else if (action == SPEARMAN) {
+	strcat(title, _("rank correlation"));
+	simple_selection(title, do_spearman, action, vwin);
+    } else if (action == MEANTEST || action == MEANTEST2) {
+	strcpy(title, _("gretl: means test"));
+	simple_selection(title, do_two_var_test, action, vwin);
+    } else if (action == VARTEST) {
+	strcpy(title, _("gretl: variances test"));
+	simple_selection(title, do_two_var_test, action, vwin);
     } else {
 	errbox("selector_callback: code was not recognized");
     }
@@ -514,25 +481,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	strcpy(defstr, "100");
 	okfunc = do_simdata;
 	break;         
-    case SPEARMAN:
-	strcpy(title, _("gretl: rank correlation"));
-	strcpy(query, _("Enter two variables by name or number:"));
-	okfunc = do_dialog_cmd;
-	varclick = VARCLICK_INSERT_ID;
-	break;
-    case MEANTEST:
-    case MEANTEST2:
-	strcpy(title, _("gretl: means test"));
-	strcpy(query, _("Enter two variables by name or number:"));
-	okfunc = do_dialog_cmd;
-	varclick = VARCLICK_INSERT_ID;
-	break;
-    case VARTEST:
-	strcpy(title, _("gretl: variances test"));
-	strcpy(query, _("Enter two variables by name or number:"));
-	okfunc = do_dialog_cmd;
-	varclick = VARCLICK_INSERT_ID;
-	break;
     case GENR:
 	strcpy(title, _("gretl: add var"));
 	strcpy(query, _("Enter formula for new variable:"));
@@ -549,18 +497,6 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
 	strcpy(query, _("Enter value to be read as \"missing\":"));
 	okfunc = do_global_setmiss;
 	break;
-    case MARKERS:
-	strcpy(title, _("gretl: add markers"));
-	strcpy(query, _("Supply full path to file with markers:"));
-	strcpy(defstr, paths.userdir);
-	okfunc = do_add_markers; 
-	break;
-    case CORRGM:
-	strcpy(title, _("gretl: correlogram"));
-	strcpy(query, _("Max lag length?\n(0 for automatic):"));
-	strcpy(defstr, "0");
-	okfunc = do_dialog_cmd;
-	break;
     case GR_BOX:
     case GR_NBOX:
 	strcpy(title, _("gretl: boxplots"));
@@ -586,32 +522,26 @@ void gretl_callback (gpointer data, guint action, GtkWidget *widget)
     }
 
     edit_dialog(title, query, defstr, okfunc, mydata, 
-		action, varclick);   
+		action, varclick, 0);   
 }
 
 /* ........................................................... */
 
 void text_copy_callback (GtkWidget *w, gpointer data)
 {
-    text_copy(data, COPY_SELECTION, w);
+    window_copy(data, GRETL_FORMAT_SELECTION, w);
 }
 
-/* ........................................................... */
-
 void text_paste_callback (GtkWidget *w, gpointer data)
 {
     text_paste(data, 0, w);
 }
 
-/* ........................................................... */
-
 void text_replace_callback (GtkWidget *w, gpointer data)
 {
     text_replace(data, 0, w);
 }
 
-/* ........................................................... */
-
 void text_undo_callback (GtkWidget *w, gpointer data)
 {
     text_undo(data, 0, w);
@@ -671,42 +601,34 @@ void add_random_callback (gpointer data, guint code, GtkWidget *widget)
 		       "minimum and maximum values:"), 
 		     "unif 0 1",  
 		     do_random, NULL, 
-		     GENR_UNIFORM, GENR);
+		     GENR_UNIFORM, GENR, 0);
     } else if (code == GENR_NORMAL) {
 	edit_dialog (_("gretl: normal variable"), 
 		     _("Enter name, mean and standard deviation:"), 
 		     "norm 0 1", 
 		     do_random, NULL, 
-		     GENR_NORMAL, GENR);
+		     GENR_NORMAL, GENR, 0);
     }
 }
 
 /* ........................................................... */
 
-static void name_first_var (GtkWidget *widget, dialog_t *ddata) 
+static void name_first_ssheet_var (GtkWidget *widget, dialog_t *dlg) 
 {
-    DATAINFO *pdinfo = (DATAINFO *) dialog_data_get_data(ddata);
     const gchar *buf;
 
-    buf = dialog_data_get_text(ddata);
+    buf = edit_dialog_get_text(dlg);
 
-    if (buf == NULL || validate_varname(buf)) return;
+    if (buf == NULL || validate_varname(buf)) {
+	return;
+    }
 
-    pdinfo->varname[1][0] = 0;
-    strncat(pdinfo->varname[1], buf, VNAMELEN - 1);
+    datainfo->varname[1][0] = 0;
+    strncat(datainfo->varname[1], buf, VNAMELEN - 1);
 
-    close_dialog(ddata);
+    close_dialog(dlg);
 
-    show_spreadsheet(datainfo);
-}
-
-static void first_var_dialog (DATAINFO *pdinfo) 
-{
-    edit_dialog (_("gretl: name variable"), 
-		 _("Enter name for new variable\n"
-		   "(max. 8 characters)"),
-		 NULL, name_first_var, pdinfo, 
-		 0, 0);
+    show_spreadsheet(SHEET_NEW_DATASET);
 }
 
 static int prep_spreadsheet (const char *dataspec)
@@ -735,18 +657,22 @@ static int prep_spreadsheet (const char *dataspec)
     start_new_Z(&Z, datainfo, 0);
     datainfo->markers = 0;
 
-    first_var_dialog(datainfo);
+    edit_dialog (_("gretl: name variable"), 
+		 _("Enter name for new variable\n"
+		   "(max. 8 characters)"),
+		 NULL, name_first_ssheet_var, NULL, 
+		 0, 0, 0);
 
     return 0;
 }
 
-static void n_obs_callback (GtkWidget *w, dialog_t *ddata)
+static void n_obs_callback (GtkWidget *w, dialog_t *dlg)
 {
     const gchar *buf;
     char obsstr[32];
     int n;
 
-    buf = dialog_data_get_text(ddata);
+    buf = edit_dialog_get_text(dlg);
     if (buf == NULL) return;
 
     n = atoi(buf);
@@ -755,7 +681,7 @@ static void n_obs_callback (GtkWidget *w, dialog_t *ddata)
 	return;
     }
 
-    close_dialog(ddata);
+    close_dialog(dlg);
 
     datainfo->n = n;
     sprintf(obsstr, "1 %d", n);
@@ -764,29 +690,32 @@ static void n_obs_callback (GtkWidget *w, dialog_t *ddata)
 
 void newdata_callback (gpointer data, guint pd_code, GtkWidget *widget) 
 {
-    gchar *obsstr = NULL;
+    if (dataset_locked()) {
+	return;
+    }
 
     if (pd_code == 0) {
+	/* cross-sectional dataset */
 	datainfo->structure = CROSS_SECTION;
 	datainfo->pd = 1;
 	datainfo->sd0 = 1.0;
 	strcpy(datainfo->stobs, "1");
 	edit_dialog (_("gretl: create data set"), 
 		     _("Number of observations:"), "50",
-		     n_obs_callback, NULL, 0, 0);
-	return;
+		     n_obs_callback, NULL, 0, 0, 0);
     } else {
+	gchar *obsstr = NULL;
+
 	datainfo->structure = TIME_SERIES;
 	datainfo->pd = pd_code;
-    }
+	compute_default_ts_info(datainfo, 1);
 
-    compute_default_ts_info(datainfo, 1);
+	sample_range_dialog(&obsstr, CREATE_DATASET, NULL);
 
-    sample_range_dialog(&obsstr, CREATE_DATASET, NULL);
-
-    if (obsstr != NULL) {
-	prep_spreadsheet(obsstr);
-	g_free(obsstr);
+	if (obsstr != NULL) {
+	    prep_spreadsheet(obsstr);
+	    g_free(obsstr);
+	}
     }
 }
 
