@@ -18,9 +18,9 @@
  */
 
 #include "gretl.h"
-#include "gretl_private.h"
-
 #include "cmdstack.h"
+#include "lib_private.h"
+#include "cmd_private.h"
 
 #undef CMD_DEBUG
 
@@ -165,21 +165,26 @@ static model_stack *mstack_from_model_id (int ID)
    they may be reconstructed later as part of the session mechanism
 */
 
-int model_command_init (char *line, CMD *cmd, int ID)
+int model_command_init (int model_ID)
 {
+    char *line;
+    CMD *libcmd;
     model_stack *mstack;
     PRN *echo;
     int err = 0;
 
+    line = get_lib_cmdline();
+    libcmd = get_lib_cmd();
+
     /* pre-process the line */
-    if (check_cmd(line)) {
+    if (check_specific_command(line)) {
 	return 1;
     }
 
-    mstack = mstack_from_model_id(ID);
+    mstack = mstack_from_model_id(model_ID);
 
     if (mstack == NULL) {
-	mstack = add_model_stack(ID);
+	mstack = add_model_stack(model_ID);
 	if (mstack == NULL) {
 	    return 1;
 	}
@@ -189,9 +194,9 @@ int model_command_init (char *line, CMD *cmd, int ID)
 	return 1;
     }
 
-    echo_cmd(cmd, datainfo, line, 0, 1, 0, echo);
+    echo_cmd(libcmd, datainfo, line, 0, echo);
 
-    if (add_command_to_mstack(mstack, echo->buf)) {
+    if (add_command_to_mstack(mstack, gretl_print_get_buffer(echo))) {
 	err = 1;
     }
     
@@ -241,7 +246,6 @@ static int vnum_from_data_command (const char *s)
 static int parse_store_cmd (const char *sline, CMD *scmd)
 {
     char *linecpy;
-    int ignore = 0;
     int err = 0;
 
     linecpy = gretl_strdup(sline);
@@ -249,12 +253,7 @@ static int parse_store_cmd (const char *sline, CMD *scmd)
 	return 1;
     }
 
-    scmd->opt = get_gretl_options(linecpy, &err);
-
-    if (!err) {
-	getcmd(linecpy, datainfo, scmd, &ignore, &Z, NULL); 
-	err = scmd->errcode;
-    }
+    err = parse_command_line(linecpy, scmd, &Z, datainfo); 
 
     free(linecpy);
 
