@@ -23,6 +23,9 @@
 #include "console.h"
 #include "menustate.h"
 
+#include "libset.h"
+#include "gretl_func.h"
+
 static GtkWidget *console_view;
 static PRN *console_prn;
 static gchar *cbuf;
@@ -34,26 +37,31 @@ static int hl, hlmax, hlines;
 
 static LOOPSET *loop;
 static int loopstack, looprun;
-static int echo_off;
 
 static int gretl_console_init (void)
 {
-    int i;
     char *hstr;
+    int i;
 
     hlines = 0;
 
     hstr = getenv("GRETL_HISTORY_LINES");
-    if (hstr != NULL) hlines = atoi(hstr);
+    if (hstr != NULL) {
+	hlines = atoi(hstr);
+    }
 
-    if (hlines <= 2 || hlines > 128) 
+    if (hlines <= 2 || hlines > 128) {
 	hlines = DEFAULT_HLINES;
+    }
 
     cmd_history = mymalloc(hlines * sizeof *cmd_history);
-    if (cmd_history == NULL) return E_ALLOC;
+    if (cmd_history == NULL) {
+	return E_ALLOC;
+    }
 
-    for (i=0; i<hlines; i++) 
+    for (i=0; i<hlines; i++) {
 	cmd_history[i] = NULL;
+    }
 
     hlmax = 0;
     hl = -1;
@@ -62,7 +70,8 @@ static int gretl_console_init (void)
 
     loop = NULL;
     loopstack = looprun = 0;
-    echo_off = 0;
+
+    set_gretl_echo(1);
 
     return 0;
 }
@@ -72,7 +81,9 @@ static void gretl_console_free (GtkWidget *w, gpointer p)
     int i;
 
     if (cmd_history != NULL) {
-	for (i=0; i<hlines; i++) free(cmd_history[i]);
+	for (i=0; i<hlines; i++) {
+	    free(cmd_history[i]);
+	}
 	free(cmd_history);
 	cmd_history = NULL;
     }
@@ -105,7 +116,9 @@ static int push_history_line (const char *line)
     /* add the new line */
     cmd_history[0] = g_strdup(line);
     
-    if (hlmax < hlines) hlmax++;
+    if (hlmax < hlines) {
+	hlmax++;
+    }
 
     hl = -1;
 
@@ -140,7 +153,9 @@ static char *pop_history_line (int keyval)
     else if (keyval == GDK_Down) {
 	if (hl >= 0) hl--;
 	if (hl < 0) {
-	    if (beeptime) beep();
+	    if (beeptime) {
+		beep();
+	    }
 	    beeptime = 1;
 	} else {
 	    beeptime = 0;
@@ -159,7 +174,9 @@ static int last_console_line_len (int len)
 
     for (i=len; i>0; i--) {
 	c = GTK_TEXT_INDEX(GTK_TEXT(console_view), i - 1);
-	if (c == '\n') break; 
+	if (c == '\n') {
+	    break; 
+	}
     }
     return len - i - 2;
 }
@@ -191,7 +208,7 @@ enum {
     SAMPLE_CHECK
 };
 
-static int console_sample (const DATAINFO *pdinfo, int code)
+static int console_sample_handler (const DATAINFO *pdinfo, int code)
 {
     static int pd, t1, t2, ts;
     static double sd0;
@@ -219,19 +236,38 @@ static int console_sample (const DATAINFO *pdinfo, int code)
 
 void console_record_sample (const DATAINFO *pdinfo)
 {
-    console_sample(pdinfo, SAMPLE_RECORD);
+    console_sample_handler(pdinfo, SAMPLE_RECORD);
 }
 
 static int sample_changed (const DATAINFO *pdinfo)
 {
-    return console_sample(pdinfo, SAMPLE_CHECK);
+    return console_sample_handler(pdinfo, SAMPLE_CHECK);
+}
+
+static int console_function_exec (char *execline)
+{
+    char *gotline;
+    int err = 0;
+
+    while (!looprun) {
+	gotline = gretl_function_get_line(execline, MAXLINE, &Z, datainfo, &err);
+	if (gotline == NULL || *gotline == '\0') {
+	    break;
+	}
+	if (!err) {
+	    err = gui_exec_line(execline, &loop, &loopstack, &looprun, 
+				console_prn, SCRIPT_EXEC, NULL);
+	}
+    }
+
+    return err;
 }
 
 static void console_exec (void)
 {
     static int redirected;
     int oldv = datainfo->v;
-    char execline[MAXLEN];
+    char execline[MAXLINE];
 #ifndef OLD_GTK
     GtkTextBuffer *buf;
     GtkTextIter start, end;
@@ -268,7 +304,7 @@ static void console_exec (void)
     }
 
     *execline = 0;
-    strncat(execline, cbuf, MAXLEN - 1);
+    strncat(execline, cbuf, MAXLINE - 1);
     g_free(cbuf);
     cbuf = NULL;
 
@@ -276,20 +312,32 @@ static void console_exec (void)
 
     push_history_line(execline); 
 
+    /* actually execute the command line */
     gui_exec_line(execline, &loop, &loopstack, &looprun, console_prn, 
 		  CONSOLE_EXEC, NULL);
 
+    /* the control structure below is rather weird and needs more
+       testing/thought.  The issue is the nesting of loops inside
+       functions or vice versa */
+
     if (looprun) { 
-	loop_exec(loop, execline, &Z, &datainfo, models,
-		  &echo_off, console_prn);
+    fn_run_loop:
+	loop_exec(loop, execline, &Z, &datainfo, models, console_prn);
 	gretl_loop_destroy(loop);
 	loop = NULL;
 	looprun = 0;
-    }    
+	if (gretl_executing_function()) {
+	    console_function_exec(execline);
+	}
+    } else if (gretl_executing_function()) {
+	console_function_exec(execline);
+	if (looprun) {
+	    /* the function we are exec'ing includes a loop */
+	    goto fn_run_loop;
+	}
+    } 
 
-    if (console_prn->fp == NULL) {
-	redirected = 0;
-    }
+    redirected = printing_is_redirected(console_prn);
 
 #ifndef OLD_GTK
     gtk_text_buffer_get_end_iter(buf, &start);
@@ -301,29 +349,24 @@ static void console_exec (void)
 #endif
 
     if (!redirected) {
-	/* put results into console window */
+	const char *cbuf = gretl_print_get_buffer(console_prn);
+
+        /* put results into console window */
 #ifndef OLD_GTK
-	if (!g_utf8_validate(console_prn->buf, -1, NULL)) {
+	if (!g_utf8_validate(cbuf, -1, NULL)) {
 	    fprintf(stderr, "text did not validate as utf8:\n'%s'\n", 
-		    console_prn->buf);
+		    cbuf);
 	} else {
-	    gtk_text_buffer_insert(buf, &start, console_prn->buf, 
-				   strlen(console_prn->buf));
+	    gtk_text_buffer_insert(buf, &start, cbuf, strlen(cbuf));
 	}
 #else
 	gtk_text_insert(GTK_TEXT(console_view), fixed_font, 
-			NULL, NULL, console_prn->buf, 
-			strlen(console_prn->buf));
+			NULL, NULL, cbuf, strlen(cbuf));
 #endif
-    }
-
-    if (console_prn->fp == NULL) {
 	gretl_print_destroy(console_prn);
 	console_prn = NULL;
     } else {
-	/* user has redirected console output to file */
-	redirected = 1;
-	*console_prn->buf = 0;
+	gretl_print_reset_buffer(console_prn);
     }
 
 #ifndef OLD_GTK
@@ -374,8 +417,13 @@ void show_gretl_console (void)
 	return;
     }
 
-    if (!user_fopen("console_tmp", fname, &prn)) return;
-    if (gretl_console_init()) return;
+    if (user_fopen("console_tmp", fname, &prn)) {
+	return;
+    }
+
+    if (gretl_console_init()) {
+	return;
+    }
 
     pputs(prn, _(intro));
     gretl_print_destroy(prn);
@@ -414,7 +462,7 @@ void show_gretl_console (void)
     gtk_editable_changed(GTK_EDITABLE(console_view));
 #endif
 
-    gtk_widget_grab_focus (console_view);
+    gtk_widget_grab_focus(console_view);
 }
 
 #define IS_BACKKEY(k) (k == GDK_BackSpace || k == GDK_Left)
@@ -432,7 +480,9 @@ static int bslash_cont (const gchar *line)
 	}
     }
 
-    if (cbuf == NULL) return 0;
+    if (cbuf == NULL) {
+	return 0;
+    }
 
     if (bslash) {
 	char *p = strrchr(cbuf, '\\');
@@ -721,20 +771,4 @@ gint console_mouse_handler (GtkWidget *w, GdkEventButton *event,
     return FALSE;
 }
 
-#if 0
-gint console_click_handler (GtkWidget *w, GdkEventButton *event,
-			    gpointer p)
-{
-    if (!on_last_line()) {
-	GdkModifierType mods;
-
-	gdk_window_get_pointer(w->window, NULL, NULL, &mods);
-	if (mods & GDK_BUTTON2_MASK) {
-	    return TRUE;
-	}
-    }
-
-    return FALSE;
-}
-#endif
 
