@@ -49,6 +49,8 @@ void copy_format_dialog (windata_t *vwin, int multicopy);
 
 void varinfo_dialog (int varnum, int full);
 
+void database_description_dialog (const char *binname);
+
 int select_var_from_list (const int *list, const char *query);
 
 void sample_range_dialog (gpointer p, guint u, GtkWidget *w);
@@ -65,10 +67,29 @@ int density_dialog (int vnum, double *bw);
 int radio_dialog (const char *title, const char **opts, 
 		  int nopts, int deflt, int helpcode);
 
-int checks_dialog (const char *title, const char **opts, 
-		   int nopts, int *active, int *spinval,
-		   const char *spintext, int spinmax,
-		   int helpcode);
+int radio_dialog_with_spinner (const char *title, const char **opts, 
+			       int nopts, int deflt, int helpcode,
+			       int *spinvar, const char *spintxt,
+			       int spinmin, int spinmax);
+
+int checks_dialog (const char *title, const char **opts, int nopts, 
+		   int *active, int *spinvar, const char *spintxt, 
+		   int spinmin, int spinmax, int helpcode);
+
+int spin_dialog (const char *title, int *spinvar, const char *spintxt, 
+		 int spinmin, int spinmax, int helpcode);
+
+int get_obs_dialog (const char *title, const char *text,
+		    const char *t1str, const char *t2str,
+		    int t1min, int t1max, int *t1, 
+		    int t2min, int t2max, int *t2);
+
+int add_obs_dialog (const char *blurb, int addmin);
+
+int forecast_dialog (int t1min, int t1max, int *t1, 
+		     int t2min, int t2max, int *t2,
+		     int pmin, int pmax, int *p,
+		     int dyn);
 
 void compute_default_ts_info (DATAINFO *dwinfo, int newdata);
 
