@@ -34,6 +34,10 @@
 # endif
 #endif
 
+#ifdef OLD_GTK
+# include "menustate.h"
+#endif
+
 #define IS_DAT_ACTION(i) (i == SAVE_DATA || \
                           i == SAVE_DATA_AS || \
                           i == SAVE_GZDATA || \
@@ -44,21 +48,18 @@
 #define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
                               i == OPEN_CSV || \
                               i == OPEN_ASCII || \
+                              i == OPEN_OCTAVE || \
 	                      i == OPEN_BOX || \
                               i == OPEN_GNUMERIC || \
 	                      i == OPEN_EXCEL)
 
 #define APPEND_DATA_ACTION(i) (i == APPEND_DATA || \
                                i == APPEND_CSV || \
+                               i == APPEND_OCTAVE || \
                                i == APPEND_GNUMERIC || \
                                i == APPEND_EXCEL || \
                                i == APPEND_ASCII)
 
-#define SAVE_TEX_ACTION(i) (i == SAVE_TEX_TAB || \
-                            i == SAVE_TEX_EQ || \
-	                    i == SAVE_TEX_TAB_FRAG || \
-                            i == SAVE_TEX_EQ_FRAG)
-
 #define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
                               i == SAVE_THIS_GRAPH || \
                               i == SAVE_LAST_GRAPH || \
@@ -72,7 +73,9 @@
                           i == EXPORT_CSV || \
                           i == EXPORT_DAT)
 
+#ifdef REMEMBER_DIR
 static char remember_dir[MAXLEN];
+#endif
 
 struct extmap {
     int action;
@@ -85,6 +88,7 @@ static struct extmap action_map[] = {
     { SAVE_GZDATA,       ".gdt" },
     { SAVE_BIN1,         ".gdt" },
     { SAVE_BIN2,         ".gdt" },
+    { SAVE_DBDATA,       ".bin" },
     { SAVE_CMDS,         ".inp" },
     { SAVE_SCRIPT,       ".inp" },
     { SAVE_CONSOLE,      ".inp" },
@@ -97,13 +101,12 @@ static struct extmap action_map[] = {
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
     { EXPORT_R_ALT,      ".R" },
+    { OPEN_OCTAVE,       ".m" },
+    { APPEND_OCTAVE,     ".m" },
     { EXPORT_OCTAVE,     ".m" },
     { EXPORT_DAT,        ".dat" },
     { SAVE_OUTPUT,       ".txt" },
-    { SAVE_TEX_TAB,      ".tex" },
-    { SAVE_TEX_EQ,       ".tex" },
-    { SAVE_TEX_TAB_FRAG, ".tex" },
-    { SAVE_TEX_EQ_FRAG,  ".tex" },
+    { SAVE_TEX,          ".tex" },
     { OPEN_DATA,         ".gdt" },
     { APPEND_DATA,       ".gdt" },    
     { OPEN_SCRIPT,       ".inp" },
@@ -128,17 +131,16 @@ static gretlopt action_to_opt (const int action)
     case SAVE_GZDATA:   return OPT_Z;
     case SAVE_BIN1:     return OPT_S;
     case SAVE_BIN2:     return OPT_O;
+    case SAVE_DBDATA:   return OPT_D;
     case EXPORT_OCTAVE: return OPT_M;
     case EXPORT_R:      return OPT_R;
     case EXPORT_R_ALT:  return OPT_A;
     case EXPORT_CSV:    return OPT_C;
-    case EXPORT_DAT:    return OPT_D;
+    case EXPORT_DAT:    return OPT_G; /* PcGive */
     default: return 0L;
     }
 }
 
-/* ........................................................... */
-
 static const char *get_gp_ext (const char *termtype)
 {
     if (!strncmp(termtype, "postscript", 10))    return ".eps";
@@ -152,20 +154,26 @@ static const char *get_gp_ext (const char *termtype)
 
 /* ........................................................... */
 
-static int dat_ext (const char *str, int err)
+static int dat_ext (const char *str, int showerr)
 {
     const char *suff;
+    int err = 0;
+
+    if (str == NULL) {
+	return 0;
+    }
 
-    if (str == NULL) return 0;
     suff = strrchr(str, '.');
-    if (suff != NULL && 
-	(!strcmp(suff, ".dat") || !strcmp(suff, ".gdt"))) {
-	if (err) 
+
+    if (suff != NULL && (!strcmp(suff, ".dat") || !strcmp(suff, ".gdt"))) {
+	if (showerr) {
 	    errbox(_("The suffix you selected should be used\n"
 		   "only for gretl datafiles"));
-	return 1;
+	}
+	err = 1;
     }
-    return 0;
+
+    return err;
 }
 
 /* ........................................................... */
@@ -181,11 +189,9 @@ static const char *get_ext (int action, gpointer data)
     if (action == SAVE_GNUPLOT || action == SAVE_THIS_GRAPH) {
 	GPT_SPEC *plot = (GPT_SPEC *) data;
 	s = get_gp_ext(plot->termtype);
-    }
-    else if (action == SAVE_LAST_GRAPH) {
+    } else if (action == SAVE_LAST_GRAPH) {
 	s = get_gp_ext(data);
-    } 
-    else {
+    } else {
 	size_t i;
 
 	for (i=0; i < sizeof action_map / sizeof *action_map; i++) {
@@ -195,6 +201,7 @@ static const char *get_ext (int action, gpointer data)
 	    }
 	}
     }
+
     return s;
 }
 
@@ -205,10 +212,14 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
     FILE *fp;
     const char *ext = NULL;
 
-    if (fname == NULL) return 1;
+    if (fname == NULL) {
+	return 1;
+    }
 
     /* don't mess if the fname is really a dir */
-    if (isdir(fname)) return 1;
+    if (isdir(fname)) {
+	return 1;
+    }
 
     /* don't mess with the name of a previously existing file */
     fp = gretl_fopen(fname, "r");
@@ -310,13 +321,17 @@ static void save_editable_content (int action, const char *fname,
 
 /* ........................................................... */
 
-static void set_startdir (char *startdir)
+static void set_startdir (char *startdir, int action)
 {
+#ifdef REMEMBER_DIR
     if (*remember_dir != '\0') {
 	strcpy(startdir, remember_dir);
     } else {
-	get_default_dir(startdir);
+	get_default_dir(startdir, action);
     }
+#else
+    get_default_dir(startdir, action);
+#endif
 
 #ifndef G_OS_WIN32
     if (startdir[strlen(startdir) - 1] != '/') {
@@ -429,6 +444,7 @@ static char *suggested_exportname (const char *fname, int action)
     return s;
 }
 
+#ifdef REMEMBER_DIR
 static void remember_this_dir (const char *fname)
 {
     int spos = slashpos(fname);
@@ -438,6 +454,7 @@ static void remember_this_dir (const char *fname)
 	strncat(remember_dir, fname, spos);
     }
 }
+#endif
 
 static void
 file_selector_process_result (const char *in_fname, int action, gpointer data)
@@ -459,9 +476,11 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	}
     } 
 
+#ifdef REMEMBER_DIR
     if (action != SET_PATH) {
 	remember_this_dir(fname);
     }
+#endif
 
     if (OPEN_DATA_ACTION(action)) {
 	strcpy(trydatfile, fname);
@@ -473,9 +492,13 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	filesel_open_script(fname);
     } else if (action == OPEN_SESSION) {
 	filesel_open_session(fname);
+    } else if (action == OPEN_MARKERS) {
+	do_add_markers(fname);
     }
 
-    if (action < END_OPEN) return;
+    if (action < END_OPEN) {
+	return;
+    }
 
     /* now for the save options */
 
@@ -490,7 +513,7 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
     if (SAVE_DATA_ACTION(action)) {
 	int overwrite = 0;
 
-	if (!strcmp(fname, paths.datfile)) {
+	if (!strcmp(fname, paths.datfile) || action == SAVE_DBDATA) {
 	    overwrite = 1;
 	}
 	do_store(fname, action_to_opt(action), overwrite);
@@ -518,10 +541,10 @@ file_selector_process_result (const char *in_fname, int action, gpointer data)
 	else errbox(_("boxplot save failed"));
     } else if (action == SAVE_SESSION) {
 	save_session(fname);
-    } else if (SAVE_TEX_ACTION(action)) {
-	MODEL *pmod = (MODEL *) data;
+    } else if (action == SAVE_TEX) {
+	PRN *prn = (PRN *) data;
 
-	do_save_tex(fname, action, pmod); 
+	save_latex(prn, fname);
     } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
@@ -591,6 +614,7 @@ static struct winfilter get_filter (int action, gpointer data)
 	{SAVE_GZDATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{SAVE_BIN1,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{SAVE_BIN2,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
+	{SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
 	{SAVE_CMDS,    { N_("gretl command files (*.inp)"), "*.inp" }},
 	{SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
 	{SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
@@ -603,12 +627,11 @@ static struct winfilter get_filter (int action, gpointer data)
 	{EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
 	{EXPORT_R_ALT, { N_("GNU R files (*.R)"), "*.R" }},
 	{EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{EXPORT_DAT  , { N_("PcGive files (*.dat)"), "*.dat" }},
+	{OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
+	{APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
+	{EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
 	{SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
-	{SAVE_TEX_TAB, { N_("TeX files (*.tex)"), "*.tex" }},
-	{SAVE_TEX_EQ,  { N_("TeX files (*.tex)"), "*.tex" }},
-	{SAVE_TEX_TAB_FRAG, { N_("TeX files (*.tex)"), "*.tex" }},
-	{SAVE_TEX_EQ_FRAG,  { N_("TeX files (*.tex)"), "*.tex" }},
+	{SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
 	{OPEN_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{APPEND_DATA,  { N_("gretl data files (*.gdt)"), "*.gdt" }},
 	{OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
@@ -624,8 +647,13 @@ static struct winfilter get_filter (int action, gpointer data)
 	{APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
 	{SET_PATH,     { N_("program files (*.exe)"), "*.exe" }}
     };
+
     static struct winfilter olddat_filter = {
 	N_("gretl data files (*.dat)"), "*.dat"
+    };    
+
+    static struct winfilter default_filter = {
+	N_("all files (*.*)"), "*.*" 
     };
 
     if (using_olddat() && IS_DAT_ACTION(action)) {
@@ -637,9 +665,13 @@ static struct winfilter get_filter (int action, gpointer data)
 
 	return get_gp_filter(plot->termtype);
     }
+
     if (action == SAVE_LAST_GRAPH) {
 	return get_gp_filter(data);
     }
+
+    filter = default_filter;
+
     for (i=0; i< sizeof map / sizeof *map; i++) {
 	if (action == map[i].action) {
 	    filter = map[i].filter;
@@ -656,18 +688,23 @@ static char *make_winfilter (int action, gpointer data)
     char *start = p;
     struct winfilter filter;
 
-    if (p == NULL) return NULL;
+    if (p == NULL) {
+	return NULL;
+    }
 
     filter = get_filter(action, data);
 
     strcpy(p, I_(filter.descrip));
-
     p += strlen(p) + 1;
     strcpy(p, filter.pat);
-    p += strlen(p) + 1;
-    strcpy(p, I_("all files (*.*)"));
-    p += strlen(p) + 1;
-    strcpy(p, "*.*");
+
+    if (strncmp(filter.descrip, "all", 3)) {
+	p += strlen(p) + 1;
+	strcpy(p, I_("all files (*.*)"));
+	p += strlen(p) + 1;
+	strcpy(p, "*.*");
+    }
+
     p += strlen(p) + 1;
     *p = '\0';
 
@@ -687,7 +724,7 @@ void file_selector (const char *msg, int action, gpointer data)
     *fname = '\0';
     *endname = '\0';
 
-    set_startdir(startdir);
+    set_startdir(startdir, action);
 
     /* special cases */
     if ((action == SAVE_DATA || action == SAVE_GZDATA) && paths.datfile[0]) {
@@ -698,17 +735,13 @@ void file_selector (const char *msg, int action, gpointer data)
 	if (!(data_status & BOOK_DATA)) {
 	    get_base(startdir, paths.datfile, SLASH);
 	}
-    }
-
-    else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	strcpy(fname, savename);
 	g_free(savename);
 	get_base(startdir, paths.datfile, SLASH);
-    }	
-
-    else if (action == SET_PATH) {
+    } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
 	if (strvar != NULL && *strvar != '\0') {
@@ -792,11 +825,15 @@ static void filesel_callback (GtkWidget *w, struct fsinfo_t *fsinfo)
 
 # ifndef OLD_GTK
     path = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsinfo->w));
-    if (path == NULL || *path == '\0' || isdir(path)) return;
+    if (path == NULL || *path == '\0' || isdir(path)) {
+	return;
+    }
     strcpy(fsinfo->fname, path);
 # else
     test = gtk_entry_get_text(GTK_ENTRY(fsel->file_entry));
-    if (test == NULL || *test == '\0') return;    
+    if (test == NULL || *test == '\0') {
+	return;
+    }
     path = gtk_file_list_get_path(GTK_FILE_LIST(fsel->file_list));
     sprintf(fsinfo->fname, "%s%s", path, test);
 # endif
@@ -844,7 +881,7 @@ void file_selector (const char *msg, int action, gpointer data)
     char startdir[MAXLEN];
     GtkFileFilter *filter;
 
-    set_startdir(startdir);
+    set_startdir(startdir, action);
 
     /* FIXME: parent window below should not always be mdata->w,
        in particular when action == SAVE_THIS_GRAPH
@@ -866,6 +903,7 @@ void file_selector (const char *msg, int action, gpointer data)
 
     filter = get_file_filter(action, data);
     gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
 
     /* special cases */
 
@@ -875,22 +913,16 @@ void file_selector (const char *msg, int action, gpointer data)
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    }
-
-    else if (EXPORT_ACTION(action) && paths.datfile[0]) {
+    } else if (EXPORT_ACTION(action) && paths.datfile[0]) {
 	char *savename = suggested_exportname(paths.datfile, action);
 
 	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
 					  savename);
 	g_free(savename);
-    }	
-
-    else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
+    } else if ((action == SAVE_SESSION) && *scriptfile != '\0') {
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
 				      scriptfile);
-    }
-
-    else if (action == SET_PATH) {
+    } else if (action == SET_PATH) {
 	char *strvar = (char *) data;
 
 	if (strvar != NULL && slashpos(strvar) > 0) {
@@ -900,7 +932,7 @@ void file_selector (const char *msg, int action, gpointer data)
 	    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), 
 						"/usr/bin");
 	}	    
-    }	
+    } 
 
     if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
 	char *fname;
@@ -960,7 +992,7 @@ void file_selector (const char *msg, int action, gpointer data)
 
     *fsinfo.fname = '\0';
 
-    set_startdir(startdir);
+    set_startdir(startdir, action);
 
     filesel = gtk_file_selection_new(msg);
     fsinfo.w = filesel;
@@ -1045,7 +1077,7 @@ void file_selector (const char *msg, int action, gpointer data)
 
     *fsinfo.fname = '\0';
 
-    set_startdir(startdir);
+    set_startdir(startdir, action);
 
     filesel = gtk_icon_file_selection_new(msg);
 
@@ -1117,7 +1149,7 @@ void file_selector (const char *msg, int action, gpointer data)
 			      GTK_OBJECT(filesel));
 
     gtk_widget_show(filesel);
-    gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);
+    gretl_set_window_modal(filesel);
     gtk_main(); 
 
     if (*fsinfo.fname != '\0') {
