@@ -78,7 +78,7 @@ drag_data_received  (GtkWidget          *widget,
 
 #ifdef USE_GNOME
 static char *optrun, *optdb;
-static int opteng, optdump;
+static int opteng, optbasque, optdump;
 
 static const struct poptOption options[] = {
     { "run", 'r', POPT_ARG_STRING, &optrun, 0, 
@@ -89,6 +89,8 @@ static const struct poptOption options[] = {
       N_("open a remote (web) database on startup"), "REMOTE_DB" },
     { "english", 'e', POPT_ARG_NONE, &opteng, 0, 
       N_("force use of English"), NULL },
+    { "basque", 'q', POPT_ARG_NONE, &optbasque, 0, 
+      N_("force use of Basque"), NULL },
     { "dump", 'c', POPT_ARG_NONE, &optdump, 0, 
       N_("dump gretl configuration to file"), NULL },
     { NULL, '\0', 0, NULL, 0, NULL, NULL },
@@ -100,7 +102,6 @@ DATAINFO *datainfo;
 char *errtext;
 char cmdfile[MAXLEN], scriptfile[MAXLEN];
 char trydatfile[MAXLEN], tryscript[MAXLEN];
-char line[1024];
 PATHS paths;                /* useful paths */
 double **Z;                 /* data set */
 MODEL **models;             /* gretl models structs */
@@ -118,10 +119,12 @@ char dbproxy[21];
 #ifdef G_OS_WIN32
 char Rcommand[MAXSTR] = "RGui.exe";
 char calculator[MAXSTR] = "calc.exe";
+char latex[MAXSTR] = "latex.exe";
 char viewdvi[MAXSTR] = "windvi.exe";
 char viewps[MAXSTR] = "gsview.exe";
 #else
 char calculator[MAXSTR] = "xcalc";
+char latex[MAXSTR] = "latex";
 char viewdvi[MAXSTR] = "xdvi";
 char viewps[MAXSTR] = "gv";
 # ifdef USE_GNOME
@@ -139,7 +142,7 @@ char tramodir[MAXSTR] = "";
 
 static void spreadsheet_edit (gpointer p, guint u, GtkWidget *w) 
 {
-    show_spreadsheet(NULL);
+    show_spreadsheet(SHEET_EDIT_DATASET);
 }
 
 static void manual_update_query (gpointer p, guint u, GtkWidget *w)
@@ -215,6 +218,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Open data/sep1"), NULL, NULL, 0, "<Separator>", GNULL },    
     { N_("/File/Open data/import CSV..."), NULL, open_data, OPEN_CSV, NULL, GNULL },
     { N_("/File/Open data/import ASCII..."), NULL, open_data, OPEN_ASCII, NULL, GNULL },
+    { N_("/File/Open data/import Octave..."), NULL, open_data, OPEN_OCTAVE, NULL, GNULL },
     { N_("/File/Open data/import BOX..."), NULL, open_data, OPEN_BOX, NULL, GNULL },
 #ifdef G_OS_WIN32
     { N_("/File/Open data/import Excel..."), NULL, open_data, 
@@ -233,6 +237,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/File/Append data/standard format..."), NULL, open_data, APPEND_DATA, NULL, GNULL },
     { N_("/File/Append data/from CSV..."), NULL, open_data, APPEND_CSV, NULL, GNULL },
     { N_("/File/Append data/from ASCII..."), NULL, open_data, APPEND_ASCII, NULL, GNULL },
+    { N_("/File/Append data/from Octave..."), NULL, open_data, APPEND_OCTAVE, NULL, GNULL },
     { N_("/File/Append data/from Gnumeric..."), NULL, open_data, 
       APPEND_GNUMERIC, NULL, GNULL },
     { N_("/File/Append data/from Excel..."), NULL, open_data, 
@@ -245,6 +250,8 @@ GtkItemFactoryEntry data_items[] = {
       "<StockItem>", GTK_STOCK_SAVE_AS },
     { N_("/File/Save data as/_gzipped..."), NULL, file_save, SAVE_GZDATA, 
       "<StockItem>", GTK_STOCK_SAVE_AS },
+    { N_("/File/Save data as/_database..."), NULL, file_save, SAVE_DBDATA, 
+      "<StockItem>", GTK_STOCK_SAVE_AS },
 
     /* File, Export data */
     { N_("/File/_Export data"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -402,10 +409,11 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/sep4"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Data/Difference of means"), NULL, NULL, 0, "<Branch>", NULL },
     { N_("/Data/Difference of means/assuming equal variances..."), NULL, 
-      gretl_callback, MEANTEST, NULL, GNULL },
+      selector_callback, MEANTEST, NULL, GNULL },
     { N_("/Data/Difference of means/assuming unequal variances..."), NULL, 
-      gretl_callback, MEANTEST2, NULL, GNULL },
-    { N_("/Data/Difference of variances..."), NULL, gretl_callback, VARTEST, NULL, GNULL },
+      selector_callback, MEANTEST2, NULL, GNULL },
+    { N_("/Data/Difference of variances..."), NULL, selector_callback, VARTEST, NULL, 
+      GNULL },
     { N_("/Data/sep5"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Data/Add variables"), NULL, NULL, 0, "<Branch>", NULL },
     { N_("/Data/Add variables/time trend"), NULL, add_index, 1, NULL, GNULL },
@@ -433,6 +441,8 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Data/Add variables/sep2"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Data/Add variables/Define _new variable..."), NULL, gretl_callback, 
       GENR, NULL, GNULL },
+    { N_("/Data/Add observations..."), NULL, do_add_obs, 0, NULL, GNULL },
+    { N_("/Data/Remove extra observations"), NULL, do_remove_obs, 0, NULL, GNULL },
     { N_("/Data/Refresh window"), NULL, refresh_data, 0, NULL, GNULL },
 
     /* Sample menu */
@@ -455,7 +465,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Sample/Set missing _value code..."), NULL, gretl_callback, 
       GSETMISS, NULL, GNULL },
     { N_("/Sample/sep4"), NULL, NULL, 0, "<Separator>", NULL },  
-    { N_("/Sample/_Add case markers..."), NULL, gretl_callback, MARKERS, NULL, GNULL },
+    { N_("/Sample/_Add case markers..."), NULL, open_data, OPEN_MARKERS, NULL, GNULL },
     { N_("/Sample/Remove case _markers"), NULL, do_remove_markers, 0, NULL, GNULL },
     { N_("/Sample/sep5"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Sample/Restructure panel..."), NULL, panel_restructure_dialog, 0, NULL, GNULL },
@@ -472,14 +482,14 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Variable/Frequency plot"), NULL, NULL, 0, "<Branch>", NULL },
     { N_("/Variable/Frequency plot/simple"), NULL, do_freqplot, 0, NULL, GNULL },
     { N_("/Variable/Frequency plot/against Normal"), NULL, do_freqplot, 
-      NORMAL, NULL, GNULL },
+      DIST_NORMAL, NULL, GNULL },
     { N_("/Variable/Frequency plot/against Gamma"), NULL, do_freqplot, 
-      GAMMA, NULL, GNULL },
+      DIST_GAMMA, NULL, GNULL },
     { N_("/Variable/Estimated density plot..."), NULL, do_kernel, 0, NULL, GNULL },
     { N_("/Variable/Range-mean graph"), NULL, do_range_mean, 0, NULL, GNULL }, 
     { N_("/Variable/sep1"), NULL, NULL, 0, "<Separator>", NULL },
     { N_("/Variable/_Time series plot"), NULL, ts_plot_var, 0, NULL, GNULL },
-    { N_("/Variable/Correlogram"), NULL, gretl_callback, CORRGM, NULL, GNULL },
+    { N_("/Variable/Correlogram"), NULL, do_corrgm, CORRGM, NULL, GNULL },
     { N_("/Variable/Spectrum"), NULL, NULL, 0, "<Branch>", NULL },
     { N_("/Variable/Spectrum/sample periodogram"), NULL, do_pergm, 0, NULL, GNULL }, 
     { N_("/Variable/Spectrum/Bartlett lag window"), NULL, do_pergm, 1, NULL, GNULL }, 
@@ -523,7 +533,7 @@ GtkItemFactoryEntry data_items[] = {
     { N_("/Model/Poi_sson..."), NULL, model_callback, POISSON, NULL, GNULL },
     { N_("/Model/Lo_gistic..."), NULL, model_callback, LOGISTIC, NULL, GNULL },
     { N_("/Model/Least _Absolute Deviation..."), NULL, model_callback, LAD, NULL, GNULL },
-    { N_("/Model/_Rank correlation..."), NULL, gretl_callback, SPEARMAN, NULL, GNULL },
+    { N_("/Model/_Rank correlation..."), NULL, selector_callback, SPEARMAN, NULL, GNULL },
     { N_("/Model/_Pooled OLS (panel)..."), NULL, model_callback, POOLED, NULL, GNULL },
     { N_("/Model/Nonlinear Least Squares..."), NULL, gretl_callback, NLS, NULL, GNULL },
 #ifdef ENABLE_GMP
@@ -552,11 +562,12 @@ GtkItemFactoryEntry data_items[] = {
 
 static void gui_usage (void)
 {
-    gui_logo(stdout);
+    gui_logo(NULL);
     printf(I_("You may supply the name of a data file on the command line.\n"));
     printf(I_("Or you may do \"gretl -r script_file\" to open a script.\n"));
     printf(I_("Or you may do \"gretl -d database\" to open a gretl database.\n"));
     printf(I_("You may do \"gretl -e\" to force use of English.\n"));
+    printf(I_("You may do \"gretl -q\" to force use of Basque.\n"));
     exit(0);
 }
 
@@ -630,11 +641,11 @@ static void real_nls_init (void)
     }
 
     build_path(gretldir, "locale", localedir, NULL);
-    loc = setlocale (LC_ALL, "");
+    loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
-    bindtextdomain (PACKAGE, localedir);
-    textdomain (PACKAGE);
-    bind_textdomain_codeset (PACKAGE, "UTF-8");
+    bindtextdomain(PACKAGE, localedir);
+    textdomain(PACKAGE);
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
 # elif defined(OSX_PKG)
@@ -648,11 +659,11 @@ static void real_nls_init (void)
     if (prefix == NULL) return;
     
     localedir = g_strdup_printf("%s/share/locale", prefix);
-    loc = setlocale (LC_ALL, "");
+    loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
-    bindtextdomain (PACKAGE, localedir);
-    textdomain (PACKAGE);
-    bind_textdomain_codeset (PACKAGE, "UTF-8");
+    bindtextdomain(PACKAGE, localedir);
+    textdomain(PACKAGE);
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
     g_free(localedir);
 }
 
@@ -662,11 +673,11 @@ static void real_nls_init (void)
 {
     char *loc;
 
-    loc = setlocale (LC_ALL, "");
+    loc = setlocale(LC_ALL, "");
     set_gretl_charset(loc);
     bindtextdomain (PACKAGE, LOCALEDIR);
-    textdomain (PACKAGE);
-    bind_textdomain_codeset (PACKAGE, "UTF-8");
+    textdomain(PACKAGE);
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
 }
 
 # endif
@@ -683,17 +694,32 @@ void nls_init (void)
     real_nls_init();
 }
 
-static void force_english (void)
+static void force_language (int f)
 {
-    setlocale (LC_ALL, "C");
+    if (f == ENGLISH) {
+	setlocale (LC_ALL, "C");
+    } else {
+# ifdef G_OS_WIN32
+	setlocale (LC_ALL, "eu");
+#else
+	setlocale (LC_ALL, "eu_ES");
+# endif
+    }
 
 # ifdef G_OS_WIN32
-    SetEnvironmentVariable("LC_ALL", "C");
-    putenv("LC_ALL=C");
-    textdomain("none");
+    if (f == ENGLISH) {
+	SetEnvironmentVariable("LC_ALL", "C");
+	putenv("LC_ALL=C");
+	textdomain("none");
+    } else {
+	SetEnvironmentVariable("LC_ALL", "eu");
+	putenv("LC_ALL=eu");
+    }
 # endif
 
-    force_english_help();
+    if (f == ENGLISH) {
+	force_english_help();
+    }
 }
 
 #endif /* ENABLE_NLS */
@@ -743,32 +769,37 @@ int main (int argc, char *argv[])
     gtk_init(&argc, &argv);
 #endif /* USE_GNOME */
 
+    libgretl_init();
     set_paths(&paths, OPT_D | OPT_X); /* defaults, gui */
 
 #ifdef G_OS_WIN32
     gretl_win32_init(argv[0]);
 #else 
-    root_check();
     set_rcfile(); /* also calls read_rc() */
+    if (!expert) {
+	root_check();
+    }
 #endif/* G_OS_WIN32 */
 
     if (argc > 1) {
-	int english = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &english);
+	int force_lang = 0;
+	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
 
 	switch (opt) {
 	case OPT_HELP:
 	    gui_usage();
 	    break;
 	case OPT_VERSION:
-	    gui_logo(stdout);
+	    gui_logo(NULL);
 	    exit(EXIT_SUCCESS);
 	    break;
 	case OPT_RUNIT:
 #ifdef USE_GNOME
 	    get_runfile(optrun);
 #else
-	    if (*filearg == '\0') gui_usage();
+	    if (*filearg == '\0') {
+		gui_usage();
+	    }
 	    get_runfile(filearg);
 #endif
 	    gui_get_data = 1;
@@ -778,10 +809,14 @@ int main (int argc, char *argv[])
 #ifdef USE_GNOME
 	    strncpy(dbname, optdb, MAXLEN-1);
 #else
-	    if (*filearg == '\0') gui_usage();
+	    if (*filearg == '\0') {
+		gui_usage();
+	    }
 	    strcpy(dbname, filearg);
 #endif
-	    if (opt == OPT_DBOPEN) fix_dbname(dbname);
+	    if (opt == OPT_DBOPEN) {
+		fix_dbname(dbname);
+	    }
 	    gui_get_data = opt;
 	    break;
 	case OPT_DUMP:
@@ -794,8 +829,8 @@ int main (int argc, char *argv[])
 	}
 
 #ifdef ENABLE_NLS
-	if (english) {
-	    force_english();
+	if (force_lang) {
+	    force_language(force_lang);
 	    if (argc == 2) {
 		gui_get_data = 1;
 	    }	
@@ -822,8 +857,9 @@ int main (int argc, char *argv[])
     models[1] = gretl_model_new();
     models[2] = gretl_model_new();
 
-    if (models[0] == NULL || models[1] == NULL || models[2] == NULL) 
+    if (models[0] == NULL || models[1] == NULL || models[2] == NULL) {
 	noalloc(_("models")); 
+    }
 
     library_command_init();
 
@@ -836,14 +872,15 @@ int main (int argc, char *argv[])
 	int ftype;
 	PRN *prn; 
 
-	prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
+	prn = gretl_print_new(GRETL_PRINT_STDERR);
 	if (prn == NULL) exit(EXIT_FAILURE);
 
 	*paths.datfile = '\0';
 
 #ifdef G_OS_WIN32
-	if (unmangle(filearg, paths.datfile)) 
+	if (unmangle(filearg, paths.datfile)) {
 	    exit(EXIT_FAILURE);
+	}
 #else
 	strcpy(paths.datfile, filearg);
 #endif
@@ -862,7 +899,10 @@ int main (int argc, char *argv[])
 			      prn, 0);
 	    break;
 	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, &datainfo, paths.datfile, &paths, prn);
+	    err = import_csv(&Z, &datainfo, paths.datfile, prn);
+	    break;
+	case GRETL_OCTAVE:
+	    err = import_octave(&Z, &datainfo, paths.datfile, prn);
 	    break;
 	case GRETL_BOX_DATA:
 	    err = import_box(&Z, &datainfo, paths.datfile, prn);
@@ -978,6 +1018,7 @@ int main (int argc, char *argv[])
     free(models);
 
     library_command_free();
+    libgretl_cleanup();
 
     if (data_status) {
 	free_datainfo(datainfo);
@@ -1061,10 +1102,10 @@ void populate_varlist (void)
     store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(mdata->listbox)));
     gtk_list_store_clear(store);
 
-    gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     for (i=0; i<datainfo->v; i++) {
-	if (hidden_var(i, datainfo)) continue;
+	if (is_hidden_variable(i, datainfo)) continue;
 	gtk_list_store_append(store, &iter);
 	sprintf(id, "%d", i);
 	gtk_list_store_set (store, &iter, 
@@ -1210,7 +1251,7 @@ static GtkWidget *make_main_window (int gui_get_data)
 	_("Variable name"), 
 	_("Descriptive label")
     };
-    int mainwin_width = 550;  
+    int mainwin_width = 580;  
     int mainwin_height = 420; 
 
     gui_scale = get_gui_scale();
@@ -1234,10 +1275,10 @@ static GtkWidget *make_main_window (int gui_get_data)
     set_up_windows_look();
 #endif
 
-    g_signal_connect (G_OBJECT (mdata->w), "delete_event",
-		      G_CALLBACK (exit_check), NULL);
-    g_signal_connect (G_OBJECT (mdata->w), "destroy",
-		      G_CALLBACK (destroy), NULL);
+    g_signal_connect(G_OBJECT(mdata->w), "delete_event",
+		     G_CALLBACK(exit_check), NULL);
+    g_signal_connect(G_OBJECT(mdata->w), "destroy",
+		     G_CALLBACK(destroy), NULL);
 
     gtk_window_set_title(GTK_WINDOW (mdata->w), "gretl");
     gtk_window_set_default_size(GTK_WINDOW (mdata->w), 
@@ -1350,14 +1391,13 @@ int restore_sample (gretlopt opt)
 
 static void restore_sample_callback (gpointer p, int verbose, GtkWidget *w)
 {
-    restore_sample(OPT_C); 
+    int err = restore_sample(OPT_C); 
 
-    if (verbose) {
+    if (verbose && !err) {
 	infobox(_("Full sample range restored"));
 	set_sample_label(datainfo);    
-	restore_sample_state(FALSE);
-	strcpy(line, "smpl full");
-	verify_and_record_command(line);
+	gretl_command_strcpy("smpl full");
+	check_and_record_command();
     }
 }
 
