@@ -77,7 +77,6 @@ extern int orig_vars;
 
 /* global state variables */
 extern int data_status;
-extern char line[MAXLINE];
 extern char *storelist;
 extern float gui_scale;
 
@@ -97,6 +96,7 @@ extern int updater;
 extern int wimp;
 #endif
 extern char calculator[MAXSTR];
+extern char latex[MAXSTR];
 extern char viewdvi[MAXSTR];
 extern char viewps[MAXSTR];
 #if defined(HAVE_AUDIO) && !defined(G_OS_WIN32)
