@@ -78,15 +78,19 @@ enum file_ops {
     OPEN_ASCII,
     APPEND_ASCII,
     OPEN_BOX,
+    OPEN_OCTAVE,
+    APPEND_OCTAVE,
     OPEN_GNUMERIC,
     APPEND_GNUMERIC,
     OPEN_EXCEL,
     APPEND_EXCEL,
     OPEN_SESSION,
+    OPEN_MARKERS,
     END_OPEN,      /* marker for end of file open section */
     SAVE_DATA,
     SAVE_DATA_AS,
     SAVE_GZDATA,
+    SAVE_DBDATA,
     SAVE_BIN1,
     SAVE_BIN2,
     EXPORT_OCTAVE,
@@ -97,10 +101,7 @@ enum file_ops {
     COPY_CSV,
     END_SAVE_DATA,  /* marker for end of data-saving section */
     SAVE_CMDS,
-    SAVE_TEX_TAB,
-    SAVE_TEX_EQ,
-    SAVE_TEX_TAB_FRAG,
-    SAVE_TEX_EQ_FRAG,
+    SAVE_TEX,
     SAVE_SCRIPT,
     SAVE_OUTPUT,
     SAVE_SESSION,
@@ -143,11 +144,12 @@ enum stat_codes {
 };
 
 enum exec_codes {
-    CONSOLE_EXEC,
-    SCRIPT_EXEC,
-    SESSION_EXEC,
-    REBUILD_EXEC,
-    SAVE_SESSION_EXEC
+    CONSOLE_EXEC      = 1 << 0,
+    SCRIPT_EXEC       = 1 << 1,
+    SESSION_EXEC      = 1 << 2,
+    REBUILD_EXEC      = 1 << 3,
+    SAVE_SESSION_EXEC = 1 << 4,
+    INCLUDE_EXEC      = 1 << 5
 };
 
 enum clipstuff {
@@ -157,16 +159,6 @@ enum clipstuff {
     TARGET_RTF
 };
 
-enum copy_variants {
-    COPY_SELECTION = 1,
-    COPY_TEXT,
-    COPY_LATEX,
-    COPY_LATEX_EQUATION,
-    COPY_RTF,
-    COPY_TEXT_AS_RTF,
-    COPY_EMF
-};
-
 enum data_status {
     HAVE_DATA     = 1 << 0,
     BOOK_DATA     = 1 << 1,
@@ -207,16 +199,22 @@ enum font_selections {
     GRAPH_FONT_SELECTION
 };
 
-enum latex_views {
-    LATEX_VIEW_TABULAR,
-    LATEX_VIEW_EQUATION,
-    LATEX_VIEW_MODELTABLE
-};
-
 enum calc_functions {
     CALC_PVAL,
     CALC_DIST,
     CALC_TEST
 };
 
+enum dynamic_forecast_status {
+    DYNAMIC_NA,
+    DYNAMIC_OK,
+    DYNAMIC_FORCED
+};
+
+#define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
+	                         c == CORR || c == FCASTERR || \
+	                         c == FCAST || c == COEFFINT || \
+	                         c == COVAR || c == VIEW_MODEL || \
+                                 c == VIEW_MODELTABLE || c == VAR)
+
 #endif /* GRETL_ENUMS_H */
