@@ -74,9 +74,10 @@ int create_child_process (char *prog, char *env)
 
 void startR (const char *Rcommand)
 {
-    char Rprofile[MAXLEN], Rdata[MAXLEN], line[MAXLEN];
+    char Rprofile[MAXLEN], Rdata[MAXLEN], Rline[MAXLEN];
     const char *supp1 = "--no-init-file";
     const char *supp2 = "--no-restore-data";
+    int *list;
     FILE *fp;
     int enverr;
 
@@ -100,15 +101,20 @@ void startR (const char *Rcommand)
     } 	
 
     build_path(paths.userdir, "Rdata.tmp", Rdata, NULL);
-    sprintf(line, "store \"%s\" -r", Rdata); 
-    if (verify_and_record_command(line) ||
-	write_data(Rdata, get_cmd_list(), (const double **) Z, datainfo, 
+
+    sprintf(Rline, "store \"%s\" -r", Rdata);
+    list = command_list_from_string(Rline);
+
+    if (list == NULL ||
+	write_data(Rdata, list, (const double **) Z, datainfo, 
 		   OPT_R, NULL)) {
 	errbox(_("Write of R data file failed"));
 	fclose(fp);
 	return; 
     }
 
+    free(list);
+
     if (dataset_is_time_series(datainfo)) {
 	fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fp);
 	fputs("if (vnum > 1.89) library(stats) else library(ts)\n", fp);
@@ -131,15 +137,17 @@ void startR (const char *Rcommand)
 
     fclose(fp);
 
-    sprintf(line, "\"%s\" %s %s", Rcommand, supp1, supp2);
-    create_child_process(line, NULL);
+    sprintf(Rline, "\"%s\" %s %s", Rcommand, supp1, supp2);
+    create_child_process(Rline, NULL);
 }
 
 char *slash_convert (char *str, int which)
 {
     char *p;
 
-    if (str == NULL) return NULL;
+    if (str == NULL) {
+	return NULL;
+    }
 
     p = str;
     while (*p) {
@@ -366,11 +374,38 @@ static int set_network_cfg_filename (const char *prog)
     return 0;
 }
 
+static int set_gd_fontpath (void)
+{
+    char fpath[MAX_PATH];
+    char *envstr;
+    UINT usz;
+
+    usz = GetWindowsDirectory(fpath, MAX_PATH);
+    if (usz == 0 || usz > MAX_PATH - 6) {
+        return 1;
+    }
+
+    fpath[2] = '/';
+    strcat(fpath, "/fonts");
+
+    SetEnvironmentVariable("GDFONTPATH", fpath);
+
+    envstr = malloc(strlen(fpath) + 12);
+    if (envstr != NULL) {
+	sprintf(envstr, "GDFONTPATH=%s", fpath);
+	putenv(envstr);
+	free(envstr);
+    }
+
+    return 0;        
+}
+
 void gretl_win32_init (const char *progname)
 {
     set_network_cfg_filename(progname);
 
     read_rc(); /* get config info from registry */
+    set_gd_fontpath();
 
 # ifdef HUSH_RUNTIME_WARNINGS
     hush_warnings();
@@ -443,7 +478,7 @@ void win32_make_user_dirs (void)
     win_mkdir(dirname);
 }
 
-static int win_copy_buf (char *buf, int format, size_t buflen)
+static int win_copy_buf (const char *buf, int fmt, size_t buflen)
 {
     HGLOBAL winclip;
     LPTSTR ptr;
@@ -464,15 +499,15 @@ static int win_copy_buf (char *buf, int format, size_t buflen)
 
     EmptyClipboard();
 
-    if (doing_nls() && (format == COPY_TEXT || format == COPY_TEXT_AS_RTF)) { 
+    if (doing_nls() && (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT)) { 
 	tr = my_locale_from_utf8(buf);
 	if (tr == NULL) {
 	    CloseClipboard();
 	    return 1;
 	}
-	winbuf = dosify_buffer(tr, format);
+	winbuf = dosify_buffer(tr, fmt);
     } else {
-	winbuf = dosify_buffer(buf, format);
+	winbuf = dosify_buffer(buf, fmt);
     }
 
     if (winbuf == NULL) {
@@ -492,9 +527,9 @@ static int win_copy_buf (char *buf, int format, size_t buflen)
     memcpy(ptr, winbuf, len);
     GlobalUnlock(winclip); 
 
-    if (format == COPY_RTF || format == COPY_TEXT_AS_RTF) { 
+    if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) { 
 	clip_format = RegisterClipboardFormat("Rich Text Format");
-    } else if (format == COPY_CSV) {
+    } else if (fmt == GRETL_FORMAT_CSV) {
 	clip_format = RegisterClipboardFormat("CSV");
     } else {
 	clip_format = CF_TEXT;
@@ -513,9 +548,11 @@ static int win_copy_buf (char *buf, int format, size_t buflen)
     return 0;
 }
 
-int prn_to_clipboard (PRN *prn, int copycode)
+int prn_to_clipboard (PRN *prn, int fmt)
 {
-    return win_copy_buf(prn->buf, copycode, 0);
+    const char *buf = gretl_print_get_buffer(prn);
+
+    return win_copy_buf(buf, fmt, 0);
 }
 
 int win_buf_to_clipboard (const char *buf)
