@@ -7,10 +7,14 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
     GdkPixbuf *pbuf = NULL;
 
     sprintf(temp, "%sgpttmp.XXXXXX", paths.userdir);
-    if (mktemp(temp) == NULL) return NULL;
+    if (mktemp(temp) == NULL) {
+	return NULL;
+    }
 
     ftmp = gretl_fopen(temp, "w");
-    if (ftmp == NULL) return NULL;
+    if (ftmp == NULL) {
+	return NULL;
+    }
 
     fsrc = gretl_fopen(fname, "r");
     if (fsrc == NULL) {
@@ -60,8 +64,6 @@ static GdkPixbuf *png_mono_pixbuf (const char *fname)
 #define STATS_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
                    "\\cellx2700\\cellx4000\\cellx6700\\cellx8000\n\\intbl"
 
-/* ............................................................. */
-
 static void printfrtf (double zz, PRN *prn, int endrow)
 {
     /* was using "qr", for right alignment */
@@ -90,11 +92,8 @@ static void printfrtf (double zz, PRN *prn, int endrow)
 #define VAR_SUMM_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
                       "\\cellx2000\\cellx4000\\cellx6000\\cellx8000\n"
 
-/* ............................................................. */
-
-void rtfprint_summary (GRETLSUMMARY *summ,
-		       const DATAINFO *pdinfo,
-		       PRN *prn)
+static void 
+rtfprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN], tmp[128];
     int i, vi;
@@ -113,8 +112,10 @@ void rtfprint_summary (GRETLSUMMARY *summ,
 	pprintf(prn, "%s\\par\n\n", tmp);
 	pputs(prn, "{" VAR_SUMM_ROW "\\intbl ");
     } else {
-	strcpy(tmp, I_("(missing values will be skipped)"));
-	pprintf(prn, "%s\\par\n\n", tmp);
+	if (summ->missing) {
+	    strcpy(tmp, I_("(missing values were skipped)"));
+	    pprintf(prn, "%s\\par\n\n", tmp); /* FIXME */
+	}
 	pprintf(prn, "{" SUMM_ROW
 		"\\intbl \\qc %s\\cell", I_("Variable"));
     }
@@ -163,8 +164,6 @@ void rtfprint_summary (GRETLSUMMARY *summ,
     pputs(prn, "}}\n");
 }
 
-/* ............................................................. */
-
 static void printftex (double zz, PRN *prn, int endrow)
 {
     if (na(zz)) {
@@ -179,22 +178,18 @@ static void printftex (double zz, PRN *prn, int endrow)
 	tex_dcolumn_double(zz, s);
 
 	if (endrow) {
-	    pprintf(prn, "%s\\\\", s);
+	    pprintf(prn, "$%s$\\\\", s);
 	} else {
-	    pprintf(prn, "%s & ", s);
+	    pprintf(prn, "$%s$ & ", s);
 	}
     }	
 }
 
-/* ............................................................. */
-
-void texprint_summary (GRETLSUMMARY *summ,
-		       const DATAINFO *pdinfo,
-		       PRN *prn)
+static void 
+texprint_summary (const Summary *summ, const DATAINFO *pdinfo, PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN], vname[16], tmp[128];
     int i, vi;
-    char pt = get_local_decpoint();
 
     ntodate(date1, pdinfo->t1, pdinfo);
     ntodate(date2, pdinfo->t2, pdinfo);
@@ -211,12 +206,12 @@ void texprint_summary (GRETLSUMMARY *summ,
 	pprintf(prn, "%s\\\\[8pt]\n\n", tmp);
 	pputs(prn, "\\begin{tabular}{rrrr}\n");
     } else {
-	strcpy(tmp, I_("(missing values denoted by $-999$ will be "
-		"skipped)"));
-	pprintf(prn, "%s\\\\[8pt]\n\n", tmp);
-	pprintf(prn, "\\begin{tabular}{lD{%c}{%c}{-1}"
-		"D{%c}{%c}{-1}D{%c}{%c}{-1}D{%c}{%c}{-1}}\n", 
-		pt, pt, pt, pt, pt, pt, pt, pt);
+	if (summ->missing) {
+	    pprintf(prn, "%s\\\\[8pt]\n\n", I_("(missing values were skipped)"));
+	} else {
+	    pputs(prn, "\n\\vspace{8pt}\n\n");
+	}
+	pputs(prn, "\\begin{tabular}{lrrrr}\n");
 	pprintf(prn, "%s &", I_("Variable"));
     }
 
@@ -267,10 +262,17 @@ void texprint_summary (GRETLSUMMARY *summ,
     }
 
     pputs(prn, "\\end{tabular}\n\\end{center}\n");
-    
 }
 
-/* ......................................................... */ 
+void special_print_summary (const Summary *summ, const DATAINFO *pdinfo,
+			    PRN *prn)
+{
+    if (tex_format(prn)) {
+	texprint_summary(summ, pdinfo, prn);
+    } else if (rtf_format(prn)) {
+	rtfprint_summary(summ, pdinfo, prn);
+    }
+}
 
 static void tex_outxx (double xx, PRN *prn)
 {
@@ -281,8 +283,6 @@ static void tex_outxx (double xx, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void rtf_outxx (double xx, PRN *prn)
 {
     if (na(xx)) {
@@ -292,70 +292,62 @@ static void rtf_outxx (double xx, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
-static void rtf_corr_row (int lo, PRN *prn)
+static void rtf_vmat_row (int lo, PRN *prn)
 {
+    int i, w = 1400;
+    int cmax = (lo + 1 > 6)? 6 : lo + 1;
+
     pputs(prn, "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262");
 
-    if (lo == 2) {
-	pputs(prn, "\\cellx1500\\cellx3000\\cellx3500\n");
-    }
-    else if (lo == 3) {
-	pputs(prn, "\\cellx1500\\cellx3000\\cellx4500\\cellx5000\n");
-    }
-    else if (lo == 4) {
-	pputs(prn, "\\cellx1500\\cellx3000\\cellx4500\\cellx6000"
-		"\\cellx6500\n");
-    }
-    else {
-	pputs(prn, "\\cellx1500\\cellx3000\\cellx4500\\cellx6000"
-		"\\cellx7500\\cellx8000\n");
+    for (i=1; i<=cmax; i++) {
+	pprintf(prn, "\\cellx%d", w * i);
     }
 
-    pputs(prn, "\\intbl ");
+    pputs(prn, "\n\\intbl ");
 }
 
-/* ......................................................... */ 
-
 static void rtf_table_pad (int pad, PRN *prn)
 {
     while (pad--) pputs(prn, "\\cell ");
 }
 
-/* ......................................................... */ 
+static void rtf_vmat_blank_row (int lo, int n, PRN *prn)
+{
+    rtf_vmat_row(lo, prn);
+    while (n--) pputs(prn, "\\cell ");
+    pputs(prn, "\\intbl \\row\n");
+}
 
 static void
-rtfprint_matrix (const double *vec, const int *list,
-		 int t1, int t2, int n, int ci,
-		 const DATAINFO *pdinfo, PRN *prn)
+rtfprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 {
     register int i, j;
-    int lo, ljnf, nf, li2, p, k, index, ij2;
+    int n = vmat->t2 - vmat->t1 + 1;
+    int lo, nf, li2, p, k, idx, ij2;
     char tmp[128];
     enum { FIELDS = 5 };
 
-    if (ci == CORR) {
+    if (vmat->ci == CORR) {
 	char date1[OBSLEN], date2[OBSLEN];
 
-	ntodate(date1, t1, pdinfo);
-	ntodate(date2, t2, pdinfo);
+	ntodate(date1, vmat->t1, pdinfo);
+	ntodate(date2, vmat->t2, pdinfo);
 
 	sprintf(tmp, I_("Correlation coefficients, using the observations "
 			"%s - %s"), date1, date2);
-	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n(%s)\\par\n",
-		tmp, I_("skipping any missing values"));
-
+	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n", tmp);
+	if (vmat->missing) {
+	    pprintf(prn, "(%s)\\par\n", I_("(missing values were skipped)"));
+	}
 	sprintf(tmp, I_("5%% critical value (two-tailed) = %.4f for n = %d"), 
 		rhocrit95(n), n);
 	pprintf(prn, "%s\\par\n\\par\n{", tmp);
-    } 
-    else if (ci == COVAR) {
+    } else {
 	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n\\par\n{",
 		I_("Coefficient covariance matrix"));
     }
     
-    lo = list[0];
+    lo = vmat->dim;
 
     for (i=0; i<=lo/FIELDS; i++) {
 	int pad;
@@ -367,30 +359,31 @@ rtfprint_matrix (const double *vec, const int *list,
 
 	pad = (lo > FIELDS)? FIELDS - p : lo - p;
 
-	rtf_corr_row(lo, prn);
+	rtf_vmat_row(lo, prn);
 
 	if (pad) rtf_table_pad(pad, prn);
 
 	/* print the varname headings */
-	for (j=1; j<=p; ++j)  {
-	    ljnf = list[j + nf];
-	    pprintf(prn, "%d) %s\\cell %s", ljnf, pdinfo->varname[ljnf],
-		    (j == p)? "\\cell \\intbl \\row\n" : "");
+	for (j=0; j<p; j++)  {
+	    pprintf(prn, "%s\\cell %s", vmat->names[j + nf],
+		    (j == p - 1)? "\\cell \\intbl \\row\n" : "");
 	}
 
 	/* print rectangular part, if any, of matrix */
 	for (j=0; j<nf; j++) {
 	    pputs(prn, "\\intbl "); 
-	    if (pad) rtf_table_pad(pad, prn);
+	    if (pad) {
+		rtf_table_pad(pad, prn);
+	    }
 	    for (k=0; k<p; k++) {
-		index = ijton(j, nf+k, lo);
-		if (ci == CORR) {
-		    rtf_outxx(vec[index], prn);
+		idx = ijton(j, nf+k, lo);
+		if (vmat->ci == CORR) {
+		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
-		    printfrtf(vec[index], prn, 0);
+		    printfrtf(vmat->vec[idx], prn, 0);
 		}
 	    }
-	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[j+1]);
+	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", vmat->names[j]);
 	}
 
 	/* print upper triangular part of matrix */
@@ -399,152 +392,130 @@ rtfprint_matrix (const double *vec, const int *list,
 	    rtf_table_pad(pad + j, prn);
 	    ij2 = nf + j;
 	    for (k=j; k<p; k++) {
-		index = ijton(ij2, nf+k, lo);
-		if (ci == CORR) {
-		    rtf_outxx(vec[index], prn);
+		idx = ijton(ij2, nf+k, lo);
+		if (vmat->ci == CORR) {
+		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
-		    printfrtf(vec[index], prn, 0);
+		    printfrtf(vmat->vec[idx], prn, 0);
 		}
 	    }
-	    pprintf(prn, "\\ql (%d\\cell \\intbl \\row\n", list[ij2+1]);
+	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", vmat->names[ij2]);
 	}
+	
+	rtf_vmat_blank_row(lo, pad + p, prn);
     }
+
     pputs(prn, "}}\n");
 }
 
-/* ........................................................... */
-
-void rtfprint_corrmat (CORRMAT *corr,
-		       const DATAINFO *pdinfo, 
-		       PRN *prn)
-{
-    rtfprint_matrix(corr->xpx, corr->list, corr->t1, corr->t2,
-		    corr->n, CORR, pdinfo, prn);
-}
-
-/* ......................................................... */
-
 static void
-texprint_matrix (const double *vec, const int *list,
-		 int t1, int t2, int n, int ci,
-		 const DATAINFO *pdinfo, PRN *prn)
+texprint_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, PRN *prn)
 {
     register int i, j;
-    int lo, ljnf, nf, li2, p, k, index, ij2;
-    char vname[16], tmp[128];
-    int fields;
+    int n = vmat->t2 - vmat->t1 + 1;
+    int lo, nf, li2, p, k, idx, ij2;
+    char vname[16];
+    int fields = 5;
 
-    if (ci == CORR) fields = 5;
-    else fields = 4;
+    lo = vmat->dim;
 
-    lo = list[0];
-
-    if (ci == CORR) {
+    if (vmat->ci == CORR) {
 	char date1[OBSLEN], date2[OBSLEN];
 
-	ntodate(date1, t1, pdinfo);
-	ntodate(date2, t2, pdinfo);
+	ntodate(date1, vmat->t1, pdinfo);
+	ntodate(date2, vmat->t2, pdinfo);
 
-	sprintf(tmp, I_("Correlation coefficients, using the observations "
+	pputs(prn, "\\begin{center}\n");
+	pprintf(prn, I_("Correlation coefficients, using the observations "
 			"%s--%s"), date1, date2);
-	pprintf(prn, "\\begin{center}\n%s\\\\\n(%s)\\\\\n", 
-		tmp, I_("skipping any missing values"));
-
-	sprintf(tmp, I_("5\\%% critical value (two-tailed) = %.4f for n = %d"), 
+	pputs(prn, "\\\\\n");
+	if (vmat->missing) {
+	    pputs(prn, I_("(missing values were skipped)"));
+	    pputs(prn, "\\\\\n");
+	}
+	pprintf(prn, I_("5\\%% critical value (two-tailed) = %.4f for n = %d"), 
 		rhocrit95(n), n);
-	pprintf(prn, "%s\\\\\n", tmp);
-    }
-    else if (ci == COVAR) {
+	pputs(prn, "\\\\\n");
+    } else {
 	pprintf(prn, "\\begin{center}\n%s\\\\\n", 
 		I_("Coefficient covariance matrix"));
     }
 
     pputs(prn, "\\vspace{8pt}\n");
 
-    if (ci == CORR) {
-	pprintf(prn, "\\begin{tabular}{rrr%s}\n",
-		(lo == 3)? "r" : (lo == 4)? "rr" : "rrr");
-    } else {
-	char pt = get_local_decpoint();
-
-	pputs(prn, "\\begin{tabular}{");
-	for (i=0; i<=lo && i<fields; i++) {
-	    pprintf(prn, "D{%c}{%c}{-1}", pt, pt);
-	}
-	pputs(prn, "r}\n");
-    }
-
     for (i=0; i<=lo/fields; i++) {
 	nf = i * fields;
 	li2 = lo - nf;
+	/* p = number of cols we'll print */
 	p = (li2 > fields) ? fields : li2;
 	if (p == 0) break;
 
+	pputs(prn, "\\begin{tabular}{");
+	for (j=0; j<p; j++) {
+	    pputc(prn, 'r');
+	}
+	pputs(prn, "l}\n");
+
 	/* print the varname headings */
-	for (j=1; j<=p; ++j)  {
-	    ljnf = list[j + nf];
-	    tex_escape(vname, pdinfo->varname[ljnf]);
-	    if (ci == CORR) {
-		pprintf(prn, "%d) %s%s", ljnf, vname,
-			(j == p)? " &\\\\" : " & ");
+	for (j=0; j<p; j++)  {
+	    tex_escape(vname, vmat->names[j + nf]);
+	    if (vmat->ci == CORR) {
+		pprintf(prn, "%s%s", vname,
+			(j == p - 1)? " &\\\\\n" : " & ");
 	    } else {
-		pprintf(prn, "\\multicolumn{1}{c}{%d) %s}%s", ljnf, vname,
-			(j == p)? " &\\\\\n" : " &\n");
+		pprintf(prn, "\\multicolumn{1}{c}{%s}%s", vname,
+			(j == p - 1)? " &\\\\\n" : " &\n");
 	    }
 	}
 	
-	/* insert spacers */
-	if (ci == CORR) {
-	    for (j=1; j<=p; ++j) {
-		pputs(prn, "\\rule{13ex}{0pt} & ");
-	    }
-	    pputs(prn, "\\\\\[-6pt]\n"); 
-	}   
-
 	/* print rectangular part, if any, of matrix */
 	for (j=0; j<nf; j++) {
 	    for (k=0; k<p; k++) {
-		index = ijton(j, nf+k, lo);
-		if (ci == CORR) {
-		    tex_outxx(vec[index], prn);
+		idx = ijton(j, nf+k, lo);
+		if (vmat->ci == CORR) {
+		    tex_outxx(vmat->vec[idx], prn);
 		} else {
-		    printftex(vec[index], prn, 0);
+		    printftex(vmat->vec[idx], prn, 0);
 		}
 	    }
-	    pprintf(prn, "(%d\\\\\n", list[j+1]);
+	    tex_escape(vname, vmat->names[j]);
+	    pprintf(prn, "%s\\\\\n", vname);
 	}
 
 	/* print upper triangular part of matrix */
 	for (j=0; j<p; ++j) {
 	    ij2 = nf + j;
-	    for (k=0; k<j; k++) pputs(prn, " & ");
+	    for (k=0; k<j; k++) {
+		pputs(prn, " & ");
+	    }
 	    for (k=j; k<p; k++) {
-		index = ijton(ij2, nf+k, lo);
-		if (ci == CORR) {
-		    tex_outxx(vec[index], prn);
+		idx = ijton(ij2, nf+k, lo);
+		if (vmat->ci == CORR) {
+		    tex_outxx(vmat->vec[idx], prn);
 		} else {
-		    printftex(vec[index], prn, 0);
+		    printftex(vmat->vec[idx], prn, 0);
 		}
 	    }
-	    pprintf(prn, "(%d\\\\\n", list[ij2+1]);
+	    tex_escape(vname, vmat->names[ij2]);
+	    pprintf(prn, "%s\\\\\n", vname);
 	}
-	pputs(prn, "\\\\\n");
+
+	pputs(prn, "\\end{tabular}\n\n");
     }
-    pputs(prn, "\\end{tabular}\n\\end{center}\n");
+
+    pputs(prn, "\\end{center}\n");
 }
 
-/* ........................................................... */
-
-void texprint_corrmat (CORRMAT *corr,
-		       const DATAINFO *pdinfo, 
-		       PRN *prn)
+void special_print_vmatrix (const VMatrix *vmat, const DATAINFO *pdinfo, 
+			    PRN *prn)
 {
-    texprint_matrix(corr->xpx, corr->list, corr->t1, corr->t2,
-		    corr->n, CORR, pdinfo, prn);
+    if (tex_format(prn)) {
+	texprint_vmatrix(vmat, pdinfo, prn);
+    } else if (rtf_format(prn)) {
+	rtfprint_vmatrix(vmat, pdinfo, prn);
+    }
 }
 
-/* ........................................................... */
-
 static 
 void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo, 
 			 PRN *prn)
@@ -564,8 +535,6 @@ void tex_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     pputs(prn, "\n\\end{raggedright}\n");
 }
 
-/* ........................................................... */
-
 static 
 void rtf_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo, 
 			 PRN *prn)
@@ -585,10 +554,26 @@ void rtf_fit_resid_head (const FITRESID *fr, const DATAINFO *pdinfo,
     pprintf(prn, "\\qc %s\\par\n\\par\n", tmp);
 }
 
-/* ........................................................... */
+static void tex_print_x (double x, int pmax, PRN *prn)
+{
+    if (x < 0) {
+	pputs(prn, "$-$");
+    } 
 
-void 
-texprint_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
+    x = fabs(x);
+
+    if (pmax != PMAX_NOT_AVAILABLE) {
+	pprintf(prn, "%.*f", pmax, x);
+    } else {
+	pprintf(prn, "%g", x);
+    }
+
+    pputs(prn, " & ");
+}
+
+static void texprint_fit_resid (const FITRESID *fr, 
+				const DATAINFO *pdinfo, 
+				PRN *prn)
 {
     int t, anyast = 0;
     double xx;
@@ -599,57 +584,52 @@ texprint_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo, PRN *prn)
     tex_escape(vname, fr->depvar);
 
     pprintf(prn, "\n\\begin{center}\n"
-	    "\\begin{tabular}{rrrrl}\n"
+	    "\\begin{longtable}{rrrrl}\n"
 	    " & \n"
 	    " \\multicolumn{1}{c}{%s} & \n"
 	    "  \\multicolumn{1}{c}{%s} & \n"
 	    "   \\multicolumn{1}{c}{%s}\\\\\n",
-	    vname, I_("fitted"), I_("residuals"));
+	    vname, I_("fitted"), I_("residual"));
 
     for (t=0; t<fr->nobs; t++) {
-	print_obs_marker(t + fr->t1, pdinfo, prn);
+	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
 	pputs(prn, " & ");
 
 	if (na(fr->actual[t])) {
-	    pputs(prn, "\\\\\n");
+	    ;
 	} else if (na(fr->fitted[t])) {
-	    pprintf(prn, "%13.*f \\\\\n", fr->pmax, fr->actual[t]);
+	    tex_print_x(fr->actual[t], fr->pmax, prn);
 	} else {
 	    int ast;
 
 	    xx = fr->actual[t] - fr->fitted[t];
 	    ast = (fabs(xx) > 2.5 * fr->sigma);
 	    if (ast) anyast = 1;
-	    if (fr->pmax != PMAX_NOT_AVAILABLE) {
-		pprintf(prn, "%13.*f & %13.*f & %13.*f & %s \\\\\n", 
-			fr->pmax, fr->actual[t],
-			fr->pmax, fr->fitted[t], fr->pmax, xx,
-			(ast)? " *" : "");
-	    } else {
-		pprintf(prn, "%13g & %13g & %13g & %s \\\\\n", 
-			fr->actual[t],
-			fr->fitted[t], xx,
-			(ast)? " *" : "");
+	    tex_print_x(fr->actual[t], fr->pmax, prn);
+	    tex_print_x(fr->fitted[t], fr->pmax, prn);
+	    tex_print_x(xx, fr->pmax, prn);
+	    if (ast) {
+		pputs(prn, " *");
 	    }
 	}
+	pputs(prn, " \\\\\n");
     }
 
-    pputs(prn, "\\end{tabular}\n"
-	  "\\end{center}\n\n");
+    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
 
-    if (anyast) pputs(prn, I_("\\textit{Note}: * denotes a residual "
-			      "in excess of 2.5 standard errors\n\n"));
+    if (anyast) {
+	pputs(prn, I_("\\textit{Note}: * denotes a residual "
+		      "in excess of 2.5 standard errors\n\n"));
+    }
 }
 
-/* .................................................................. */
-
 #define FR_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
                 "\\cellx800\\cellx2400\\cellx4000\\cellx5600" \
                 "\\cellx6100\n"
 
-void rtfprint_fit_resid (const FITRESID *fr, 
-			 const DATAINFO *pdinfo, 
-			 PRN *prn)
+static void rtfprint_fit_resid (const FITRESID *fr, 
+				const DATAINFO *pdinfo, 
+				PRN *prn)
 {
     double xx;
     int anyast = 0;
@@ -703,26 +683,90 @@ void rtfprint_fit_resid (const FITRESID *fr,
     pputs(prn, "}\n");
 }
 
+void special_print_fit_resid (const FITRESID *fr, 
+			      const DATAINFO *pdinfo, 
+			      PRN *prn)
+{
+    if (tex_format(prn)) {
+	texprint_fit_resid(fr, pdinfo, prn);
+    } else if (rtf_format(prn)) {
+	rtfprint_fit_resid(fr, pdinfo, prn);
+    }
+}
+
 /* .................................................................. */
 
-void texprint_fcast_with_errs (const FITRESID *fr, 
-			       const DATAINFO *pdinfo, 
-			       PRN *prn)
+static void texprint_fcast_x (double x, int places, char *str)
 {
+    if (places != PMAX_NOT_AVAILABLE && !na(x)) {
+	sprintf(str, "%.*f", places, x);
+    } else {
+	tex_dcolumn_double(x, str);
+    }
+}
+
+static void texprint_fcast_without_errs (const FITRESID *fr, 
+					 const DATAINFO *pdinfo, 
+					 PRN *prn)
+{
+    char actual[32], fitted[32];
+    char vname[16];
+    char pt = get_local_decpoint();
     int t;
+
+    pputs(prn, "%% The table below needs the \"dcolumn\" and "
+	  "\"longtable\" packages\n\n");
+
+    pprintf(prn, "\\begin{center}\n"
+	    "\\begin{longtable}{%%\n"
+	    "r%% col 1: obs\n"
+	    "  l%% col 2: varname\n"
+	    "    D{%c}{%c}{-1}}%% col 3: fitted\n",
+	    pt, pt);
+
+    tex_escape(vname, fr->depvar);
+
+    pprintf(prn, "%s & %s & \\multicolumn{1}{c}{%s} \\\\ [4pt] \n",
+	    I_("Obs"), vname, I_("prediction"));
+
+    for (t=fr->pre_n; t<fr->nobs; t++) {
+	texprint_fcast_x(fr->actual[t], fr->pmax, actual);
+	texprint_fcast_x(fr->fitted[t], fr->pmax, fitted);
+	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
+	pprintf(prn, " & %s & %s \\\\\n",
+		actual, fitted);
+    }
+
+    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
+}
+
+static void texprint_fcast_with_errs (const FITRESID *fr, 
+				      const DATAINFO *pdinfo, 
+				      PRN *prn)
+{
     double maxerr;
+    int pmax = fr->pmax;
+    int errpmax = fr->pmax;
     char actual[32], fitted[32], sderr[32], lo[32], hi[32];
     char vname[16];
     char pt = get_local_decpoint();
+    int t;
 
-    pprintf(prn, I_("For 95 percent confidence intervals, "
-		    "$t(%d, .025) = %.3f$\n\n"), 
-	    fr->df, fr->tval);
+    pputs(prn, "\\begin{center}\n");
+    if (fr->model_ci == ARMA) {
+	pprintf(prn, _("For 95\\%% confidence intervals, $z(.025) = %.2f$\n\n"), 
+		1.96);
+    } else {
+	pprintf(prn, I_("For 95\\%% confidence intervals, $t(%d, .025) = %.3f$\n\n"), 
+		fr->df, fr->tval);
+    }
+    pputs(prn, "\\end{center}\n");
 
-    pputs(prn, "%% The table below needs the \"dcolumn\" package\n\n");
+    pputs(prn, "%% The table below needs the \"dcolumn\" and "
+	  "\"longtable\" packages\n\n");
 
     pprintf(prn, "\\begin{center}\n"
-	    "\\begin{tabular}{%%\n"
+	    "\\begin{longtable}{%%\n"
 	    "r%% col 1: obs\n"
 	    "  l%% col 2: varname\n"
 	    "    D{%c}{%c}{-1}%% col 3: fitted\n"
@@ -744,43 +788,86 @@ void texprint_fcast_with_errs (const FITRESID *fr,
     pputs(prn, "& & & & \\multicolumn{1}{c}{low} & "
 	  "\\multicolumn{1}{c}{high} \\\\\n");
 
-    for (t=0; t<fr->nobs; t++) {
-	maxerr = fr->tval * fr->sderr[t];
-	tex_dcolumn_double(fr->actual[t], actual);
-	tex_dcolumn_double(fr->fitted[t], fitted);
-	tex_dcolumn_double(fr->sderr[t], sderr);
-	tex_dcolumn_double(fr->fitted[t] - maxerr, lo);
-	tex_dcolumn_double(fr->fitted[t] + maxerr, hi);
-	print_obs_marker(t + fr->t1, pdinfo, prn);
+    if (pmax < 4) {
+	errpmax = pmax + 1;
+    }
+
+    for (t=fr->pre_n; t<fr->nobs; t++) {
+	double xlo, xhi;
+
+	if (na(fr->sderr[t])) {
+	    xlo = xhi = NADBL;
+	} else {
+	    maxerr = fr->tval * fr->sderr[t];
+	    xlo = fr->fitted[t] - maxerr;
+	    xhi = fr->fitted[t] + maxerr;
+	}
+	texprint_fcast_x(fr->actual[t], pmax, actual);
+	texprint_fcast_x(fr->fitted[t], pmax, fitted);
+	texprint_fcast_x(fr->sderr[t], errpmax, sderr);
+	texprint_fcast_x(xlo, pmax, lo);
+	texprint_fcast_x(xhi, pmax, hi);
+	tex_print_obs_marker(t + fr->t1, pdinfo, prn);
 	pprintf(prn, " & %s & %s & %s & %s & %s \\\\\n",
 		actual, fitted, sderr, lo, hi);
     }
 
-    pputs(prn, "\\end{tabular}\n"
-	  "\\end{center}\n\n");
+    pputs(prn, "\\end{longtable}\n\\end{center}\n\n");
 }
 
-/* .................................................................. */
-
 #define FC_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
-                "\\cellx800\\cellx2200\\cellx3600\\cellx5000" \
-                "\\cellx7800\n"
+                "\\cellx800\\cellx2200\\cellx3600\n"
 
-void rtfprint_fcast_with_errs (const FITRESID *fr, 
-			       const DATAINFO *pdinfo, 
-			       PRN *prn)
+#define FCE_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
+                 "\\cellx800\\cellx2200\\cellx3600\\cellx5000" \
+                 "\\cellx7800\n"
+
+static void rtfprint_fcast_without_errs (const FITRESID *fr, 
+					 const DATAINFO *pdinfo, 
+					 PRN *prn)
+{
+    int t;
+
+    pputs(prn, "{\\rtf1\\par\n\n");
+
+    pputs(prn, "{" FC_ROW "\\intbl ");
+
+    pprintf(prn, 
+	    " \\qc %s\\cell"
+	    " \\qc %s\\cell"
+	    " \\qc %s\\cell"
+	    " \\intbl \\row\n", 
+	    I_("Obs"), fr->depvar, I_("prediction")); 
+
+    for (t=fr->pre_n; t<fr->nobs; t++) {
+	pputs(prn, "\\qr ");
+	print_obs_marker(t + fr->t1, pdinfo, prn);
+	pputs(prn, "\\cell"); 
+	printfrtf(fr->actual[t], prn, 0);
+	printfrtf(fr->fitted[t], prn, 0);
+    }
+
+    pputs(prn, "}}\n");
+}
+
+static void rtfprint_fcast_with_errs (const FITRESID *fr, 
+				      const DATAINFO *pdinfo, 
+				      PRN *prn)
 {
     int t;
     double maxerr;
     char tmp[128];
 
-    sprintf(tmp, I_("For 95 percent confidence intervals, "
-		    "t(%d, .025) = %.3f"), 
-	    fr->df, fr->tval);
-
+    if (fr->model_ci == ARMA) {
+	sprintf(tmp, I_("For 95%% confidence intervals, z(.025) = %.2f"), 
+		1.96);
+    } else {
+	sprintf(tmp, I_("For 95%% confidence intervals, t(%d, .025) = %.3f"), 
+		fr->df, fr->tval);
+    }
     pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n\\par\n", tmp);
 
-    pputs(prn, "{" FC_ROW "\\intbl ");
+    pputs(prn, "{" FCE_ROW "\\intbl ");
     pprintf(prn, 
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
@@ -793,7 +880,7 @@ void rtfprint_fcast_with_errs (const FITRESID *fr,
 	    /* xgettext:no-c-format */
 	    I_("95% confidence interval"));
 
-    for (t=0; t<fr->nobs; t++) {
+    for (t=fr->pre_n; t<fr->nobs; t++) {
 	pputs(prn, "\\qr ");
 	print_obs_marker(t + fr->t1, pdinfo, prn);
 	pputs(prn, "\\cell"); 
@@ -801,118 +888,128 @@ void rtfprint_fcast_with_errs (const FITRESID *fr,
 	printfrtf(fr->actual[t], prn, 0);
 	printfrtf(fr->fitted[t], prn, 0);
 	printfrtf(fr->sderr[t], prn, 0);
-	pprintf(prn, "\\qc (%#.*g, %#.*g)\\cell \\intbl \\row\n", 
-		GRETL_DIGITS, fr->fitted[t] - maxerr, 
-		GRETL_DIGITS, fr->fitted[t] + maxerr);
+	if (na(fr->sderr[t])) {
+	    pputs(prn, "\\qc \\cell \\intbl \\row\n");
+	} else {
+	    maxerr = fr->tval * fr->sderr[t];
+	    pprintf(prn, "\\qc (%#.*g, %#.*g)\\cell \\intbl \\row\n", 
+		    GRETL_DIGITS, fr->fitted[t] - maxerr, 
+		    GRETL_DIGITS, fr->fitted[t] + maxerr);
+	}
     }
 
     pputs(prn, "}}\n");
 }
 
-/* .................................................................. */
+void special_print_forecast (const FITRESID *fr, 
+			     const DATAINFO *pdinfo, 
+			     PRN *prn)
+{
+    if (tex_format(prn)) {
+	if (fr->sderr != NULL) {
+	    texprint_fcast_with_errs(fr, pdinfo, prn);
+	} else {
+	    texprint_fcast_without_errs(fr, pdinfo, prn);
+	}
+    } else if (rtf_format(prn)) {
+	if (fr->sderr != NULL) {
+	    rtfprint_fcast_with_errs(fr, pdinfo, prn);
+	} else {
+	    rtfprint_fcast_without_errs(fr, pdinfo, prn);
+	}
+    }
+}
 
 static void 
-texprint_coeff_interval (const CONFINT *cf, const DATAINFO *pdinfo, 
-			 int c, PRN *prn)
+texprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
     char vname[16];
 
-    tex_escape(vname, pdinfo->varname[cf->list[c]]);
-    pprintf(prn, " %3d) & %8s & ", cf->list[c], vname);
+    tex_escape(vname, cf->names[i]);
+    pprintf(prn, " %s & ", vname);
 
-    if (isnan(cf->coeff[c-2])) {
+    if (isnan(cf->coeff[i])) {
 	pprintf(prn, "\\multicolumn{1}{c}{%s} & ", I_("undefined"));
     } else {
 	char coeff[32];
 
-	tex_dcolumn_double(cf->coeff[c-2], coeff);
+	tex_dcolumn_double(cf->coeff[i], coeff);
 	pprintf(prn, "%s & ", coeff);
     }
 
-    if (isnan(cf->maxerr[c-2])) {
+    if (isnan(cf->maxerr[i])) {
 	pprintf(prn, "\\multicolumn{2}{c}{%s}", I_("undefined"));
     } else {
 	char lo[32], hi[32];
 
-	tex_dcolumn_double(cf->coeff[c-2] - cf->maxerr[c-2], lo);
-	tex_dcolumn_double(cf->coeff[c-2] + cf->maxerr[c-2], hi);
+	tex_dcolumn_double(cf->coeff[i] - cf->maxerr[i], lo);
+	tex_dcolumn_double(cf->coeff[i] + cf->maxerr[i], hi);
 	pprintf(prn, "%s & %s", lo, hi);
     }
     pputs(prn, "\\\\\n");
 }
 
-/* .................................................................. */
-
-void texprint_confints (const CONFINT *cf, const DATAINFO *pdinfo, 
-			PRN *prn)
+static void texprint_confints (const CoeffIntervals *cf, PRN *prn)
 {
-    int i, ncoeff = cf->list[0];
     char pt = get_local_decpoint();
+    int i;
 
     pprintf(prn, "$t(%d, .025) = %.3f$\n\n", cf->df, tcrit95(cf->df));
 
     pputs(prn, "%% The table below needs the \"dcolumn\" package\n\n");
 
     pprintf(prn, "\\begin{center}\n"
-	    "\\begin{tabular}{rrD{%c}{%c}{-1}D{%c}{%c}{-1}D{%c}{%c}{-1}}\n",
+	    "\\begin{tabular}{rD{%c}{%c}{-1}D{%c}{%c}{-1}D{%c}{%c}{-1}}\n",
 	    pt, pt, pt, pt, pt, pt);
 
-    pprintf(prn, " & %s%%\n"
+    pprintf(prn, " %s%%\n"
 	    " & \\multicolumn{1}{c}{%s}%%\n"
 	    "  & \\multicolumn{2}{c}{%s}\\\\\n",
 	    I_("Variable"), I_("Coefficient"),
 	    /* xgettext:no-c-format */
 	    I_("95\\% confidence interval"));
 
-    pprintf(prn, " & & & \\multicolumn{1}{c}{%s}%%\n"
+    pprintf(prn, " & & \\multicolumn{1}{c}{%s}%%\n"
 	    "  & \\multicolumn{1}{c}{%s}\\\\\n",
 	    I_("low"), I_("high"));
 
-    for (i=2; i<=ncoeff; i++) {
-	texprint_coeff_interval(cf, pdinfo, i, prn);
+    for (i=0; i<cf->ncoeff; i++) {
+	texprint_coeff_interval(cf, i, prn);
     }
 
     pputs(prn, "\\end{tabular}\n"
 	  "\\end{center}\n");
 }
 
-/* .................................................................. */
-
 static void 
-rtfprint_coeff_interval (const CONFINT *cf, const DATAINFO *pdinfo, 
-			 int c, PRN *prn)
+rtfprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
-    pprintf(prn, "\\qr %d)\\cell \\qc %s\\cell", cf->list[c], 
-	    pdinfo->varname[cf->list[c]]);
+    pprintf(prn, "\\qc %s\\cell", cf->names[i]);
 
-    printfrtf(cf->coeff[c-2], prn, 0);
+    printfrtf(cf->coeff[i], prn, 0);
 
-    if (isnan(cf->maxerr[c-2])) {
+    if (isnan(cf->maxerr[i])) {
 	pprintf(prn, "\\qc %s\\cell ", I_("undefined"));
     } else {
 	pprintf(prn, "\\qc (%#.*g, %#.*g)\\cell ", 
-		GRETL_DIGITS, cf->coeff[c-2] - cf->maxerr[c-2], 
-		GRETL_DIGITS, cf->coeff[c-2] + cf->maxerr[c-2]);
+		GRETL_DIGITS, cf->coeff[i] - cf->maxerr[i], 
+		GRETL_DIGITS, cf->coeff[i] + cf->maxerr[i]);
     }
     pputs(prn, " \\intbl \\row\n");
 }
 
-/* .................................................................. */
+#define CF_ROW  "\\trowd \\trgaph60\\trleft-30\\trrh262" \
+                "\\cellx2400\\cellx4000\\cellx7200\n" 
 
-#define CF_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
-                "\\cellx800\\cellx2400\\cellx4000\\cellx7200\n" 
-
-void rtfprint_confints (const CONFINT *cf, const DATAINFO *pdinfo, 
-			PRN *prn)
+static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
 {
-    int i, ncoeff = cf->list[0];
+    int i;
 
     pprintf(prn, "{\\rtf1\\par\n\\qc t(%d, .025) = %.3f\\par\n\\par\n", 
 	    cf->df, tcrit95(cf->df));
 
     pputs(prn, "{" CF_ROW "\\intbl ");
     pprintf(prn, 
-	    " \\qc \\cell"
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
@@ -921,31 +1018,20 @@ void rtfprint_confints (const CONFINT *cf, const DATAINFO *pdinfo,
 	    /* xgettext:no-c-format */
 	    I_("95% confidence interval"));
 
-    for (i=2; i<=ncoeff; i++) {
-	rtfprint_coeff_interval(cf, pdinfo, i, prn);
+    for (i=0; i<cf->ncoeff; i++) {
+	rtfprint_coeff_interval(cf, i, prn);
     }
 
     pputs(prn, "}}\n");
 }
 
-/* .................................................................. */
-
-void texprint_vcv (const VCV *vcv, 
-                   const DATAINFO *pdinfo, 
-                   PRN *prn)
+void special_print_confints (const CoeffIntervals *cf, PRN *prn)
 {
-    texprint_matrix(vcv->vec, vcv->list, 0, 0,
-                    0, COVAR, pdinfo, prn);
-}
-
-/* .................................................................. */
-
-void rtfprint_vcv (const VCV *vcv,
-                   const DATAINFO *pdinfo, 
-                   PRN *prn)
-{
-    rtfprint_matrix(vcv->vec, vcv->list, 0, 0,
-                    0, COVAR, pdinfo, prn);
+    if (tex_format(prn)) {
+	texprint_confints(cf, prn);
+    } else if (rtf_format(prn)) {
+	rtfprint_confints(cf, prn);
+    }
 }
 
 /* copy data to buffer in CSV format and place on clipboard */
@@ -1021,11 +1107,39 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     setlocale(LC_NUMERIC, "");
 #endif
 
-    if (pmax) free(pmax);
+    if (pmax != NULL) {
+	free(pmax);
+    }
 
     return 0;
 }
 
+static int real_csv_to_clipboard (const char *liststr)
+{
+    char line[MAXLINE];
+    int *list = NULL;
+    PRN *prn = NULL;
+    int err = 0;
+
+    sprintf(line, "store csv %s", liststr);
+    list = command_list_from_string(line);
+
+    if (list != NULL) {
+	err = bufopen(&prn);
+	if (!err) {
+	    err = data_to_buf_as_csv(list, prn);
+	}
+	if (!err) {
+	    err = prn_to_clipboard(prn, COPY_CSV);
+	}
+    }
+
+    free(list);
+    gretl_print_destroy(prn);
+
+    return err;
+}
+
 int csv_to_clipboard (void)
 {
     int err = 0;
@@ -1034,23 +1148,7 @@ int csv_to_clipboard (void)
     data_save_selection_wrapper(COPY_CSV);
 
     if (storelist != NULL && *storelist != 0) {
-	PRN *prn;
-
-	*line = 0;
-	sprintf(line, "store csv %s", storelist);
-
-	err = check_cmd(line);
-	if (!err) {
-	    err = bufopen(&prn);
-	}
-	if (!err) {
-	    err = data_to_buf_as_csv(get_cmd_list(), prn);
-	}
-	if (!err) {
-	    err = prn_to_clipboard(prn, COPY_CSV);
-	}
-
-        gretl_print_destroy(prn);
+	err = real_csv_to_clipboard(storelist);
 	free(storelist);
 	storelist = NULL;
     }
@@ -1061,31 +1159,14 @@ int csv_to_clipboard (void)
 int csv_selected_to_clipboard (void)
 {
     char *liststr;
-    PRN *prn = NULL;
     int err = 0;
 
-    liststr = mdata_selection_to_string(0);
-    if (liststr == NULL) return 0;
+    liststr = main_window_selection_as_string();
 
-    delimiter_dialog();
-
-    *line = 0;
-    sprintf(line, "store csv %s", liststr);
-    free(liststr);
-
-    err = check_cmd(line);
-    if (!err) {
-	err = bufopen(&prn);
-    }
-    if (!err) {
-	err = data_to_buf_as_csv(get_cmd_list(), prn);
-    }
-    if (!err) {
-	err = prn_to_clipboard(prn, COPY_CSV);
-    }
-
-    if (prn != NULL) {
-	gretl_print_destroy(prn);
+    if (liststr != NULL) {
+	delimiter_dialog();
+	err = real_csv_to_clipboard(liststr);
+	free(liststr);
     }
 
     return err;
