@@ -47,21 +47,19 @@ void set_replay_on (void);
 
 void set_replay_off (void);
 
-const int *get_cmd_list (void);
+int gretl_command_sprintf (const char *template, ...);
 
-char *user_fopen (const char *fname, char *fullname, PRN **pprn);
+int gretl_command_strcpy (const char *s);
+
+int *command_list_from_string (char *s);
+
+int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
 gint bufopen (PRN **pprn);
 
-PRN *bufopen_with_size (size_t sz);
-
-gint check_cmd (char *line);
- 
-gint cmd_init (char *line);
-
 void do_menu_op (gpointer data, guint action, GtkWidget *widget);
 
-void do_dialog_cmd (GtkWidget *widget, dialog_t *ddata);
+void do_two_var_test (GtkWidget *widget, gpointer p);
 
 void do_run_script (gpointer data, guint code, GtkWidget *w);
 
@@ -77,13 +75,15 @@ void exit_free_modelspec (void);
 
 int bool_subsample (gretlopt opt);
 
-void do_samplebool (GtkWidget *widget, dialog_t *ddata);
+void do_samplebool (GtkWidget *widget, dialog_t *dlg);
+
+int do_set_sample (void);
 
 void drop_all_missing (gpointer data, guint opt, GtkWidget *w);
 
 void count_missing (void);
 
-void do_add_markers (GtkWidget *widget, dialog_t *ddata);
+void do_add_markers (const char *fname);
 
 void do_remove_markers (gpointer data, guint u, GtkWidget *w);
 
@@ -93,13 +93,19 @@ int maybe_restore_full_data (int action);
 
 void gui_transpose_data (gpointer p, guint u, GtkWidget *w);
 
+int dataset_is_subsampled (void);
+
+void set_original_n (int n);
+
+int get_original_n (void);
+
 /* model-related functions */
 
 void do_coint (GtkWidget *widget, gpointer p);
 
 void do_coint2 (GtkWidget *widget, gpointer p);
 
-void do_forecast (GtkWidget *widget, dialog_t *ddata);
+void do_forecast (gpointer data, guint u, GtkWidget *w);
 
 void do_coeff_sum (GtkWidget *widget, gpointer p);
 
@@ -107,21 +113,19 @@ void do_add_omit (GtkWidget *widget, gpointer p);
 
 void do_lmtest (gpointer data, guint aux_code, GtkWidget *widget);
 
-void do_autocorr (GtkWidget *widget, dialog_t *ddata);
+void do_autocorr (gpointer data, guint u, GtkWidget *widget);
 
-void do_chow (GtkWidget *widget, dialog_t *ddata);
-
-void do_cusum (gpointer data, guint u, GtkWidget *widget);
+void do_chow_cusum (gpointer data, guint action, GtkWidget *w);
 
 void do_reset (gpointer data, guint u, GtkWidget *widget);
 
 void unit_root_test (gpointer data, guint u, GtkWidget *widget);
 
-void do_arch (GtkWidget *widget, dialog_t *ddata);
+void do_arch (gpointer data, guint u, GtkWidget *widget);
 
-void do_restrict (GtkWidget *widget, dialog_t *ddata);
+void do_restrict (GtkWidget *widget, dialog_t *dlg);
 
-void do_nls_model (GtkWidget *widget, dialog_t *ddata);
+void do_nls_model (GtkWidget *widget, dialog_t *dlg);
 
 void do_model (GtkWidget *widget, gpointer p);
 
@@ -139,32 +143,42 @@ void do_coeff_intervals (gpointer data, guint i, GtkWidget *w);
 
 void do_panel_diagnostics (gpointer data, guint u, GtkWidget *w);
 
+void do_spearman (GtkWidget *widget, gpointer p);
+
 #ifdef ENABLE_GMP
 void do_mp_ols (GtkWidget *widget, gpointer p);
 #endif
 
+int out_of_sample_info (int add_ok, int *t2);
+
 /* variable-related functions */
 
-void do_simdata (GtkWidget *widget, dialog_t *ddata);
+void do_simdata (GtkWidget *widget, dialog_t *dlg);
 
-void do_genr (GtkWidget *widget, dialog_t *ddata);
+void do_genr (GtkWidget *widget, dialog_t *dlg);
 
-void do_model_genr (GtkWidget *widget, dialog_t *ddata);
+void do_model_genr (GtkWidget *widget, dialog_t *dlg);
 
-void do_random (GtkWidget *widget, dialog_t *ddata);
+void do_random (GtkWidget *widget, dialog_t *dlg);
 
-void do_seed (GtkWidget *widget, dialog_t *ddata);
+void do_seed (GtkWidget *widget, dialog_t *dlg);
 
-void do_global_setmiss (GtkWidget *widget, dialog_t *ddata);
+void do_global_setmiss (GtkWidget *widget, dialog_t *dlg);
 
-void do_variable_setmiss (GtkWidget *widget, dialog_t *ddata);
+void do_variable_setmiss (GtkWidget *widget, dialog_t *dlg);
 
-void do_edit_label (GtkWidget *widget, dialog_t *ddata);
+void do_edit_label (GtkWidget *widget, dialog_t *dlg);
+
+int do_rename_variable (int v, const char *newname, int full);
+
+int record_varlabel_change (int v);
 
 void do_resid_freq (gpointer data, guint action, GtkWidget *widget);
 
 void do_freqplot (gpointer data, guint gamma, GtkWidget *widget);
 
+void do_corrgm (gpointer data, guint u, GtkWidget *widget);
+
 void do_pergm (gpointer data, guint opt, GtkWidget *widget);
 
 #if defined (HAVE_TRAMO) || defined (HAVE_X12A)
@@ -181,6 +195,10 @@ void add_dummies (gpointer data, guint action, GtkWidget *widget);
 
 void add_index (gpointer data, guint tm, GtkWidget *widget);
 
+void do_add_obs (gpointer data, guint u, GtkWidget *widget);
+
+void do_remove_obs (gpointer data, guint u, GtkWidget *widget);
+
 void add_logs_etc (gpointer data, guint action, GtkWidget *widget);
 
 int add_fit_resid (MODEL *pmod, int code, int undo);
@@ -213,7 +231,7 @@ void do_splot_from_selector (GtkWidget *widget, gpointer p);
 
 void plot_from_selection (gpointer data, guint action, GtkWidget *widget);
 
-void do_box_graph (GtkWidget *widget, dialog_t *ddata);
+void do_box_graph (GtkWidget *widget, dialog_t *dlg);
 
 void do_dummy_graph (GtkWidget *widget, gpointer p);
 
@@ -235,9 +253,9 @@ void do_open_csv_box (char *fname, int code, int append);
 
 int do_store (char *mydatfile, gretlopt oflag, int overwrite);
 
-void view_latex (gpointer data, guint prn_code, GtkWidget *widget);
+void view_latex (PRN *prn);
 
-void do_save_tex (char *fname, int code, MODEL *pmod);
+void save_latex (PRN *prn, const char *fname);
 
 void do_save_text (char *fname, MODEL *pmod);
 
@@ -249,7 +267,7 @@ int gui_exec_line (char *line,
 		   PRN *prn, int exec_code, 
 		   const char *myname); 
 
-int verify_and_record_command (char *line);
+int check_and_record_command (void);
 
 /* other */
 
@@ -257,5 +275,9 @@ int latex_compile (char *texshort);
 
 void add_mahalanobis_data (windata_t *vwin);
 
+void add_pca_data (windata_t *vwin);
+
+void add_fcast_data (windata_t *vwin);
+
 #endif /* LIBRARY_H */
 
