@@ -41,6 +41,44 @@ void add_remove_markers_state (gboolean s)
     flip(mdata->ifac, "/Sample/Remove case markers", s);
 }
 
+/* ......................................................... */
+
+/* by using gretl_set_window_modal() we make the main
+   window visibly insensitive */
+
+static int modcount;
+
+static void increment_modal_count (GtkWidget *w)
+{
+    if (modcount == 0) {
+	gtk_widget_set_sensitive(mdata->w, FALSE);
+    }
+
+    modcount++;
+}
+
+static void decrement_modal_count (GtkWidget *w, gpointer p)
+{
+    if (modcount > 0) {
+	modcount--;
+    }
+
+    if (modcount == 0) {
+	gtk_widget_set_sensitive(mdata->w, TRUE);
+    }
+}
+
+void gretl_set_window_modal (GtkWidget *w)
+{
+    gtk_window_set_modal(GTK_WINDOW(w), TRUE);
+    increment_modal_count(w);
+    g_signal_connect(G_OBJECT(w), "destroy", 
+		     G_CALLBACK(decrement_modal_count),
+		     NULL);
+}
+
+/* ......................................................... */
+
 void main_menubar_state (gboolean s)
 {
     if (mdata->ifac == NULL) return;
@@ -102,46 +140,57 @@ static GtkItemFactoryEntry time_series_model_items[] = {
                                 (d->pd == 4 || d->pd == 12 || \
                                  d->pd == 5 || d->pd == 7))
 
+#define DATASET_DB_OK(d) (d->pd == 1 || d->pd == 4 || d->pd == 12)
+
 void time_series_menu_state (gboolean s)
 {
-    if (mdata->ifac != NULL) {
-	/* Data menu */
-	flip(mdata->ifac, "/Data/Graph specified vars/Time series plot...", s);
-	/* Variable menu */
-	flip(mdata->ifac, "/Variable/Time series plot", s);
-	flip(mdata->ifac, "/Variable/Correlogram", s);
-	flip(mdata->ifac, "/Variable/Spectrum", s);
-	flip(mdata->ifac, "/Variable/Runs test", s);
-	flip(mdata->ifac, "/Variable/Augmented Dickey-Fuller test", s);
-	flip(mdata->ifac, "/Variable/KPSS test", s);
-	flip(mdata->ifac, "/Variable/ARMA model", s);
+    gboolean dbsave;
+
+    if (mdata->ifac == NULL) {
+	return;
+    }
+
+    /* File menu */
+    dbsave = s && (DATASET_DB_OK(datainfo));
+    flip(mdata->ifac, "/File/Save data as/database...", dbsave);
+
+    /* Data menu */
+    flip(mdata->ifac, "/Data/Graph specified vars/Time series plot...", s);
+
+    /* Variable menu */
+    flip(mdata->ifac, "/Variable/Time series plot", s);
+    flip(mdata->ifac, "/Variable/Correlogram", s);
+    flip(mdata->ifac, "/Variable/Spectrum", s);
+    flip(mdata->ifac, "/Variable/Runs test", s);
+    flip(mdata->ifac, "/Variable/Augmented Dickey-Fuller test", s);
+    flip(mdata->ifac, "/Variable/KPSS test", s);
+    flip(mdata->ifac, "/Variable/ARMA model", s);
 #ifdef HAVE_X12A
-	flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", s);
+    flip(mdata->ifac, "/Variable/X-12-ARIMA analysis", s);
 #endif
 #ifdef HAVE_TRAMO
-	flip(mdata->ifac, "/Variable/TRAMO analysis", s);
+    flip(mdata->ifac, "/Variable/TRAMO analysis", s);
 #endif
-	flip(mdata->ifac, "/Variable/Hurst exponent", s);
-	/* Model menu */
-	flip(mdata->ifac, "/Model/Time series", s);
-	/* Sample menu */
-	flip(mdata->ifac, "/Sample/Compact data...", 
-	     s && DATASET_COMPACTABLE(datainfo));
+    flip(mdata->ifac, "/Variable/Hurst exponent", s);
+    /* Model menu */
+    flip(mdata->ifac, "/Model/Time series", s);
+    /* Sample menu */
+    flip(mdata->ifac, "/Sample/Compact data...", 
+	 s && DATASET_COMPACTABLE(datainfo));
 
-	if (s) {
-	    GtkWidget *w =  
-		gtk_item_factory_get_widget(mdata->ifac, 
-					    "/Model/Time series/Cochrane-Orcutt...");
+    if (s) {
+	GtkWidget *w =  
+	    gtk_item_factory_get_widget(mdata->ifac, 
+					"/Model/Time series/Cochrane-Orcutt...");
 
-	    if (w == NULL) {
-		int i, n = sizeof time_series_model_items / 
-		    sizeof time_series_model_items[0];
+	if (w == NULL) {
+	    int i, n = sizeof time_series_model_items / 
+		sizeof time_series_model_items[0];
 
-		for (i=0; i<n; i++) {
-		    gtk_item_factory_create_item(mdata->ifac, 
-						 &time_series_model_items[i], 
-						 mdata, 1);
-		}
+	    for (i=0; i<n; i++) {
+		gtk_item_factory_create_item(mdata->ifac, 
+					     &time_series_model_items[i], 
+					     mdata, 1);
 	    }
 	}
     }
@@ -183,6 +232,13 @@ void restore_sample_state (gboolean s)
     }
 }
 
+void drop_obs_state (gboolean s)
+{
+    if (mdata->ifac != NULL) {
+	flip(mdata->ifac, "/Data/Remove extra observations", s);
+    }
+}
+
 void compact_data_state (gboolean s)
 {
     if (mdata->ifac != NULL) {
@@ -206,8 +262,8 @@ static gint var_popup_click (GtkWidget *widget, gpointer data)
 	do_freqplot(NULL, 0, NULL);
     else if (!strcmp(item, _("Boxplot")))
 	do_boxplot_var(mdata->active_var);
-    else if (!strcmp(item, _("Correlogram"))) 
-	gretl_callback(NULL, CORRGM, NULL);
+    else if (!strcmp(item, _("Correlogram")))
+	do_corrgm(NULL, CORRGM, NULL);
     else if (!strcmp(item, _("Spectrum"))) 
 	do_pergm(NULL, 0, NULL);
     else if (!strcmp(item, _("Spectrum (Bartlett)"))) 
@@ -388,6 +444,8 @@ void set_sample_label (DATAINFO *pdinfo)
 	case 6:
 	case 7:
 	    strcpy(pdstr, _("Daily")); break;
+	case 10:
+	    strcpy(pdstr, _("Decennial")); break;
 	default:
 	    strcpy(pdstr, _("Unknown")); break;
 	}
