@@ -3,6 +3,8 @@
 
 void refresh_data (void);
 
+void gretl_set_window_modal (GtkWidget *w);
+
 void edit_info_state (gboolean s);
 void add_remove_markers_state (gboolean s);
 void main_menubar_state (gboolean s);
@@ -12,6 +14,7 @@ void ts_or_panel_menu_state (gboolean s);
 void session_menu_state (gboolean s);
 void restore_sample_state (gboolean s);
 void compact_data_state (gboolean s);
+void drop_obs_state (gboolean s);
 
 GtkWidget *build_var_popup (void);
 GtkWidget *build_selection_popup (void);
