@@ -347,14 +347,14 @@ static void center_in_field (const char *s, int width, PRN *prn)
     }
 }
 
-static const char *short_estimator_string (int ci, int format)
+static const char *short_estimator_string (int ci, PRN *prn)
 {
     if (ci == HSK) return N_("HSK");
     else if (ci == CORC) return N_("CORC");
     else if (ci == HILU) return N_("HILU");
     else if (ci == ARCH) return N_("ARCH");
     else if (ci == POOLED) return N_("OLS");
-    else return estimator_string(ci, format);
+    else return estimator_string(ci, prn);
 }
 
 static const char *get_asts (double pval)
@@ -377,8 +377,8 @@ static void print_model_table_coeffs (PRN *prn)
     int i, j, k;
     const MODEL *pmod;
     char tmp[16];
-    int tex = (prn->format == GRETL_PRINT_FORMAT_TEX);
-    int rtf = (prn->format == GRETL_PRINT_FORMAT_RTF);
+    int tex = tex_format(prn);
+    int rtf = rtf_format(prn);
 
     /* loop across all variables that appear in any model */
     for (i=2; i<=grand_list[0]; i++) {
@@ -388,8 +388,7 @@ static void print_model_table_coeffs (PRN *prn)
 	if (tex) {
 	    tex_escape(tmp, datainfo->varname[v]);
 	    pprintf(prn, "%s ", tmp);
-	}
-	else if (rtf) {
+	} else if (rtf) {
 	    print_rtf_row_spec(prn, 0);
 	    pprintf(prn, "\\intbl \\qc %s\\cell ", datainfo->varname[v]);
 	} else {
@@ -407,24 +406,25 @@ static void print_model_table_coeffs (PRN *prn)
 		char numstr[32];
 
 		if (floateq(s, 0.0)) {
-		    if (floateq(x, 0.0)) pval = 1.0;
-		    else pval = 0.0001;
+		    if (floateq(x, 0.0)) {
+			pval = 1.0;
+		    } else {
+			pval = 0.0001;
+		    }
 		} else {
 		    pval = coeff_pval(pmod, x / s, pmod->dfd);
 		}
 
-		if (!tex) {
-		    sprintf(numstr, "%#.4g", x);
-		    gretl_fix_exponent(numstr);
-		}
+		sprintf(numstr, "%#.4g", x);
+		gretl_fix_exponent(numstr);
 
 		if (tex) {
 		    if (x < 0) {
-			pprintf(prn, "& %s$-$%#.4g%s ", get_pre_asts(pval),
-				fabs(x), tex_get_asts(pval));
+			pprintf(prn, "& %s$-$%s%s ", get_pre_asts(pval),
+				numstr + 1, tex_get_asts(pval));
 		    } else {
-			pprintf(prn, "& %s%#.4g%s ", get_pre_asts(pval), 
-				x, tex_get_asts(pval));
+			pprintf(prn, "& %s%s%s ", get_pre_asts(pval), 
+				numstr, tex_get_asts(pval));
 		    }
 		} else if (rtf) {
 		    pprintf(prn, "\\qc %s%s\\cell ", numstr, get_asts(pval));
@@ -434,9 +434,13 @@ static void print_model_table_coeffs (PRN *prn)
 		}
 		f = 0;
 	    } else {
-		if (tex) pputs(prn, "& ");
-		else if (rtf) pputs(prn, "\\qc \\cell ");
-		else pputs(prn, "            ");
+		if (tex) {
+		    pputs(prn, "& ");
+		} else if (rtf) {
+		    pputs(prn, "\\qc \\cell ");
+		} else {
+		    pputs(prn, "            ");
+		}
 	    }
 	}
 
@@ -455,8 +459,11 @@ static void print_model_table_coeffs (PRN *prn)
 	f = 1;
 	for (j=0; j<model_list_len; j++) {
 	    pmod = model_list[j];
-	    if (pmod == NULL) continue;
+	    if (pmod == NULL) {
+		continue;
+	    }
 	    if ((k = var_is_in_model(v, pmod))) {
+		char numstr[32];
 		double val;
 
 		if (use_tstats) {
@@ -465,31 +472,43 @@ static void print_model_table_coeffs (PRN *prn)
 		    val = pmod->sderr[k-2];
 		}
 
-		if (tex) {
-		    pprintf(prn, "& \\footnotesize{(%#.4g)} ", val);
-		} else {
-		    char numstr[32];
+		sprintf(numstr, "%#.4g", val);
+		gretl_fix_exponent(numstr);
 
-		    sprintf(numstr, "%#.4g", val);
-		    gretl_fix_exponent(numstr);
-		    if (rtf) {
-			if (f == 1) pputs(prn, "\\qc \\cell ");
-			pprintf(prn, "\\qc (%s)\\cell ", numstr);
-			f = 0;
+		if (tex) {
+		    if (val < 0) {
+			pprintf(prn, "& \\footnotesize{($-$%s)} ", numstr + 1);
 		    } else {
-			sprintf(tmp, "(%s)", numstr);
-			pprintf(prn, "%12s", tmp);
+			pprintf(prn, "& \\footnotesize{(%s)} ", numstr);
 		    }
+		} else if (rtf) {
+		    if (f == 1) {
+			pputs(prn, "\\qc \\cell ");
+		    }
+		    pprintf(prn, "\\qc (%s)\\cell ", numstr);
+		    f = 0;
+		} else {
+		    sprintf(tmp, "(%s)", numstr);
+		    pprintf(prn, "%12s", tmp);
 		}
 	    } else {
-		if (tex) pputs(prn, "& ");
-		else if (rtf) pputs(prn, "\\qc \\cell ");
-		else pputs(prn, "            ");
+		if (tex) {
+		    pputs(prn, "& ");
+		} else if (rtf) {
+		    pputs(prn, "\\qc \\cell ");
+		} else {
+		    pputs(prn, "            ");
+		}
 	    }
 	}
-	if (tex) pputs(prn, "\\\\ [4pt] \n");
-	else if (rtf) pputs(prn, "\\intbl \\row\n");
-	else pputs(prn, "\n\n");
+
+	if (tex) {
+	    pputs(prn, "\\\\ [4pt] \n");
+	} else if (rtf) {
+	    pputs(prn, "\\intbl \\row\n");
+	} else {
+	    pputs(prn, "\n\n");
+	}
     }
 }
 
@@ -522,8 +541,8 @@ static void print_n_r_squared (PRN *prn, int *binary)
     int j;
     int same_df, any_R2, any_ll;
     const MODEL *pmod;
-    int tex = (prn->format == GRETL_PRINT_FORMAT_TEX);
-    int rtf = (prn->format == GRETL_PRINT_FORMAT_RTF);
+    int tex = tex_format(prn);
+    int rtf = rtf_format(prn);
 
     if (rtf) print_rtf_row_spec(prn, 0);
 
@@ -665,7 +684,7 @@ int display_model_table (int gui)
     if (ci > 0) {
 	/* all models use same estimation procedure */
 	pprintf(prn, _("%s estimates"), 
-		_(estimator_string(ci, prn->format)));
+		_(estimator_string(ci, prn)));
 	pputc(prn, '\n');
     }
 
@@ -691,7 +710,7 @@ int display_model_table (int gui)
 	    if (model_list[j] == NULL) continue;
 	    strcpy(est, 
 		   _(short_estimator_string((model_list[j])->ci,
-					    prn->format)));
+					    prn)));
 	    center_in_field(est, 12, prn);
 	}
 	pputc(prn, '\n');
@@ -723,12 +742,11 @@ int display_model_table (int gui)
     return 0;
 }
 
-int tex_print_model_table (int view)
+static int tex_print_model_table (PRN *prn)
 {
     int j, ci;
     int binary = 0;
     char tmp[16];
-    PRN *prn;
 
     if (model_list_empty()) {
 	mtable_errmsg(_("The model table is empty"), 1);
@@ -737,22 +755,16 @@ int tex_print_model_table (int view)
 
     if (make_grand_varlist()) return 1;
 
-    if (bufopen(&prn)) return 1;
-
-    prn->format = GRETL_PRINT_FORMAT_TEX;
+    gretl_print_set_format(prn, GRETL_FORMAT_TEX);
 
     ci = common_estimator();
 
-    if (view) {
-	gretl_tex_preamble(prn, 0);
-    }
-
     pputs(prn, "\\begin{center}\n");
 
     if (ci > 0) {
 	/* all models use same estimation procedure */
 	pprintf(prn, I_("%s estimates"), 
-		I_(estimator_string(ci, prn->format)));
+		I_(estimator_string(ci, prn)));
 	pputs(prn, "\\\\\n");
     }
 
@@ -784,7 +796,7 @@ int tex_print_model_table (int view)
 	    if (model_list[j] == NULL) continue;
 	    strcpy(est, 
 		   I_(short_estimator_string((model_list[j])->ci,
-					     prn->format)));
+					     prn)));
 	    pprintf(prn, " & %s ", est);
 	}
 	pputs(prn, "\\\\ ");
@@ -816,16 +828,6 @@ int tex_print_model_table (int view)
 
     pputs(prn, "\\end{center}\n");
 
-    if (view) {
-	pputs(prn, "\n\\end{document}\n");
-    }
-
-    if (view) {
-	view_latex(prn, LATEX_VIEW_MODELTABLE, NULL);
-    } else {
-	prn_to_clipboard(prn, COPY_LATEX);
-    }
-
     return 0;
 }
 
@@ -842,11 +844,10 @@ static void print_rtf_row_spec (PRN *prn, int tall)
     pputc(prn, '\n');
 }
 
-int rtf_print_model_table (void)
+static int rtf_print_model_table (PRN *prn)
 {
     int j, ci;
     int binary = 0;
-    PRN *prn;
 
     if (model_list_empty()) {
 	mtable_errmsg(_("The model table is empty"), 1);
@@ -855,9 +856,7 @@ int rtf_print_model_table (void)
 
     if (make_grand_varlist()) return 1;
 
-    if (bufopen(&prn)) return 1;
-
-    prn->format = GRETL_PRINT_FORMAT_RTF;
+    gretl_print_set_format(prn, GRETL_FORMAT_RTF);
 
     ci = common_estimator();
 
@@ -867,7 +866,7 @@ int rtf_print_model_table (void)
 	/* all models use same estimation procedure */
 	pputs(prn, "\\par \\qc ");
 	pprintf(prn, I_("%s estimates"), 
-		I_(estimator_string(ci, prn->format)));
+		I_(estimator_string(ci, prn)));
 	pputc(prn, '\n');
     }
 
@@ -897,7 +896,7 @@ int rtf_print_model_table (void)
 	    if (model_list[j] == NULL) continue;
 	    strcpy(est, 
 		   I_(short_estimator_string((model_list[j])->ci,
-					     prn->format)));
+					     prn)));
 	    pprintf(prn, "\\qc %s\\cell ", est);
 	}
 	pputs(prn, "\\intbl \\row\n");
@@ -922,11 +921,20 @@ int rtf_print_model_table (void)
 
     pputs(prn, "\\par\n}\n");
 
-    prn_to_clipboard(prn, COPY_RTF);
-
     return 0;
 }
 
+int special_print_model_table (PRN *prn)
+{
+    if (tex_format(prn)) {
+	return tex_print_model_table(prn);
+    } else if (rtf_format(prn)) {
+	return rtf_print_model_table(prn);
+    } else {
+	return 1;
+    }
+}
+
 int modeltab_parse_line (const char *line, const MODEL *pmod, PRN *prn)
 {
     char cmdword[8];
