@@ -23,6 +23,7 @@
 #include "selector.h"
 #include "treeutils.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 enum {
     SR_VARLIST,
@@ -42,6 +43,7 @@ struct _selector {
     GtkWidget *default_check;
     GtkWidget *extra;
     GtkWidget *extra2;
+    GtkWidget *add_button;
     int code;
     int active_var;
     int error;
@@ -50,8 +52,14 @@ struct _selector {
     gpointer data;
 };
 
-#define WANT_TOGGLES(c) (c == OLS || c == TOBIT || c == ARMA || \
-                         c == GARCH || c == COINT2 || c == TSLS || \
+#define WANT_TOGGLES(c) (c == ARMA || \
+                         c == COINT || \
+                         c == COINT2 || \
+                         c == GARCH || \
+                         c == HILU || \
+                         c == OLS || \
+                         c == TOBIT || \
+                         c == TSLS || \
                          c == VAR)
 
 static int default_var;
@@ -262,6 +270,78 @@ static void set_dependent_var_callback (GtkWidget *w, selector *sr)
 					 sr);
 }
 
+static void set_right_var_from_main (GtkTreeModel *model, GtkTreePath *path,
+				     GtkTreeIter *iter, selector *sr)
+{
+    GtkTreeModel *rightmod;
+    GtkTreeIter r_iter;
+    gchar *vname = NULL;
+    gchar *vnum = NULL;
+    int v;
+
+    gtk_tree_model_get(model, iter, 0, &vnum, 1, &vname, -1);
+
+    rightmod = gtk_tree_view_get_model(GTK_TREE_VIEW(sr->rightvars));
+    if (rightmod == NULL) {
+	g_free(vname);
+	g_free(vnum);
+	return;
+    }
+
+    v = atoi(vnum);
+
+    if (gtk_tree_model_get_iter_first(rightmod, &r_iter)) {
+	while (gtk_tree_model_iter_next(rightmod, &r_iter)) {
+	    ;
+	}
+    }
+
+    gtk_list_store_append(GTK_LIST_STORE(rightmod), &r_iter);
+    gtk_list_store_set(GTK_LIST_STORE(rightmod), &r_iter, 
+		       0, v, 1, vname, -1);
+
+    g_free(vname);
+    g_free(vnum);
+}
+
+static void set_vars_from_main (selector *sr)
+{
+    GtkTreeSelection *selection;
+
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(mdata->listbox));
+    gtk_tree_selection_selected_foreach(selection, 
+					(GtkTreeSelectionForeachFunc) 
+					set_right_var_from_main,
+					sr);
+}
+
+static void set_single_var (selector *sr, int v)
+{
+    GtkTreeModel *mod;
+    GtkTreeIter iter;
+
+    mod = gtk_tree_view_get_model(GTK_TREE_VIEW(sr->rightvars));
+    if (mod == NULL) {
+	return;
+    }
+
+    gtk_tree_model_get_iter_first(mod, &iter);
+    gtk_list_store_append(GTK_LIST_STORE(mod), &iter);
+    gtk_list_store_set(GTK_LIST_STORE(mod), &iter, 
+		       0, v, 1, datainfo->varname[v], -1);
+}
+
+static int selection_at_max (selector *sr, int nsel)
+{
+    int ret = 0;
+
+    if (TWO_VARS_CODE(sr->code) && nsel == 2) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 static void real_add_generic (GtkTreeModel *model, GtkTreeIter *iter, 
 			      selector *sr, int which)
 {
@@ -271,8 +351,9 @@ static void real_add_generic (GtkTreeModel *model, GtkTreeIter *iter,
     gint vnum, test;
     gchar *vname = NULL;
     gint already_there = 0;
+    gint at_max = 0;
 
-    gtk_tree_model_get (model, iter, 0, &vnum, 1, &vname, -1);
+    gtk_tree_model_get(model, iter, 0, &vnum, 1, &vname, -1);
 
     if (which == SR_AUXVARS) {
 	list = sr->auxvars;
@@ -289,21 +370,35 @@ static void real_add_generic (GtkTreeModel *model, GtkTreeIter *iter,
     }
 
     if (gtk_tree_model_get_iter_first(orig_model, &orig_iter)) {
+	int j = 1;
+
 	while (1) {
+	    if (selection_at_max(sr, j)) {
+		at_max = 1;
+		break;
+	    }
 	    gtk_tree_model_get(orig_model, &orig_iter, 0, &test, -1);
 	    if (test == vnum) {
 		already_there = 1; 
 		break;
 	    }
-	    if (!gtk_tree_model_iter_next(orig_model, &orig_iter)) break;
+	    if (!gtk_tree_model_iter_next(orig_model, &orig_iter)) {
+		break;
+	    }
+	    j++;
 	}
     }
 
-    if (!already_there) {
+    if (!already_there && !at_max) {
         gtk_list_store_append(GTK_LIST_STORE(orig_model), &orig_iter);
         gtk_list_store_set(GTK_LIST_STORE(orig_model), &orig_iter, 
 			   0, vnum, 1, vname, -1);
     }
+
+    if (sr->add_button != NULL && at_max) {
+	gtk_widget_set_sensitive(sr->add_button, FALSE);
+    }
+
     g_free(vname);
 }
 
@@ -364,24 +459,44 @@ static void remove_from_right_callback (GtkWidget *w, gpointer data)
     GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
     GtkTreePath *path;
     GtkTreeIter iter, last;
+    selector *sr;
+    int nsel = 0;
 
-    if (model == NULL || selection == NULL) return;
+    if (model == NULL || selection == NULL) {
+	return;
+    }
 
     /* get to the last row */
     if (gtk_tree_model_get_iter_first(model, &iter)) {
 	last = iter;
-	while (gtk_tree_model_iter_next(model, &iter)) last = iter;
-    } else return;
+	nsel = 1;
+	while (gtk_tree_model_iter_next(model, &iter)) {
+	    last = iter;
+	    nsel++;
+	}
+    } else {
+	return;
+    }
     
     /* work back up, deleting selected rows */
     path = gtk_tree_model_get_path (model, &last);
     while (1) {
-	if (gtk_tree_model_get_iter (model, &last, path) &&
-	    gtk_tree_selection_iter_is_selected (selection, &last)) {
-	    gtk_list_store_remove (GTK_LIST_STORE(model), &last);
+	if (gtk_tree_model_get_iter(model, &last, path) &&
+	    gtk_tree_selection_iter_is_selected(selection, &last)) {
+	    gtk_list_store_remove(GTK_LIST_STORE(model), &last);
+	    nsel--;
 	}
-	if (!gtk_tree_path_prev(path)) break;
-    }    
+	if (!gtk_tree_path_prev(path)) {
+	    break;
+	}
+    } 
+
+    sr = g_object_get_data(G_OBJECT(data), "selector");
+    if (sr != NULL && sr->add_button != NULL &&
+	!GTK_WIDGET_SENSITIVE(sr->add_button) &&
+	!selection_at_max(sr, nsel)) {
+	gtk_widget_set_sensitive(sr->add_button, TRUE);
+    }
 }
 
 /* callbacks from button presses in list boxes: double and right
@@ -415,7 +530,7 @@ static gint listvar_special_click (GtkWidget *widget, GdkEventButton *event,
     }
 
     if (mods & GDK_BUTTON3_MASK) {
-	remove_from_right_callback (NULL, data);
+	remove_from_right_callback(NULL, data);
 	return TRUE;
     } 
 
@@ -454,6 +569,9 @@ static void clear_vars (GtkWidget *w, selector *sr)
 	gtk_entry_set_text(GTK_ENTRY(sr->rightvars), "");
     } else {
 	clear_varlist(sr->rightvars);
+	if (sr->add_button != NULL) {
+	    gtk_widget_set_sensitive(sr->add_button, TRUE);
+	}
     }
 
     if (MODEL_CODE(sr->code)) {
@@ -1189,6 +1307,8 @@ static void selector_init (selector *sr, guint code, const char *title)
     sr->auxvars = NULL;
     sr->default_check = NULL;
     sr->extra = NULL;
+    sr->extra2 = NULL;
+    sr->add_button = NULL;
     sr->cmdlist = NULL;
     sr->data = NULL;
     sr->active_var = 0;
@@ -1263,6 +1383,24 @@ static void verbose_callback (GtkWidget *w,  selector *sr)
     }
 }
 
+static void corc_callback (GtkWidget *w,  selector *sr)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	sr->opts &= ~OPT_B;
+    } else {
+	sr->opts |= OPT_B;
+    }
+}
+
+static void engle_granger_callback (GtkWidget *w,  selector *sr)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
+	sr->opts &= ~OPT_N;
+    } else {
+	sr->opts |= OPT_N;
+    }
+}
+
 static void build_pq_spinners (selector *sr)
 {
     GtkWidget *hbox, *tmp;
@@ -1363,6 +1501,34 @@ build_selector_switches (selector *sr)
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
 	gtk_widget_show(tmp);
 
+	gtk_box_pack_start(GTK_BOX(sr->vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+    } else if (sr->code == HILU) {
+	tmp = gtk_check_button_new_with_label
+	    (_("Fine-tune using Cochrane-Orcutt"));
+	g_signal_connect(G_OBJECT(tmp), "toggled",
+			 G_CALLBACK(corc_callback), sr);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+	gtk_widget_show(tmp);
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+
+	gtk_box_pack_start(GTK_BOX(sr->vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+    } else if (sr->code == COINT) {
+	tmp = gtk_check_button_new_with_label
+	    (_("Cointegrating regression includes a constant"));
+	g_signal_connect(G_OBJECT(tmp), "toggled",
+			 G_CALLBACK(engle_granger_callback), sr);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), tmp, TRUE, TRUE, 0);
+	gtk_widget_show(tmp);
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), TRUE);
+
 	gtk_box_pack_start(GTK_BOX(sr->vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
     }
@@ -1463,9 +1629,15 @@ void selection_dialog (const char *title, void (*okfunc)(), guint cmdcode)
     gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);
     
     for (i=0; i<datainfo->v; i++) {
-	if (i == 0 && !MODEL_CODE(cmdcode)) continue;
-        if (hidden_var(i, datainfo)) continue;
-	if (screen_scalar(i, cmdcode)) continue;
+	if (i == 0 && !MODEL_CODE(cmdcode)) {
+	    continue;
+	}
+        if (is_hidden_variable(i, datainfo)) {
+	    continue;
+	}
+	if (screen_scalar(i, cmdcode)) {
+	    continue;
+	}
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter, 0, i, 1, datainfo->varname[i], -1);
     }
@@ -1615,6 +1787,11 @@ static char *get_topstr (int cmdnum)
 	return N_("Select variables to omit");
     case COEFFSUM:
 	return N_("Select coefficients to sum");
+    case SPEARMAN:
+    case MEANTEST:
+    case MEANTEST2:
+    case VARTEST:
+	return N_("Select two variables");
     case PRINT:
 	return N_("Select variables to display");
     case GR_PLOT: 
@@ -1649,7 +1826,12 @@ static void add_omit_list (gpointer p, selector *sr)
     
     if (sr->code == OMIT || sr->code == COEFFSUM) {
 	for (i=2; i<=pmod->list[0]; i++) {
-	    if (pmod->list[i] == 0) continue;
+	    if (pmod->list[i] == 0) {
+		continue;
+	    }
+	    if (pmod->list[i] == LISTSEP) {
+		break;
+	    }
 	    gtk_list_store_append(store, &iter);
 	    gtk_list_store_set(store, &iter, 
 			       0, pmod->list[i], 
@@ -1698,16 +1880,19 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     GtkListStore *store;
     GtkTreeIter iter;
     selector *sr;
-    int i;
+    int i, vnum = 0;
 
     open_dialog = get_open_dialog();
+
     if (open_dialog != NULL) {
 	gdk_window_raise(open_dialog->window);
 	return;
     }
 
     sr = mymalloc(sizeof *sr);
-    if (sr == NULL) return;
+    if (sr == NULL) {
+	return;
+    }
 
     selector_init(sr, cmdcode, title);
 
@@ -1752,13 +1937,21 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     if (cmdcode == OMIT || cmdcode == ADD || cmdcode == COEFFSUM) {
         add_omit_list(p, sr);
     } else {
+	int nleft = 0;
+
 	for (i=1; i<datainfo->v; i++) {
-	    if (hidden_var(i, datainfo)) continue;
-	    if (screen_scalar(i, cmdcode)) continue;
+	    if (is_hidden_variable(i, datainfo) || screen_scalar(i, cmdcode)) {
+		continue;
+	    }
 	    gtk_list_store_append(store, &iter);
 	    gtk_list_store_set(store, &iter, 0, i, 
 			       1, datainfo->varname[i], 
 			       -1);
+	    vnum = i;
+	    nleft++;
+	}
+	if (nleft != 1) {
+	    vnum = 0;
 	}
     }
 
@@ -1768,13 +1961,13 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     /* middle: vertical holder for push/pull buttons */
     mid_vbox = gtk_vbox_new(FALSE, 5);
 
-    tmp = gtk_button_new_with_label (_("Select ->"));
-    gtk_box_pack_start(GTK_BOX(mid_vbox), tmp, TRUE, FALSE, 0);
-    g_signal_connect (G_OBJECT(tmp), "clicked", 
+    sr->add_button = gtk_button_new_with_label (_("Select ->"));
+    gtk_box_pack_start(GTK_BOX(mid_vbox), sr->add_button, TRUE, FALSE, 0);
+    g_signal_connect (G_OBJECT(sr->add_button), "clicked", 
 		      G_CALLBACK(add_to_right_callback), sr);
-    gtk_widget_show(tmp);
+    gtk_widget_show(sr->add_button);
 
-    if (p == NULL) {
+    if (p == NULL && !TWO_VARS_CODE(sr->code)) {
 	/* data save action */
 	tmp = gtk_button_new_with_label (_("All ->"));
 	gtk_box_pack_start(GTK_BOX(mid_vbox), tmp, TRUE, FALSE, 0);
@@ -1794,6 +1987,7 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     right_vbox = gtk_vbox_new(FALSE, 5);
 
     sr->rightvars = var_list_box_new(GTK_BOX(right_vbox), sr, SR_RIGHTVARS);
+    g_object_set_data(G_OBJECT(sr->rightvars), "selector", sr);
 
     gtk_box_pack_start(GTK_BOX(big_hbox), right_vbox, TRUE, TRUE, 0);
     gtk_widget_show(right_vbox);
@@ -1810,10 +2004,16 @@ void simple_selection (const char *title, void (*okfunc)(), guint cmdcode,
     /* buttons: "OK", Clear, Cancel, Help */
     build_selector_buttons(sr, okfunc);
 
+    if (TWO_VARS_CODE(sr->code) && mdata_selection_count() == 2) {
+	set_vars_from_main(sr);
+    } else if (SAVE_DATA_ACTION(sr->code) && vnum > 0) {
+	set_single_var(sr, vnum);
+    }
+
     gtk_widget_show(sr->dlg);
-    
+
     if (SAVE_DATA_ACTION(sr->code)) {
-	gtk_window_set_modal(GTK_WINDOW(sr->dlg), TRUE);
+	gretl_set_window_modal(sr->dlg);
     }
 }
 
@@ -1842,7 +2042,7 @@ static void selection_add_item (GtkTreeModel *model, GtkTreePath *path,
     lmkr->n_items += 1;
 }
 
-char *mdata_selection_to_string (int n_required) 
+char *main_window_selection_as_string (void) 
 {
     GtkTreeSelection *select;
     struct list_maker lmkr;
@@ -1869,17 +2069,6 @@ char *mdata_selection_to_string (int n_required)
 	return lmkr.liststr;
     }
 
-    if (n_required && lmkr.n_items != n_required) {
-	gchar *msg;
-
-	msg = g_strdup_printf(_("Please select %d variables first"),
-			      n_required);
-	errbox(msg);
-	g_free(msg);
-	free(lmkr.liststr);
-	lmkr.liststr = NULL;
-    }
-
     return lmkr.liststr;
 }
 
@@ -1941,7 +2130,13 @@ int selector_code (const selector *sr)
 
 const char *selector_list (const selector *sr)
 {
-    return sr->cmdlist;
+    const char *ret = NULL;
+
+    if (sr->cmdlist != NULL && *sr->cmdlist != '\0') {
+	ret = sr->cmdlist;
+    }
+
+    return ret;
 }
 
 int selector_list_hasconst (const selector *sr)
