@@ -421,7 +421,7 @@ void add_graph_to_session (gpointer data, guint code, GtkWidget *w)
     
     errno = 0;
 
-    get_default_dir(savedir);
+    get_default_dir(savedir, SAVE_THIS_GRAPH);
 
     if (code == GRETL_GNUPLOT_GRAPH) {
 	GPT_SPEC *plot = (GPT_SPEC *) data;
@@ -541,14 +541,15 @@ static int real_add_var_to_session (GRETL_VAR *var)
 
 int real_add_text_to_session (PRN *prn, const char *tname)
 {
+    const char *pbuf;
     int nt = look_up_text_by_name(tname);
     int replace = 0;
 
     if (nt >= 0) {
+	pbuf = gretl_print_get_buffer(prn);
+	free(session.texts[nt]->buf);
+	session.texts[nt]->buf = g_strdup(pbuf);
 	replace = 1;
-	free((session.texts[nt])->buf);
-	(session.texts[nt])->buf = prn->buf;  
-	prn->buf = NULL;
     } else {
 	GRETL_TEXT **texts;
 
@@ -566,8 +567,8 @@ int real_add_text_to_session (PRN *prn, const char *tname)
 	    return ADD_OBJECT_FAIL;
 	}
 
-	(session.texts[nt])->buf = prn->buf;
-	prn->buf = NULL;
+	pbuf = gretl_print_get_buffer(prn);
+	session.texts[nt]->buf = g_strdup(pbuf);
 
 	strcpy((session.texts[nt])->name, tname);
 
@@ -672,11 +673,13 @@ int try_add_var_to_session (GRETL_VAR *var)
 
 void remember_model (gpointer data, guint close, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data;
-    MODEL *pmod = (MODEL *) mydata->data;
+    windata_t *vwin = (windata_t *) data;
+    MODEL *pmod = (MODEL *) vwin->data;
     gchar *buf;
 
-    if (pmod == NULL) return;
+    if (pmod == NULL) {
+	return;
+    }
 
     if (model_already_saved(pmod)) {
 	infobox(_("Model is already saved"));
@@ -695,9 +698,9 @@ void remember_model (gpointer data, guint close, GtkWidget *widget)
 
     session_changed(1);
 
-    /* close model window */
-    if (close) {
-	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(mydata->w)));
+    /* close model window? */
+    if (close && !window_is_busy(vwin)) {
+	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(vwin->w)));
     } 
 }
 
@@ -705,8 +708,8 @@ void remember_model (gpointer data, guint close, GtkWidget *widget)
 
 void remember_var (gpointer data, guint close, GtkWidget *widget)
 {
-    windata_t *mydata = (windata_t *) data;
-    GRETL_VAR *var = (GRETL_VAR *) mydata->data;
+    windata_t *vwin = (windata_t *) data;
+    GRETL_VAR *var = (GRETL_VAR *) vwin->data;
     gchar *buf;
 
     if (var == NULL) return;
@@ -730,7 +733,7 @@ void remember_var (gpointer data, guint close, GtkWidget *widget)
 
     /* close VAR window */
     if (close) {
-	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(mydata->w)));
+	gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(vwin->w)));
     } 
 }
 
@@ -829,7 +832,7 @@ void do_open_session (GtkWidget *w, gpointer data)
 	    fwin = (windata_t *) data;
 	} else {
 	    d = (dialog_t *) data;
-	    fwin = (windata_t *) dialog_data_get_data(d);
+	    fwin = (windata_t *) edit_dialog_get_data(d);
 	}
     }
 
@@ -899,6 +902,10 @@ void do_open_session (GtkWidget *w, gpointer data)
 
 void verify_clear_data (void)
 {
+    if (dataset_locked()) {
+	return;
+    }
+
     if (!expert) {
         if (yes_no_dialog ("gretl",                      
 			   _("Clearing the data set will end\n"
@@ -907,6 +914,7 @@ void verify_clear_data (void)
             return;
 	}
     }
+
     close_session();
 }
 
@@ -1321,7 +1329,7 @@ int parse_savefile (const char *fname)
 
 int recreate_session (const char *fname)
 {
-    PRN *prn = gretl_print_new(GRETL_PRINT_NULL, NULL);
+    PRN *prn = gretl_print_new(GRETL_PRINT_NULL);
 
 #ifdef SESSION_DEBUG
     fprintf(stderr, "recreate_session: fname = %s\n", fname);
@@ -1497,13 +1505,11 @@ static void open_boxplot (gui_obj *gobj)
 static void open_gui_text (gui_obj *gobj)
 { 
     GRETL_TEXT *text = (GRETL_TEXT *) gobj->data;
-    size_t sz = strlen(text->buf);
     PRN *prn;
 
-    prn = bufopen_with_size(sz + 1);
+    prn = gretl_print_new_with_buffer(g_strdup(text->buf));
 
     if (prn != NULL) { 
-	strcpy(prn->buf, text->buf);
 	view_buffer(prn, 80, 400, gobj->name, INFO, NULL);
     }
 }
@@ -1881,15 +1887,12 @@ void display_text_by_name (const char *tname)
 {
     GRETL_TEXT *text;
     PRN *prn;
-    size_t sz;
 
     text = get_text_by_name(tname);
     if (text == NULL) return;
 
-    sz = strlen(text->buf);
-    prn = bufopen_with_size(sz + 1);
+    prn = gretl_print_new_with_buffer(g_strdup(text->buf));
     if (prn != NULL) { 
-	strcpy(prn->buf, text->buf);
 	view_buffer(prn, 80, 400, tname, INFO, NULL);
     }
 }
@@ -2249,7 +2252,7 @@ static gboolean session_icon_click (GtkWidget *widget,
 	case 'x':
 	    open_gui_text(gobj); break;
 	case 'd':
-	    show_spreadsheet(NULL); break;
+	    show_spreadsheet(SHEET_EDIT_DATASET); break;
 	case 'i':
 	    open_info(NULL, 0, NULL); break;
 	case 'p':
@@ -2323,7 +2326,7 @@ static void data_popup_activated (GtkWidget *widget, gpointer data)
     gchar *item = (gchar *) data;
 
     if (strcmp(item, _("Edit")) == 0) 
-	show_spreadsheet(NULL);
+	show_spreadsheet(SHEET_EDIT_DATASET);
     else if (strcmp(item, _("Save...")) == 0) 
 	file_save(mdata, SAVE_DATA, NULL);
     else if (strcmp(item, _("Export as CSV...")) == 0) 
@@ -3218,15 +3221,15 @@ static void auto_save_gp (gpointer data, guint quiet, GtkWidget *w)
     FILE *fp;
     char msg[MAXLEN];
     gchar *savestuff;
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
 
-    if ((fp = gretl_fopen(mydata->fname, "w")) == NULL) {
-	sprintf(msg, _("Couldn't write to %s"), mydata->fname);
+    if ((fp = gretl_fopen(vwin->fname, "w")) == NULL) {
+	sprintf(msg, _("Couldn't write to %s"), vwin->fname);
 	errbox(msg); 
 	return;
     }
 
-    savestuff = gtk_editable_get_chars(GTK_EDITABLE(mydata->w), 0, -1);
+    savestuff = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0, -1);
     fprintf(fp, "%s", savestuff);
     g_free(savestuff); 
     fclose(fp);
@@ -3242,17 +3245,17 @@ static void auto_save_gp (gpointer data, guint quiet, GtkWidget *w)
 {
     FILE *fp;
     gchar *msg, *savestuff;
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
 # ifdef ENABLE_NLS
     gchar *trbuf;
 # endif
 
-    savestuff = textview_get_text(GTK_TEXT_VIEW(mydata->w));
+    savestuff = textview_get_text(GTK_TEXT_VIEW(vwin->w));
 
     if (savestuff == NULL) return;
 
-    if ((fp = gretl_fopen(mydata->fname, "w")) == NULL) {
-	msg = g_strdup_printf(_("Couldn't write to %s"), mydata->fname);
+    if ((fp = gretl_fopen(vwin->fname, "w")) == NULL) {
+	msg = g_strdup_printf(_("Couldn't write to %s"), vwin->fname);
 	errbox(msg); 
 	g_free(msg);
 	g_free(savestuff);
@@ -3324,11 +3327,11 @@ static char *add_pause_to_plotfile (const char *fname)
 void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 {
     gchar *buf = NULL;
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
 
     auto_save_gp(data, 1, NULL);
 
-    buf = g_strdup_printf("gnuplot -persist \"%s\"", mydata->fname);
+    buf = g_strdup_printf("gnuplot -persist \"%s\"", vwin->fname);
 
     if (system(buf)) {
         errbox(_("gnuplot command failed"));
@@ -3342,7 +3345,7 @@ void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 {
     gchar *buf = NULL;
-    windata_t *mydata = (windata_t *) data;
+    windata_t *vwin = (windata_t *) data;
     int err = 0;
 # ifdef G_OS_WIN32
     gchar *tmpfile;
@@ -3351,7 +3354,7 @@ void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
     auto_save_gp(data, 1, NULL);
 
 # ifdef G_OS_WIN32
-    tmpfile = add_pause_to_plotfile(mydata->fname);
+    tmpfile = add_pause_to_plotfile(vwin->fname);
     if (tmpfile != NULL) {
 	buf = g_strdup_printf("\"%s\" \"%s\"", paths.gnuplot, tmpfile);
 	err = (WinExec(buf, SW_SHOWNORMAL) < 32);
@@ -3361,7 +3364,7 @@ void gp_to_gnuplot (gpointer data, guint i, GtkWidget *w)
 	err = 1;
     }
 # else
-    buf = g_strdup_printf("gnuplot -persist \"%s\"", mydata->fname);
+    buf = g_strdup_printf("gnuplot -persist \"%s\"", vwin->fname);
     err = gretl_spawn(buf);
 # endif
 
