@@ -184,6 +184,8 @@ RCVAR rc_vars[] = {
       ROOTSET, MAXLEN, 3, NULL },
     { "Rcommand", N_("Command to launch GNU R"), NULL, Rcommand, 
       ROOTSET, MAXSTR, 3, NULL },
+    { "latex", N_("Command to compile TeX files"), NULL, latex, 
+      ROOTSET, MAXSTR, 3, NULL },
     { "viewdvi", N_("Command to view DVI files"), NULL, viewdvi, 
       ROOTSET, MAXSTR, 3, NULL },
 #ifndef G_OS_WIN32
@@ -365,11 +367,11 @@ static void slash_terminate (char *path)
     }
 }
 
-void get_default_dir (char *s)
+void get_default_dir (char *s, int action)
 {
     *s = '\0';
 
-    if (usecwd) {
+    if (usecwd && action != SAVE_DBDATA) {
 	char *test = getcwd(s, MAXLEN);
 
 	if (test == NULL) {
@@ -1168,8 +1170,8 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     show_or_hide_toolbar(want_toolbar);
 
+    /* register these for session using libset apparatus */
     set_use_qr(useqr);
-
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
     set_garch_robust_vcv(hc_garch);
@@ -1816,17 +1818,17 @@ static void font_selection_ok (GtkWidget *w, GtkFontSelectionDialog *fs)
 {
     gchar *fstring = gtk_font_selection_dialog_get_font_name(fs);
 
-    if (strlen(fstring)) {
+    if (fstring != NULL && *fstring != '\0') {
+	fprintf(stderr, "fstring = '%s'\n", fstring);
         strcpy(fixedfontname, fstring);
         gdk_font_unref(fixed_font);
         fixed_font = gdk_font_load(fixedfontname);
         write_rc();
     }
-    g_free(fstring);
-    gtk_widget_destroy(GTK_WIDGET (fs));
-}
 
-/* .................................................................. */
+    g_free(fstring);
+    gtk_widget_destroy(GTK_WIDGET(fs));
+}
 
 void font_selector (gpointer data, guint u, GtkWidget *w)
 {
@@ -1837,15 +1839,15 @@ void font_selector (gpointer data, guint u, GtkWidget *w)
 	fontsel = gtk_font_selection_dialog_new 
 	    (_("Font for gretl output windows"));
 
-	gtk_window_set_position (GTK_WINDOW (fontsel), GTK_WIN_POS_MOUSE);
+	gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
 
-	gtk_font_selection_dialog_set_filter(GTK_FONT_SELECTION_DIALOG (fontsel),
+	gtk_font_selection_dialog_set_filter(GTK_FONT_SELECTION_DIALOG(fontsel),
 					     GTK_FONT_FILTER_BASE, GTK_FONT_ALL,
 					     NULL, NULL, NULL, NULL, 
 					     spacings, NULL);
 
 	gtk_font_selection_dialog_set_font_name 
-	    (GTK_FONT_SELECTION_DIALOG (fontsel), fixedfontname);
+	    (GTK_FONT_SELECTION_DIALOG(fontsel), fixedfontname);
 
 	gtk_signal_connect (GTK_OBJECT(fontsel), "destroy",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroyed),
@@ -1862,7 +1864,7 @@ void font_selector (gpointer data, guint u, GtkWidget *w)
 				    (fontsel)->cancel_button),
 				   "clicked", 
 				   GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				   GTK_OBJECT (fontsel));
+				   GTK_OBJECT(fontsel));
     }
 
     if (!GTK_WIDGET_VISIBLE(fontsel)) {
@@ -1878,9 +1880,15 @@ void font_selector (gpointer data, guint u, GtkWidget *w)
 
 static const char *font_weight_string (int weight)
 {
-    if (weight >= FW_THIN && weight <= FW_LIGHT) return " Light";
-    if (weight >= FW_NORMAL && weight <= FW_DEMIBOLD) return "";
-    if (weight >= FW_BOLD) return " Bold";
+    if (weight >= FW_THIN && weight <= FW_LIGHT) {
+	return " Light";
+    }
+    if (weight >= FW_NORMAL && weight <= FW_DEMIBOLD) {
+	return "";
+    }
+    if (weight >= FW_BOLD) {
+	return " Bold";
+    }
     return "";
 }
 
@@ -2265,11 +2273,11 @@ static int validate_dir (const char *dirname)
     return err;
 }
 
-static void real_set_userdir (GtkWidget *widget, dialog_t *ddata)
+static void real_set_userdir (GtkWidget *widget, dialog_t *dlg)
 {
     const gchar *dirname;
 
-    dirname = dialog_data_get_text(ddata);
+    dirname = edit_dialog_get_text(dlg);
 
     if (validate_dir(dirname)) {
 	return;
@@ -2278,7 +2286,7 @@ static void real_set_userdir (GtkWidget *widget, dialog_t *ddata)
 #if defined(HAVE_TRAMO) || defined(HAVE_X12A)
 	set_tramo_x12a_dirs();
 #endif
-	close_dialog(ddata);
+	close_dialog(dlg);
     }
 }
 
@@ -2301,7 +2309,7 @@ void first_time_set_user_dir (void)
 		   "Please enter a directory for gretl user files."),
                  paths.userdir, 
                  real_set_userdir, NULL, 
-                 CREATE_USERDIR, 0);
+                 CREATE_USERDIR, 0, 0);
 }
 
 #endif /* G_OS_WIN32 */
