@@ -21,6 +21,7 @@
 #include "treeutils.h"
 #include "ssheet.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 #include <errno.h>
 #include <ctype.h>
@@ -31,6 +32,12 @@
 #endif
 
 #undef SSDEBUG
+#undef CELLDEBUG
+
+enum {
+    SHEET_AT_END,
+    SHEET_AT_POINT
+};
 
 typedef struct {
     GtkWidget *view;
@@ -40,26 +47,25 @@ typedef struct {
     GtkCellRenderer *dumbcell;
     GtkCellRenderer *datacell;
     gchar location[20];
-    gint datacols, datarows;
-    gint totcols;
-    gint n_scalars;
+    int datacols, datarows;
+    int totcols;
+    int orig_vars;
+    int added_vars;
+    int orig_main_v;
+    int modified;
     guint cid;
     guint point;
-} spreadsheet;
-
-enum {
-    SHEET_AT_END,
-    SHEET_AT_POINT
-};
+} Spreadsheet;
 
 static void sheet_add_var_callback (gpointer data, guint code, GtkWidget *w);
 static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w);
-static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet);
+static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet);
 static void set_up_sheet_column (GtkTreeViewColumn *column, gint width, 
 				 gboolean expand);
 static gint get_data_col_width (void);
-static int add_data_column (spreadsheet *sheet);
-static void create_sheet_cell_renderers (spreadsheet *sheet);
+static int add_data_column (Spreadsheet *sheet);
+static void create_sheet_cell_renderers (Spreadsheet *sheet);
+static void set_dataset_locked (gboolean s);
 
 static GtkItemFactoryEntry sheet_items[] = {
     { N_("/_Observation"), NULL, NULL, 0, "<Branch>", GNULL },
@@ -67,10 +73,29 @@ static GtkItemFactoryEntry sheet_items[] = {
       NULL, GNULL },
     { N_("/Observation/_Insert obs"), NULL, sheet_add_obs_callback, SHEET_AT_POINT, 
       NULL, GNULL },
-    { N_("/Add _Variable"), NULL, sheet_add_var_callback, 0, NULL, GNULL }
+    { N_("/_Variable"), NULL, NULL, 0, "<Branch>", GNULL },
+    { N_("/Variable/_Add"), NULL, sheet_add_var_callback, 0, NULL, GNULL }
 };
 
-static int sheet_modified;
+static void disable_obs_menu (GtkItemFactory *ifac)
+{
+    GtkWidget *w = gtk_item_factory_get_item(ifac, "/Observation");
+
+    gtk_widget_set_sensitive(w, FALSE);
+}
+
+static int spreadsheet_hide (int i, const DATAINFO *pdinfo)
+{
+    int ret = 0;
+
+    if (pdinfo->vector[i] == 0) {
+	ret = 1;
+    } else if (is_hidden_variable(i, pdinfo)) {
+	ret = 1;
+    }
+
+    return ret;
+}
 
 /* .................................................................. */
 
@@ -95,7 +120,7 @@ static char *single_underscores (char *targ, const char *src)
 
 /* .................................................................. */
 
-static void set_locator_label (spreadsheet *sheet, GtkTreePath *path,
+static void set_locator_label (Spreadsheet *sheet, GtkTreePath *path,
 			       GtkTreeViewColumn *column)
 {
     GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(sheet->view));
@@ -119,31 +144,7 @@ static void set_locator_label (spreadsheet *sheet, GtkTreePath *path,
 
 #ifndef OLD_SELECTION
 
-static GtkTreeViewColumn *get_column_by_number (GtkTreeView *view, int colnum)
-{
-    GList *collist = gtk_tree_view_get_columns(view);
-    GtkTreeViewColumn *column = NULL;
-    gpointer p;
-    int gotcol = 0;
-
-    while (collist != NULL) {
-	column = collist->data;
-	p = g_object_get_data(G_OBJECT(column), "colnum");
-	if (p != NULL) {
-	    if (colnum == GPOINTER_TO_INT(p)) {
-		gotcol = 1;
-		break;
-	    }
-	}
-	collist = collist->next;
-    }
-
-    if (collist) g_list_free(collist);
-
-    return (gotcol)? column : NULL;
-}
-
-static void move_to_next_cell (spreadsheet *sheet, GtkTreePath *path,
+static void move_to_next_cell (Spreadsheet *sheet, GtkTreePath *path,
 			       GtkTreeViewColumn *column)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
@@ -168,7 +169,7 @@ static void move_to_next_cell (spreadsheet *sheet, GtkTreePath *path,
 	gpointer p = g_object_get_data(G_OBJECT(column), "colnum");
 
 	if (p != NULL && (colnum = GPOINTER_TO_INT(p)) < sheet->datacols) {
-	    GtkTreeViewColumn *nextcol = get_column_by_number(view, colnum + 1);
+	    GtkTreeViewColumn *nextcol = gtk_tree_view_get_column(view, colnum + 1);
 
 	    if (nextcol != NULL) {
 		gtk_tree_view_set_cursor(view, path, nextcol, FALSE);
@@ -186,7 +187,7 @@ static void move_to_next_cell (spreadsheet *sheet, GtkTreePath *path,
 static gint sheet_cell_edited (GtkCellRendererText *cell,
 			       const gchar *path_string,
 			       const gchar *user_text,
-			       spreadsheet *sheet)
+			       Spreadsheet *sheet)
 {
     const gchar *new_text = NULL;
     int err = 0;
@@ -221,7 +222,7 @@ static gint sheet_cell_edited (GtkCellRendererText *cell,
 	if (old_text != NULL && strcmp(old_text, new_text)) {
 	    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
 			       colnum, new_text, -1);
-	    sheet_modified = 1;
+	    sheet->modified = 1;
 	}
 #ifndef OLD_SELECTION
 	move_to_next_cell(sheet, path, column);
@@ -235,22 +236,37 @@ static gint sheet_cell_edited (GtkCellRendererText *cell,
 
 /* .................................................................. */
 
-static int real_add_new_var (spreadsheet *sheet, const char *varname)
+static void 
+spreadsheet_scroll_to_new_col (Spreadsheet *sheet, GtkTreeViewColumn *column)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkTreePath *path;
+    gchar *pathstr;
+    GtkAdjustment *adj;
+    GtkWidget *vp;
+
+    pathstr = g_strdup("0");
+    path = gtk_tree_path_new_from_string(pathstr);
+    gtk_tree_view_set_cursor(view, path, column, TRUE);
+    vp = gtk_widget_get_ancestor(GTK_WIDGET(view), GTK_TYPE_BIN);
+    adj = gtk_viewport_get_hadjustment(GTK_VIEWPORT(vp));
+    gtk_adjustment_set_value(adj, adj->upper);
+    gtk_tree_path_free(path);
+    g_free(pathstr);
+}
+
+static int real_add_new_var (Spreadsheet *sheet, const char *varname)
 {
     GtkTreeViewColumn *column;
-    GList *collist = NULL, *clstart = NULL;
-    gint i, cols;
+    gint cols, colnum;
     char tmp[16];
 
-#ifdef SSDEBUG
-    fprintf(stderr, "real_add_new_var: sheet->totcols=%d\n", 
-	    sheet->totcols);
-#endif
+    if (add_data_column(sheet)) {
+	return 1;
+    }
 
-    if (add_data_column(sheet)) return 1;
-
-#ifdef SSDEBUG
-    fprintf(stderr, "real_add_new_var: now: sheet->totcols=%d\n", 
+#if SSDEBUG
+    fprintf(stderr, "real_add_new_var, after add_data_column: sheet->totcols=%d\n", 
 	    sheet->totcols);
 #endif
 
@@ -259,62 +275,70 @@ static int real_add_new_var (spreadsheet *sheet, const char *varname)
     gtk_tree_view_column_set_title(column, tmp);
     set_up_sheet_column(column, get_data_col_width(), TRUE);
 
-    cols = gtk_tree_view_insert_column(GTK_TREE_VIEW(sheet->view), 
-				       column, sheet->datacols);
-	
-    clstart = collist = gtk_tree_view_get_columns(GTK_TREE_VIEW(sheet->view));
+    cols = gtk_tree_view_insert_column(GTK_TREE_VIEW(sheet->view), column, -1);
+    colnum = cols - 1;
 
+#if 0	
     create_sheet_cell_renderers(sheet);
+#endif
 
-    i = 0;
-    while (collist != NULL && i < sheet->totcols) {
-	GtkCellRenderer *renderer;
-	gint idx;
+    gtk_tree_view_column_pack_start(column, sheet->datacell, TRUE);
+    gtk_tree_view_column_set_attributes(column, 
+					sheet->datacell,
+					"text", colnum,
+					NULL);
+    g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(colnum));
 
-	column = GTK_TREE_VIEW_COLUMN(collist->data);
-	gtk_tree_view_column_clear(column);
+    /* scroll to editing position if need be */
+    spreadsheet_scroll_to_new_col(sheet, column);
 
-	if (i > 0 && i <= sheet->datacols) {
-	    renderer = sheet->datacell;
-	    idx = i;
-	} else {
-	    renderer = sheet->dumbcell;
-	    idx = 0;
-	}
-
-	gtk_tree_view_column_pack_start(column, renderer, TRUE);
-	gtk_tree_view_column_set_attributes (column, 
-					     renderer,
-					     "text", i,
-					     NULL);
-	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(idx));
-
-	collist = collist->next;
-	i++;
-    }
-
-    if (clstart) g_list_free(clstart);
-
-    sheet_modified = 1;
+    sheet->modified = 1;
 
     return 0;
 }
 
 /* .................................................................. */
 
-static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
+static void 
+spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    GtkTreePath *path;
+    gchar *pathstr;
+    GtkTreeViewColumn *column;
+    GtkAdjustment *adj;
+    GtkWidget *vp;
+
+    pathstr = g_strdup_printf("%d", row);
+    path = gtk_tree_path_new_from_string(pathstr);
+    column = gtk_tree_view_get_column(view, col);
+    gtk_tree_view_set_cursor(view, path, column, FALSE);
+    vp = gtk_widget_get_ancestor(GTK_WIDGET(view), GTK_TYPE_BIN);
+    adj = gtk_viewport_get_vadjustment(GTK_VIEWPORT(vp));
+    gtk_adjustment_set_value(adj, adj->upper);
+    gtk_tree_path_free(path);
+    g_free(pathstr);
+}
+
+static void 
+real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     gint pointpath = 0;
+    gint oldrows = sheet->datarows;
     GtkListStore *store;
+    gchar *pathstr = NULL;
     GtkTreeIter iter;
     gchar rowlabel[10];
-    gint i;
+    gint i, j;
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
 
     if (sheet->point == SHEET_AT_END) {
-	gtk_list_store_append(store, &iter);
+	pathstr = g_strdup_printf("%d", sheet->datarows - 1);
+	for (i=0; i<n; i++) {
+	    gtk_list_store_append(store, &iter);
+	}
     } else if (sheet->point == SHEET_AT_POINT) {
 	GtkTreePath *path;
 
@@ -327,18 +351,34 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
 	return;
     }
 
-    sheet->datarows += 1;
-
     if (datainfo->markers && obsname != NULL) {
 	gtk_list_store_set(store, &iter, 0, obsname, -1);
     } else if (sheet->point == SHEET_AT_END) {
-	get_full_obs_string(rowlabel, sheet->datarows - 1, datainfo);
-	gtk_list_store_set(store, &iter, 0, rowlabel, -1);
+	gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store), &iter,
+					    pathstr);
+	for (j=0; j<n; j++) {
+	    gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+	    get_full_obs_string(rowlabel, sheet->datarows + j, datainfo);
+	    gtk_list_store_set(store, &iter, 0, rowlabel, -1);
+	}
     }
 
-    for (i=1; i<=sheet->datacols; i++) {
-	gtk_list_store_set(store, &iter, i, "", -1);
-    }
+    sheet->datarows += n;
+
+    if (pathstr != NULL) {
+	gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store), &iter,
+					    pathstr);
+	for (j=0; j<n; j++) {
+	    gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+	    for (i=1; i<=sheet->datacols; i++) {
+		gtk_list_store_set(store, &iter, i, "", -1);
+	    }	
+	}
+    } else {
+	for (i=1; i<=sheet->datacols; i++) {
+	    gtk_list_store_set(store, &iter, i, "", -1);
+	}
+    }	
 
     if (sheet->point == SHEET_AT_POINT && !datainfo->markers) {
 	for (i=pointpath; i<sheet->datarows; i++) {
@@ -349,108 +389,89 @@ static void real_add_new_obs (spreadsheet *sheet, const char *obsname)
     } 
 
     if (sheet->point == SHEET_AT_END) {
-	GtkTreePath *path;
-	GtkTreeViewColumn *column;
-	GtkAdjustment *adj;
-	gchar *pathstr;
-	gdouble adjval;
-
-	pathstr = g_strdup_printf("%d", sheet->datarows - 1);
-	path = gtk_tree_path_new_from_string(pathstr);
-	column = gtk_tree_view_get_column(view, 1);
-	gtk_tree_view_set_cursor(view, path, column, FALSE);
-	adj = 
-	    gtk_viewport_get_vadjustment(GTK_VIEWPORT(gtk_widget_get_ancestor
-						      (GTK_WIDGET(view),
-						       GTK_TYPE_BIN)));
-	adjval = gtk_adjustment_get_value(adj);
-	gtk_adjustment_set_value (adj, adjval + 30); /* why is this hack needed? */
-	gtk_tree_path_free(path);
-	g_free(pathstr);
+	spreadsheet_scroll_to_foot(sheet, oldrows, 1);
     } else {
-	gchar *pathstr = g_strdup_printf("%d", pointpath);
-	GtkTreePath *path = gtk_tree_path_new_from_string(pathstr);
+	GtkTreePath *path;
 	GtkTreeIter insiter;
 
+	pathstr = g_strdup_printf("%d", pointpath);
+	path = gtk_tree_path_new_from_string(pathstr);
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &insiter, path);
 	gtk_list_store_set(store, &insiter, 1, "", -1);
 	gtk_tree_path_free(path);
+    }
+
+    if (pathstr != NULL) {
 	g_free(pathstr);
     }
 
-    sheet_modified = 1;
+    sheet->modified = 1;
 }
 
 /* ........................................................... */
 
-static void name_new_var (GtkWidget *widget, dialog_t *ddata) 
+static void name_new_var (GtkWidget *widget, dialog_t *dlg) 
 {
-    spreadsheet *sheet = (spreadsheet *) dialog_data_get_data(ddata);
+    Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
     const gchar *buf;
     char varname[VNAMELEN];
 
-    buf = dialog_data_get_text(ddata);
+    buf = edit_dialog_get_text(dlg);
 
     if (buf == NULL || validate_varname(buf)) return;
 
     *varname = 0;
     strncat(varname, buf, VNAMELEN - 1);
 
-    close_dialog(ddata);
+    close_dialog(dlg);
 
     if (real_add_new_var(sheet, varname)) {
 	errbox(_("Out of memory attempting to add variable"));
     }
 }
 
-/* ........................................................... */
-
-static void name_new_obs (GtkWidget *widget, dialog_t *ddata) 
+static void name_new_obs (GtkWidget *widget, dialog_t *dlg) 
 {
-    spreadsheet *sheet = (spreadsheet *) dialog_data_get_data(ddata);
+    Spreadsheet *sheet = (Spreadsheet *) edit_dialog_get_data(dlg);
     const gchar *buf;
     char obsmarker[OBSLEN];
 
-    buf = dialog_data_get_text(ddata);
+    buf = edit_dialog_get_text(dlg);
     if (buf == NULL) return;
 
     *obsmarker = 0;
     strncat(obsmarker, buf, OBSLEN - 1);
 
-    close_dialog(ddata);
-    real_add_new_obs(sheet, obsmarker);
+    close_dialog(dlg);
+    real_add_new_obs(sheet, obsmarker, 1);
 }
 
-/* ........................................................... */
-
-static void name_var_dialog (spreadsheet *sheet) 
+static void name_var_dialog (Spreadsheet *sheet) 
 {
     edit_dialog (_("gretl: name variable"), 
 		 _("Enter name for new variable\n"
 		   "(max. 8 characters)"),
 		 NULL, name_new_var, sheet, 
-		 0, 0);
+		 0, 0, 0);
 }
 
-/* ........................................................... */
-
-static void new_case_dialog (spreadsheet *sheet) 
+static void new_case_dialog (Spreadsheet *sheet) 
 {
     edit_dialog (_("gretl: case marker"), 
 		 _("Enter case marker for new obs\n"
 		   "(max. 8 characters)"),
 		 NULL, name_new_obs, sheet, 
-		 0, 0);
+		 0, 0, 0);
 }
 
 /* ........................................................... */
 
-static int add_data_column (spreadsheet *sheet)
+static int add_data_column (Spreadsheet *sheet)
 {
     GType *types;
     GtkListStore *old_store, *new_store;
     GtkTreeIter old_iter, new_iter;
-    gint i, row, newcol;
+    gint i, row;
 
     /* This is relatively complex because, so far as I can tell, you can't
        append or insert additional columns in a GtkListStore: we have to
@@ -471,8 +492,7 @@ static int add_data_column (spreadsheet *sheet)
 	types[i] = G_TYPE_STRING;
     }
 
-    newcol = sheet->datacols;
-
+    /* get pointers to original and new stores */
     old_store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(sheet->view)));
     new_store = gtk_list_store_newv(sheet->totcols, types);
     free(types);
@@ -483,19 +503,18 @@ static int add_data_column (spreadsheet *sheet)
 
     /* construct the new table */
     for (row=0; row<sheet->datarows; row++) {
-	int col;
 	gchar *str;
+	int col;
 
 	gtk_list_store_append(new_store, &new_iter);
 
 	for (col=0; col<sheet->totcols; col++) {
-	    if (col < newcol) {
+	    if (col < sheet->datacols) {
 		/* copy labels and original data */
 		gtk_tree_model_get(GTK_TREE_MODEL(old_store), &old_iter, col, &str, -1);
 		gtk_list_store_set(new_store, &new_iter, col, str, -1);
 		g_free(str);
-	    }
-	    else if (col >= newcol && col <= newcol) {
+	    } else { 
 		/* new data values: set blank */
 		gtk_list_store_set(new_store, &new_iter, col, "", -1);
 	    }
@@ -506,6 +525,13 @@ static int add_data_column (spreadsheet *sheet)
     gtk_tree_view_set_model(GTK_TREE_VIEW(sheet->view), GTK_TREE_MODEL(new_store));
     g_object_unref(G_OBJECT(new_store));
 
+    sheet->added_vars += 1;
+
+#if SSDEBUG
+    fprintf(stderr, "add_data_column: sheet->added_vars now = %d\n",
+	    sheet->added_vars);
+#endif
+
     return 0;
 }
 
@@ -513,46 +539,46 @@ static int add_data_column (spreadsheet *sheet)
 
 static void sheet_add_var_callback (gpointer data, guint u, GtkWidget *w)
 {
-    spreadsheet *sheet = (spreadsheet *) data;
+    Spreadsheet *sheet = (Spreadsheet *) data;
 
     name_var_dialog(sheet);
 }
 
-/* ........................................................... */
-
 static void sheet_add_obs_callback (gpointer data, guint where, GtkWidget *w)
 {
-    spreadsheet *sheet = (spreadsheet *) data;
+    Spreadsheet *sheet = (Spreadsheet *) data;
 
     sheet->point = where;
 
     if (datainfo->markers) {
 	new_case_dialog(sheet);
+    } else if (where == SHEET_AT_END) {
+	int n = add_obs_dialog(NULL, 1);
+
+	if (n > 0) {
+	    real_add_new_obs(sheet, NULL, n);
+	}
     } else {
-	real_add_new_obs(sheet, NULL);
+	real_add_new_obs(sheet, NULL, 1);
     }
 }
 
-/* ........................................................... */
-
-static void popup_sheet_add_obs (GtkWidget *w, spreadsheet *sheet)
+static void popup_sheet_add_obs (GtkWidget *w, Spreadsheet *sheet)
 {
     sheet_add_obs_callback(sheet, SHEET_AT_END, NULL);
 }
 
-static void popup_sheet_insert_obs (GtkWidget *w, spreadsheet *sheet)
+static void popup_sheet_insert_obs (GtkWidget *w, Spreadsheet *sheet)
 {
     sheet_add_obs_callback(sheet, SHEET_AT_POINT, NULL);
 }
 
-static void popup_sheet_add_var (GtkWidget *w, spreadsheet *sheet)
+static void popup_sheet_add_var (GtkWidget *w, Spreadsheet *sheet)
 {
     sheet_add_var_callback(sheet, 0, NULL);
 }
 
-/* ........................................................... */
-
-static void build_sheet_popup (spreadsheet *sheet)
+static void build_sheet_popup (Spreadsheet *sheet)
 {
     if (sheet->popup != NULL) return;
 
@@ -573,7 +599,7 @@ static void build_sheet_popup (spreadsheet *sheet)
 
 #ifndef OLD_SELECTION
 
-static gboolean update_cell_position (GtkTreeView *view, spreadsheet *sheet)
+static gboolean update_cell_position (GtkTreeView *view, Spreadsheet *sheet)
 {
     GtkTreePath *path = NULL;
     GtkTreeViewColumn *column;
@@ -581,7 +607,7 @@ static gboolean update_cell_position (GtkTreeView *view, spreadsheet *sheet)
 
     /* this is connected to the "cursor-changed" signal */
 
-#ifdef SSDEBUG
+#if CELLDEBUG
     fprintf(stderr, "** update_cell_position()\n");
 #endif
 
@@ -599,7 +625,7 @@ static gboolean update_cell_position (GtkTreeView *view, spreadsheet *sheet)
 	}
 
 	if (newrow != oldrow || newcol != oldcol) {
-#ifdef SSDEBUG
+#if CELLDEBUG
 	    fprintf(stderr, " activating cell(%d, %d)\n", newrow, newcol);
 #endif
 	    set_locator_label(sheet, path, column);
@@ -608,73 +634,110 @@ static gboolean update_cell_position (GtkTreeView *view, spreadsheet *sheet)
 	    gtk_tree_view_set_cursor(view, path, column, 
 				     FALSE); /* start editing? */
 	} else {
-#ifdef SSDEBUG
+#if CELLDEBUG
 	   fprintf(stderr, " still in cell(%d, %d)\n", oldrow, oldcol); 
 #endif
 	}
     }
 
-    if (path != NULL) gtk_tree_path_free(path);
+    if (path != NULL) {
+	gtk_tree_path_free(path);
+    }
 
     return TRUE; /* is this right? */
 }
 
 #endif /* !OLD_SELECTION */
 
-/* ........................................................... */
-
-static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet)
+static int 
+var_added_since_ssheet_opened (int i, Spreadsheet *sheet, int main_v)
 {
-    int n = datainfo->n, oldv = datainfo->v;
-    int i, t;
-    int orig_cols, newvars, newobs, missobs = 0;
+    return (i >= sheet->orig_main_v && i < main_v);
+}
+
+/* pull modified values from the data-editing spreadsheet
+   into the main dataset */
+
+static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet)
+{
+    int oldv = datainfo->v;
+    int newvars = sheet->added_vars;
+    int newobs = sheet->datarows - datainfo->n;
+    int missobs = 0;
+
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreeIter iter;
     GtkTreeViewColumn *column;
     GtkTreeModel *model;
-    gint colnum;
 
-    newobs = sheet->datarows - n;
-    orig_cols = datainfo->v - 1 - sheet->n_scalars;
-    newvars = sheet->datacols - orig_cols;
+    int i, colnum, t;
+
+    if (!sheet->modified) {
+	infobox(_("No changes were made"));
+	return;
+    }    
+
+#if SSDEBUG
+    fprintf(stderr, "get_data_from_sheet: oldv = %d, newvars = %d, newobs = %d\n",
+	    oldv, newvars, newobs);
+#endif
 
     model = gtk_tree_view_get_model(view);
 
     if (newobs > 0) {
-	if (grow_nobs(newobs, &Z, datainfo)) {
+	/* extend series length first, if needed */
+	if (dataset_add_observations(newobs, &Z, datainfo) ||
+	    dataset_destroy_hidden_variables(&Z, datainfo)) {
 	    errbox(_("Failed to allocate memory for new data"));
 	    return;
 	}
-	n = datainfo->n;
     }
 
     if (newvars > 0) {
-	if (dataset_add_vars(newvars, &Z, datainfo)) {
+	/* then add variables, if needed */
+	if (dataset_add_series(newvars, &Z, datainfo)) {
 	    errbox(_("Failed to allocate memory for new data"));
 	    return;
 	}
 	for (i=0; i<newvars; i++) { 
 	    const gchar *newname;
 
-	    column = gtk_tree_view_get_column(view, orig_cols + 1 + i);
+	    colnum = sheet->orig_vars + 1 + i;
+	    column = gtk_tree_view_get_column(view, colnum);
 	    newname = gtk_tree_view_column_get_title(column);
-	    strcpy(datainfo->varname[i + oldv], newname);
-	    strcpy(VARLABEL(datainfo, i + oldv), "");
+	    strcpy(datainfo->varname[oldv + i], newname); 
+#if SSDEBUG
+	    fprintf(stderr, " added var %d (%s) from column %d\n",
+		    oldv + i, newname, colnum);
+#endif
+
 	}
     }
 
-    colnum = 0;
+    colnum = 1;
     for (i=1; i<datainfo->v; i++) {
-	if (datainfo->vector[i] == 0) {
+
+	if (spreadsheet_hide(i, datainfo)) {
+	    /* hidden vars were not put into the spreadsheet */
 	    continue;
 	}
-	colnum++;
-	gtk_tree_model_get_iter_first(model, &iter);	
-	for (t=0; t<n; t++) {
+
+	if (var_added_since_ssheet_opened(i, sheet, oldv)) {
+	    continue;
+	}
+
+	gtk_tree_model_get_iter_first(model, &iter);
+
+#if SSDEBUG
+	fprintf(stderr, " updating data for var %d (%s) from column %d\n",
+		i, datainfo->varname[i], colnum);
+#endif
+	
+	for (t=0; t<datainfo->n; t++) {
 	    gchar *numstr;
 
 	    gtk_tree_model_get(model, &iter, colnum, &numstr, -1);
-	    if (*numstr) {
+	    if (*numstr != '\0') {
 		Z[i][t] = atof(numstr); 
 	    } else {
 		Z[i][t] = NADBL;
@@ -683,13 +746,15 @@ static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet)
 	    g_free(numstr);
 	    gtk_tree_model_iter_next(model, &iter);
 	}
+	colnum++;
     }
 
     if (datainfo->markers && datainfo->S != NULL) {
-	gtk_tree_model_get_iter_first(model, &iter);
-	for (t=0; t<n; t++) {
-	    gchar *marker;
+	gchar *marker;
 
+	gtk_tree_model_get_iter_first(model, &iter);
+
+	for (t=0; t<datainfo->n; t++) {
 	    gtk_tree_model_get(model, &iter, 0, &marker, -1);
 	    strcpy(datainfo->S[t], marker);
 	    g_free(marker);
@@ -706,12 +771,12 @@ static void get_data_from_sheet (GtkWidget *w, spreadsheet *sheet)
 	infobox(_("Data updated OK"));
     }
 
-    sheet_modified = 0;
+    sheet->modified = 0;
 }
 
 /* ........................................................... */
 
-static void select_first_editable_cell (spreadsheet *sheet)
+static void select_first_editable_cell (Spreadsheet *sheet)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreePath *path;
@@ -727,44 +792,53 @@ static void select_first_editable_cell (spreadsheet *sheet)
 
 /* ........................................................... */
 
-static int add_data_to_sheet (spreadsheet *sheet, int new)
+static int add_data_to_sheet (Spreadsheet *sheet, SheetCode code)
 {
     gchar rowlabel[OBSLEN];
-    gint i, t, colnum, n = datainfo->n;
+    gint colnum;
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreeIter iter;
     GtkListStore *store;
+    int i, t;
+
+#if SSDEBUG
+    fprintf(stderr, "Doing add_data_to_sheet\n");
+#endif
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
 
     /* insert observation markers */
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    for (t=0; t<n; t++) {
+    for (t=0; t<datainfo->n; t++) {
 	get_full_obs_string(rowlabel, t, datainfo);
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter, 0, rowlabel, -1);
     }
 
-    sheet->datarows = t;
+    sheet->datarows = datainfo->n;
 
-    /* insert data values */
+    /* now insert data values */
 
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
-    for (t=0; t<n; t++) {
-	if (new) {
-	    /* insert "missing" values */
+    for (t=0; t<datainfo->n; t++) {
+	if (code == SHEET_NEW_DATASET) {
+	    /* no hidden vars to consider; insert "missing" values */
 	    for (i=1; i<datainfo->v; i++) {
 		gtk_list_store_set(store, &iter, i, "", -1);
 	    }
 	} else {	
-	    /* insert actual values */
 	    char numstr[32];
 
 	    colnum = 0;
 	    for (i=1; i<datainfo->v; i++) {
-		/* don't put scalars into the spreadsheet */
-		if (datainfo->vector[i] == 0) continue;
+		if (spreadsheet_hide(i, datainfo)) {
+		    /* don't put scalars or hidden vars into the spreadsheet */
+#if SSDEBUG
+		    if (t == 0) fprintf(stderr, " skipping hidden var, ID %d\n", i);
+#endif
+		    continue;
+		}
 		if (na(Z[i][t])) {
 		    *numstr = '\0';
 		} else {
@@ -772,11 +846,22 @@ static int add_data_to_sheet (spreadsheet *sheet, int new)
 		}
 		colnum++;
 		gtk_list_store_set(store, &iter, colnum, numstr, -1);
+#if SSDEBUG
+		if (t == 0) fprintf(stderr, " adding data for var %d in col %d\n", 
+				    i, colnum);
+#endif
 	    }
 	}
 	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
     }
 
+    sheet->orig_main_v = datainfo->v;
+
+#if SSDEBUG
+    fprintf(stderr, " datarows=%d, orig_vars=%d, orig_main_v=%d\n", 
+	    sheet->datarows, sheet->orig_vars, sheet->orig_main_v);
+#endif
+
     return 0;
 }
 
@@ -807,7 +892,9 @@ static gint get_obs_col_width (void)
 {
     static gint width;
 
-    if (width == 0) width = get_string_width("XXXXXXXXX");
+    if (width == 0) {
+	width = get_string_width("XXXXXXXXX");
+    }
     return width;
 }
 
@@ -815,7 +902,9 @@ static gint get_data_col_width (void)
 {
     static gint width;
 
-    if (width == 0) width = get_string_width("-00.000000");
+    if (width == 0) {
+	width = get_string_width("-00.000000");
+    }
     return width;
 }
 
@@ -835,7 +924,7 @@ set_up_sheet_column (GtkTreeViewColumn *column, gint width, gboolean expand)
 
 /* ........................................................... */
 
-static void create_sheet_cell_renderers (spreadsheet *sheet)
+static void create_sheet_cell_renderers (Spreadsheet *sheet)
 {
     GtkCellRenderer *r;
 
@@ -851,7 +940,7 @@ static void create_sheet_cell_renderers (spreadsheet *sheet)
 		 "xalign", 1.0, 
 		 "editable", TRUE, NULL);
     g_signal_connect (G_OBJECT (r), "edited",
-		      G_CALLBACK (sheet_cell_edited), sheet);
+		      G_CALLBACK(sheet_cell_edited), sheet);
     sheet->datacell = r;
 }
 
@@ -860,7 +949,7 @@ static void create_sheet_cell_renderers (spreadsheet *sheet)
 /* relatively minimal version for gtk 2.0.N */
 
 static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
-				     spreadsheet *sheet)
+				     Spreadsheet *sheet)
 {   
     GdkModifierType mods; 
     gint ret = FALSE;
@@ -903,7 +992,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
     return ret;
 }
 
-static gboolean update_selected (GtkTreeSelection *selection, spreadsheet *sheet)
+static gboolean update_selected (GtkTreeSelection *selection, Spreadsheet *sheet)
 {
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
     GtkTreePath *path;
@@ -950,7 +1039,7 @@ static void manufacture_keystroke (GtkWidget *widget, guint uval)
 }
 
 static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key, 
-				   spreadsheet *sheet)
+				   Spreadsheet *sheet)
 {
     if (key->keyval == GDK_Tab) {
 	/* FIXME */
@@ -971,9 +1060,6 @@ static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key,
 	    if (key->keyval == GDK_Left && colnum == 1) {
 		return TRUE;
 	    }
-	    if (key->keyval == GDK_Right && colnum == sheet->datacols) {
-		return TRUE;
-	    }
 	}
     } 
 
@@ -1000,12 +1086,12 @@ static gint catch_spreadsheet_key (GtkWidget *view, GdkEventKey *key,
 /* fuller version of "catch_spreadsheet_click" for gtk 2.2 and higher */
 
 static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
-				     spreadsheet *sheet)
+				     Spreadsheet *sheet)
 {   
     GdkModifierType mods; 
     gint ret = FALSE;
 
-# ifdef SSDEBUG
+# if CELLDEBUG
     fprintf(stderr, "** catch_spreadsheet_click()\n");
 # endif
 
@@ -1031,7 +1117,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	GtkTreePath *path;
 	GtkTreeViewColumn *column;
 
-# ifdef SSDEBUG
+# if CELLDEBUG
 	fprintf(stderr, "Got button 1 click\n");
 # endif
 
@@ -1044,7 +1130,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	    gpointer p = g_object_get_data(G_OBJECT(column), "colnum");
 	    gint colnum = GPOINTER_TO_INT(p);
 
-# ifdef SSDEBUG
+# if CELLDEBUG
 	    fprintf(stderr, "Clicked column: colnum = %d\n", colnum);
 # endif
 
@@ -1061,7 +1147,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 	gtk_tree_path_free(path);
     }
 
-# ifdef SSDEBUG
+# if CELLDEBUG
     fprintf(stderr, "catch_spreadsheet_click returning %d\n", ret);
 # endif
 
@@ -1070,9 +1156,7 @@ static gint catch_spreadsheet_click (GtkWidget *view, GdkEvent *event,
 
 #endif /* end of gtk >= 2.2 code */
 
-/* ........................................................... */
-
-static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
+static int build_sheet_view (Spreadsheet *sheet)
 {
     GtkListStore *store; 
     GtkWidget *view;
@@ -1081,44 +1165,45 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
     GType *types;
     gint i, width, colnum;
 
-    sheet->datacols = nvars - 1; /* don't show the constant */
+    sheet->datacols = 0;
 
-    /* we'll drop any scalar variables from the spreadsheet */
     for (i=1; i<datainfo->v; i++) {
-	if (datainfo->vector[i] == 0) {
-	    sheet->datacols -= 1;
-	    sheet->n_scalars += 1;
+	if (!spreadsheet_hide(i, datainfo)) {
+	    sheet->datacols += 1;
 	}
     }
 
-    /* obs, data, padding, boolean cols */
-    sheet->totcols = 1 + sheet->datacols;
+    /* obs col, data cols */
+    sheet->totcols = sheet->datacols + 1;
+    sheet->orig_vars = sheet->datacols;
 
     types = mymalloc(sheet->totcols * sizeof *types);
-    if (types == NULL) return NULL;
+    if (types == NULL) {
+	return 1;
+    }
 
     /* configure the types */
     for (i=0; i<sheet->totcols; i++) {
 	types[i] = G_TYPE_STRING;
     }
 
-    store = gtk_list_store_newv (sheet->totcols, types);
+    store = gtk_list_store_newv(sheet->totcols, types);
     free(types);
 
     view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
     g_object_unref(G_OBJECT(store));
 
-    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW(view), TRUE);
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(view), TRUE);
 
     /* build and attach the (two) cell renderers */
     create_sheet_cell_renderers(sheet);
 
     /* construct the observation marker column */
     width = get_obs_col_width();
-    column = gtk_tree_view_column_new_with_attributes (NULL,
-						       sheet->dumbcell,
-						       "text", 0, 
-						       NULL);
+    column = gtk_tree_view_column_new_with_attributes(NULL,
+						      sheet->dumbcell,
+						      "text", 0, 
+						      NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
     set_up_sheet_column(column, width, FALSE);
     g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(0));
@@ -1126,20 +1211,22 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
     /* construct the data columns */
     width = get_data_col_width();
     colnum = 0;
-    for (i=1; i<nvars; i++) {
+    for (i=1; i<datainfo->v; i++) {
 	char tmp[16];
 
-	if (datainfo->vector[i] == 0) continue;
+	if (spreadsheet_hide(i, datainfo)) {
+	    continue;
+	}
 	colnum++;
 	double_underscores(tmp, datainfo->varname[i]);
-	column = gtk_tree_view_column_new_with_attributes (tmp,
-							   sheet->datacell,
-							   "text", 
-							   colnum, 
-							   NULL);
+	column = gtk_tree_view_column_new_with_attributes(tmp,
+							  sheet->datacell,
+							  "text", 
+							  colnum, 
+							  NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 	set_up_sheet_column(column, width, TRUE);
-	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(i));
+	g_object_set_data(G_OBJECT(column), "colnum", GINT_TO_POINTER(colnum));
     }
 
     /* set the selection property on the tree view */
@@ -1158,14 +1245,17 @@ static GtkWidget *data_sheet_new (spreadsheet *sheet, gint nobs, gint nvars)
 		      G_CALLBACK(catch_spreadsheet_key), sheet);
 #endif
 
-    return view;
+    /* attach to sheet struct */
+    sheet->view = view;
+
+    return 0;
 }
 
 /* ........................................................... */
 
-static void free_spreadsheet (GtkWidget *widget, spreadsheet **psheet) 
+static void free_spreadsheet (GtkWidget *widget, Spreadsheet **psheet) 
 {
-    spreadsheet *sheet = *psheet;
+    Spreadsheet *sheet = *psheet;
 
     if (sheet->popup != NULL) {
 	gtk_widget_destroy(sheet->popup);
@@ -1173,13 +1263,13 @@ static void free_spreadsheet (GtkWidget *widget, spreadsheet **psheet)
 
     free(sheet);
     *psheet = NULL;
+
+    set_dataset_locked(FALSE);
 }
 
-/* ........................................................... */
-
-static spreadsheet *sheet_new (void)
+static Spreadsheet *spreadsheet_new (void)
 {
-    spreadsheet *sheet;
+    Spreadsheet *sheet;
 
     sheet = mymalloc(sizeof *sheet);
     if (sheet == NULL) return NULL;
@@ -1192,7 +1282,10 @@ static spreadsheet *sheet_new (void)
     sheet->datacell = NULL;
     sheet->datacols = sheet->datarows = 0;
     sheet->totcols = 0;
-    sheet->n_scalars = 0;
+    sheet->orig_vars = 0;
+    sheet->added_vars = 0;
+    sheet->orig_main_v = 0;
+    sheet->modified = 0;
     sheet->cid = 0;
 
     return sheet;
@@ -1200,11 +1293,11 @@ static spreadsheet *sheet_new (void)
 
 /* ........................................................... */
 
-static gint maybe_exit_sheet (GtkWidget *w, spreadsheet *sheet)
+static gint maybe_exit_sheet (GtkWidget *w, Spreadsheet *sheet)
 {
     int resp;
 
-    if (sheet_modified) {
+    if (sheet->modified) {
 	resp = yes_no_dialog ("gretl", 
 			      _("Do you want to save changes you have\n"
 				"made to the current data set?"), 1);
@@ -1219,14 +1312,12 @@ static gint maybe_exit_sheet (GtkWidget *w, spreadsheet *sheet)
     return FALSE;
 }
 
-/* ........................................................... */
-
 static gint sheet_delete_event (GtkWidget *w, GdkEvent *event,
-				spreadsheet *sheet)
+				Spreadsheet *sheet)
 {
     int resp;
 
-    if (sheet_modified) {
+    if (sheet->modified) {
 	resp = yes_no_dialog ("gretl", 
 			      _("Do you want to save changes you have\n"
 				"made to the current data set?"), 1);
@@ -1242,9 +1333,9 @@ static gint sheet_delete_event (GtkWidget *w, GdkEvent *event,
 
 /* ........................................................... */
 
-void show_spreadsheet (DATAINFO *pdinfo) 
+void show_spreadsheet (SheetCode code) 
 {
-    static spreadsheet *sheet;    
+    static Spreadsheet *sheet;    
     GtkWidget *tmp, *button_box;
     GtkWidget *scroller, *main_vbox;
     GtkWidget *hbox, *padbox;
@@ -1254,7 +1345,7 @@ void show_spreadsheet (DATAINFO *pdinfo)
     int err = 0;
 
 #ifdef G_OS_WIN32
-    if (pdinfo == NULL && datainfo->n > 1600) {
+    if (datainfo->n > 1600) {
 	errbox(_("Sorry, can't edit more than 1600 rows"));
 	return;
     }    
@@ -1265,10 +1356,12 @@ void show_spreadsheet (DATAINFO *pdinfo)
 	return;
     }
 
-    sheet = sheet_new();
-    if (sheet == NULL) return;
+    sheet = spreadsheet_new();
+    if (sheet == NULL) {
+	return;
+    }
 
-    if (pdinfo == NULL && datainfo->v == 1) {
+    if (datainfo->v == 1) {
 	errbox(_("Please add a variable to the dataset first"));
 	return;
     }
@@ -1296,6 +1389,11 @@ void show_spreadsheet (DATAINFO *pdinfo)
     gtk_item_factory_create_items(ifac, 
 				  sizeof sheet_items / sizeof sheet_items[0],
 				  sheet_items, sheet);
+
+    if (complex_subsampled()) {
+	disable_obs_menu(ifac);
+    }
+
     mbar = gtk_item_factory_get_widget(ifac, "<main>");
     gtk_box_pack_start(GTK_BOX(main_vbox), mbar, FALSE, FALSE, 0);
     gtk_widget_show(mbar);
@@ -1326,11 +1424,12 @@ void show_spreadsheet (DATAINFO *pdinfo)
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller), hbox);
     gtk_widget_show(hbox);
 
-    sheet->view = data_sheet_new(sheet, datainfo->n, datainfo->v);
+    build_sheet_view(sheet);
     gtk_container_add(GTK_CONTAINER(hbox), sheet->view);
     gtk_widget_show(sheet->view); 
 
     if (datainfo->v < 7) {
+	/* padding to fill out the spreadsheet */
 	padbox = gtk_vbox_new(FALSE, 1);
 	gtk_container_add(GTK_CONTAINER(hbox), padbox);
 	gtk_widget_show(padbox);
@@ -1361,11 +1460,7 @@ void show_spreadsheet (DATAINFO *pdinfo)
 		      G_CALLBACK(catch_spreadsheet_click),
 		      sheet);
 
-    if (pdinfo == NULL) {
-	err = add_data_to_sheet(sheet, 0);
-    } else {
-	err = add_data_to_sheet(sheet, 1);
-    }
+    add_data_to_sheet(sheet, code);
 
     if (err) {
 	gtk_widget_destroy(sheet->win);
@@ -1373,8 +1468,31 @@ void show_spreadsheet (DATAINFO *pdinfo)
     }
 
     select_first_editable_cell(sheet);
-    sheet_modified = 0;
 
     gtk_widget_show(sheet->win);
+
+    /* we can't have the user making confounding changes elsewhere,
+       while editing the dataset here */
+    set_dataset_locked(TRUE);
 }
 
+/* mechanism for locking dataset against changes while editing */
+
+static int locked;
+
+static void set_dataset_locked (gboolean s)
+{
+    locked = s;
+    flip(mdata->ifac, "/Sample", !s);
+}
+
+int dataset_locked (void)
+{
+    if (locked) {
+	errbox(_("You can't do this while editing the dataset"));
+    }
+
+    return locked;
+}
+
+
