@@ -20,6 +20,13 @@
 #ifndef SSHEET_H
 #define SSHEET_H
 
-void show_spreadsheet (DATAINFO *pdinfo);
+typedef enum {
+    SHEET_EDIT_DATASET,
+    SHEET_NEW_DATASET
+} SheetCode;
+
+void show_spreadsheet (SheetCode code);
+
+int dataset_locked (void);
 
 #endif /* SSHEET_H */
