@@ -25,6 +25,10 @@
 #include "model_table.h"
 #include "clipboard.h"
 
+#ifdef OLD_GTK
+# include "menustate.h"
+#endif
+
 /* find-and-replace related materials */
 
 struct search_replace {
@@ -121,7 +125,7 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_widget_grab_focus(s->f_entry);
     gtk_widget_show (s->w);
 #ifdef OLD_GTK /* ?? */
-    gtk_window_set_modal(GTK_WINDOW(s->w), TRUE);
+    gretl_set_window_modal(s->w);
 #endif
     gtk_main();
 }
@@ -386,202 +390,153 @@ void text_replace (windata_t *mydata, guint u, GtkWidget *widget)
 
 #endif /* old versus new gtk */
 
-/* copying text from gretl windows */
+enum {
+    W_PREVIEW,
+    W_COPY,
+    W_SAVE
+};
 
-#define SPECIAL_COPY(h) (h == COPY_LATEX || h == COPY_RTF)
-
-void text_copy (gpointer data, guint how, GtkWidget *w) 
+static int special_text_handler (int cmd, guint fmt, int what, 
+				 gpointer data)
 {
-    windata_t *vwin = (windata_t *) data;
-    gchar *msg = NULL;
-    PRN *prn;
+    PRN *prn = NULL;
+    int err = 0;
 
-    /* descriptive statistics */
-    if ((vwin->role == SUMMARY || vwin->role == VAR_SUMMARY)
-	&& SPECIAL_COPY(how)) {
-	GRETLSUMMARY *summ = (GRETLSUMMARY *) vwin->data;
-	
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) {
-	    texprint_summary(summ, datainfo, prn);
-	} else if (how == COPY_RTF) { 
-	    rtfprint_summary(summ, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
+    if (bufopen(&prn)) {
+	return 1;
     }
 
-    /* correlation matrix */
-    else if (vwin->role == CORR && SPECIAL_COPY(how)) {
-	CORRMAT *corr = (CORRMAT *) vwin->data;
+    gretl_print_set_format(prn, fmt);
 
-	if (bufopen(&prn)) return;
+    if (cmd == SUMMARY || cmd == VAR_SUMMARY) {
+	Summary *summ = (Summary *) data;
 
-	if (how == COPY_LATEX) { 
-	    texprint_corrmat(corr, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    rtfprint_corrmat(corr, datainfo, prn);
-	}
+	special_print_summary(summ, datainfo, prn);
+    } else if (cmd == CORR || cmd == COVAR) {
+	VMatrix *corr = (VMatrix *) data;
 
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }
+	special_print_vmatrix(corr, datainfo, prn);
+    } else if (cmd == FCAST) {
+	FITRESID *fr = (FITRESID *) data;
 
-    /* display for fitted, actual, resid */
-    else if (vwin->role == FCAST && SPECIAL_COPY(how)) {
-	FITRESID *fr = (FITRESID *) vwin->data;
+	special_print_fit_resid(fr, datainfo, prn);
+    } else if (cmd == FCASTERR) {
+	FITRESID *fr = (FITRESID *) data;
 
-	if (bufopen(&prn)) return;
+	special_print_forecast(fr, datainfo, prn);
+    } else if (cmd == COEFFINT) {
+	CoeffIntervals *cf = (CoeffIntervals *) data;
 
-	if (how == COPY_LATEX) { 
-	    texprint_fit_resid(fr, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    rtfprint_fit_resid(fr, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }   
-
-    /* forecasts with standard errors */
-    else if (vwin->role == FCASTERR && SPECIAL_COPY(how)) {
-	FITRESID *fr = (FITRESID *) vwin->data;
-
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) { 
-	    texprint_fcast_with_errs(fr, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    rtfprint_fcast_with_errs(fr, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }  
-
-    /* coefficient confidence intervals */
-    else if (vwin->role == COEFFINT && SPECIAL_COPY(how)) {
-	CONFINT *cf = (CONFINT *) vwin->data;
-
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) { 
-	    texprint_confints(cf, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    rtfprint_confints(cf, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }  
-
-    /* coefficient covariance matrix */
-    else if (vwin->role == COVAR && SPECIAL_COPY(how)) {
-	VCV *vcv = (VCV *) vwin->data;
-
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) { 
-	    texprint_vcv(vcv, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    rtfprint_vcv(vcv, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }      
-
-    /* multiple-precision OLS (gtk-1.2?) */
-    else if (vwin->role == MPOLS && SPECIAL_COPY(how)) {
-	mp_results *mpvals = (mp_results *) vwin->data;
-
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) { 
-	    prn->format = GRETL_PRINT_FORMAT_TEX;
-	    print_mpols_results (mpvals, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    prn->format = GRETL_PRINT_FORMAT_RTF;
-	    print_mpols_results (mpvals, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }
-
-    /* VAR system */
-    else if (vwin->role == VAR && SPECIAL_COPY(how)) {
-	GRETL_VAR *var = (GRETL_VAR *) vwin->data;
-
-	if (bufopen(&prn)) return;
-
-	if (how == COPY_LATEX) { 
-	    prn->format = GRETL_PRINT_FORMAT_TEX;
-	    gretl_var_print(var, datainfo, prn);
-	} 
-	else if (how == COPY_RTF) { 
-	    prn->format = GRETL_PRINT_FORMAT_RTF;
-	    gretl_var_print(var, datainfo, prn);
-	}
-
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
-    }    
-
-    /* or it's a model window we're copying from? */
-    else if (vwin->role == VIEW_MODEL &&
-	(how == COPY_RTF || how == COPY_LATEX ||
-	 how == COPY_LATEX_EQUATION)) {
-	MODEL *pmod = (MODEL *) vwin->data;
+	special_print_confints(cf, prn);
+    } else if (cmd == VIEW_MODEL) { 
+	MODEL *pmod = (MODEL *) data;
 
 	if (pmod->errcode) { 
 	    errbox("Couldn't format model");
-	    return;
+	    err = 1;
+	} else {
+	    if (tex_format(prn)) {
+		tex_print_model(pmod, datainfo, prn);
+	    } else {
+		printmodel(pmod, datainfo, OPT_NONE, prn);
+	    }
 	}
-	if (bufopen(&prn)) return;
+    } else if (cmd == MPOLS) {
+	/* this is not actually ready -- see modelprint.c */
+	mp_results *mpvals = (mp_results *) data;
 
-	if (how == COPY_RTF) {
-	    prn->format = GRETL_PRINT_FORMAT_RTF;
-	    printmodel(pmod, datainfo, OPT_NONE, prn);
+	print_mpols_results(mpvals, datainfo, prn);
+    } else if (cmd == VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) data;
+
+	gretl_var_print(var, datainfo, prn);
+    } else if (cmd == VIEW_MODELTABLE) {
+	err = special_print_model_table(prn);
+    } 
+
+    if (!err) {
+	if (what == W_PREVIEW) {
+	    /* there's no RTF preview option */
+	    view_latex(prn);
+	} else if (what == W_COPY) {
+	    prn_to_clipboard(prn, fmt);
+	} else if (what == W_SAVE) {
+	    file_selector(_("Save LaTeX file"), SAVE_TEX, prn);
 	}
-	else if (how == COPY_LATEX) {
-	    prn->format = GRETL_PRINT_FORMAT_TEX;
-	    printmodel(pmod, datainfo, OPT_NONE, prn);
-	}
-	else if (how == COPY_LATEX_EQUATION) {
-	    tex_print_equation(pmod, datainfo, 0, prn);
-	}
-	prn_to_clipboard(prn, how);
-	gretl_print_destroy(prn);
     }
 
-    /* or from the model table? */
-    else if (vwin->role == VIEW_MODELTABLE && SPECIAL_COPY(how)) {
-	if (how == COPY_LATEX) {
-	    if (tex_print_model_table(0)) return;
+    gretl_print_destroy(prn);
+
+    return err;
+}
+
+void window_tex_callback (GtkWidget *w, windata_t *vwin)
+{
+    const char *opts[] = {
+	N_("View"),
+	N_("Copy"),
+	N_("Save")
+    };
+    int opt = radio_dialog("gretl: LaTeX", opts, 3, 0, 0);
+
+    if (opt >= 0) {
+	special_text_handler(vwin->role, GRETL_FORMAT_TEX, opt, vwin->data);
+    }
+}
+
+void model_tex_view (gpointer data, guint fmt, GtkWidget *w)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    special_text_handler(vwin->role, fmt, W_PREVIEW, vwin->data);
+}
+
+void model_tex_save (gpointer data, guint fmt, GtkWidget *w)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    special_text_handler(vwin->role, fmt, W_SAVE, vwin->data);
+}
+
+void var_tex_callback (gpointer data, guint opt, GtkWidget *w)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    special_text_handler(vwin->role, GRETL_FORMAT_TEX, opt, vwin->data);
+}
+
+/* copying text from gretl windows */
+
+#define SPECIAL_FORMAT(f) ((f & GRETL_FORMAT_TEX) || \
+                           (f & GRETL_FORMAT_RTF)) 
+
+void window_copy (gpointer data, guint fmt, GtkWidget *w) 
+{
+    windata_t *vwin = (windata_t *) data;
+    gchar *msg = NULL;
+    int err = 0;
+
+    /* copying from window with special stuff enabled */
+    if (MULTI_FORMAT_ENABLED(vwin->role) && SPECIAL_FORMAT(fmt)) {
+	err = special_text_handler(vwin->role, fmt, W_COPY, vwin->data);
+	if (err) {
+	    return;
 	}
-	else if (how == COPY_RTF) {
-	    if (rtf_print_model_table()) return;
-	} 
     }
 
     /* copying plain text from window */
 #ifndef OLD_GTK
-    else if (how == COPY_TEXT || how == COPY_TEXT_AS_RTF || how == COPY_SELECTION) {
+    else if (fmt == GRETL_FORMAT_TXT || 
+	     fmt == GRETL_FORMAT_RTF_TXT || 
+	     fmt == GRETL_FORMAT_SELECTION) {
 	GtkTextBuffer *textbuf = 
 	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-	PRN textprn;
-	int myhow = how;
+	PRN *textprn;
+	int myfmt = fmt;
 
-	if (myhow == COPY_SELECTION) myhow = COPY_TEXT;
+	if (myfmt == GRETL_FORMAT_SELECTION) {
+	    myfmt = GRETL_FORMAT_TXT;
+	}
 
 	if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL)) {
 	    /* there is a selection in place */
@@ -590,30 +545,31 @@ void text_copy (gpointer data, guint how, GtkWidget *w)
 
 	    gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend);
 	    selbuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
-	    gretl_print_attach_buffer(&textprn, selbuf);
-	    prn_to_clipboard(&textprn, myhow);
-	    g_free(selbuf);
+	    textprn = gretl_print_new_with_buffer(selbuf);
+	    prn_to_clipboard(textprn, myfmt);
+	    gretl_print_destroy(textprn);
 	    if (w != NULL) {
 		infobox(_("Copied selection to clipboard"));
 	    }
 	    return;
 	} else {
 	    /* no selection: copy everything */
-	    gretl_print_attach_buffer(&textprn,
-				      textview_get_text(GTK_TEXT_VIEW(vwin->w))); 
-	    prn_to_clipboard(&textprn, myhow);
-	    g_free(textprn.buf);
+	    char *buf = textview_get_text(GTK_TEXT_VIEW(vwin->w)); 
+
+	    textprn = gretl_print_new_with_buffer(buf);
+	    prn_to_clipboard(textprn, myfmt);
+	    gretl_print_destroy(textprn);
 	}
     }
 #else
-    else if (how == COPY_TEXT) {
-	PRN textprn;
+    else if (fmt == GRETL_FORMAT_TXT) {
+	char *buf;
+	PRN *textprn;
 
-	gretl_print_attach_buffer(&textprn, 
-				  gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 
-							 0, -1));
-	prn_to_clipboard(&textprn, 0);
-	g_free(textprn.buf);
+	buf = gtk_editable_get_chars(GTK_EDITABLE(vwin->w), 0, -1);
+	textprn = gretl_print_new_with_buffer(buf);
+	prn_to_clipboard(textprn, 0);
+	gretl_print_destroy(textprn);
     } else { /* COPY_SELECTION */
 	gtk_editable_copy_clipboard(GTK_EDITABLE(vwin->w));
 	return;
@@ -622,9 +578,10 @@ void text_copy (gpointer data, guint how, GtkWidget *w)
 
     if (w != NULL) {
 	msg = g_strdup_printf(_("Copied contents of window as %s"),
-			      (how == COPY_LATEX)? "LaTeX" :
-			      (how == COPY_RTF || how == COPY_TEXT_AS_RTF)? 
-			      "RTF" : _("plain text"));
+			      (fmt & GRETL_FORMAT_TEX)? "LaTeX" :
+			      (fmt & GRETL_FORMAT_RTF)? "RTF" : 
+			      (fmt & GRETL_FORMAT_RTF_TXT)? "RTF" :
+			      _("plain text"));
 	infobox(msg);
 	g_free(msg);
     }
@@ -652,10 +609,11 @@ void window_print (windata_t *vwin, guint u, GtkWidget *widget)
     GtkEditable *gedit = GTK_EDITABLE(vwin->w);
 
     buf = gtk_editable_get_chars(gedit, 0, -1);
-    if (gedit->has_selection)
+    if (gedit->has_selection) {
 	selbuf = gtk_editable_get_chars(gedit, 
 					gedit->selection_start_pos,
 					gedit->selection_end_pos);
+    }
 # endif /* OLD_GTK */
 
     winprint(buf, selbuf);
@@ -694,6 +652,7 @@ void system_print_buf (const gchar *buf, FILE *fp)
 char *dosify_buffer (const char *buf, int format)
 {
     int extra = 0, nlines = 0;
+    int rtf = (format == GRETL_FORMAT_RTF_TXT);
     char *targ, *q;
     const char *p;
     const char *rtf_preamble = "{\\rtf1\n"
@@ -711,7 +670,7 @@ char *dosify_buffer (const char *buf, int format)
     }
     extra = nlines + 1;
 
-    if (format == COPY_TEXT_AS_RTF) {
+    if (rtf) {
 	extra *= 5;
 	extra += rtf_add_bytes;
     }
@@ -721,7 +680,7 @@ char *dosify_buffer (const char *buf, int format)
 	return NULL;
     }
 
-    if (format == COPY_TEXT_AS_RTF) {
+    if (rtf) {
 	strcpy(targ, rtf_preamble);
 	q = targ + strlen(targ);
     } else {
@@ -731,7 +690,7 @@ char *dosify_buffer (const char *buf, int format)
     p = buf;
     while (*p) {
 	if (*p == '\n') {
-	    if (format == COPY_TEXT_AS_RTF) {
+	    if (rtf) {
 		*q++ = '\\';
 		*q++ = 'p';
 		*q++ = 'a';
@@ -746,7 +705,7 @@ char *dosify_buffer (const char *buf, int format)
     } 
     *q = 0;
 
-    if (format == COPY_TEXT_AS_RTF) {
+    if (rtf) {
 	strcat(q, "}\n");
     }
 
