@@ -22,15 +22,15 @@
 #include <stdio.h>
 #include "gretl_matrix.h"
 
-enum series_save_opts {
+typedef enum {
     SAVE_LEVERAGE  = 1 << 0,
     SAVE_INFLUENCE = 1 << 1,
     SAVE_DFFITS    = 1 << 2,
     SAVE_TREND     = 1 << 3,
     SAVE_CYCLE     = 1 << 4
-};
+} SeriesSaveCode;
 
-enum aux_codes {
+typedef enum {
     AUX_NONE,  /* not an auxiliary regression */
     AUX_SQ,    /* aux. regression for nonlinearity (squared terms) */
     AUX_LOG,   /* aux. regression for nonlinearity (log terms) */
@@ -51,38 +51,37 @@ enum aux_codes {
     AUX_JOHANSEN,  /* Johansen cointegration test */
     AUX_GROUPWISE, /* testing for groupwise heteroskedasticity */
     AUX_AUX    /* auxiliary regression not otherwise specified */
-};
+} ModelAuxCode;
 
 /* functions follow */
  
 double robust_omit_F (const int *list, MODEL *pmod);
 
-int add_test (int *addvars,  MODEL *orig, MODEL *new,
+int add_test (const int *addvars,  MODEL *orig, MODEL *new,
 	      double ***pZ, DATAINFO *pdinfo, 
 	      gretlopt opt, PRN *prn);
 
-int omit_test (int *omitvars, MODEL *orig, MODEL *new, 
+int omit_test (const int *omitvars, MODEL *orig, MODEL *new, 
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
 int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
-		       int aux_code, gretlopt opt, PRN *prn, 
-		       GRETLTEST *test);
+		       int aux_code, gretlopt opt, PRN *prn); 
 
 int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
-		PRN *prn, GRETLTEST *test);
+		gretlopt opt, PRN *prn);
 
 int autocorr_test (MODEL *pmod, int order,
 		   double ***pZ, DATAINFO *pdinfo, 
-		   PRN *prn, GRETLTEST *test);
+		   gretlopt opt, PRN *prn);
 
 int chow_test (const char *line, MODEL *pmod, 
 	       double ***pZ, DATAINFO *pdinfo, 
-	       PRN *prn, GRETLTEST *test);
+	       gretlopt opt, PRN *prn);
 
 int cusum_test (MODEL *pmod, 
 		double ***pZ, DATAINFO *pdinfo, 
-		PRN *prn, GRETLTEST *test);
+		gretlopt opt, PRN *prn);
 
 int hausman_test (MODEL *pmod, 
 		  double ***pZ, DATAINFO *pdinfo, 
