@@ -19,10 +19,12 @@
 
 /* dbread.c for gretl */
 
-#include <glib.h>
-
 #include "libgretl.h"
 
+#ifdef USE_GTK2
+# include <glib.h>
+#endif
+
 #undef DB_DEBUG
 
 #define RECNUM long
@@ -111,6 +113,9 @@ int get_native_db_data (const char *dbbase, SERIESINFO *sinfo,
 	fread(&val, sizeof val, 1, fp);
 	sprintf(numstr, "%g", val);
 	Z[1][t] = atof(numstr);
+	if (Z[1][t] == DBNA) {
+	    Z[1][t] = NADBL;
+	}
     }
 
     fclose(fp);
@@ -135,11 +140,13 @@ static void get_native_series_comment (SERIESINFO *sinfo, const char *s)
     n = strlen(sinfo->descrip) - 1;
     
     for (i=n; i>0; i--) {
-	if (isspace(sinfo->descrip[i])) sinfo->descrip[i] = 0;
-	else if (sinfo->descrip[i] == '\r' || sinfo->descrip[i] == '\n') {
+	if (isspace(sinfo->descrip[i])) {
 	    sinfo->descrip[i] = 0;
+	} else if (sinfo->descrip[i] == '\r' || sinfo->descrip[i] == '\n') {
+	    sinfo->descrip[i] = 0;
+	} else {
+	    break;
 	}
-	else break;
     }
 }
 
@@ -214,21 +221,18 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
 
     while (!gotit) {
 
-	if (fgets(line1, 255, fp) == NULL) break;
+	if (fgets(line1, sizeof line1, fp) == NULL) break;
 
 	if (*line1 == '#') continue;
-	line1[255] = 0;
 
 	if (sscanf(line1, "%8s", sername) != 1) break;
-	sername[8] = 0;
 
 	if (!strcmp(series, sername)) {
 	    gotit = 1;
 	    strcpy(sinfo->varname, sername);
 	}
 
-	fgets(line2, 71, fp);
-	line2[71] = 0;
+	fgets(line2, sizeof line2, fp);
 
 	if (gotit) {
 	    get_native_series_comment(sinfo, line1);
@@ -263,8 +267,6 @@ get_native_series_info (const char *series, SERIESINFO *sinfo)
     return err;
 }
 
-/* ........................................................... */
-
 /* Figure the ending observation date of a series */
 
 static int get_endobs (char *datestr, int startyr, int startfrac, 
@@ -337,12 +339,10 @@ static int dinfo_to_sinfo (const DATEINFO *dinfo, SERIESINFO *sinfo,
 	} else {
 	    startfrac = 4;
 	}
-    }
-    else if (dinfo->info == 12) {
+    } else if (dinfo->info == 12) {
 	sprintf(pdstr, ".%02d", dinfo->month);
 	startfrac = dinfo->month;
-    }
-    else if (dinfo->info == 1) {
+    } else if (dinfo->info == 1) {
 	startfrac = 0;
     } else {
 	fprintf(stderr, I_("frequency (%d) does not make seem to make sense"),
@@ -389,13 +389,11 @@ static int dinfo_to_tbl_row (const DATEINFO *dinfo, db_table_row *row,
 	else if (dinfo->month > 1 && dinfo->month <= 4) startfrac = 2;
 	else if (dinfo->month > 4 && dinfo->month <= 7) startfrac = 3;
 	else startfrac = 4;
-    }
-    else if (dinfo->info == 12) {
+    } else if (dinfo->info == 12) {
 	pd = 'M';
 	sprintf(pdstr, ".%02d", dinfo->month);
 	startfrac = dinfo->month;
-    }
-    else if (dinfo->info == 1) {
+    } else if (dinfo->info == 1) {
 	pd = 'A';
 	startfrac = 0;
     } else {
@@ -409,10 +407,17 @@ static int dinfo_to_tbl_row (const DATEINFO *dinfo, db_table_row *row,
 
     if (!err) {
 	get_endobs(endobs, dinfo->year, startfrac, dinfo->info, n);
-	row->varname = g_strdup(varname);
-	row->comment = g_strdup(comment);
+	row->varname = gretl_strdup(varname);
+	row->comment = gretl_strdup(comment);
+#ifdef USE_GTK2
 	row->obsinfo = g_strdup_printf("%c  %d%s - %s  n = %d", pd, 
 				       (int) dinfo->year, pdstr, endobs, n);
+#else
+	row->obsinfo = malloc(64);
+	sprintf(row->obsinfo, "%c  %d%s - %s  n = %d", pd, 
+		(int) dinfo->year, pdstr, endobs, n);
+
+#endif
     } 
 
     return err;
@@ -634,8 +639,6 @@ static int find_rats_dir_by_number (FILE *fp, int first_dir,
     return (int) forward;
 }
 
-/* ........................................................... */
-
 static int get_rats_series_offset_by_number (FILE *fp, 
 					     int series_number)
 {
@@ -654,11 +657,10 @@ static int get_rats_series_offset_by_number (FILE *fp,
     return offset;
 }
 
-/* ........................................................... */
+/* retrieve the actual data values from the data blocks */
 
 static int get_rats_series (int offset, SERIESINFO *sinfo, FILE *fp, 
 			    double **Z)
-/* retrieve the actual data values from the data blocks */
 {
     RATSData rdata;
     char numstr[16];
@@ -788,8 +790,6 @@ get_rats_series_info_by_name (const char *series_name,
     return ret;
 }
 
-/* ........................................................... */
-
 static int 
 get_rats_data_by_offset (const char *fname, 
 			 SERIESINFO *sinfo,
@@ -1213,7 +1213,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    pdinfo->n, pdinfo->v, (void *) pdinfo->varname);
 #endif
 
-    if (new && dataset_add_vars(1, pZ, pdinfo)) {
+    if (new && dataset_add_series(1, pZ, pdinfo)) {
 	strcpy(gretl_errmsg, _("Out of memory adding series"));
 	return 1;
     }
@@ -1226,7 +1226,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    sinfo->pd != 12) {
 	    strcpy(gretl_errmsg, _("Sorry, can't handle this conversion yet!"));
 	    if (new) {
-		dataset_drop_vars(1, pZ, pdinfo);
+		dataset_drop_last_variables(1, pZ, pdinfo);
 	    }
 	    return 1;
 	}
@@ -1234,7 +1234,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    sprintf(gretl_errmsg, _("%s: you must specify a compaction method"), 
 		    sinfo->varname);
 	    if (new) {
-		dataset_drop_vars(1, pZ, pdinfo);
+		dataset_drop_last_variables(1, pZ, pdinfo);
 	    }
 	    return 1;
 	}
@@ -1252,7 +1252,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     if (xvec == NULL) {
 	strcpy(gretl_errmsg, _("Out of memory adding series"));
 	if (new) {
-	    dataset_drop_vars(1, pZ, pdinfo);
+	    dataset_drop_last_variables(1, pZ, pdinfo);
 	}
 	return 1;
     }
@@ -1784,7 +1784,7 @@ static int dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 	pdinfo->t1 = 0;
 	pdinfo->t2 = pdinfo->n - 1;
 
-	destroy_dataset_markers(pdinfo);
+	dataset_destroy_obs_markers(pdinfo);
     }
 
     return err;
@@ -1812,7 +1812,7 @@ insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
 	return E_ALLOC;
     }
 
-    err = grow_nobs(nmiss, pZ, pdinfo);
+    err = dataset_add_observations(nmiss, pZ, pdinfo);
     if (err) {
 	free(tmp);
 	return err;
@@ -1841,7 +1841,7 @@ insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
     free(tmp);
 
     if (!err) {
-	destroy_dataset_markers(pdinfo);
+	dataset_destroy_obs_markers(pdinfo);
 	pdinfo->t2 = pdinfo->n - 1;
 	ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
     }
@@ -1957,7 +1957,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     
     if (oldpd >= 5 && oldpd <= 7) {
 	/* remove any daily date strings */
-	destroy_dataset_markers(pdinfo);
+	dataset_destroy_obs_markers(pdinfo);
     }
 
     /* compact the individual data series */
