@@ -17,11 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-typedef struct GRETLSUMMARY_ GRETLSUMMARY;
-typedef struct FREQDIST_ FREQDIST;
+typedef struct Summary_ Summary;
+typedef struct FreqDist_ FreqDist;
+typedef struct MahalDist_ MahalDist;
 
-struct GRETLSUMMARY_ {
+struct Summary_ {
     int n;
+    int missing;
     int *list;
     double *mean;
     double *median;
@@ -33,7 +35,7 @@ struct GRETLSUMMARY_ {
     double *cv;
 };
 
-struct FREQDIST_ {
+struct FreqDist_ {
     char varname[VNAMELEN];  /* for ID purposes */
     int dist;                /* code for theoretical distribution */
     int numbins;             /* number of bins or intervals */
@@ -43,17 +45,40 @@ struct FREQDIST_ {
     int *f;                  /* frequencies in the intervals */
     double test;             /* either Chi-squared statistic for testing
                                 for a Gaussian distribution, or z statistic
-			        for testing for Gamma dist.
-			     */
+			        for testing for Gamma dist. */
     int n;
     int t1, t2;
 };
 
 /* functions follow */
 
-void free_freq (FREQDIST *freq);
+int gretl_minmax (int t1, int t2, const double *x, 
+		  double *min, double *max);
 
-FREQDIST *get_freq (int varno, const double **Z, const DATAINFO *pdinfo, 
+double gretl_mean (int t1, int t2, const double *x);
+
+double gretl_median (int t1, int t2, const double *x);
+
+double gretl_sst (int t1, int t2, const double *x);
+
+double gretl_variance (int t1, int t2, const double *x);
+
+double gretl_stddev (int t1, int t2, const double *x);
+
+double gretl_covar (int t1, int t2, const double *x, const double *y);
+
+double gretl_corr (int t1, int t2, const double *x, const double *y,
+		   int *missing);
+
+double gretl_corr_rsq (int t1, int t2, const double *x, const double *y);
+
+int gretl_moments (int t1, int t2, const double *x, 
+		   double *xbar, double *sd, 
+		   double *skew, double *kurt, int k);
+
+void free_freq (FreqDist *freq);
+
+FreqDist *get_freq (int varno, const double **Z, const DATAINFO *pdinfo, 
 		    int params, gretlopt opt);
 
 int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
@@ -62,6 +87,8 @@ int freqdist (int varno, const double **Z, const DATAINFO *pdinfo,
 int model_error_dist (const MODEL *pmod, double ***pZ,
 		      DATAINFO *pdinfo, PRN *prn);
 
+int auto_acf_order (int pd, int nobs);
+
 int corrgram (int varno, int order, 
 	      double ***pZ, DATAINFO *pdinfo, 
 	      int batch, PRN *prn);
@@ -70,19 +97,21 @@ int periodogram (int varno,
 		 double ***pZ, const DATAINFO *pdinfo, 
 		 int batch, int opt, PRN *prn);
 
-GRETLSUMMARY *summary (const int *list, const double **Z, 
+Summary *summary (const int *list, const double **Z, 
 		       const DATAINFO *pdinfo,
 		       PRN *prn);
 
-void print_summary (const GRETLSUMMARY *summ,
+void print_summary (const Summary *summ,
 		    const DATAINFO *pdinfo,
 		    PRN *prn); 
 
-void free_summary (GRETLSUMMARY *summ);
+void free_summary (Summary *summ);
 
-CORRMAT *corrlist (int *list, const double **Z, const DATAINFO *pdinfo);
+VMatrix *corrlist (int *list, const double **Z, const DATAINFO *pdinfo);
 
-void free_corrmat (CORRMAT *corrmat);
+VMatrix *vmatrix_new (void);
+
+void free_vmatrix (VMatrix *vmat);
 
 int gretl_corrmx (int *list, const double **Z, const DATAINFO *pdinfo, 
 		  PRN *prn);
@@ -94,14 +123,7 @@ int means_test (const int *list, const double **Z,
 int vars_test (const int *list, const double **Z, 
 	       const DATAINFO *pdinfo, PRN *prn);
 
-void matrix_print_corr (CORRMAT *corr, const DATAINFO *pdinfo,
-			PRN *prn);
-
-double gretl_median (const double *x, int n);
-
-int moments (int t1, int t2, const double *zx, 
-	     double *xbar, double *std, 
-	     double *skew, double *kurt, int k);
+void matrix_print_corr (VMatrix *corr, const DATAINFO *pdinfo, PRN *prn);
 
 double doornik_chisq (double skew, double kurt, int n);
 
@@ -109,5 +131,12 @@ int mahalanobis_distance (const int *list, double ***pZ,
 			  DATAINFO *pdinfo, gretlopt opt,
 			  PRN *prn);
 
-char *unique_savename (char *vname, DATAINFO *pdinfo, int vmax);
+MahalDist *get_mahal_distances (const int *list, double ***pZ,
+				DATAINFO *pdinfo, gretlopt opt,
+				PRN *prn);
 
+void free_mahal_dist (MahalDist *md);
+
+const double *mahal_dist_get_distances (const MahalDist *md);
+int mahal_dist_get_n (const MahalDist *md);
+const int *mahal_dist_get_varlist(const MahalDist *md);
