@@ -20,7 +20,6 @@
 /* discrete.c for gretl: logit and probit models */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #define TINY 1.0e-13
 
@@ -139,30 +138,6 @@ static int add_slopes_to_model (MODEL *pmod, double fbx)
 
 /* .......................................................... */
 
-int dmod_isdummy (const double *x, int t1, int t2)
-{
-    int t, m = 0, goodobs = 0;
-
-    for (t=t1; t<=t2; t++) {
-	if (na(x[t])) {
-	    continue;
-	}
-	if (x[t] != 0.0 && x[t] != 1.0) {
-	    return 0;
-	}
-	if (x[t] == 1.0) {
-	    m++;
-	}
-	goodobs++;
-    }
-
-    if (m < goodobs) return m;
-
-    return 0;
-} 
-
-/* .......................................................... */
-
 static double *hess_wts (MODEL *pmod, const double **Z, int opt) 
 {
     int t, tw, n = pmod->t2 - pmod->t1 + 1;
@@ -257,17 +232,18 @@ static double *hessian (MODEL *pmod, const double **Z, int opt)
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, int opt)
+MODEL logit_probit (const int *list, double ***pZ, DATAINFO *pdinfo, int opt)
 {
-    int i, t, v, misst, depvar = list[1];
+    int i, t, v, depvar = list[1];
     int oldt1 = pdinfo->t1;
     int oldt2 = pdinfo->t2;
     int oldv = pdinfo->v;
     int itermax = 250;
     double tol = 1.0e-9; /* ? */
     int *dmodlist = NULL;
+    int *act_pred = NULL;
     MODEL dmod;
-    int dummy, n_correct;
+    int dummy;
     double xx, zz, fx, Fx, fbx;
     double lldiff, llbak;
     int iters;
@@ -279,7 +255,7 @@ MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, int opt)
     gretl_model_init(&dmod);
     
     /* check whether depvar is binary */
-    dummy = dmod_isdummy((*pZ)[depvar], pdinfo->t1, pdinfo->t2);
+    dummy = gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[depvar]);
     if (!dummy) {
 	dmod.errcode = E_UNSPEC;
 	sprintf(gretl_errmsg, _("The dependent variable '%s' is not a 0/1 "
@@ -307,16 +283,24 @@ MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, int opt)
 	goto bailout;
     }
 
+    /* space for actual/predicated matrix */
+    act_pred = malloc(4 * sizeof *act_pred);
+    if (act_pred != NULL) {
+	for (i=0; i<4; i++) {
+	    act_pred[i] = 0;
+	}
+    }
+
     /* make room for full set of transformed vars */
-    if (dataset_add_vars(list[0], pZ, pdinfo)) {
+    if (dataset_add_series(list[0], pZ, pdinfo)) {
 	dmod.errcode = E_ALLOC;
 	return dmod;
     }
 
     v = oldv; /* the first newly created variable */
 
-    adjust_t1t2(NULL, list, &pdinfo->t1, &pdinfo->t2, 
-		(const double **) *pZ, &misst);
+    varlist_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
+			  (const double **) *pZ);
 
     dmod = lsq(list, pZ, pdinfo, OLS, OPT_A, 0);
     if (dmod.ifc == 0) {
@@ -507,21 +491,36 @@ MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, int opt)
 
     /* calculate additional statistics */
     xx = 0.0;
-    n_correct = 0;
     for (t=dmod.t1; t<=dmod.t2; t++) {
+	double xb = dmod.yhat[t];
+
 	if (model_missing(&dmod, t)) {
 	    continue;
 	}
 	zz = (*pZ)[depvar][t];
 	xx += zz;
-	n_correct += ((dmod.yhat[t] > 0.0 && floateq(zz, 1.0)) ||
-		      (dmod.yhat[t] <= 0.0 && floateq(zz, 0.0)));
+
+	if (act_pred != NULL) {
+	    i = 2 * (floateq(zz, 1.0)) + (xb > 0.0);
+	    act_pred[i] += 1;
+	}
+
+	if (dmod.ci == LOGIT) {
+	   dmod.yhat[t] = exp(xb) / (1.0 + exp(xb)); 
+	} else {
+	   dmod.yhat[t] = normal_cdf(xb); 
+	}
+	dmod.uhat[t] = zz - dmod.yhat[t];
     }
 
     xx /= dmod.nobs;
     dmod.ybar = xx;
     dmod.sdy = fbx;
-    gretl_model_set_int(&dmod, "correct", n_correct);
+
+    if (act_pred != NULL) {
+	gretl_model_set_data(&dmod, "discrete_act_pred", act_pred, 
+			     4 * sizeof *act_pred);
+    }
 
     mle_aic_bic(&dmod, 0);
 
@@ -536,7 +535,7 @@ MODEL logit_probit (int *list, double ***pZ, DATAINFO *pdinfo, int opt)
     pdinfo->t1 = oldt1;
     pdinfo->t2 = oldt2;
 
-    dataset_drop_vars(pdinfo->v - oldv, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
 
     return dmod;
 }
