@@ -23,39 +23,33 @@
 
 /* functions follow */
 
-MODEL lsq (int *list, 
-	   double ***pZ, DATAINFO *pdinfo, 
-	   int ci, gretlopt opts, double rho);
+MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
+	   GretlCmdIndex ci, gretlopt opt, double rho);
 
-double estimate_rho (int *list, double ***pZ, DATAINFO *pdinfo,
-		     int batch, int opt, int *err, PRN *prn);
+double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
+		     GretlCmdIndex ci, int *err, gretlopt opt, PRN *prn);
 
-int hilu_corc (double *toprho, int *list, 
-	       double ***pZ, DATAINFO *pdinfo,
-	       PATHS *ppaths, int batch,
-	       int opt, PRN *prn);
+MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo); 
 
-MODEL lad (int *list, double ***pZ, DATAINFO *pdinfo); 
+MODEL arma (const int *list, const double **Z, DATAINFO *pdinfo, 
+	    gretlopt opt, PRN *prn);
 
-MODEL arma (int *list, const double **Z, DATAINFO *pdinfo, 
-	    PRN *prn);
+MODEL arma_x12 (const int *list, const double **Z, DATAINFO *pdinfo, 
+		const PATHS *ppaths, gretlopt opt, PRN *prn);
 
-MODEL arma_x12 (int *list, const double **Z, DATAINFO *pdinfo, 
-		PRN *prn, const PATHS *ppaths);
-
-MODEL logistic_model (int *list, double ***pZ, DATAINFO *pdinfo,
+MODEL logistic_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 		      const char *param);
 
-MODEL tobit_model (int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		   PRN *prn);
 
-MODEL poisson_model (int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL poisson_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
 		     PRN *prn);
 
-MODEL garch (int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
+MODEL garch (const int *list, double ***pZ, DATAINFO *pdinfo, gretlopt opt,
 	     PRN *prn);
 
-MODEL pooled (int *list, double ***pZ, DATAINFO *pdinfo,
+MODEL pooled (const int *list, double ***pZ, DATAINFO *pdinfo,
 	      gretlopt opt, PRN *prn);
 
 int groupwise_hetero_test (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
@@ -65,27 +59,31 @@ double *tsls_get_Xi (const MODEL *pmod, double **Z, int i);
 
 void tsls_free_data (const MODEL *pmod);
 
-MODEL tsls_func (int *list, int pos_in, 
+MODEL tsls_func (const int *list, int pos_in, 
 		 double ***pZ, DATAINFO *pdinfo,
 		 gretlopt opt);
 
-MODEL hsk_func (int *list, double ***pZ, DATAINFO *pdinfo);
+MODEL hsk_func (const int *list, double ***pZ, DATAINFO *pdinfo);
 
 int whites_test (MODEL *pmod, 
 		 double ***pZ, DATAINFO *pdinfo, 
-		 PRN *prn, GRETLTEST *test);
+		 gretlopt opt, PRN *prn);
 
-MODEL ar_func (int *list, int pos, 
+MODEL ar_func (const int *list, int pos, 
 	       double ***pZ, DATAINFO *pdinfo, 
 	       gretlopt opt, PRN *prn);
 
-MODEL arch (int order, int *list, 
-	    double ***pZ, DATAINFO *pdinfo, 
-	    GRETLTEST *test, gretlopt opt, PRN *prn);
+MODEL arch_test (MODEL *pmod, int order, 
+		 double ***pZ, DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn);
+
+MODEL arch_model (const int *list, int order, 
+		  double ***pZ, DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn);
 
 int makevcv (MODEL *pmod);
 
-VCV *get_vcv (MODEL *pmod);
+int *augment_regression_list (const int *orig, int aux, 
+			      double ***pZ, DATAINFO *pdinfo);
 
-void free_vcv (VCV *vcv);
 
