@@ -20,8 +20,8 @@
 #ifndef GENERATE_H
 #define GENERATE_H
 
-int generate (double ***pZ, DATAINFO *pdinfo, 
-	      const char *line, MODEL *pmod); 
+int generate (const char *line, double ***pZ, DATAINFO *pdinfo, 
+	      MODEL *pmod, gretlopt opt); 
 
 int genrtime (double ***pZ, DATAINFO *pdinfo, int tm);
 
@@ -40,6 +40,8 @@ int plotvar (double ***pZ, DATAINFO *pdinfo,
 
 void varlist (const DATAINFO *pdinfo, PRN *prn);
 
+void maybe_list_vars (const DATAINFO *pdinfo, PRN *prn);
+
 int varindex (const DATAINFO *pdinfo, const char *varname);
 
 int genr_fit_resid (const MODEL *pmod, 
@@ -50,5 +52,12 @@ int get_generated_value (const char *argv, double *val,
 			 double ***pZ, DATAINFO *pdinfo,
 			 MODEL *pmod, int t);
 
+int gretl_reserved_word (const char *str);
+
+int genr_function_from_string (const char *s);
+
+int get_t_from_obs_string (char *s, const double **Z, 
+			   const DATAINFO *pdinfo);
+
 #endif /* GENERATE_H */
 
