@@ -56,8 +56,10 @@ struct atomset_ {
 #define must_know_children(f) (f == T_NOBS || f == T_MEAN || f == T_SUM || \
                                f == T_SD || f == T_VAR || f == T_SST || \
                                f == T_MEDIAN || f == T_MIN || f == T_MAX || \
-                               f == T_DIFF || f == T_LDIFF || \
-                               f == T_CUM || f == T_SORT || f == T_VARNUM || \
+                               f == T_DIFF || f == T_LDIFF || f == T_T1 || f == T_T2 || \
+                               f == T_CUM || f == T_SORT || \
+                               f == T_VARNUM || f == T_VECTOR || \
+                               f == T_ISLIST || f == T_NELEM || \
                                f == T_RESAMPLE || f == T_HPFILT || f == T_BKFILT)
 
 static int all_children_scalar (genatom **atoms, int n,
@@ -374,7 +376,7 @@ void reset_calc_stack (GENERATE *genr)
     calc_stack(0., STACK_RESET, genr);
 }
 
-#ifdef GENR_DEBUG
+#if GENR_DEBUG
 # include <stdarg.h>
 void dprintf (const char *format, ...)
 {
@@ -386,7 +388,4 @@ void dprintf (const char *format, ...)
 
    return;
 }
-# define DPRINTF(x) dprintf x
-#else 
-# define DPRINTF(x)
 #endif /* GENR_DEBUG */
