@@ -20,20 +20,21 @@
 /* graphing.c for gretl */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 #include "var.h"
 #include "libset.h"
 #include "../../cephes/libprob.h"
 
 #include <unistd.h>
 
+#undef GP_DEBUG
+
 #ifdef _WIN32
 # include <windows.h>
 # define EMF_USER_COLORS
 #else
-# include <glib.h>
-# if GLIB_CHECK_VERSION(2,0,0)
+# ifdef USE_GTK2
 #  define GLIB2
+#  include <glib.h>
 #  include <signal.h>
 #  if HAVE_SYS_WAIT_H
 #   include <sys/wait.h>
@@ -44,7 +45,7 @@
 #  ifndef WIFEXITED
 #   define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #  endif
-# endif /* GLIB_CHECK_VERSION */
+# endif /* GTK2 */
 #endif /* ! _WIN32 */
 
 
@@ -56,7 +57,7 @@ struct gnuplot_info {
     unsigned char flags;
     int ts_plot;
     int yscale;
-    int pdist;
+    int impulses;
     int lo;
     int ols_ok;
     int t1;
@@ -74,6 +75,17 @@ struct gnuplot_info {
 
 #undef SPAWN_DEBUG
 
+/**
+ * gnuplot_test_command:
+ * @cmd: gnuplot command string.
+ * 
+ * See if the installed version of gnuplot will accept a given
+ * command.
+ *
+ * Returns: 0 if gnuplot successfully handles the given command,
+ * 1 on error.
+ */
+
 int gnuplot_test_command (const char *cmd)
 {
     int ok, ret = 1;
@@ -136,6 +148,10 @@ int gnuplot_test_command (const char *cmd)
 	g_error_free(error);
     }
 
+#ifdef SPAWN_DEBUG
+    fprintf(stderr, "gnuplot test: ret = %d\n", ret);
+#endif
+
     return ret;
 }
 
@@ -159,8 +175,6 @@ int gnuplot_test_command (const char *cmd)
 
 #endif /* ! GLIB2, ! WIN32 */
 
-/* ........................................................ */
-
 static int printvars (FILE *fp, int t, const int *list, const double **Z,
 		      const char *label, double offset)
 {
@@ -189,7 +203,16 @@ static int printvars (FILE *fp, int t, const int *list, const double **Z,
     return miss;
 }
 
-/* ........................................................... */
+/**
+ * get_timevar_name:
+ * @pdinfo: information on dataset.
+ *
+ * Based on the properties and frequency of the dataset, 
+ * figure out the appropriate (internal) name for the variable 
+ * to be used on the x-axis in a time-series plot.
+ *
+ * Returns: the name.
+ */
 
 const char *get_timevar_name (DATAINFO *pdinfo)
 {
@@ -203,7 +226,8 @@ const char *get_timevar_name (DATAINFO *pdinfo)
 	return "months";
     } else if (pdinfo->pd == 24) {
 	return "hrs";
-    } else if (calendar_data(pdinfo)) {
+    } else if (calendar_data(pdinfo) || 
+	       dataset_is_decennial(pdinfo)) {
 	return "decdate";
     } else {
 	return "time";
@@ -296,6 +320,7 @@ int gnuplot_has_ttf (void)
 	    err = gnuplot_test_command("set term png font arial 8");
 	}
     }
+
     return !err;
 }
 
@@ -308,6 +333,7 @@ int gnuplot_has_specified_colors (void)
 	   if it fails, we have the new driver, we hope */
 	err = gnuplot_test_command("set term png color");
     }
+
     return err;
 }
 
@@ -318,6 +344,7 @@ static int gnuplot_has_style_fill (void)
     if (err == -1) {
 	err = gnuplot_test_command("set style fill solid");
     }
+
     return !err;
 }
 
@@ -329,8 +356,11 @@ static char *label_front (void)
 	err = gnuplot_test_command("set label 'foo' at 0,0 front");
     }
 
-    if (err) return "";
-    else return " front";
+    if (err) {
+	return "";
+    } else {
+	return " front";
+    }
 }
 
 
@@ -338,17 +368,18 @@ static char *label_front (void)
 
 /**
  * get_gretl_png_term_line:
- * @plottype: 
+ * @ptype: indication of the sort of plot to be made, which
+ * may made a difference to the color palette chosen.
  *
  * Constructs a suitable line for sending to gnuplot to invoke
  * the PNG "terminal".  Checks the environment for setting of 
- * GRETL_PNG_GRAPH_FONT. Also appends a color-specification string 
+ * %GRETL_PNG_GRAPH_FONT.  Also appends a color-specification string 
  * if the gnuplot PNG driver supports this.
  *
- * Returns: the term string, "set term png ..."
+ * Returns: the terminal string, "set term png ..."
  */
 
-const char *get_gretl_png_term_line (int plottype)
+const char *get_gretl_png_term_line (PlotType ptype)
 {
     static char png_term_line[256];
     char font_string[128];
@@ -382,7 +413,7 @@ const char *get_gretl_png_term_line (int plottype)
 	strcpy(color_string, " xffffff x000000 x202020");
 	for (i=0; i<3; i++) {
 	    strcat(color_string, " ");
-	    strcat(color_string, get_gnuplot_pallette(i, plottype));
+	    strcat(color_string, get_gnuplot_pallette(i, ptype));
 	}
     } else {
 	strcpy(color_string, " color"); /* old PNG driver */
@@ -391,7 +422,7 @@ const char *get_gretl_png_term_line (int plottype)
     sprintf(png_term_line, "set term png%s%s",
 	    font_string, color_string);
 
-#ifdef GNUPLOT_DEBUG
+#if GP_DEBUG
     fprintf(stderr, "png term line:\n'%s'\n", png_term_line);
 #endif
 
@@ -400,7 +431,8 @@ const char *get_gretl_png_term_line (int plottype)
 
 /**
  * get_gretl_emf_term_line:
- * @plottype: 
+ * @ptype: indication of the sort of plot to be made.
+ * @color: 1 if graph is to be in color, else 0.
  *
  * Constructs a suitable line for sending to gnuplot to invoke
  * the EMF "terminal".  
@@ -408,7 +440,7 @@ const char *get_gretl_png_term_line (int plottype)
  * Returns: the term string, "set term emf ..."
  */
 
-const char *get_gretl_emf_term_line (int plottype, int color)
+const char *get_gretl_emf_term_line (PlotType ptype, int color)
 {
     static char emf_term_line[256];
 
@@ -418,7 +450,7 @@ const char *get_gretl_emf_term_line (int plottype, int color)
     }
 
 #ifdef EMF_USER_COLORS
-    if (frequency_plot_code(plottype)) {
+    if (frequency_plot_code(ptype)) {
 	sprintf(emf_term_line, "set term emf color %s",
 		get_gnuplot_pallette(0, PLOT_FREQ_SIMPLE));
     } else {
@@ -443,20 +475,27 @@ const char *get_gretl_emf_term_line (int plottype, int color)
 
 /**
  * gnuplot_init:
- * @plottype: code for the type of plot.
+ * @ptype: indication of the sort of plot to be made.
  * @fpp: pointer to stream to be opened.
  *
- * If we're in GUI mode, writes a unique temporary filename into
- * gretl_plotfile; opens plotfile for writing as @fpp; if
- * we're in GUI mode, writes PNG terminal type into plotfile.
+ * If we're in GUI mode: writes a unique temporary filename into
+ * the internal variable #gretl_plotfile; opens plotfile for writing 
+ * as @fpp; and writes initial lines into the output file to select 
+ * the PNG terminal type, and direct gnuplot's ouput to a temporary
+ * file in the gretl user directory.  
  *
- * Returns: 0 on success, 1 on failure
+ * If not in GUI mode, opens as @fpp the file %gpttmp.plt in the
+ * gretl user directory.  
+ *
+ * This function is not used in batch mode.
+ *
+ * Returns: 0 on success, 1 on failure.
  */
 
-int gnuplot_init (int plottype, FILE **fpp)
+int gnuplot_init (PlotType ptype, FILE **fpp)
 {
     int gui = gretl_in_gui_mode();
-    char plotfile[MAXLEN];
+    char plotfile[MAXLEN] = {0};
 
     if (gretl_looping()) {
 	return E_OK;
@@ -480,17 +519,81 @@ int gnuplot_init (int plottype, FILE **fpp)
 
     *fpp = gretl_fopen(plotfile, "w");
     if (*fpp == NULL) {
+	fprintf(stderr, "gnuplot_init: couldn't write to %s\n", plotfile);
 	return E_FOPEN;
-    }
+    } 
 
     if (gui) {
-	fprintf(*fpp, "%s\n", get_gretl_png_term_line(plottype));
+	fprintf(*fpp, "%s\n", get_gretl_png_term_line(ptype));
 	fprintf(*fpp, "set output '%sgretltmp.png'\n", gretl_user_dir());
     }
 
+#if GP_DEBUG
+    fprintf(stderr, "gnuplot_init: set plotfile = '%s'\n", 
+	    plotfile);
+#endif
+
     return 0;
 }
 
+#ifdef ENABLE_NLS
+
+#undef RECODE_DBG
+
+static int recode_gnuplot_file (const char *fname)
+{
+    FILE *fp, *fq;
+    const char *font;
+    char oldline[512], newline[1024];
+    char rname[FILENAME_MAX];
+    int ttf = 0;
+
+    fp = fopen(fname, "r");
+    if (fp == NULL) {
+	return 1;
+    }
+
+    strcpy(rname, fname);
+    strcat(rname, "l2");
+
+    fq = fopen(rname, "w");
+    if (fq == NULL) {
+	fclose(fp);
+	return 1;
+    }
+
+    font = gretl_png_font();
+    if (font != NULL && *font != '\0') {
+	ttf = 1;
+    }
+
+    while (fgets(oldline, sizeof oldline, fp)) {
+	if (isdigit((unsigned char) oldline[0])) {
+	    fputs(oldline, fq);
+	} else if (ttf) {
+	    sprint_l2_to_html(newline, oldline, sizeof newline);
+	    fputs(newline, fq);
+#if RECODE_DBG
+	    fprintf(stderr, "recode (sprint_l2_to_html):\n"
+		    " original: '%s'\n modified: '%s'\n",
+		    oldline, newline);
+#endif
+	} else {
+	    fputs(oldline, fq);
+	}
+    }
+
+    fclose(fp);
+    fclose(fq);
+	    
+    remove(fname);
+    rename(rname, fname);
+
+    return 0;
+}
+
+#endif
+
 /**
  * gnuplot_make_graph:
  *
@@ -504,6 +607,15 @@ int gnuplot_make_graph (void)
     int err = 0;
     char plotcmd[MAXLEN];
 
+#ifdef ENABLE_NLS  
+    if (use_latin_2()) {
+# if GP_DEBUG
+	fprintf(stderr, "gnuplot_make_graph: calling recode_gnuplot_file()\n");
+# endif
+	recode_gnuplot_file(gretl_plotfile());
+    } 
+#endif
+
 #ifdef WIN32
     sprintf(plotcmd, "\"%s\" \"%s\"", gretl_gnuplot_path(), gretl_plotfile());
     err = winfork(plotcmd, NULL, SW_SHOWMINIMIZED, 0);
@@ -513,7 +625,7 @@ int gnuplot_make_graph (void)
     err = gretl_spawn(plotcmd);  
 #endif
 
-#if 0
+#if GP_DEBUG
     fprintf(stderr, "gnuplot_make_graph:\n"
 	    " plotcmd='%s', err = %d\n", plotcmd, err);
 #endif
@@ -532,6 +644,7 @@ static int auto_plot_var (const char *s)
 {
     if (strcmp(s, "t") == 0 ||
 	strcmp(s, "annual") == 0 ||
+	strcmp(s, "decdate") == 0 ||
 	strcmp(s, "qtrs") == 0 ||
 	strcmp(s, "months") == 0 ||
 	strcmp(s, "hrs") == 0) {
@@ -544,6 +657,7 @@ static int auto_plot_var (const char *s)
 static void make_gtitle (FILE *fp, int code, const char *s1, const char *s2)
 {
     char title[128];
+    char depvar[9];
 
     switch (code) {
     case GTITLE_VLS:
@@ -551,7 +665,10 @@ static void make_gtitle (FILE *fp, int code, const char *s1, const char *s2)
 		s1, s2);
 	break;
     case GTITLE_RESID:
-	sprintf(title, I_("Regression residuals (= observed - fitted %s)"), s1);
+	if (sscanf(s1, "residual for %8s", depvar) == 1) {
+	    sprintf(title, I_("Regression residuals (= observed - fitted %s)"), 
+		    depvar);
+	}
 	break;
     case GTITLE_AF:
 	sprintf(title, I_("Actual and fitted %s"), s1);
@@ -647,13 +764,15 @@ get_gnuplot_output_file (FILE **fpp, unsigned char flags,
 	if (*plotfile == '\0' || strstr(plotfile, "gpttmp") != NULL) {
 	    *plot_count += 1; 
 	    sprintf(fname, "%sgpttmp%02d.plt", gretl_user_dir(), *plot_count);
-	}
-	set_gretl_plotfile(fname);
-	*fpp = gretl_fopen(fname, "w");
+	    set_gretl_plotfile(fname);
+	} 
+	plotfile = gretl_plotfile();
+	*fpp = gretl_fopen(plotfile, "w");
 	if (*fpp == NULL) {
 	    err = E_FOPEN;
 	}
     } else {
+	/* note: gnuplot_init not used in batch mode */
 	err = gnuplot_init(code, fpp);
     }
 
@@ -668,6 +787,10 @@ get_ols_line (struct gnuplot_info *gpinfo, const int *list,
     int olslist[4];
     int err = 0;
 
+#if GP_DEBUG
+    fprintf(stderr, "gnuplot: doing get_ols_line\n");
+#endif
+
     olslist[0] = 3;
     olslist[1] = list[1];
     olslist[2] = 0;
@@ -682,7 +805,7 @@ get_ols_line (struct gnuplot_info *gpinfo, const int *list,
 
     if (!err) {
 	/* is the fit significant? */
-	if (tprob(plotmod.coeff[1] / plotmod.sderr[1], plotmod.dfd) < .10) {
+	if (t_pvalue_2(plotmod.coeff[1] / plotmod.sderr[1], plotmod.dfd) < .10) {
 	    sprintf(ols_line, "%g + %g*x title '%s' w lines\n", 
 		    plotmod.coeff[0], plotmod.coeff[1], 
 		    I_("least squares fit"));
@@ -702,8 +825,8 @@ get_ols_line (struct gnuplot_info *gpinfo, const int *list,
 static void 
 print_x_range (struct gnuplot_info *gpinfo, const double *x, FILE *fp)
 {
-    if (isdummy(x, gpinfo->t1, gpinfo->t2)) {
-	fputs("set xrange[-1:2]\n", fp);	
+    if (gretl_isdummy(gpinfo->t1, gpinfo->t2, x)) {
+	fputs("set xrange [-1:2]\n", fp);	
 	fputs("set xtics (\"0\" 0, \"1\" 1)\n", fp);
 	gpinfo->xrange = 3;
     } else {
@@ -729,6 +852,10 @@ check_for_yscale (struct gnuplot_info *gpinfo, const int *list,
     double ratio;
     int i, j, oddcount;
 
+#if GP_DEBUG
+    fprintf(stderr, "gnuplot: doing check_for_yscale\n");
+#endif
+
     /* find minima, maxima of the y-axis vars */
     for (i=1; i<list[0]; i++) {
 	gretl_minmax(gpinfo->t1, gpinfo->t2, Z[list[i]], &ymin[i], &ymax[i]);
@@ -843,7 +970,7 @@ gp_info_init (struct gnuplot_info *gpinfo, unsigned char flags,
     gpinfo->flags = flags;
     gpinfo->ts_plot = 1;   /* plotting against time on x-axis? */
     gpinfo->yscale = 0;    /* two y axis scales needed? */
-    gpinfo->pdist = 0;     /* plotting probability dist? */
+    gpinfo->impulses = 0;  /* plotting with impulses? */
     gpinfo->lo = lo;
     gpinfo->ols_ok = 0;    /* plot automatic OLS line? */
     gpinfo->t1 = t1;
@@ -863,9 +990,120 @@ gp_info_init (struct gnuplot_info *gpinfo, unsigned char flags,
 	gpinfo->yscale = 1;
     } 
 
+    if (flags & GP_IMPULSES) {
+	gpinfo->impulses = 1;
+    }    
+
     gpinfo->yvar1 = gpinfo->yvar2 = NULL;
 }
 
+#if GP_DEBUG
+static void print_gnuplot_flags (unsigned char flags)
+{
+    fprintf(stderr, "*** gnuplot() called with flags:\n");
+
+    if (flags & GP_IMPULSES) {
+	fprintf(stderr, " GP_IMPULSES\n");
+    }
+    if (flags & GP_RESIDS) {
+	fprintf(stderr, " GP_RESIDS\n");
+    }	
+    if (flags & GP_FA) {
+	fprintf(stderr, " GP_FA\n");
+    }
+    if (flags & GP_DUMMY) {
+	fprintf(stderr, " GP_DUMMY\n");
+    }
+    if (flags & GP_BATCH) {
+	fprintf(stderr, " GP_BATCH\n");
+    }
+    if (flags & GP_GUI) {
+	fprintf(stderr, " GP_GUI\n");
+    }
+    if (flags & GP_OLS_OMIT) {
+	fprintf(stderr, " GP_OLS_OMIT\n");
+    }
+    if (flags & GP_FILE) {
+	fprintf(stderr, " GP_FILE\n");
+    }
+}
+#endif
+
+static void set_withstr (unsigned char flags, const int *lines, 
+			 int i, char *str)
+{
+    int ltest = 0;
+
+    if (lines != NULL) {
+	ltest = lines[(flags & GP_GUI)? i - 1 : 0];
+    }
+
+    if (ltest) {
+	strcpy(str, "w lines");
+    } else {
+	strcpy(str, "w points");
+    }
+}
+
+static void graph_list_adjust_sample (int *list, 
+				      struct gnuplot_info *ginfo,
+				      const double **Z)
+{
+    int t1min = ginfo->t1;
+    int t2max = ginfo->t2;
+    int t_ok;
+    int i, t;
+
+    for (t=t1min; t<=t2max; t++) {
+	t_ok = 0;
+	for (i=1; i<=list[0]; i++) {
+	    if (!na(Z[list[i]][t])) {
+		t_ok = 1;
+		break;
+	    }
+	}
+	if (t_ok) {
+	    break;
+	} 
+	t1min++;
+    }
+
+    for (t=t2max; t>t1min; t--) {
+	t_ok = 0;
+	for (i=1; i<=list[0]; i++) {
+	    if (!na(Z[list[i]][t])) {
+		t_ok = 1;
+		break;
+	    }
+	}
+	if (t_ok) {
+	    break;
+	}
+	t2max--;
+    }
+
+    if (t2max > t1min) {
+	for (i=1; i<=list[0]; i++) {
+	    int all_missing = 1;
+
+	    for (t=t1min; t<=t2max; t++) {
+		if (!na(Z[list[i]][t])) {
+		    all_missing = 0;
+		    break;
+		}
+	    }
+	    if (all_missing) {
+		gretl_list_delete_at_pos(list, i);
+		i--;
+	    }
+	}
+    }
+
+    ginfo->t1 = t1min;
+    ginfo->t2 = t2max;
+    ginfo->lo = list[0];
+}
+
 /**
  * gnuplot:
  * @list: list of variables to plot, by ID number.
@@ -892,7 +1130,6 @@ int gnuplot (int *list, const int *lines, const char *literal,
     char s1[MAXDISP] = {0};
     char s2[MAXDISP] = {0};
     char xlabel[MAXDISP] = {0};
-    char depvar[VNAMELEN] = {0};
     char withstr[16] = {0};
     char keystr[48] = {0};
     char ols_line[128] = {0};
@@ -902,6 +1139,10 @@ int gnuplot (int *list, const int *lines, const char *literal,
 
     *gretl_errmsg = '\0';
 
+#if GP_DEBUG
+    print_gnuplot_flags(flags);
+#endif
+
 #ifdef WIN32
     /* only gnuplot 3.8 or higher will accept "height" here */
     strcpy(keystr, "set key left top height 1 width 1 box\n");
@@ -915,19 +1156,8 @@ int gnuplot (int *list, const int *lines, const char *literal,
 
     gp_info_init(&gpinfo, flags, list[0], pdinfo->t1, pdinfo->t2);
 
-    /* special for probability distribution plot */
-    if ((flags & GP_IMPULSES) || lines == NULL) {
-	if (!(flags & ~GP_OLS_OMIT)) { 
-	    strcpy(withstr, "w i");
-	}
-	gpinfo.pdist = 1;
-    }
-
-    /* hack to get the name of the dependent variable into the
-       graph */
-    if (flags & GP_RESIDS) {
-	strcpy(depvar, pdinfo->varname[list[gpinfo.lo]]);
-	gpinfo.lo -= 1;
+    if (gpinfo.impulses) {
+	strcpy(withstr, "w i");
     }
 
     /* check whether we're doing a time-series plot */
@@ -955,15 +1185,14 @@ int gnuplot (int *list, const int *lines, const char *literal,
     }	
 
     /* add a simple regression line if appropriate */
-    if (!gpinfo.pdist && !(flags & GP_OLS_OMIT) && gpinfo.lo == 2 && 
+    if (!gpinfo.impulses && !(flags & GP_OLS_OMIT) && gpinfo.lo == 2 && 
 	!gpinfo.ts_plot && !(flags & GP_RESIDS)) {
 	get_ols_line(&gpinfo, list, pZ, pdinfo, ols_line);
     }
 
     /* adjust sample range, and reject if it's empty */
-    adjust_t1t2(NULL, list, &gpinfo.t1, &gpinfo.t2, 
-		(const double **) *pZ, NULL);
-    if (gpinfo.t2 == gpinfo.t1) {
+    graph_list_adjust_sample(list, &gpinfo, (const double **) *pZ);
+    if (gpinfo.t1 == gpinfo.t2 || gpinfo.lo == 1) {
 	fclose(fp);
 	return GRAPH_NO_DATA;
     }
@@ -1015,14 +1244,14 @@ int gnuplot (int *list, const int *lines, const char *literal,
 	    }
 	}
 	if (flags & GP_RESIDS && !(flags & GP_DUMMY)) { 
-	    make_gtitle(fp, GTITLE_RESID, depvar, NULL);
+	    make_gtitle(fp, GTITLE_RESID, VARLABEL(pdinfo, list[1]), NULL);
 	    fprintf(fp, "set ylabel '%s'\n", I_("residual"));
 	} else {
 	    fprintf(fp, "set ylabel '%s'\n", series_name(pdinfo, list[1]));
 	}
 	strcpy(keystr, "set nokey\n");
     } else if ((flags & GP_RESIDS) && (flags & GP_DUMMY)) { 
-	make_gtitle(fp, GTITLE_RESID, depvar, NULL);
+	make_gtitle(fp, GTITLE_RESID, VARLABEL(pdinfo, list[1]), NULL);
 	fprintf(fp, "set ylabel '%s'\n", I_("residual"));
     } else if (flags & GP_FA) {
 	if (list[3] == pdinfo->v - 1) { 
@@ -1070,7 +1299,7 @@ int gnuplot (int *list, const int *lines, const char *literal,
 		    (i == gpinfo.oddman)? "x1y2" : "x1y1",
 		    series_name(pdinfo, list[i]), 
 		    (i == gpinfo.oddman)? I_("right") : I_("left"),
-		    (gpinfo.pdist)? "w impulses" : 
+		    (gpinfo.impulses)? "w impulses" : 
 		    (gpinfo.ts_plot)? "w lines" : "w points",
 		    (i == gpinfo.lo - 1)? "\n" : " , \\\n");
 	}
@@ -1089,14 +1318,8 @@ int gnuplot (int *list, const int *lines, const char *literal,
 	    } else {
 		strcpy(s1, series_name(pdinfo, list[i]));
 	    }
-	    if (!gpinfo.pdist) { 
-		int ltest = lines[(flags & GP_GUI)? i - 1 : 0];
-
-		if (ltest) {
-		    strcpy(withstr, "w lines");
-		} else {
-		    strcpy(withstr, "w points");
-		}
+	    if (!gpinfo.impulses) { 
+		set_withstr(flags, lines, i, withstr);
 	    }
 	    fprintf(fp, " '-' using 1:($2) title '%s' %s", 
 		    s1, withstr);
@@ -1289,6 +1512,7 @@ static int get_3d_output_file (FILE **fpp)
 
     sprintf(fname, "%sgpttmp.plt", gretl_user_dir());
     *fpp = gretl_fopen(fname, "w");
+
     if (*fpp == NULL) {
 	err = E_FOPEN;
     } else {
@@ -1368,7 +1592,8 @@ int gnuplot_3d (int *list, const char *literal,
 	return E_FOPEN;
     }
 
-    adjust_t1t2(NULL, list, &t1, &t2, (const double **) *pZ, NULL);
+    varlist_adjust_sample(list, &t1, &t2, (const double **) *pZ);
+
     /* if resulting sample range is empty, complain */
     if (t2 == t1) {
 	fclose(fq);
@@ -1452,10 +1677,10 @@ static void print_freq_dist_label (char *s, int dist, double x, double y)
     }
 #endif
 
-    if (dist == NORMAL) {
+    if (dist == DIST_NORMAL) {
 	sprintf(s, "N(%.5g%c%.5g)", x, 
 		((dcomma)? ' ' : ','), y);
-    } else if (dist == GAMMA) {
+    } else if (dist == DIST_GAMMA) {
 	sprintf(s, "gamma(%.5g%c%.5g)", x, 
 		((dcomma)? ' ' : ','), y);
     }
@@ -1467,16 +1692,16 @@ static void print_freq_dist_label (char *s, int dist, double x, double y)
 
 /**
  * plot_freq:
- * @freq: frequency distribution struct.
- * @dist: distribution code (see #dist_codes).
+ * @freq: pointer to frequency distribution struct.
+ * @dist: probability distribution code.
  *
  * Plot the actual frequency distribution for a variable versus a
- * theoretical distribution, Gaussian or gamma (or none).
+ * theoretical distribution: Gaussian, gamma or none.
  *
  * Returns: 0 on successful completion, error code on error.
  */
 
-int plot_freq (FREQDIST *freq, int dist)
+int plot_freq (FreqDist *freq, DistCode dist)
 {
     double alpha = 0.0, beta = 0.0, lambda = 1.0;
     FILE *fp = NULL;
@@ -1495,7 +1720,11 @@ int plot_freq (FREQDIST *freq, int dist)
 
     if ((err = gnuplot_init(plottype, &fp))) {
 	return err;
-    }    
+    }  
+
+#if GP_DEBUG
+    fprintf(stderr, "*** plot_freq called\n");
+#endif  
 
     barwidth = freq->endpt[K-1] - freq->endpt[K-2];
     barskip = 0.005 * (freq->endpt[K] - freq->endpt[0]);
@@ -1504,9 +1733,9 @@ int plot_freq (FREQDIST *freq, int dist)
 	barskip /= 2.0;
     }
 
-    if (dist == NORMAL) {
+    if (dist == DIST_NORMAL) {
 	plottype = PLOT_FREQ_NORMAL;
-    } else if (dist == GAMMA) {
+    } else if (dist == DIST_GAMMA) {
 	plottype = PLOT_FREQ_GAMMA;
     }
 
@@ -1530,11 +1759,11 @@ int plot_freq (FREQDIST *freq, int dist)
 	   theoretical distribution they enclose, and calculate a
 	   height adjustment factor for the impulses */
 
-	if (dist == NORMAL) {
+	if (dist == DIST_NORMAL) {
 	    fputs("(against normal)\n", fp);
 
-	    propn = normal((freq->endpt[i-1] - freq->xbar)/freq->sdx) -
-		normal((freq->endpt[i] - freq->xbar)/freq->sdx);
+	    propn = normal_pvalue_1((freq->endpt[i-1] - freq->xbar)/freq->sdx) -
+		normal_pvalue_1((freq->endpt[i] - freq->xbar)/freq->sdx);
 	    lambda = 1.0 / (propn * freq->n * sqrt(2 * M_PI) * freq->sdx);
 	    fprintf(fp, "sigma = %g\n", freq->sdx);
 	    fprintf(fp, "mu = %g\n", freq->xbar);
@@ -1549,7 +1778,7 @@ int plot_freq (FREQDIST *freq, int dist)
 	    }
 
 	    if (!na(freq->test)) {
-		fprintf(fp, "set label '%s:' at graph .03, graph .97%s\n",
+		fprintf(fp, "set label \"%s:\" at graph .03, graph .97%s\n",
 			I_("Test statistic for normality"),
 			label_front());
 		print_freq_test_label(label, I_("Chi-squared(2) = %.3f pvalue = %.5f"), 
@@ -1557,7 +1786,7 @@ int plot_freq (FREQDIST *freq, int dist)
 		fprintf(fp, "set label '%s' at graph .03, graph .93%s\n", 
 			label, label_front());
 	    }	
-	} else if (dist == GAMMA) {
+	} else if (dist == DIST_GAMMA) {
 	    double xx, height, var = freq->sdx * freq->sdx;
 
 	    fputs("(against gamma)\n", fp);
@@ -1583,7 +1812,7 @@ int plot_freq (FREQDIST *freq, int dist)
 			I_("Test statistic for gamma"),
 			label_front());
 		print_freq_test_label(label, I_("z = %.3f pvalue = %.5f"), 
-				      freq->test, 2.0 * normal(fabs(freq->test)));
+				      freq->test, normal_pvalue_2(freq->test));
 		fprintf(fp, "set label '%s' at graph .03, graph .93%s\n", 
 			label, label_front());
 	    }	
@@ -1631,14 +1860,14 @@ int plot_freq (FREQDIST *freq, int dist)
 
     if (!dist) {
 	fprintf(fp, "plot '-' using 1:($2) %s\n", withstr);
-    } else if (dist == NORMAL) {
+    } else if (dist == DIST_NORMAL) {
 	print_freq_dist_label(label, dist, freq->xbar, freq->sdx);
 	fputs("plot \\\n", fp);
 	fprintf(fp, "'-' using 1:($2) title '%s' %s , \\\n"
 		"(1/(sqrt(2*pi)*sigma)*exp(-(x-mu)**2/(2*sigma**2))) "
 		"title '%s' w lines\n",
 		freq->varname, withstr, label);
-    } else if (dist == GAMMA) {
+    } else if (dist == DIST_GAMMA) {
 	print_freq_dist_label(label, dist, alpha, beta);
 	fputs("plot \\\n", fp);
 	fprintf(fp, "'-' using 1:($2) title '%s' %s ,\\\n"
@@ -1673,44 +1902,87 @@ int plot_fcast_errs (int n, const double *obs,
 {
     FILE *fp = NULL;
     double xmin, xmax, xrange;
+    int depvar_present = 0;
+    int do_errs = (maxerr != NULL);
     int t, err;
 
     if ((err = gnuplot_init(PLOT_FORECAST, &fp))) {
 	return err;
     }
 
+    /* don't graph empty portion of forecast */
+    for (t=n-1; t>0; t--) {
+	if (na(depvar[t]) && na(yhat[t])) {
+	    n--;
+	} else {
+	    break;
+	}
+    }
+
+    if (n < 3) {
+	/* won't draw a graph for 2 datapoints or less */
+	return 1;
+    }
+
+    /* check that we have any values for the actual var */
+    for (t=0; t<n; t++) {
+	if (!na(depvar[t])) {
+	    depvar_present = 1;
+	    break;
+	}
+    }
+
     fputs("# forecasts with 95 pc conf. interval\n", fp);
 
     gretl_minmax(0, n - 1, obs, &xmin, &xmax);
     xrange = xmax - xmin;
     xmin -= xrange * .025;
     xmax += xrange * .025;
-    fprintf(fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
-    fputs("set missing \"?\"\n", fp);
-    if (!time_series) {
-	fputs("set xtics 1\n", fp);
-    } else {
-	fprintf(fp, "# timeseries %d\n", time_series);
-    }
 
-    fprintf(fp, "set key left top\n"
-	    "plot \\\n'-' using 1:2 title '%s' w lines , \\\n"
-	    "'-' using 1:2 title '%s' w lines , \\\n"
-	    "'-' using 1:2:3 title '%s' w errorbars\n", 
-	    varname, I_("fitted"), I_("95 percent confidence interval"));
-
-    /* send data inline */
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
 #endif
-    for (t=0; t<n; t++) {
-	if (na(depvar[t])) {
-	    fprintf(fp, "%.8g ?\n", obs[t]);
-	} else {
-	    fprintf(fp, "%.8g %.8g\n", obs[t], depvar[t]);
-	}
+    fprintf(fp, "set xrange [%.7g:%.7g]\n", xmin, xmax);
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
+
+    fputs("set missing \"?\"\n", fp);
+
+    if (time_series) {
+	fprintf(fp, "# timeseries %d\n", time_series);
+    } else if (n < 33) {
+	fputs("set xtics 1\n", fp);
     }
-    fputs("e\n", fp);
+
+    fputs("set key left top\nplot \\\n", fp);
+    if (depvar_present) {
+	fprintf(fp, "'-' using 1:2 title \"%s\" w lines , \\\n",
+		varname);
+    }
+    fprintf(fp, "'-' using 1:2 title '%s' w lines", I_("fitted"));
+    if (do_errs) {
+	fprintf(fp, " , \\\n'-' using 1:2:3 title '%s' w errorbars\n",
+		I_("95 percent confidence interval"));
+    } else {
+	fputc('\n', fp);
+    }
+
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+
+    if (depvar_present) {
+	for (t=0; t<n; t++) {
+	    if (na(depvar[t])) {
+		fprintf(fp, "%.8g ?\n", obs[t]);
+	    } else {
+		fprintf(fp, "%.8g %.8g\n", obs[t], depvar[t]);
+	    }
+	}
+	fputs("e\n", fp);
+    }
+
     for (t=0; t<n; t++) {
 	if (na(yhat[t])) {
 	    fprintf(fp, "%.8g ?\n", obs[t]);
@@ -1719,14 +1991,18 @@ int plot_fcast_errs (int n, const double *obs,
 	}
     }
     fputs("e\n", fp);
-    for (t=0; t<n; t++) {
-	if (na(yhat[t]) || na(maxerr[t])) {
-	    fprintf(fp, "%.8g ? ?\n", obs[t]);
-	} else {
-	    fprintf(fp, "%.8g %.8g %.8g\n", obs[t], yhat[t], maxerr[t]);
+
+    if (do_errs) {
+	for (t=0; t<n; t++) {
+	    if (na(yhat[t]) || na(maxerr[t])) {
+		fprintf(fp, "%.8g ? ?\n", obs[t]);
+	    } else {
+		fprintf(fp, "%.8g %.8g %.8g\n", obs[t], yhat[t], maxerr[t]);
+	    }
 	}
+	fputs("e\n", fp);
     }
-    fputs("e\n", fp);
+
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "");
 #endif
@@ -1772,20 +2048,24 @@ int garch_resid_plot (const MODEL *pmod, double ***pZ, DATAINFO *pdinfo)
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "C");
 #endif
+
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	fprintf(fp, "%.8g %.8g\n", obs[t], pmod->uhat[t]);
     }
     fputs("e\n", fp);
+
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	sd2 = -sqrt(h[t]);
 	fprintf(fp, "%.8g %.8g\n", obs[t], sd2);
     }
     fputs("e\n", fp);
+
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	sd2 = sqrt(h[t]);
 	fprintf(fp, "%.8g %.8g\n", obs[t], sd2);
     }
     fputs("e\n", fp);
+
 #ifdef ENABLE_NLS
     setlocale(LC_NUMERIC, "");
 #endif
@@ -1913,10 +2193,15 @@ gp_string (FILE *fp, const char *fmt, const char *s, int png)
 
 const char *gp_justification_string (int j)
 {
-    if (j == GP_JUST_LEFT) return "left";
-    else if (j == GP_JUST_CENTER) return "center";
-    else if (j == GP_JUST_RIGHT) return "right";
-    else return "left";
+    if (j == GP_JUST_LEFT) {
+	return "left";
+    } else if (j == GP_JUST_CENTER) {
+	return "center";
+    } else if (j == GP_JUST_RIGHT) {
+	return "right";
+    } else {
+	return "left";
+    }
 }
 
 static void 
@@ -2154,10 +2439,9 @@ int print_plotspec_details (const GPT_SPEC *spec, FILE *fp)
     return miss;
 }
 
-/* ........................................................... */
+/* ship out a plot struct, to gnuplot or file */
 
 int go_gnuplot (GPT_SPEC *spec, char *fname)
-     /* ship out a plot struct, to gnuplot or file */
 {
     FILE *fp = NULL;
     int dump = 0;
@@ -2168,15 +2452,21 @@ int go_gnuplot (GPT_SPEC *spec, char *fname)
 
     if (dump) {  
 	/* dump of gnuplot commands to named file */
-	if (fname == NULL) return 1;  /* impossible */
+	if (fname == NULL) {
+	    return 1;  /* impossible */
+	}
 	fp = gretl_fopen(fname, "w");
-	if (fp == NULL) return 1;
+	if (fp == NULL) {
+	    return 1;
+	}
     } else {     
 	/* output to gnuplot, for screen or other "term" */
 	if (spec->fp == NULL) {
 	    fp = gretl_fopen(gretl_plotfile(), "w");
 	}
-	if (fp == NULL) return 1;
+	if (fp == NULL) {
+	    return 1;
+	}
 
 	if (fname != NULL) { 
 #ifdef ENABLE_NLS
@@ -2292,16 +2582,6 @@ gretl_var_plot_impulse_response (GRETL_VAR *var,
     char title[128];
     int t, err;
 
-    if (periods == 0) {
-	if (pdinfo->pd == 4) {
-	    periods = 24;
-	} else if (pdinfo->pd == 12) {
-	    periods = 36;
-	} else {
-	    periods = 12;
-	}
-    }
-
     if ((err = gnuplot_init(PLOT_REGULAR, &fp))) {
 	return err;
     }
@@ -2362,9 +2642,9 @@ static char gnuplot_pallette[4][8] = {
     "xaabbcc"
 };
 
-const char *get_gnuplot_pallette (int i, int ptype)
+const char *get_gnuplot_pallette (int i, PlotType ptype)
 {
-#ifdef GNUPLOT_DEBUG
+#if GP_DEBUG
     fprintf(stderr, "get_gnuplot_pallette: i=%d, ptype=%d\n",
 	    i, ptype);
 #endif
@@ -2384,10 +2664,16 @@ static int colstr_is_valid (const char *colstr)
     int i;
     const char *ok = "0123456789abcdef";
 
-    if (*colstr != 'x') return 0;
-    if (strlen(colstr) != 7) return 0;
+    if (*colstr != 'x') {
+	return 0;
+    }
+    if (strlen(colstr) != 7) {
+	return 0;
+    }
     for (i=1; i<7; i++) {
-	if (strchr(ok, colstr[i]) == NULL) return 0;
+	if (strchr(ok, colstr[i]) == NULL) {
+	    return 0;
+	}
     }
 
     return 1;
