@@ -32,6 +32,7 @@ struct gretl_cmd {
 static struct gretl_cmd gretl_cmds[] = {
     { SEMIC,    ";" },     
     { ADD,      "add" },
+    { ADDOBS,   "addobs" },
     { ADDTO,    "addto" },
     { ADF,      "adf" }, 
     { APPEND,   "append" },
@@ -65,6 +66,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { FIT,      "fit" }, 
     { FREQ,     "freq" }, 
     { FUNC,     "function" },
+    { FUNCERR,  "funcerr" },
     { GARCH,    "garch" },
     { GENR,     "genr" },     
     { GNUPLOT,  "gnuplot" },  
@@ -77,6 +79,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { HURST,    "hurst" },
     { IF,       "if" },
     { IMPORT,   "import" },
+    { INCLUDE,  "include" },
     { INFO,     "info" }, 
     { KPSS,     "kpss" },
     { LABEL,    "label" },
@@ -114,6 +117,7 @@ static struct gretl_cmd gretl_cmds[] = {
     { PVALUE,   "pvalue" },  
     { PWE,      "pwe" },
     { QUIT,     "quit" }, 
+    { REMEMBER, "remember" },
     { RENAME,   "rename" },
     { RESET,    "reset" },
     { RESTRICT, "restrict" },
@@ -159,8 +163,11 @@ int gretl_command_number (const char *s)
 
 const char *gretl_command_word (int i)
 {
-    if (i >= 0 && i < NC) return gretl_cmds[i].cword;
-    else return "";
+    if (i >= 0 && i < NC) {
+	return gretl_cmds[i].cword;
+    } else {
+	return "";
+    }
 }
 
 const char *gretl_command_complete_next (const char *s,
