@@ -20,9 +20,10 @@
 #ifndef COMMANDS_H
 #define COMMANDS_H
 
-enum gretl_cmd_codes {
+typedef enum {
     SEMIC,    
     ADD,
+    ADDOBS,
     ADDTO,
     ADF,
     APPEND,
@@ -56,6 +57,7 @@ enum gretl_cmd_codes {
     FIT,     
     FREQ, 
     FUNC,
+    FUNCERR,
     GARCH,
     GENR,     
     GNUPLOT, 
@@ -68,6 +70,7 @@ enum gretl_cmd_codes {
     HURST,
     IF,
     IMPORT,
+    INCLUDE,
     INFO,
     KPSS,
     LABEL,
@@ -104,7 +107,8 @@ enum gretl_cmd_codes {
     PROBIT,
     PVALUE, 
     PWE,
-    QUIT, 
+    QUIT,
+    REMEMBER,
     RENAME,
     RESET,
     RESTRICT,
@@ -134,7 +138,7 @@ enum gretl_cmd_codes {
     VIF,
     WLS,
     NC
-};
+} GretlCmdIndex;
 
 #define NEEDS_TWO_VARS(c)  ((c) == AR || (c) == ARCH || (c) == COINT || \
                            (c) == CORC || (c) == HCCM || (c) == HILU || \
@@ -178,6 +182,7 @@ enum gretl_cmd_codes {
                         c == RMPLOT || \
                         c == RUNS || \
                         c == SPEARMAN || \
+                        c == SUMMARY || \
                         c == TESTUHAT || \
                         c == VIF || \
                         c == VARTEST)
