@@ -20,6 +20,7 @@
 /* errors.c - error messages for gretl */
 
 #include "libgretl.h"
+#include "libset.h"
 
 int gretl_errno;
 char gretl_errmsg[ERRLEN];
@@ -35,23 +36,13 @@ const char *gretl_error_messages[] = {
     N_("Dependent variable is all zeros, aborting regression"),  /* E_ZERO */
     N_("Total sum of squares was not positive"),                 /* E_TSS */
     N_("Sum of squared residuals negative!"),                    /* E_ESS */
-    N_("First character of new name is not a letter"),           /* E_NOTALPH */
-    N_("You can't redefine the constant in genr"),               /* E_CONST */
     N_("Unbalanced parentheses in genr command"),                /* E_UNBAL */
-    N_("Too many nested parentheses.  genr not done"),           /* E_NEST */
-    N_("Argument must be an integer"),                           /* E_NOTINTG */
     N_("Sorry, command not available for this estimator"),       /* E_NOTIMP */
-    N_("Evaluation of genr expression failed"),                  /* E_IGNONZERO */
-    N_("Unrecognized term in genr formula"),                     /* E_CASEU */
     N_("Unspecified error -- FIXME"),                            /* E_UNSPEC */
     N_("Syntax error in genr formula"),                          /* E_SYNTAX */
-    N_("\"getxvec\" failed in generating new variable"),         /* E_ER */
-    N_("Unrecognized operator in genr formula"),                 /* E_BADOP */
     N_("This command won't work with the current periodicity"),  /* E_PDWRONG */
-    N_("The \"-o\" flag is not implemented for this command"),   /* E_OFLAG */
     N_("Error attempting to open file"),                         /* E_FOPEN */
     N_("Out of memory error"),                                   /* E_ALLOC */
-    N_("Missing equals sign in genr"),                           /* E_NOEQ */
     N_("No formula supplied in genr"),                           /* E_EQN */
     N_("Unknown variable name in command"),                      /* E_UNKVAR */
     N_("The observations specified for the regression "
@@ -60,7 +51,6 @@ const char *gretl_error_messages[] = {
     N_("This command is implemented only for OLS models"),       /* E_OLSONLY */
     N_("Invalid argument for coeff, corr, stderr, rho, "
        "pvalue, critical or mpow"),                              /* E_INVARG */
-    N_("Invalid lag order for adf command"),                     /* E_ADF */
     N_("Invalid sample split for Chow test"),                    /* E_SPLIT */
     N_("Syntax error in command line"),                          /* E_PARSE */
     N_("No independent variables left after omissions"),         /* E_NOVARS */
@@ -72,12 +62,10 @@ const char *gretl_error_messages[] = {
     N_("Error generating logarithms"),                           /* E_LOGS */
     N_("Error generating squares"),                              /* E_SQUARES */
     N_("Error generating lagged variables"),                     /* E_LAGS */
-    N_("Error in auxiliary regression for rho"),                 /* E_RHO */
     N_("Attempting to take square root of negative number"),     /* E_SQRT */
     N_("Excessive exponent in genr formula"),                    /* E_HIGH */
     N_("Weight variable is all zeros, aborting regression"),     /* E_WTZERO */
     N_("Need valid starting and ending observations"),           /* E_OBS */
-    N_("New variable name was not supplied"),                    /* E_NOVAR */
     N_("You must include a constant in this sort of model"),     /* E_NOCONST */
     N_("There were missing observations for the added "
        "variable(s).\nReset the sample and rerun the original "
@@ -88,8 +76,10 @@ const char *gretl_error_messages[] = {
     N_("The convergence criterion was not met"),                 /* E_NOCONV */
     N_("The operation was canceled"),                            /* E_CANCEL */
     N_("Missing values encountered"),                            /* E_MISSDATA */
+    NULL,                                                        /* E_NAN */
+    NULL,                                                        /* E_DB_DUP */
     NULL,                                                        /* E_OK */
-    NULL                                                         /* E_NAN */
+    NULL                                                         /* E_MAX */
 };
 
 /**
@@ -107,17 +97,23 @@ const char *gretl_error_messages[] = {
 
 char *get_errmsg (const int errcode, char *errtext, PRN *prn)
 {
-    if (errcode > 0 && errcode < E_MAX && gretl_error_messages[errcode]) {
-	if (errtext == NULL) {
-	    pprintf(prn, "%s\n", _(gretl_error_messages[errcode]));
-	    return NULL;
-	} else {
-	    strcpy(errtext, _(gretl_error_messages[errcode]));
-	    return errtext;
+    char *msg = NULL;
+
+    if (errcode > 0 && errcode < E_MAX) {
+	if (gretl_error_messages[errcode] != NULL) {
+	    if (errtext != NULL) {
+		strcpy(errtext, _(gretl_error_messages[errcode]));
+		msg = errtext;
+	    } else {
+		pprintf(prn, "%s\n", _(gretl_error_messages[errcode]));
+	    }
 	}
     } else {
-	return NULL;
+	fprintf(stderr, "get_errmsg: out of bounds errcode %d\n", 
+		errcode);
     }
+
+    return msg;
 }
 
 /**
@@ -151,8 +147,15 @@ const char *get_gretl_errmsg (void)
 
 const char *get_gretl_msg (void)
 {
-    if (*gretl_msg == '\0') return NULL;
-    return gretl_msg;
+    const char *ret = NULL;
+
+    if (gretl_messages_on()) {
+	if (*gretl_msg != '\0') {
+	    ret = gretl_msg;
+	}
+    }
+
+    return ret;
 }
 
 int print_gretl_errmsg (PRN *prn)
@@ -162,23 +165,25 @@ int print_gretl_errmsg (PRN *prn)
     if (*gretl_errmsg != '\0') {
 	pprintf(prn, "%s\n", gretl_errmsg);
 	ret = 1;
-    } else {
-	char *str = get_errmsg(gretl_errno, NULL, prn);
-
-	if (str != NULL) ret = 1;
-    } 
+    } else if (get_errmsg(gretl_errno, NULL, prn)) {
+	ret = 1;
+    }
 
     return ret;
 }
 
 int print_gretl_msg (PRN *prn)
 {
-    if (*gretl_msg != '\0') {
-	pprintf(prn, "%s\n", gretl_msg);
-	return 1;
-    } else {
-	return 0;
+    int ret = 0;
+
+    if (gretl_messages_on()) {
+	if (*gretl_msg != '\0') {
+	    pprintf(prn, "%s\n", gretl_msg);
+	    ret = 1;
+	}
     }
+
+    return ret;
 }
 
 /**
