@@ -23,6 +23,9 @@
 #include "gretl_matrix.h"
 #include "gretl_matrix_private.h"
 
+#include "f2c.h"
+#include "clapack_double.h"
+
 static const char *wspace_fail = "gretl_matrix: workspace query failed\n";
 
 static int packed_idx (int nrows, int i, int j);
@@ -68,7 +71,6 @@ static gretl_matrix *real_gretl_matrix_alloc (int rows, int cols,
  * Returns: pointer to a newly allocated gretl_matrix, or %NULL
  * on failure.  Note that the actual data storage is not
  * initialized.
- * 
  */
 
 gretl_matrix *gretl_matrix_alloc (int rows, int cols)
@@ -82,7 +84,6 @@ gretl_matrix *gretl_matrix_alloc (int rows, int cols)
  *
  * Returns: pointer to a newly allocated gretl_matrix, or %NULL
  * on failure.  The matrix is in packed (triangular) form.
- * 
  */
 
 gretl_matrix *gretl_packed_matrix_alloc (int rows)
@@ -100,11 +101,10 @@ gretl_matrix *gretl_packed_matrix_alloc (int rows)
  * Returns: pointer to a newly allocated gretl_vector containing
  * the elements of x (or their squares), or %NULL on failure.  
  * Missing elements of x are skipped.
- * 
  */
 
 gretl_vector *
-gretl_column_vector_from_array (const double *x, int n, int mod)
+gretl_column_vector_from_array (const double *x, int n, GretlMatrixMod mod)
 {
     gretl_matrix *v;
     double xi;
@@ -128,6 +128,41 @@ gretl_column_vector_from_array (const double *x, int n, int mod)
     return v;
 }
 
+/**
+ * gretl_data_series_to_vector:
+ * @Z: data array.
+ * @varno: ID number of variable
+ * @t1: starting observation.
+ * @t2: ending observation.
+ *
+ * Returns: a newly allocated gretl_vector containing the values
+ * of the given variable (data series) for the given range,
+ * or %NULL on failure.  
+ */
+
+gretl_vector *gretl_data_series_to_vector (const double **Z, int varno, 
+					   int t1, int t2)
+{
+    gretl_matrix *v;
+    int t, n = t2 - t1 + 1;
+
+    if (n <= 0) {
+	return NULL;
+    }
+
+    v = gretl_column_vector_alloc(n);
+    if (v == NULL) {
+	return NULL;
+    }
+
+    for (t=0; t<n; t++) {
+	v->val[t] = Z[varno][t + t1];
+    }
+
+    return v;
+}
+
+
 /* ....................................................... */
 
 static gretl_matrix *
@@ -183,7 +218,6 @@ gretl_matrix_copy_mod (const gretl_matrix *m, int mod)
  * @m: source matrix to be copied.
  *
  * Returns: an allocated copy of matrix @m, or %NULL on failure.  
- * 
  */
 
 gretl_matrix *gretl_matrix_copy (const gretl_matrix *m)
@@ -196,7 +230,6 @@ gretl_matrix *gretl_matrix_copy (const gretl_matrix *m)
  * @m: source matrix to be copied.
  *
  * Returns: an allocated copy of the tranpose of @m, or %NULL on failure.  
- * 
  */
 
 gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m)
@@ -204,13 +237,11 @@ gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m)
     return gretl_matrix_copy_mod(m, GRETL_MOD_TRANSPOSE);
 }
 
-
 /**
  * gretl_matrix_free:
  * @m: matrix to be freed.
  *
  * Frees the allocated storage in @m, then @m itself.
- * 
  */
 
 void gretl_matrix_free (gretl_matrix *m)
@@ -244,6 +275,112 @@ void gretl_matrix_zero (gretl_matrix *m)
     for (i=0; i<n; i++) m->val[i] = 0.0;
 }
 
+/**
+ * gretl_matrix_log:
+ * @m: input matrix.
+ *
+ * Sets all elements of @m to logs of original values.
+ */
+
+void gretl_matrix_log (gretl_matrix *m)
+{
+    int i, n;
+
+    if (m == NULL || m->val == NULL) return;
+
+    if (m->packed) {
+	n = (m->rows * m->rows + m->rows) / 2;
+    } else {
+	n = m->rows * m->cols;
+    }
+    
+    for (i=0; i<n; i++) m->val[i] = log(m->val[i]);
+}
+
+/**
+ * gretl_vector_mean:
+ * @v: input vector.
+ *
+ * Returns: the arithmetic mean of the elements of @v, or
+ * #NADBL on failure.
+ */
+
+double gretl_vector_mean (const gretl_vector *v)
+{
+    double ret = 0.0;
+    int i, n, den;
+
+    if (v == NULL || v->val == NULL) {
+	return NADBL;
+    }
+
+    if (v->rows > 1 && v->cols > 1) {
+	return NADBL;
+    }
+    
+    if (v->rows > 1) {
+	den = n = v->rows;
+    } else {
+	den = n = v->cols;
+    }
+
+    for (i=0; i<n; i++) {
+	if (!na(v->val[i])) {
+	    ret += v->val[i];
+	} else {
+	    den--;
+	}
+    }
+
+    if (den > 0) {
+	ret /= den;
+    } else {
+	ret = NADBL;
+    }
+
+    return ret;
+}
+
+/**
+ * gretl_vector_variance:
+ * @v: input vector.
+ *
+ * Returns: the variance of the elements of @v, or
+ * #NADBL on failure.
+ */
+
+double gretl_vector_variance (const gretl_vector *v)
+{
+    double s2 = 0.0;
+    double x, xbar;
+    int i, n, den;
+
+    if (v == NULL || v->val == NULL) {
+	return NADBL;
+    }
+
+    den = n = gretl_vector_get_length(v);
+    xbar = gretl_vector_mean(v);
+
+    for (i=0; i<n; i++) {
+	x = v->val[i];
+	if (!na(x)) {
+	    x -= xbar;
+	    s2 += x * x;
+	} else {
+	    den--;
+	}
+    }
+
+    if (den > 0) {
+	s2 /= den;
+    } else {
+	s2 = NADBL;
+    }
+
+    return s2;
+}
+
 static int gretl_matrix_zero_triangle (gretl_matrix *m, char t)
 {
     int i, j;
@@ -293,7 +430,6 @@ int gretl_matrix_zero_upper (gretl_matrix *m)
  * Sets the elements of @m outside of the upper triangle to zero.
  * 
  * Returns: %GRETL_MATRIX_OK on success, non-zero error code otherwise.
- * 
  */
 
 int gretl_matrix_zero_lower (gretl_matrix *m)
@@ -307,7 +443,6 @@ int gretl_matrix_zero_lower (gretl_matrix *m)
  * @x: scalar by which to multiply.
  *
  * Multiplies all elements of @m by @x.
- * 
  */
 
 void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x)
@@ -331,7 +466,6 @@ void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x)
  * @x: scalar by which to divide.
  *
  * Divides all elements of @m by @x.
- * 
  */
 
 void gretl_matrix_divide_by_scalar (gretl_matrix *m, double x)
@@ -349,6 +483,31 @@ void gretl_matrix_divide_by_scalar (gretl_matrix *m, double x)
     for (i=0; i<n; i++) m->val[i] /= x;
 }
 
+/**
+ * gretl_matrix_dot_pow:
+ * @m: matrix to operate on.
+ * @x: scalar to use for exponentiation.
+ *
+ * Raises all elements of @m to the power @x.
+ */
+
+void gretl_matrix_dot_pow (gretl_matrix *m, double x)
+{
+    int i, n;
+
+    if (m == NULL || m->val == NULL) return;
+
+    if (m->packed) {
+	n = (m->rows * m->rows + m->rows) / 2;
+    } else {
+	n = m->rows * m->cols;
+    }
+    
+    for (i=0; i<n; i++) {
+	m->val[i] = pow(m->val[i], x);
+    }
+}
+
 /**
  * gretl_matrix_copy_values:
  * @targ: target matrix.
@@ -360,7 +519,6 @@ void gretl_matrix_divide_by_scalar (gretl_matrix *m, double x)
  * Returns: %GRETL_MATRIX_OK on successful completion, or
  * %GRETL_MATRIX_NON_CONFORM if the two matrices are not
  * conformable for the operation.
- * 
  */
 
 int gretl_matrix_copy_values (gretl_matrix *targ, 
@@ -398,7 +556,6 @@ int gretl_matrix_copy_values (gretl_matrix *targ,
  * Returns: %GRETL_MATRIX_OK on successful completion, or
  * %GRETL_MATRIX_NON_CONFORM if the two matrices are not
  * conformable for the operation.
- * 
  */
 
 int 
@@ -436,7 +593,6 @@ gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src)
  * Returns: %GRETL_MATRIX_OK on successful completion, or
  * %GRETL_MATRIX_NON_CONFORM if the two matrices are not
  * conformable for the operation.
- * 
  */
 
 int 
@@ -463,6 +619,36 @@ gretl_matrix_subtract_from (gretl_matrix *targ, const gretl_matrix *src)
     return GRETL_MATRIX_OK;
 }
 
+/**
+ * gretl_matrix_transpose:
+ * @targ: target matrix.
+ * @src: source matrix.
+ *
+ * Fills out @targ (which must be pre-allocated and of the right
+ * dimensions) with the transpose of @src.
+ *
+ * Returns: %GRETL_MATRIX_OK on success, non-zero error code otherwise.
+ */
+
+int gretl_matrix_transpose (gretl_matrix *targ, const gretl_matrix *src)
+{
+    int i, j;
+    double x;
+
+    if (targ->rows != src->cols || targ->cols != src->rows) {
+	return GRETL_MATRIX_NON_CONFORM;
+    }
+
+    for (i=0; i<src->rows; i++) {
+	for (j=0; j<src->cols; j++) {
+	    x = src->val[mdx(src, i, j)];
+	    targ->val[mdx(targ, j, i)] = x;
+	}
+    }
+
+    return GRETL_MATRIX_OK;
+}
+
 /**
  * gretl_square_matrix_transpose:
  * @m: square matrix to operate on.
@@ -503,7 +689,6 @@ int gretl_square_matrix_transpose (gretl_matrix *m)
  * 
  * Returns: %GRETL_MATRIX_OK on successful completion, or
  * %GRETL_MATRIX_ERR if the source matrix is not square.
- * 
  */
 
 int gretl_matrix_add_self_transpose (gretl_matrix *m)
@@ -555,8 +740,6 @@ double *gretl_matrix_steal_data (gretl_matrix *m)
     return vals;
 }
 
-/* ....................................................... */
-
 static int packed_idx (int nrows, int i, int j)
 {
     int idx;
@@ -583,7 +766,6 @@ static int packed_idx (int nrows, int i, int j)
  * 
  * Returns: the data value, or #NADBL if the indices are out
  * of range for @m.
- * 
  */
 
 double gretl_matrix_get (const gretl_matrix *m, int i, int j)
@@ -612,7 +794,6 @@ double gretl_matrix_get (const gretl_matrix *m, int i, int j)
  * 
  * Returns: the data value, or #NADBL if the index is out
  * of range for @v.
- * 
  */
 
 double gretl_vector_get (const gretl_vector *v, int i)
@@ -663,7 +844,6 @@ int gretl_matrix_set (gretl_matrix *m, int i, int j, double x)
  * 
  * Returns: %GRETL_MATRIX_OK, or %GRETL_MATRIX_ERR if the
  * given index is out of range for @v.
- * 
  */
 
 int gretl_vector_set (gretl_vector *v, int i, double x)
@@ -681,22 +861,19 @@ int gretl_vector_set (gretl_vector *v, int i, double x)
  * gretl_matrix_print:
  * @m: matrix to print.
  * @msg: accompanying message text (or %NULL if no message is wanted).
- * @prn: pointer to gretl printing struct.
+ * @prn: pointer to gretl printing struct (or %NULL).
  *
- * Prints the matrix @m to @prn.
- * 
+ * Prints the matrix @m to @prn (or to %stdout if @prn is %NULL).
  */
 
 void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn)
 {
     int i, j;
-    PRN myprn;
+    PRN *myprn = NULL;
 
     if (prn == NULL) {
-	myprn.fp = stdout;
-	myprn.fpaux = NULL;
-	myprn.buf = NULL;
-	prn = &myprn;
+	myprn = gretl_print_new(GRETL_PRINT_STDOUT);
+	prn = myprn;
     }
 
     if (msg != NULL && *msg != '\0') {
@@ -711,6 +888,10 @@ void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn)
     }
 
     pputc(prn, '\n');
+
+    if (myprn != NULL) {
+	gretl_print_destroy(myprn);
+    }
 }
 
 /**
@@ -722,7 +903,6 @@ void gretl_matrix_print (const gretl_matrix *m, const char *msg, PRN *prn)
  * preserved: it is overwritten by the decomposition.
  * 
  * Returns: the log determinant, or #NABDL on failure.
- * 
  */
 
 double gretl_vcv_log_determinant (gretl_matrix *a)
@@ -846,7 +1026,6 @@ static double gretl_LU_determinant (gretl_matrix *a, int logdet, int absval)
  * by the factorization.
  * 
  * Returns: the determinant, or #NABDL on failure.
- * 
  */
 
 double gretl_matrix_determinant (gretl_matrix *a)
@@ -863,7 +1042,6 @@ double gretl_matrix_determinant (gretl_matrix *a)
  * by the factorization.
  * 
  * Returns: the determinant, or #NABDL on failure.
- * 
  */
 
 double gretl_matrix_log_determinant (gretl_matrix *a)
@@ -880,7 +1058,6 @@ double gretl_matrix_log_determinant (gretl_matrix *a)
  * preserved: it is overwritten by the factorization.
  * 
  * Returns: the determinant, or #NABDL on failure.
- * 
  */
 
 double gretl_matrix_log_abs_determinant (gretl_matrix *a)
@@ -894,12 +1071,11 @@ double gretl_matrix_log_abs_determinant (gretl_matrix *a)
  * @b: gretl_vector.
  *
  * Solves ax = b for the unknown vector x, using LU decomposition.
- * On exit, b is replaced by the solution and a is replaced by its 
+ * On exit, @b is replaced by the solution and @a is replaced by its 
  * LU decomposition.
  * 
  * Returns: 0 on successful completion, or a non-zero error code
  * (from the lapack function %dgetrs) on error.
- * 
  */
 
 int gretl_LU_solve (gretl_matrix *a, gretl_vector *b)
@@ -939,7 +1115,6 @@ int gretl_LU_solve (gretl_matrix *a, gretl_vector *b)
  *
  * Returns: allocated gretl_matrix, the elements of which are set to
  * the values in @X, or %NULL on allocation failure.
- * 
  */
 
 gretl_matrix *gretl_matrix_from_2d_array (const double **X, 
@@ -987,9 +1162,9 @@ matrix_multiply_self_transpose (const gretl_matrix *a,
 /**
  * gretl_matrix_multiply_mod:
  * @a: left-hand matrix.
- * @aflag: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
+ * @amod: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
  * @b: right-hand matrix.
- * @bflag: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
+ * @bmod: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
  * @c: matrix to hold the product.
  * 
  * Multiplies @a (or a-transpose) into @b (or b transpose),
@@ -1000,15 +1175,15 @@ matrix_multiply_self_transpose (const gretl_matrix *a,
  * 
  */
 
-int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
-			       const gretl_matrix *b, int bflag,
+int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
+			       const gretl_matrix *b, GretlMatrixMod bmod,
 			       gretl_matrix *c)
 {
     register int i, j, k;
     int lrows, lcols;
     int rrows, rcols;
-    const int atr = (aflag == GRETL_MOD_TRANSPOSE);
-    const int btr = (bflag == GRETL_MOD_TRANSPOSE);
+    const int atr = (amod == GRETL_MOD_TRANSPOSE);
+    const int btr = (bmod == GRETL_MOD_TRANSPOSE);
     int aidx, bidx;
     double targ;
 
@@ -1061,19 +1236,18 @@ int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
 /**
  * gretl_matrix_dot_product:
  * @a: left-hand matrix.
- * @aflag: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
+ * @amod: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
  * @b: right-hand matrix.
- * @bflag: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
- * @err: pointer to integer error-code variable.
- * 
- * Returns: The dot (scalar) product of @a (or a-transpose) and
- * @b (or b-transpose).  @err points to %GRETL_MATRIX_ERR on
- * failure.
+ * @bmod: modifier: %GRETL_MOD_NONE or %GRETL_MOD_TRANSPOSE.
+ * @errp: pointer to receive error code (zero on success,
+ * non-zero on failure), or %NULL.
  * 
+ * Returns: The dot (scalar) product of @a (or @a-transpose) and
+ * @b (or @b-transpose).
  */
 
-double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
-				 const gretl_matrix *b, int bflag,
+double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
+				 const gretl_matrix *b, GretlMatrixMod bmod,
 				 int *errp)
 {
     gretl_matrix *c = NULL;
@@ -1086,7 +1260,7 @@ double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
     }
 
     if (!err) {
-	err = gretl_matrix_multiply_mod(a, aflag, b, bflag, c);
+	err = gretl_matrix_multiply_mod(a, amod, b, bmod, c);
     }
 
     if (!err) {
@@ -1102,6 +1276,41 @@ double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
     return ret;
 }
 
+/**
+ * gretl_matrix_dot_multiply:
+ * @a: left-hand matrix.
+ * @b: right-hand matrix.
+ * 
+ * Returns: a new matrix, each of whose elements is the product of the
+ * corresponding elements of the matrices @a and @b (or %NULL on
+ * failure).
+ */
+
+gretl_matrix *gretl_matrix_dot_multiply (const gretl_matrix *a, 
+					 const gretl_matrix *b)
+{
+    gretl_matrix *c;
+    int i, n;
+
+    if (a->rows != b->rows || a->cols != b->cols) {
+	fputs("gretl_matrix_dot_multiply: matrices not conformable\n", stderr);
+	return NULL;
+    }
+
+    c = gretl_matrix_alloc(a->rows, a->cols);
+    if (c == NULL) {
+	return NULL;
+    }
+
+    n = a->rows * a->cols;
+
+    for (i=0; i<n; i++) {
+	c->val[i] = a->val[i] * b->val[i];
+    }
+
+    return c;
+}
+
 static double 
 gretl_matrix_column_mean (const gretl_matrix *m, int col)
 {
@@ -1120,14 +1329,13 @@ gretl_matrix_column_mean (const gretl_matrix *m, int col)
  * @m: source matrix (expected to have rows >= cols).
  *
  * Forms a variance-covariance matrix based on @m, thus:
- * - subtract the column means from the column elements of @m.
- * - multiply @m-transpose into @m.
- * - divide the elements of the product by the number of rows
+ * (1) subtract the column means from the column elements of @m;
+ * (2) multiply @m-transpose into @m; and
+ * (3) divide the elements of the product by the number of rows
  *   in @m.
  * 
  * Returns: the allocated variance-covariance matrix, or %NULL
  * on failure.
- * 
  */
 
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m)
@@ -1177,7 +1385,6 @@ gretl_matrix *gretl_matrix_vcv (gretl_matrix *m)
  *
  * Returns: %GRETL_MATRIX_OK on success; non-zero error code on
  * failure.
- * 
  */
 
 int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
@@ -1199,7 +1406,6 @@ int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
  *
  * Returns: %GRETL_MATRIX_OK on success; %GRETL_MATRIX_ERR on 
  * failure.
- * 
  */
 
 int gretl_matrix_cholesky_decomp (gretl_matrix *a)
@@ -1227,7 +1433,6 @@ int gretl_matrix_cholesky_decomp (gretl_matrix *a)
  * Uses the lapack functions %dgetrf and %dgetri.
  *
  * Returns: 0 on success; non-zero error code on failure.
- * 
  */
 
 int gretl_invert_general_matrix (gretl_matrix *a)
@@ -1343,7 +1548,6 @@ gretl_symmetric_matrix_expand (gretl_matrix *m, char uplo)
  * On exit @a is overwritten with the inverse. 
  *
  * Returns: 0 on success; non-zero error code on failure.
- * 
  */
 
 int gretl_invert_diagonal_matrix (gretl_matrix *a)
@@ -1374,7 +1578,6 @@ int gretl_invert_diagonal_matrix (gretl_matrix *a)
  * the inverse. Uses the lapack functions %dpotrf and %dpotri.
  *
  * Returns: 0 on success; non-zero error code on failure.
- * 
  */
 
 int gretl_invert_symmetric_matrix (gretl_matrix *a)
@@ -1426,7 +1629,6 @@ int gretl_invert_symmetric_matrix (gretl_matrix *a)
  *
  * Returns: allocated storage containing the eigenvalues, or %NULL
  * on failure.
- * 
  */
 
 double *gretl_general_matrix_eigenvals (gretl_matrix *m, gretl_matrix *ev) 
@@ -1513,7 +1715,6 @@ double *gretl_general_matrix_eigenvals (gretl_matrix *m, gretl_matrix *ev)
  *
  * Returns: allocated storage containing the eigenvalues, or %NULL
  * on failure.
- * 
  */
 
 double *
@@ -1581,7 +1782,6 @@ gretl_symmetric_matrix_eigenvals (gretl_matrix *m, int eigenvecs)
  * 
  * Sets an integer value on the gretl_matrix (used for internal
  * information).  
- * 
  */
 
 void gretl_matrix_set_int (gretl_matrix *m, int t)
@@ -1594,7 +1794,6 @@ void gretl_matrix_set_int (gretl_matrix *m, int t)
  * @m: matrix to read from.
  * 
  * Returns the integer that has been set on the matrix, or zero.
- * 
  */
 
 int gretl_matrix_get_int (const gretl_matrix *m)
@@ -1612,7 +1811,6 @@ int gretl_vector_get_length (const gretl_vector *v)
  * @m: matrix to query.
  * 
  * Returns: the number of columns in @m. 
- * 
  */
 
 int gretl_matrix_cols (const gretl_matrix *m)
@@ -1625,7 +1823,6 @@ int gretl_matrix_cols (const gretl_matrix *m)
  * @m: matrix to query.
  * 
  * Returns: the number of rows in @m. 
- * 
  */
 
 int gretl_matrix_rows (const gretl_matrix *m)
@@ -1633,8 +1830,6 @@ int gretl_matrix_rows (const gretl_matrix *m)
     return m->rows;
 }
 
-/* ....................................................... */
-
 static int
 get_svd_ols_vcv (const gretl_matrix *A, const gretl_matrix *B,
 		 const double *s, gretl_matrix *vcv, double *s2)
@@ -1732,11 +1927,24 @@ get_ols_uhat (const gretl_vector *y, const gretl_matrix *X,
     }
 }
 
-/* OLS for data represented as gretl matrices, using Singular
-   Value decomposition: if vcv is non-NULL compute covariance
-   matrix -- but if s2 is NULL, make this just (X'X)^{-1}.
-   if uhat is non-NULL, compute residuals in uhat
-*/
+/**
+ * gretl_matrix_svd_ols:
+ * @y: dependent variable vector.
+ * @X: matrix of independent variables.
+ * @b: vector to hold coefficient estimates.
+ * @vcv: matrix to hold the covariance matrix of the coefficients,
+ * or %NULL if this is not needed.
+ * @uhat: vector to hold the regression residuals, or %NULL if 
+ * these are not needed.
+ * @s2: pointer to receive residual variance, or %NULL.  Note:
+ * if @s2 is %NULL, the vcv estimate will be plain (X'X)^{-1}.
+ *
+ * Computes OLS estimates using SVD decomposition, and puts the
+ * coefficient estimates in @b.  Optionally, calculates the
+ * covariance matrix in @vcv and the residuals in @uhat.
+ * 
+ * Returns: 0 on success, non-zero error code on failure.
+ */
 
 int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
 			  gretl_vector *b, gretl_matrix *vcv,
@@ -1861,7 +2069,7 @@ int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
  * @uhat: vector to hold the regression residuals, or %NULL if 
  * these are not needed.
  * @s2: pointer to receive residual variance, or %NULL.  Note:
- * if s2 is NULL, the vcv estimate will be plain (X'X)^{-1}.
+ * if @s2 is %NULL, the vcv estimate will be plain (X'X)^{-1}.
  *
  * Computes OLS estimates using LU factorization, and puts the
  * coefficient estimates in @b.  Optionally, calculates the
@@ -2057,83 +2265,129 @@ gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
     return err;
 }
 
+/* computes either b'Xb (if bmod = GRETL_MOD_TRANSPOSE), 
+   or bXb' (bmod = GRETL_MOD_NONE) */
+
+static double gretl_scalar_b_X_b (const gretl_vector *b, 
+				  GretlMatrixMod bmod,
+				  const gretl_matrix *X,
+				  int *errp)
+{
+    gretl_matrix *tmp = NULL;
+    double ret = NADBL;
+    int tmpdim = (bmod == GRETL_MOD_TRANSPOSE)?
+	b->rows : b->cols;
+    int chk = (bmod == GRETL_MOD_TRANSPOSE)? 
+	b->cols : b->rows;
+    int err = 0;
+
+    if (X->rows != X->cols || tmpdim != X->rows || chk != 1) {
+	err = GRETL_MATRIX_NON_CONFORM;
+    }
+
+    if (!err) {
+	tmp = gretl_matrix_alloc(1, tmpdim);
+	if (tmp == NULL) {
+	    err = GRETL_MATRIX_NOMEM;
+	}
+    }
+
+    if (!err) {
+	err = gretl_matrix_multiply_mod(b, bmod,
+					X, GRETL_MOD_NONE,
+					tmp);
+    }
+
+    if (!err) {
+	GretlMatrixMod rmod = (bmod == GRETL_MOD_TRANSPOSE)?
+	    GRETL_MOD_NONE : GRETL_MOD_TRANSPOSE;
+
+	ret = gretl_matrix_dot_product(tmp, GRETL_MOD_NONE,
+				       b, rmod, &err);
+    }
+
+    gretl_matrix_free(tmp);
+
+    if (err) {
+	ret = NADBL;
+    }
+
+    if (errp != NULL) {
+	*errp = err;
+    }
+
+    return ret;
+}
+
 /**
  * gretl_scalar_b_prime_X_b:
  * @b: column k-vector.
  * @X: k x k matrix.
- * @err: pointer to error code variable.
+ * @errp: pointer to receive error code, or %NULL.
  *
  * Computes the scalar product, @b transpose times @X times @b.
- * On success, *err = 0, otherwise it is non-zero.
- * 
- * Returns: scalar product.
+ * If @errp is not %NULL its content is set to 0 on success, non-zero
+ * on failure.
  * 
+ * Returns: the scalar product, or #NADBL on failure.
  */
 
 double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
 				 int *errp)
 {
-    gretl_matrix *tmp = NULL;
-    double ret = NADBL;
+    return gretl_scalar_b_X_b(b, GRETL_MOD_TRANSPOSE, X, errp);
+}
 
-    *errp = 0;
+/**
+ * gretl_scalar_b_X_b_prime:
+ * @b: column k-vector.
+ * @X: k x k matrix.
+ * @errp: pointer to receive error code, or %NULL.
+ *
+ * Computes the scalar product, @b times @X times @b transpose.
+ * If @errp is not %NULL its content is set to 0 on success, non-zero
+ * on failure.
+ * 
+ * Returns: the scalar product, or #NADBL on failure.
+ */
 
-    if (b->rows != X->rows ||
-	X->rows != X->cols ||
-	b->cols != 1) {
-	*errp = GRETL_MATRIX_NON_CONFORM;
-    }
-
-    if (*errp == 0) {
-	tmp = gretl_matrix_alloc(1, b->rows);
-	if (tmp == NULL) {
-	    *errp = GRETL_MATRIX_NOMEM;
-	}
-    }
-
-    if (*errp == 0) {
-	*errp = gretl_matrix_multiply_mod(b, GRETL_MOD_TRANSPOSE,
-					  X, GRETL_MOD_NONE,
-					  tmp);
-    }
-
-    if (*errp == 0) {
-	ret = gretl_matrix_dot_product(tmp, GRETL_MOD_NONE,
-				       b, GRETL_MOD_NONE,
-				       errp);
-    }
-
-    gretl_matrix_free(tmp);
-
-    return ret;
+double gretl_scalar_b_X_b_prime (const gretl_vector *b, const gretl_matrix *X,
+				 int *errp)
+{
+    return gretl_scalar_b_X_b(b, GRETL_MOD_NONE, X, errp);
 }
 
 /**
  * gretl_matrix_A_X_A_prime:
  * @A: m * k matrix.
  * @X: k * k matrix.
- * @err: pointer to error code variable.
+ * @errp: pointer to receive error code, or %NULL.
  *
  * Computes A * X * A'.
+ * If @errp is not %NULL its content is set to 0 on success, non-zero
+ * on failure.
  * 
- * Returns: m * m matrix product, or %NULL on error (in which case
- * @err will point to a non-zero error code).
- * 
+ * Returns: m * m matrix product, or %NULL on error.
  */
 
 gretl_matrix *
 gretl_matrix_A_X_A_prime (const gretl_matrix *A, const gretl_matrix *X,
-			  int *err)
+			  int *errp)
 {
     gretl_matrix *tmp = NULL;
     gretl_matrix *ret = NULL;
     int m = A->rows;
     int k = A->cols;
+    int err = 0;
 
-    *err = 0;
+    if (errp != NULL) {
+	*errp = 0;
+    }
 
     if (X->rows != k || X->cols != k) {
-	*err = GRETL_MATRIX_NON_CONFORM;
+	if (errp != NULL) {
+	    *errp = GRETL_MATRIX_NON_CONFORM;
+	}
 	return NULL;
     }
 
@@ -2143,25 +2397,30 @@ gretl_matrix_A_X_A_prime (const gretl_matrix *A, const gretl_matrix *X,
     if (tmp == NULL || ret == NULL) {
 	gretl_matrix_free(tmp);
 	gretl_matrix_free(ret);
-	*err = GRETL_MATRIX_NOMEM;
+	if (errp != NULL) {
+	    *errp = GRETL_MATRIX_NOMEM;
+	}
 	return NULL;
     }
 
-    *err = gretl_matrix_multiply_mod(A, GRETL_MOD_NONE,
-				     X, GRETL_MOD_NONE,
-				     tmp);
+    err = gretl_matrix_multiply_mod(A, GRETL_MOD_NONE,
+				    X, GRETL_MOD_NONE,
+				    tmp);
 
-    if (!*err) {
-	*err = gretl_matrix_multiply_mod(tmp, GRETL_MOD_NONE,
-					 A, GRETL_MOD_TRANSPOSE,
-					 ret);
+    if (!err) {
+	err = gretl_matrix_multiply_mod(tmp, GRETL_MOD_NONE,
+					A, GRETL_MOD_TRANSPOSE,
+					ret);
     }
 
     gretl_matrix_free(tmp);
 
-    if (*err) {
+    if (err) {
 	gretl_matrix_free(ret);
 	ret = NULL;
+	if (errp != NULL) {
+	    *errp = err;
+	}
     }
 
     return ret;
@@ -2203,7 +2462,9 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
     int k = pmod->ncoeff;
 
     /* first ensure the model _has_ a vcv */
-    if (makevcv(pmod)) return NULL;
+    if (makevcv(pmod)) {
+	return NULL;
+    }
 
     if (select == NULL) {
 	nc = k;
@@ -2211,17 +2472,25 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
 	nc = count_selection(select, k);
     }
     
-    if (nc == 0) return NULL;
+    if (nc == 0) {
+	return NULL;
+    }
 
     vcv = gretl_matrix_alloc(nc, nc);
-    if (vcv == NULL) return NULL;
+    if (vcv == NULL) {
+	return NULL;
+    }
 
     ii = 0;
     for (i=0; i<k; i++) {
-	if (select != NULL && !select[i]) continue;
+	if (select != NULL && !select[i]) {
+	    continue;
+	}
 	jj = 0;
 	for (j=0; j<=i; j++) {
-	    if (select != NULL && !select[j]) continue;
+	    if (select != NULL && !select[j]) {
+		continue;
+	    }
 	    idx = ijton(i, j, pmod->ncoeff);
 	    gretl_matrix_set(vcv, ii, jj, pmod->vcv[idx]);
 	    if (jj != ii) {
@@ -2242,20 +2511,19 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
  * (or %NULL for the full vector).
  *
  * Produces all or part of the coefficient vector for @pmod  
- * in the form of a gretl_vector.  Storage is allocated, to be freed
+ * in the form of a gretl column vector.  Storage is allocated, to be freed
  * by the caller.  If @select is non-%NULL, it should be an array
  * with non-zero elements in positions corresponding to the
  * desired rows and zero elements otherwise.
  * 
  * Returns: the coefficient vector, or %NULL on error.
- * 
  */
 
 gretl_vector *
 gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
 {
     gretl_vector *b;
-    int i, ii, nc;
+    int i, j, nc;
     int k = pmod->ncoeff;
 
     if (select == NULL) {
@@ -2264,15 +2532,21 @@ gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
 	nc = count_selection(select, k);
     }
     
-    if (nc == 0) return NULL;
+    if (nc == 0) {
+	return NULL;
+    }
 
     b = gretl_column_vector_alloc(nc);
-    if (b == NULL) return NULL;
+    if (b == NULL) {
+	return NULL;
+    }
 
-    ii = 0;
+    j = 0;
     for (i=0; i<k; i++) {
-	if (select != NULL && !select[i]) continue;
-	gretl_vector_set(b, ii++, pmod->coeff[i]);
+	if (select != NULL && !select[i]) {
+	    continue;
+	}
+	b->val[j++] = pmod->coeff[i];
     }
 
     return b;
@@ -2329,7 +2603,8 @@ int gretl_is_zero_vector (const gretl_vector *v)
  * @Z: data array.
  * @pdinfo: pointer to data information struct.
  * @means: pointer to pick up vector of means, or %NULL to discard.
- * @err: pointer to pass back error code.
+ * @errp: pointer to receive non-zero error code in case of
+ * failure, or %NULL.
  *
  * Returns: the variance-covariance matrix of the listed variables
  * (over the currently defined data sample), or %NULL in case of
@@ -2340,7 +2615,7 @@ gretl_matrix *
 gretl_covariance_matrix_from_varlist (const int *list, const double **Z, 
 				      const DATAINFO *pdinfo, 
 				      gretl_matrix **means,
-				      int *err)
+				      int *errp)
 {
     gretl_matrix *vcv;
     gretl_vector *xbar;
@@ -2350,33 +2625,39 @@ gretl_covariance_matrix_from_varlist (const int *list, const double **Z,
 
     double vv, x, y;
     double xbi, xbj;
-    int nv;
-
-    *err = 0;
+    int nv, err = 0;
+    
+    if (errp != NULL) {
+	*errp = 0;
+    }
 
     vcv = gretl_matrix_alloc(k, k);
     if (vcv == NULL) {
-	*err = E_ALLOC;
+	if (errp != NULL) {
+	    *errp = E_ALLOC;
+	}
 	return NULL;
     }
 
     xbar = gretl_vector_alloc(k);
     if (xbar == NULL) {
-	*err = E_ALLOC;
+	if (errp != NULL) {
+	    *errp = E_ALLOC;
+	}
 	gretl_matrix_free(vcv);
 	return NULL;
     }
     
-    for (i=0; i<k && !*err; i++) {
+    for (i=0; i<k && !err; i++) {
 	xbi = gretl_mean(pdinfo->t1, pdinfo->t2, Z[list[i+1]]);
 	if (na(xbi)) {
-	    *err = E_DATA;
+	    err = E_DATA;
 	} else {
 	    gretl_vector_set(xbar, i, xbi);
 	}
     }
 
-    for (i=0; i<k && !*err; i++) {
+    for (i=0; i<k && !err; i++) {
 	xbi = gretl_vector_get(xbar, i);
 	for (j=i; j<k; j++) {
 	    xbj = gretl_vector_get(xbar, j);
@@ -2392,7 +2673,7 @@ gretl_covariance_matrix_from_varlist (const int *list, const double **Z,
 		nv++;
 	    }
 	    if (nv < 2) {
-		*err = E_DATA;
+		err = E_DATA;
 		vv = NADBL;
 	    } else {
 		vv /= (nv - 1); /* plain nv? */
@@ -2402,15 +2683,18 @@ gretl_covariance_matrix_from_varlist (const int *list, const double **Z,
 	}
     }
 
-    if (means != NULL && !*err) {
+    if (means != NULL && !err) {
 	*means = xbar;
     } else {
 	gretl_vector_free(xbar);
     }
 
-    if (*err) {
+    if (err) {
 	gretl_matrix_free(vcv);
 	vcv = NULL;
+	if (errp != NULL) {
+	    *errp = err;
+	}
     }
 
     return vcv;
