@@ -20,31 +20,48 @@
 #ifndef GRETL_MATRIX_H
 #define GRETL_MATRIX_H
 
-#include "f2c.h"
-#include "clapack_double.h"
-
 /* #define LDEBUG 1 */
 
-enum gretl_matrix_errors {
+typedef enum {
     GRETL_MATRIX_OK = 0,
     GRETL_MATRIX_NOMEM,
     GRETL_MATRIX_NON_CONFORM,
     GRETL_MATRIX_RANGE,
     GRETL_MATRIX_SINGULAR,
     GRETL_MATRIX_ERR 
-};
+} GretlMatrixError;
 
-enum gretl_matrix_mods {
+typedef enum {
     GRETL_MOD_NONE = 0,
     GRETL_MOD_TRANSPOSE,
     GRETL_MOD_SQUARE
-};
+} GretlMatrixMod;
 
 typedef struct _gretl_matrix gretl_matrix;
 typedef struct _gretl_matrix gretl_vector;
 
+/**
+ * gretl_vector_alloc:
+ * @i: number of columns.
+ *
+ * Allocates a new #gretl_vector with %i columns.
+ */
 #define gretl_vector_alloc(i) gretl_matrix_alloc(1,(i))
+
+/**
+ * gretl_column_vector_alloc:
+ * @i: number of rows.
+ *
+ * Allocates a new column gretl_vector with @i rows.
+ */
 #define gretl_column_vector_alloc(i) gretl_matrix_alloc((i),1)
+
+/**
+ * gretl_vector_free:
+ * @v: %gretl_vector to free.
+ *
+ * Frees the vector @v and its associated storage.
+ */
 #define gretl_vector_free(v) gretl_matrix_free(v)
 
 gretl_matrix *gretl_matrix_alloc (int rows, int cols);
@@ -56,11 +73,20 @@ gretl_matrix *gretl_matrix_copy (const gretl_matrix *m);
 gretl_matrix *gretl_matrix_copy_transpose (const gretl_matrix *m);
 
 gretl_vector *gretl_column_vector_from_array (const double *x, 
-					      int n, int mod);
+					      int n, GretlMatrixMod mod);
+
+gretl_vector *gretl_data_series_to_vector (const double **Z, int varno, 
+					   int t1, int t2);
 
 gretl_matrix *gretl_matrix_from_2d_array (const double **X, 
 					  int rows, int cols);
 
+void gretl_matrix_log (gretl_matrix *m);
+
+double gretl_vector_mean (const gretl_vector *v);
+
+double gretl_vector_variance (const gretl_vector *v);
+
 void gretl_matrix_zero (gretl_matrix *m);
 
 int gretl_matrix_zero_upper (gretl_matrix *m);
@@ -71,6 +97,8 @@ void gretl_matrix_multiply_by_scalar (gretl_matrix *m, double x);
 
 void gretl_matrix_divide_by_scalar (gretl_matrix *m, double x);
 
+void gretl_matrix_dot_pow (gretl_matrix *m, double x);
+
 void gretl_matrix_free (gretl_matrix *m);
 
 double *gretl_matrix_steal_data (gretl_matrix *m);
@@ -91,20 +119,25 @@ int gretl_matrix_add_to (gretl_matrix *targ, const gretl_matrix *src);
 int 
 gretl_matrix_subtract_from (gretl_matrix *targ, const gretl_matrix *src);
 
+int gretl_matrix_transpose (gretl_matrix *targ, const gretl_matrix *src);
+
 int gretl_square_matrix_transpose (gretl_matrix *m);
 
 int gretl_matrix_add_self_transpose (gretl_matrix *m);
 
-int gretl_matrix_multiply_mod (const gretl_matrix *a, int aflag,
-			       const gretl_matrix *b, int bflag,
+int gretl_matrix_multiply_mod (const gretl_matrix *a, GretlMatrixMod amod,
+			       const gretl_matrix *b, GretlMatrixMod bmod,
 			       gretl_matrix *c);
 
 int gretl_matrix_multiply (const gretl_matrix *a, const gretl_matrix *b,
 			   gretl_matrix *c);
 
-double gretl_matrix_dot_product (const gretl_matrix *a, int aflag,
-				 const gretl_matrix *b, int bflag,
-				 int *err);
+double gretl_matrix_dot_product (const gretl_matrix *a, GretlMatrixMod amod,
+				 const gretl_matrix *b, GretlMatrixMod bmod,
+				 int *errp);
+
+gretl_matrix *gretl_matrix_dot_multiply (const gretl_matrix *a, 
+					 const gretl_matrix *b);
 
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
@@ -146,11 +179,14 @@ int gretl_matrix_svd_ols (const gretl_vector *y, const gretl_matrix *X,
 			  gretl_vector *uhat, double *s2);
 
 double gretl_scalar_b_prime_X_b (const gretl_vector *b, const gretl_matrix *X,
-				 int *err);
+				 int *errp);
+
+double gretl_scalar_b_X_b_prime (const gretl_vector *b, const gretl_matrix *X,
+				 int *errp);
 
 gretl_matrix *
 gretl_matrix_A_X_A_prime (const gretl_matrix *A, const gretl_matrix *X,
-			  int *err);
+			  int *errp);
 
 gretl_matrix *
 gretl_vcv_matrix_from_model (MODEL *pmod, const char *select);
@@ -177,6 +213,6 @@ int gretl_is_zero_vector (const gretl_vector *v);
 gretl_matrix *
 gretl_covariance_matrix_from_varlist (const int *list, const double **Z, 
 				      const DATAINFO *pdinfo, 
-				      gretl_matrix **means, int *err);
+				      gretl_matrix **means, int *errp);
 
 #endif /* GRETL_MATRIX_H */
