@@ -18,7 +18,6 @@
  */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #undef MODEL_DEBUG
 
@@ -26,28 +25,45 @@ struct model_data_item_ {
     char *key;
     void *ptr;
     size_t size;
+    void (*destructor) (void *);
 };
 
-static void free_item_data (const char *key, void *ptr)
+struct ModelTest_ {
+    int type;
+    int order;
+    char *param;
+    unsigned char teststat;
+    int dfn, dfd;
+    double value;
+    double pvalue;
+};
+
+static void free_model_data_item (model_data_item *item)
 {
-    /* FIXME? (case of structs) */
-    free(ptr);
+    if (item->destructor != NULL) {
+	(*item->destructor)(item->ptr);
+    } else {
+	free(item->ptr);
+    }
+    free(item->key);
+    free(item);
 }
 
-static model_data_item *create_data_item (const char *key, void *ptr, size_t size)
+static model_data_item *create_data_item (const char *key, void *ptr, size_t size,
+					  void (*destructor) (void *))
 {
     model_data_item *item;
 
     item = malloc(sizeof *item);
     if (item != NULL) {
-	item->key = malloc(strlen(key) + 1);
+	item->key = gretl_strdup(key);
 	if (item->key == NULL) {
 	    free(item);
 	    item = NULL;
 	} else {
-	    strcpy(item->key, key);
 	    item->ptr = ptr;
 	    item->size = size;
+	    item->destructor = destructor;
 	}
     }
 
@@ -55,22 +71,33 @@ static model_data_item *create_data_item (const char *key, void *ptr, size_t siz
 }
 
 /**
- * gretl_model_set_data:
+ * gretl_model_set_data_with_destructor:
  * @pmod: pointer to #MODEL.
  * @key: key string for data, used in retrieval.
  * @ptr: data-pointer to be attached to model.
  * @size: size of data in bytes.
+ * @destructor: pointer to function that should be used to free
+ * the data-pointer in question.
  *
- * Attaches data to a model: the data can be retrieved later using
+ * Attaches data to @pmod: the data can be retrieved later using
  * gretl_model_get_data().  Note that the data are not "physically"
- * copied to the model; simply, the pointer is recorded.  The 
- * size is needed in case the model is copied.  The data pointer
- * will be freed when the model is cleared, with clear_model().
+ * copied to the model; simply, @ptr is recorded on the model.
+ * This means that the data referenced by the pointer now in 
+ * effect belong to @pmod.  When @pmod is cleared with clear_model(),
+ * @destructor will be invoked with @ptr as its single argument.
+ * If a simple "free" is OK for freeing the data, you can use
+ * gretl_model_set_data() instead.
+ *
+ * The @size is needed in case the model is copied with
+ * copy_model(), in which case the target of the copying
+ * operation receives a newly allocated copy of the data in
+ * question.
  *
  * Returns: 0 on success, 1 on failure.
  */
 
-int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
+int gretl_model_set_data_with_destructor (MODEL *pmod, const char *key, void *ptr, 
+					  size_t size, void (*destructor) (void *))
 {
     model_data_item **items;
     model_data_item *item;
@@ -81,7 +108,7 @@ int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
 
     pmod->data_items = items;
 
-    item = create_data_item(key, ptr, size);
+    item = create_data_item(key, ptr, size, destructor);
     if (item == NULL) return 1;
 
     pmod->data_items[n_items - 1] = item;
@@ -90,6 +117,35 @@ int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
     return 0;
 }
 
+/**
+ * gretl_model_set_data:
+ * @pmod: pointer to #MODEL.
+ * @key: key string for data, used in retrieval.
+ * @ptr: data-pointer to be attached to model.
+ * @size: size of data in bytes.
+ *
+ * Attaches data to @pmod: the data can be retrieved later using
+ * gretl_model_get_data().  Note that the data are not "physically"
+ * copied to the model; simply, @ptr is recorded on the model.
+ * This means that the data referenced by the pointer now in 
+ * effect belong to @pmod.  The data pointer will be freed when 
+ * @pmod is cleared with clear_model().  If the data has deep
+ * structure that requires special treatment on freeing, use
+ * gretl_model_set_data_with_destructor() instead.
+ *
+ * The @size is needed in case the model is copied with
+ * copy_model(), in which case the target of the copying
+ * operation receives a newly allocated copy of the data in
+ * question.
+ *
+ * Returns: 0 on success, 1 on failure.
+ */
+
+int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
+{
+    return gretl_model_set_data_with_destructor(pmod, key, ptr, size, NULL);
+}
+
 /**
  * gretl_model_set_int:
  * @pmod: pointer to #MODEL.
@@ -97,7 +153,7 @@ int gretl_model_set_data (MODEL *pmod, const char *key, void *ptr, size_t size)
  * @val: integer value to set.
  *
  * Records an integer value on a model: the value can be retrieved 
- * later using gretl_model_get_int().  
+ * later using gretl_model_get_int(), using the appropriate @key.  
  *
  * Returns: 0 on success, 1 on failure.
  */
@@ -127,12 +183,13 @@ int gretl_model_set_int (MODEL *pmod, const char *key, int val)
 
 /**
  * gretl_model_set_double:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @key: key string, used in retrieval.
  * @val: double-precision value to set.
  *
- * Records a floating-point value on a model: the value can be 
- * retrieved later using gretl_model_get_double().  
+ * Records a floating-point value on @pmod: the value can be 
+ * retrieved later using gretl_model_get_double() with the
+ * appropriate @key. 
  *
  * Returns: 0 on success, 1 on failure.
  */
@@ -162,11 +219,11 @@ int gretl_model_set_double (MODEL *pmod, const char *key, double val)
 
 /**
  * gretl_model_get_data_and_size:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @key: key string.
- * @sz: pointer to receive the size of the data
+ * @sz: location to receive the size of the data.
  *
- * Returns the data pointer identified by @key, or %NULL on failure.
+ * Returns: the data pointer identified by @key, or %NULL on failure.
  */
 
 void *gretl_model_get_data_and_size (const MODEL *pmod, const char *key,
@@ -188,7 +245,7 @@ void *gretl_model_get_data_and_size (const MODEL *pmod, const char *key,
 
 /**
  * gretl_model_get_data:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @key: key string.
  *
  * Returns: the data pointer identified by @key, or %NULL on failure.
@@ -201,7 +258,7 @@ void *gretl_model_get_data (const MODEL *pmod, const char *key)
 
 /**
  * gretl_model_get_int:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @key: key string.
  *
  * Returns: the integer value identified by @key, or 0 on failure.
@@ -224,7 +281,7 @@ int gretl_model_get_int (const MODEL *pmod, const char *key)
 
 /**
  * gretl_model_get_double:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @key: key string.
  *
  * Returns: the double-precision value identified by @key, or 
@@ -246,6 +303,406 @@ double gretl_model_get_double (const MODEL *pmod, const char *key)
     return NADBL;
 }
 
+static void make_cname (const char *orig, char *cname)
+{
+    char *p;
+
+    if (orig == NULL || *orig == 0) {
+	return;
+    }
+
+    p = strrchr(orig, '_');
+
+    if (p == NULL) {
+	strcpy(cname, orig);
+    } else {
+	unsigned char c = (unsigned char) *(p + 1);
+
+	if (isdigit(c)) {
+	    int lag = atoi(++p);
+
+	    sprintf(cname, "ut^2(-%d)", lag);
+	}
+    }
+}
+
+/**
+ * gretl_model_get_param_name:
+ * @pmod: pointer to model.
+ * @pdinfo: dataset information.
+ * @i: index number for parameter, zero-based, corresponding
+ * to position in the %coeff array in @pmod.
+ * @targ: string into which to write param name.
+ *
+ * Writes the appropriate parameter name into @targ, which
+ * should be at least #VNAMELEN bytes long.  Usually this is
+ * the name of a variable in the dataset, but sometimes it is
+ * a special string (e.g. for nonlinear models).
+ *
+ * Returns: @targ.
+ */
+
+char *gretl_model_get_param_name (const MODEL *pmod, const DATAINFO *pdinfo,
+				  int i, char *targ)
+{
+    *targ = '\0';
+
+    if (pmod != NULL) {
+	/* special treatment for ARCH, ARMA, GARCH, NLS */
+	if (pmod->aux == AUX_ARCH) {
+	    make_cname(pdinfo->varname[pmod->list[i + 2]], targ);
+	} else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
+	    strcpy(targ, pmod->params[i + 1]);
+	} else {
+	    strcpy(targ, pdinfo->varname[pmod->list[i + 2]]);
+	}
+    }
+
+    return targ;
+}
+
+void free_coeff_intervals (CoeffIntervals *cf)
+{
+    int i;
+
+    free(cf->coeff);
+    free(cf->maxerr);
+
+    if (cf->names != NULL) {
+	for (i=0; i<cf->ncoeff; i++) {
+	    free(cf->names[i]);
+	}
+	free(cf->names);
+    }
+
+    free(cf);
+}
+
+/**
+ * gretl_model_get_coeff_intervals:
+ * @pmod: pointer to gretl model.
+ * @pdinfo: dataset information.
+ *
+ * Save the 95 percent confidence intervals for the parameter
+ * estimates in @pmod.
+ * 
+ * Returns: pointer to #CONFINT struct containing the results.
+ */
+
+CoeffIntervals *
+gretl_model_get_coeff_intervals (const MODEL *pmod, 
+				 const DATAINFO *pdinfo)
+{
+    CoeffIntervals *cf;
+    double t = tcrit95(pmod->dfd);
+    char pname[24];
+    int i, err = 0;
+
+    cf = malloc(sizeof *cf);
+    if (cf == NULL) {
+	return NULL;
+    }
+
+    cf->ncoeff = pmod->ncoeff;
+    cf->df = pmod->dfd;
+    cf->ifc = pmod->ifc;
+
+    cf->coeff = NULL;
+    cf->maxerr = NULL;
+    cf->names = NULL;
+
+    cf->coeff = malloc(cf->ncoeff * sizeof *cf->coeff);
+    if (cf->coeff == NULL) {
+	err = 1;
+	goto bailout;
+    }
+
+    cf->maxerr = malloc(cf->ncoeff * sizeof *cf->maxerr);
+    if (cf->maxerr == NULL) {
+	err = 1;
+	goto bailout;
+    }
+
+    cf->names = malloc(cf->ncoeff * sizeof *cf->names);
+    if (cf->names == NULL) {
+	err = 1;
+	goto bailout;
+    }    
+
+    for (i=0; i<cf->ncoeff && !err; i++) { 
+	cf->coeff[i] = pmod->coeff[i];
+	cf->maxerr[i] = (pmod->sderr[i] > 0)? t * pmod->sderr[i] : 0.0;
+	gretl_model_get_param_name(pmod, pdinfo, i, pname);
+	cf->names[i] = gretl_strdup(pname);
+	if (cf->names[i] == NULL) {
+	    int j;
+	    
+	    for (j=0; j<i; j++) {
+		free(cf->names[i]);
+	    }
+	    free(cf->names);
+	    cf->names = NULL;
+	    err = 1;
+	}
+    }
+
+ bailout:
+
+    if (err) {
+	free_coeff_intervals(cf);
+	cf = NULL;
+    }
+
+    return cf;
+}
+
+/**
+ * gretl_arma_model_get_AR_order:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: the autoregressive order of @pmod, or 0 if
+ * @pmod is not an ARMA model.
+ */
+
+int gretl_arma_model_get_AR_order (const MODEL *pmod)
+{
+    int p = 0;
+
+    if (pmod->ci == ARMA) {
+	p = pmod->list[1];
+    }
+
+    return p;
+}
+
+/**
+ * gretl_arma_model_get_MA_order:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: the moving-average order of @pmod, or 0 if
+ * @pmod is not an ARMA model.
+ */
+
+int gretl_arma_model_get_MA_order (const MODEL *pmod)
+{
+    int q = 0;
+
+    if (pmod->ci == ARMA) {
+	q = pmod->list[2];
+    }
+
+    return q;
+}
+
+/**
+ * gretl_model_get_depvar:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: the ID number of the dependent variable in @pmod.
+ */
+
+int gretl_model_get_depvar (const MODEL *pmod)
+{
+    int dv = 0;
+
+    if (pmod != NULL && pmod->list != NULL) {
+	if (pmod->ci == GARCH) {
+	    dv = pmod->list[4];
+	} else if (pmod->ci == ARMA) {
+	    if (gretl_model_get_int(pmod, "seasonal")) {
+		dv = pmod->list[7];
+	    } else {
+		dv = pmod->list[4];
+	    }
+	} else {
+	    dv = pmod->list[1];
+	}
+    }
+
+    return dv;
+}
+
+/**
+ * gretl_model_get_x_list:
+ * @pmod: pointer to gretl model.
+ *
+ * Returns: an allocated copy of the list of independent
+ * variables included in @pmod, or %NULL on failure. 
+ */
+
+int *gretl_model_get_x_list (const MODEL *pmod)
+{
+    int *list = NULL;
+    int i, nx;
+
+    if (pmod->ci == ARMA) {
+	int start;
+
+	if (gretl_model_get_int(pmod, "seasonal")) {
+	    start = 7;
+	} else {
+	    start = 4;
+	}
+	nx = pmod->list[0] - start + pmod->ifc;
+	if (nx > 0) {
+	    list = gretl_list_new(nx);
+	    if (list != NULL) {
+		if (pmod->ifc) {
+		    list[1] = 0;
+		    for (i=2; i<=list[0]; i++) {
+			list[i] = pmod->list[i + start - 1];
+		    }
+		} else {
+		    for (i=1; i<=list[0]; i++) {
+			list[i] = pmod->list[i + start];
+		    }
+		}		    
+	    }
+	}
+    } else if (pmod->ci == GARCH) {
+	nx = pmod->list[0] - 4;
+	if (nx > 0) {
+	    list = gretl_list_new(nx);
+	    if (list != NULL) {
+		for (i=1; i<=list[0]; i++) {
+		    list[i] = pmod->list[i + 4];
+		}
+	    }
+	}
+    } else if (pmod->ci != NLS) {
+	nx = pmod->ncoeff;
+	list = gretl_list_new(nx);
+	if (list != NULL) {
+	    for (i=1; i<=list[0]; i++) {
+		list[i] = pmod->list[i + 1];
+	    }
+	}
+    }
+
+    return list;
+}
+
+/**
+ * gretl_model_new_vcv:
+ * @pmod: pointer to model.
+ * @nelem: pointer to receive number of elements in
+ * the packed array, or %NULL;
+ * 
+ * Allocates space for a packed coefficient covariance matrix
+ * in @pmod (if such space is not already allocated).  Sets
+ * all entries in the array to zero.
+ *
+ * Returns: 0 on success, %E_ALLOC on error.
+ */
+
+int gretl_model_new_vcv (MODEL *pmod, int *nelem)
+{
+    int nv = pmod->ncoeff;
+    int nxpx = (nv * nv + nv) / 2; 
+    int i, err = 0;
+
+    if (pmod->vcv == NULL) {
+	/* not already allocated */
+	pmod->vcv = malloc(nxpx * sizeof *pmod->vcv);
+	if (pmod->vcv == NULL) {
+	    err = E_ALLOC;
+	} 
+    }
+
+    if (pmod->vcv != NULL) {
+	for (i=0; i<nxpx; i++) {
+	    pmod->vcv[i] = 0.0;
+	}
+	if (nelem != NULL) {
+	    *nelem = nxpx;
+	}
+    }	
+
+    return err;
+}
+
+/**
+ * gretl_model_get_vcv:
+ * @pmod: pointer to model.
+ * @pdinfo: dataset information.
+ * 
+ * Supplies the caller with a copy of the variance-covariance 
+ * matrix for the parameter estimates in @pmod.  See also
+ * free_vcv().  To get the covariance matrix in gretl_matrix
+ * format, see gretl_vcv_matrix_from_model().
+ *
+ * Returns: #VMatrix struct or %NULL on error.
+ */
+
+VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo)
+{
+    char varname[VNAMELEN];
+    int i, nt, nc = pmod->ncoeff;
+    VMatrix *vcv;
+
+    vcv = vmatrix_new();
+
+    if (vcv == NULL) {
+	return NULL;
+    }
+
+    vcv->names = create_strings_array(nc);
+    if (vcv->names == NULL) {
+	free(vcv);
+	return NULL;
+    }
+
+    for (i=0; i<nc; i++) {
+	gretl_model_get_param_name(pmod, pdinfo, i, varname);
+	vcv->names[i] = gretl_strdup(varname);
+	if (vcv->names[i] == NULL) {
+	    free_vmatrix(vcv);
+	    return NULL;
+	}
+    }
+
+    if (pmod->vcv == NULL && makevcv(pmod)) {
+	free_vmatrix(vcv);
+	return NULL;
+    }
+
+    /* calculate number of elements in vcv */
+    nt = (nc * nc + nc) / 2;
+
+    /* copy vcv */
+    vcv->vec = copyvec(pmod->vcv, nt);
+    if (vcv->vec == NULL) {
+	free_vmatrix(vcv);
+	return NULL;
+    }
+
+    vcv->ci = pmod->ci;
+    vcv->dim = nc;
+    vcv->t1 = pmod->t1;
+    vcv->t2 = pmod->t2;
+    
+    return vcv;
+}
+
+/**
+ * impose_model_smpl:
+ * @pmod: pointer to model.
+ * @pdinfo: dataset information.
+ *
+ * Sets on @pdinfo the sample range (starting and ending
+ * observations) that was in effect when @pmod was estimated.
+ * This is not always the same as the data range over which
+ * @pmod was actually estimated (e.g. in case of 
+ * autoregressive models, where observations are dropped
+ * to allow for lags).
+ */
+
+void impose_model_smpl (const MODEL *pmod, DATAINFO *pdinfo)
+{
+    pdinfo->t1 = pmod->smpl.t1;
+    pdinfo->t2 = pmod->smpl.t2;
+}
+
 #ifdef HAVE_X12A
 
 static void maybe_delete_x12_file (const MODEL *pmod)
@@ -260,27 +717,26 @@ static void maybe_delete_x12_file (const MODEL *pmod)
 
 static void destroy_all_data_items (MODEL *pmod)
 {
-    model_data_item *item;
     int i;
 
-    if (pmod->n_data_items == 0) return;
+    if (pmod->n_data_items == 0) {
+	return;
+    }
 
 #ifdef HAVE_X12A
     maybe_delete_x12_file(pmod);
 #endif
 
     for (i=0; i<pmod->n_data_items; i++) {
-	item = pmod->data_items[i];
-	free_item_data(item->key, item->ptr);
-	free(item->key);
-	free(item);
+	free_model_data_item(pmod->data_items[i]);
     }
 
     free(pmod->data_items);
     pmod->data_items = NULL;
 }
 
-int gretl_model_destroy_data_item (MODEL *pmod, const char *key)
+static int discard_model_data_item (MODEL *pmod, const char *key,
+				    int free_data)
 {
     model_data_item *junk = NULL;
     int i, targ = 0;
@@ -311,16 +767,72 @@ int gretl_model_destroy_data_item (MODEL *pmod, const char *key)
 
 	pmod->n_data_items -= 1;
 
-	free(junk->key);
-	free(junk);
+	if (free_data) {
+	    /* deep free the data item */
+	    free_model_data_item(junk);
+	} else {
+	    /* just free the item, not the actual data */
+	    free(junk->key);
+	    free(junk);
+	}
     }
 
     return err;
 }
 
-/* .......................................................... */
+/**
+ * gretl_model_destroy_data_item:
+ * @pmod: pointer to model.
+ * @key: key string.
+ *
+ * Looks up the data pointer, attached to @pmod, that is
+ * identified by @key, and if a pointer is found, frees
+ * it (or applies the destructor function that was set for
+ * the item, if any) and removes it from the model's list of 
+ * data items.  If you want to remove the item from the
+ * model's list without freeing the underlying data pointer,
+ * use gretl_model_detach_data_item().
+ *
+ * Returns: 0 on success, 1 on failure (pointer not found).
+ */
 
-void gretl_model_set_auxiliary (MODEL *pmod, int aux)
+int gretl_model_destroy_data_item (MODEL *pmod, const char *key)
+{
+    return discard_model_data_item(pmod, key, 1);
+}
+
+/**
+ * gretl_model_detach_data_item:
+ * @pmod: pointer to model.
+ * @key: key string.
+ *
+ * Looks up the data item, attached to @pmod, that is
+ * identified by @key, and if an item is found, removes
+ * it from the model's list of such items.  The data
+ * pointer associated with @key is not touched.  If you
+ * want the underlying resources associated with @key to be 
+ * freed, use gretl_model_destroy_data_item().
+ *
+ * Returns: 0 on success, 1 on failure (key not found).
+ */
+
+int gretl_model_detach_data_item (MODEL *pmod, const char *key)
+{
+    return discard_model_data_item(pmod, key, 0);
+}
+
+/**
+ * gretl_model_set_auxiliary:
+ * @pmod: pointer to model.
+ * @aux: code indicating a model's function in an
+ * auxiliary role (typically, in relation to a hypothesis
+ * test on another model).
+ *
+ * Sets an auxiliary code on @pmod, which may be relevant
+ * for how the model is printed.
+ */
+
+void gretl_model_set_auxiliary (MODEL *pmod, ModelAuxCode aux)
 {
     pmod->aux = aux;
 }
@@ -328,7 +840,7 @@ void gretl_model_set_auxiliary (MODEL *pmod, int aux)
 static void gretl_model_init_pointers (MODEL *pmod)
 {
     pmod->list = NULL;
-    pmod->subdum = NULL;
+    pmod->submask = NULL;
     pmod->missmask = NULL;
     pmod->coeff = NULL;
     pmod->sderr = NULL;
@@ -347,10 +859,13 @@ static void gretl_model_init_pointers (MODEL *pmod)
 
 /**
  * gretl_model_init:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  *
  * Initializes a gretl #MODEL, including setting its pointer members
- * to %NULL.
+ * to %NULL. This initialization should be done if the caller has
+ * declared a #MODEL struct directly, rather than obtaining a pointer to
+ * #MODEL using gretl_model_new() (in which case the initialization is
+ * done automatically).
  */
 
 void gretl_model_init (MODEL *pmod)
@@ -380,11 +895,16 @@ void gretl_model_init (MODEL *pmod)
 
 /**
  * gretl_model_smpl_init:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  * @pdinfo: dataset information.
  *
- * Sets the start and end of the model's sample to the current dataset
- * values.
+ * Records the start and end of the current sample range in
+ * the model @pmod, which may be necessary for future reference
+ * if a hypothesis test is to be performed.  Note that this
+ * sample range may not be the same as the data range over
+ * which the model is actually estimated (for example, in the
+ * case of autoregressive models where observations have to
+ * be dropped to allow for lags).
  */
 
 void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo)
@@ -396,10 +916,10 @@ void gretl_model_smpl_init (MODEL *pmod, const DATAINFO *pdinfo)
 /**
  * gretl_model_new:
  * 
- * Allocates memory for a gretl MODEL struct and initializes the struct,
+ * Allocates memory for a gretl #MODEL struct and initializes the struct,
  * using gretl_model_init().
  *
- * Returns: pointer to #MODEL (or %NULL if allocation fails).
+ * Returns: pointer to model (or %NULL if allocation fails).
  */
 
 MODEL *gretl_model_new (void)
@@ -410,26 +930,6 @@ MODEL *gretl_model_new (void)
     return pmod;
 }
 
-/**
- * exchange_smpl:
- * @pmod: pointer to #MODEL.
- * @pdinfo: pointer to data information struct.
- * 
- * Swaps the starting and ending values for the data sample
- * between @pmod and @pdinfo.
- */
-
-void exchange_smpl (MODEL *pmod, DATAINFO *pdinfo)
-{
-    int t1 = pdinfo->t1, t2 = pdinfo->t2;
-
-    pdinfo->t1 = pmod->smpl.t1;
-    pdinfo->t2 = pmod->smpl.t2;
-
-    pmod->smpl.t1 = t1;
-    pmod->smpl.t2 = t2;
-}
-
 /* .......................................................... */
 
 static void clear_ar_info (MODEL *pmod)
@@ -456,7 +956,7 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
     fprintf(stderr, "%s:\n"
 	    " pmod = %p\n"
 	    " pmod->list = %p\n"
-	    " pmod->subdum = %p\n"
+	    " pmod->submask = %p\n"
 	    " pmod->missmask = %p\n"
 	    " pmod->coeff = %p\n"
 	    " pmod->sderr = %p\n"
@@ -470,7 +970,7 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
 	    " pmod->tests = %p\n"
 	    " pmod->data = %p\n", msg,
 	    (void *) pmod, (void *) pmod->list, 
-	    (void *) pmod->subdum, (void *) pmod->missmask, 
+	    (void *) pmod->submask, (void *) pmod->missmask, 
 	    (void *) pmod->coeff, (void *) pmod->sderr, 
 	    (void *) pmod->yhat, (void *) pmod->uhat, 
 	    (void *) pmod->xpx, (void *) pmod->vcv, 
@@ -481,7 +981,7 @@ debug_print_model_info (const MODEL *pmod, const char *msg)
 
 #endif /* MODEL_DEBUG */
 
-static void gretl_test_free (GRETLTEST *test)
+static void gretl_test_free (ModelTest *test)
 {
     if (test->param != NULL) {
 	free(test->param);
@@ -490,11 +990,14 @@ static void gretl_test_free (GRETLTEST *test)
 
 /**
  * clear_model:
- * @pmod: pointer to #MODEL.
+ * @pmod: pointer to model.
  *
  * Clears a gretl #MODEL, freeing all allocated storage and setting
  * pointer members to %NULL.  Also frees any data pointers attached
- * via gretl_model_set_data().
+ * via gretl_model_set_data().  The model pointer itself is not
+ * freed, so this function may be called on a #MODEL which has been 
+ * declared directly by the caller (by passing the address of the
+ * #MODEL).  
  */
 
 void clear_model (MODEL *pmod)
@@ -506,7 +1009,7 @@ void clear_model (MODEL *pmod)
 	debug_print_model_info(pmod, "Doing clear_model");
 #endif
 	if (pmod->list) free(pmod->list);
-	if (pmod->subdum) free(pmod->subdum);
+	if (pmod->submask) free(pmod->submask);
 	if (pmod->missmask) free(pmod->missmask);
 	if (pmod->coeff) free(pmod->coeff);
 	if (pmod->sderr) free(pmod->sderr);
@@ -542,10 +1045,16 @@ void clear_model (MODEL *pmod)
 
 /* ........................................................... */
 
-static void copy_test (GRETLTEST *targ, const GRETLTEST *src)
+static void copy_test (ModelTest *targ, const ModelTest *src)
 {
     targ->type = src->type;
-    targ->param = gretl_strdup(src->param);
+    
+    if (src->param != NULL) {
+	targ->param = gretl_strdup(src->param);
+    } else {
+	targ->param = NULL;
+    }
+
     targ->teststat = src->teststat;
     targ->dfn = src->dfn;
     targ->dfd = src->dfd;
@@ -557,7 +1066,9 @@ static int copy_model_tests (MODEL *targ, const MODEL *src)
 {
     int i, n = src->ntests;
 
-    if (n <= 0 || src->tests == NULL) return 0;
+    if (n <= 0 || src->tests == NULL) {
+	return 0;
+    }
 
     targ->tests = malloc(n * sizeof *targ->tests);
     if (targ->tests == NULL) return 1;
@@ -569,9 +1080,9 @@ static int copy_model_tests (MODEL *targ, const MODEL *src)
     return 0;
 }
 
-void gretl_test_init (GRETLTEST *test, int which)
+static void gretl_test_init (ModelTest *test, ModelTestType ttype)
 {
-    test->type = which;
+    test->type = ttype;
     test->order = 0;
     test->param = NULL;
     test->teststat = 0;
@@ -579,42 +1090,87 @@ void gretl_test_init (GRETLTEST *test, int which)
     test->value = test->pvalue = NADBL;
 }
 
-int add_test_to_model (MODEL *pmod, GRETLTEST *test)
+/**
+ * new_test_on_model:
+ * @pmod: pointer to model.
+ * @ttype: type of test to add.
+ *
+ * Adds a #ModelTest to @pmod, if a test of the given type has
+ * not already been performed and recorded.
+ *
+ * Returns: model test pointer, or %NULL if the test is
+ * already present or on failure.
+ */
+
+ModelTest *new_test_on_model (MODEL *pmod, ModelTestType ttype)
 {
-    GRETLTEST *tests;
+    ModelTest *tests = NULL;
+    ModelTest *ret = NULL;
     int i, nt = pmod->ntests;
+    int done = 0;
 
     for (i=0; i<nt; i++) {
-	if (test->type == pmod->tests[i].type) {
+	if (ttype == pmod->tests[i].type) {
 	    /* already done */
-	    return -1;
+	    done = 1;
 	}
     }
 
-    tests = realloc(pmod->tests, (nt + 1) * sizeof *tests);
-    if (tests == NULL) {
-	return 1;
+    if (!done) {
+	tests = realloc(pmod->tests, (nt + 1) * sizeof *tests);
     }
 
-    pmod->tests = tests;
+    if (tests != NULL) {
+	pmod->tests = tests;
+	pmod->ntests += 1;
+	ret = &pmod->tests[nt];
+	gretl_test_init(ret, ttype);
+    }
 
-    pmod->tests[nt].type = test->type;
-
-    pmod->tests[nt].param = test->param;
-    test->param = NULL;
-
-    pmod->tests[nt].teststat = test->teststat;
-    pmod->tests[nt].value = test->value;
-    pmod->tests[nt].dfn = test->dfn;
-    pmod->tests[nt].dfd = test->dfd;
-    pmod->tests[nt].pvalue = test->pvalue;
-
-    pmod->ntests += 1;
-
-    return 0;
+    return ret;
 }
 
-static int gretl_test_print_string (GRETLTEST *test, PRN *prn)
+void model_test_set_teststat (ModelTest *test, unsigned char ts)
+{
+    test->teststat = ts;
+}
+
+void model_test_set_order (ModelTest *test, int order)
+{
+    test->order = order;
+}
+
+void model_test_set_dfn (ModelTest *test, int df)
+{
+    test->dfn = df;
+}
+
+void model_test_set_dfd (ModelTest *test, int df)
+{
+    test->dfd = df;
+}
+
+void model_test_set_value (ModelTest *test, double val)
+{
+    test->value = val;
+}
+
+void model_test_set_pvalue (ModelTest *test, double pval)
+{
+    test->pvalue = pval;
+}
+
+void model_test_set_param (ModelTest *test, const char *s)
+{
+    test->param = gretl_strdup(s);
+}
+
+void model_test_set_allocated_param (ModelTest *test, char *s)
+{
+    test->param = s;
+}
+
+static int gretl_test_print_string (const ModelTest *test, PRN *prn)
 {
     const char *test_strs[] = {
 	N_("Test for addition of variables"),
@@ -630,7 +1186,7 @@ static int gretl_test_print_string (GRETLTEST *test, PRN *prn)
 	N_("Non-linearity test (squares)"),
 	N_("White's test for heteroskedasticity")
     };
-    char ord[16];
+    char ordstr[16];
     char *param = NULL;
 
     if (test->type >= GRETL_TEST_MAX) {
@@ -638,8 +1194,8 @@ static int gretl_test_print_string (GRETLTEST *test, PRN *prn)
     }
 
     if (test->order > 0) {
-	sprintf(ord, "%d", test->order);
-	param = ord;
+	sprintf(ordstr, "%d", test->order);
+	param = ordstr;
     } else if (test->type == GRETL_TEST_CHOW) {
 	param = test->param;
     }
@@ -698,7 +1254,7 @@ static int print_add_omit_varnames (const char *s, PRN *prn)
     return 0;
 }
 
-static int gretl_test_print_h_0 (GRETLTEST *test, PRN *prn)
+static int gretl_test_print_h_0 (const ModelTest *test, PRN *prn)
 {
     const char *h_0_strs[] = {
 	N_("parameters are zero for the variables"),
@@ -733,15 +1289,15 @@ static int gretl_test_print_h_0 (GRETLTEST *test, PRN *prn)
 }
 
 static void 
-get_test_stat_string (const GRETLTEST *test, char *str, int format)
+get_test_stat_string (const ModelTest *test, char *str, PRN *prn)
 {
-    int tex = is_tex(format);
+    int tex = tex_format(prn);
 
     switch (test->teststat) {
     case GRETL_STAT_TR2:
 	if (tex) {
 	    sprintf(str, "$TR^2$ = %g", test->value);
-	} else if (is_rtf(format)) {
+	} else if (rtf_format(prn)) {
 	    sprintf(str, "TR{\\super 2} = %g", test->value);
 	} else {
 	    sprintf(str, "TR^2 = %g", test->value);
@@ -773,6 +1329,7 @@ get_test_stat_string (const GRETLTEST *test, char *str, int format)
 	}
 	break;
     case GRETL_STAT_LR:
+    case GRETL_STAT_WALD_CHISQ:
 	if (tex) {
 	    sprintf(str, "$\\chi^2_%d$ = %g", test->dfn, test->value); 
 	} else {
@@ -785,9 +1342,9 @@ get_test_stat_string (const GRETLTEST *test, char *str, int format)
 }
 
 static void 
-get_test_pval_string (const GRETLTEST *test, char *str, int format)
+get_test_pval_string (const ModelTest *test, char *str, PRN *prn)
 {
-    int tex = is_tex(format);
+    int tex = tex_format(prn);
 
     switch (test->teststat) {
     case GRETL_STAT_TR2:
@@ -826,6 +1383,7 @@ get_test_pval_string (const GRETLTEST *test, char *str, int format)
 	break;
     case GRETL_STAT_NORMAL_CHISQ:
     case GRETL_STAT_LR:
+    case GRETL_STAT_WALD_CHISQ:
 	sprintf(str, "%g", test->pvalue);
 	break;
     default:
@@ -833,12 +1391,26 @@ get_test_pval_string (const GRETLTEST *test, char *str, int format)
     }
 }
 
-void gretl_model_test_print (GRETLTEST *test, PRN *prn)
+void gretl_model_test_print (const MODEL *pmod, int i, PRN *prn)
 {
+    const ModelTest *test;
     char test_str[128], pval_str[128];
+    const char *tstat;
 
-    get_test_stat_string(test, test_str, prn->format);
-    get_test_pval_string(test, pval_str, prn->format);
+    if (i >= pmod->ntests) {
+	return;
+    }
+
+    test = &pmod->tests[i];
+
+    if (test->teststat == GRETL_STAT_WALD_CHISQ) {
+	tstat = N_("Asymptotic test statistic");
+    } else {
+	tstat = N_("Test statistic");
+    }
+
+    get_test_stat_string(test, test_str, prn);
+    get_test_pval_string(test, pval_str, prn);
 
     if (plain_format(prn)) {
 	gretl_test_print_string(test, prn);
@@ -846,7 +1418,7 @@ void gretl_model_test_print (GRETLTEST *test, PRN *prn)
 	gretl_test_print_h_0(test, prn);
 	pprintf(prn, "\n  %s: %s\n"
 		"  %s = %s\n\n",
-		_("Test statistic"), test_str, 
+		_(tstat), test_str, 
 		_("with p-value"), pval_str);
     } else if (tex_format(prn)) {
 	gretl_test_print_string(test, prn);
@@ -854,7 +1426,7 @@ void gretl_model_test_print (GRETLTEST *test, PRN *prn)
 	gretl_test_print_h_0(test, prn);
 	pprintf(prn, "\\\\\n\\quad %s: %s\\\\\n"
 		"\\quad %s = %s\\\\\n",
-		I_("Test statistic"), test_str, 
+		I_(tstat), test_str, 
 		I_("with p-value"), pval_str);
     } else if (rtf_format(prn)) {
 	pputs(prn, "\\par \\ql ");
@@ -863,11 +1435,16 @@ void gretl_model_test_print (GRETLTEST *test, PRN *prn)
 	gretl_test_print_h_0(test, prn);
 	pprintf(prn, "\\par\n %s: %s\\par\n"
 		" %s = %s\\par\n\n",
-		I_("Test statistic"), test_str, 
+		I_(tstat), test_str, 
 		I_("with p-value"), pval_str);
     }
 }
 
+void gretl_model_print_last_test (const MODEL *pmod, PRN *prn)
+{
+    gretl_model_test_print(pmod, pmod->ntests - 1, prn);
+}
+
 static ARINFO *copy_ar_info (const ARINFO *src)
 {
     ARINFO *targ;
@@ -971,6 +1548,7 @@ static int copy_model_data_items (MODEL *targ, const MODEL *src)
 
 	memcpy(targitem->ptr, srcitem->ptr, srcitem->size);
 	targitem->size = srcitem->size;
+	targitem->destructor = srcitem->destructor;
     }
 
     if (err) {
@@ -1032,8 +1610,8 @@ int copy_model (MODEL *targ, const MODEL *src, const DATAINFO *pdinfo)
 	return 1;
     if ((targ->yhat = copyvec(src->yhat, pdinfo->n)) == NULL) 
 	return 1;
-    if (src->subdum != NULL && 
-	(targ->subdum = copy_subdum(src->subdum, pdinfo->n)) == NULL) 
+    if (src->submask != NULL && 
+	(targ->submask = copy_subsample_mask(src->submask, pdinfo->n)) == NULL) 
 	return 1;
     if (src->missmask != NULL && 
 	(targ->missmask = copy_missmask(src)) == NULL) 
@@ -1098,11 +1676,57 @@ int swap_models (MODEL **targ, MODEL **src)
     return 0;
 }
 
+int is_model_cmd (const char *s)
+{
+    int ret = 0;
+
+    if (s != NULL && *s != '\0') {
+	if (!strncmp(s, "ols", 3)  ||
+	    !strncmp(s, "corc", 4) ||
+	    !strncmp(s, "hilu", 4) ||
+	    !strncmp(s, "wls", 3)  ||
+	    !strncmp(s, "pwe", 3)  ||
+	    !strncmp(s, "pooled", 6)  ||
+	    !strncmp(s, "hccm", 4) ||
+	    !strncmp(s, "hsk", 3)  ||
+	    !strncmp(s, "add ", 4)  ||
+	    !strncmp(s, "lad", 3)  ||
+	    !strncmp(s, "omit", 4) ||
+	    !strncmp(s, "tsls", 4) ||
+	    !strncmp(s, "logit", 5)  ||
+	    !strncmp(s, "probit", 6) ||
+	    !strncmp(s, "tobit", 5) ||
+	    !strncmp(s, "poisson", 7) ||
+	    !strncmp(s, "garch", 5) ||
+	    !strncmp(s, "logistic", 8) ||
+	    !strncmp(s, "end nls", 7) ||
+	    !strncmp(s, "arma", 4) ||
+	    !strncmp(s, "ar ", 3) ||
+	    !strcmp(s, "ar")) {
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
+int is_quiet_model_test (int ci, gretlopt opt)
+{
+    int ret = 0;
+
+    if ((opt & OPT_Q) && (ci == OMIT || ci == ADD ||
+			  ci == OMITFROM || ci == ADDTO)) {
+	ret = 1;
+    }
+
+    return ret;
+}
+
 /**
  * command_ok_for_model:
  * @test_ci:  index of command to be tested.
- * @model_ci: command index of a gretl #MODEL (for example,
- * OLS, HCCM or CORC).
+ * @model_ci: command index of a gretl model (for example,
+ * %OLS, %HCCM or %CORC).
  *
  * Returns: 1 if the model-related command in question is
  * meaningful and acceptable in the context of the specific
@@ -1116,24 +1740,20 @@ int command_ok_for_model (int test_ci, int model_ci)
     switch (test_ci) {
     case ADD:
     case ADDTO:
-    case OMIT:
-    case OMITFROM:
     case COEFFSUM:
     case VIF:
-	if (model_ci == TSLS || model_ci == NLS || 
+	if (model_ci == NLS || model_ci == TSLS ||
 	    model_ci == ARMA || model_ci == GARCH) ok = 0;
 	break;
 
+    case OMIT:
+    case OMITFROM:
+	if (model_ci == NLS || model_ci == ARMA || 
+	    model_ci == GARCH) ok = 0;
+	break;
+
     case EQNPRINT:
-	if (model_ci != OLS) ok = 0; /* unduly restrictive? */
-	break;
-
-    case FCAST:
-    case FIT:
-	break;
-
-    case FCASTERR:
-	if (model_ci != OLS) ok = 0;
+	if (model_ci != OLS) ok = 0; /* FIXME: unduly restrictive? */
 	break;
 
     case LMTEST:
@@ -1155,7 +1775,7 @@ int command_ok_for_model (int test_ci, int model_ci)
 	if (model_ci == LAD || model_ci == NLS) ok = 0;
 	break;
     case TESTUHAT:
-	/* need to exclude garch? */
+	/* do we really need to exclude garch? */
 	if (model_ci == TOBIT || model_ci == GARCH) ok = 0;
 	break;
 
@@ -1270,9 +1890,9 @@ int mle_aic_bic (MODEL *pmod, int addk)
 double coeff_pval (const MODEL *pmod, double x, int df)
 {
     if (0 && ML_ESTIMATOR(pmod->ci)) {
-	return gaussprob(x);
+	return normal_pvalue_2(x);
     } else {
-	return tprob(x, df);
+	return t_pvalue_2(x, df);
     }
 }
 
