@@ -30,12 +30,12 @@
 # include <dirent.h>
 #endif
 
-#ifndef WIN32
+#if defined(USE_GTK2) && !defined(WIN32)
 # include <glib.h>
 # if GLIB_CHECK_VERSION(2,0,0)
 #  define GLIB2
 # endif /* GLIB_CHECK_VERSION */
-#endif /* ! WIN32 */
+#endif /* GTK2, !WIN32 */
 
 #if (GLIB_MAJOR_VERSION >= 2) && (GLIB_MINOR_VERSION >= 6)
 # ifdef WIN32
@@ -95,12 +95,14 @@ gzFile gretl_gzopen (const char *filename, const char *mode)
 
 /* .......................................................... */
 
-int path_append (char *file, const char *path)
+int gretl_path_prepend (char *file, const char *path)
 {
     char temp[MAXLEN];
     int n, pathlen = strlen(file) + strlen(path) + 1;
 
-    if (pathlen > MAXLEN) return 1;
+    if (pathlen > MAXLEN) {
+	return 1;
+    }
 
     strcpy(temp, path);
     n = strlen(temp);
@@ -296,7 +298,7 @@ static char *search_dir (char *fname, const char *topdir, int code)
 
     strcpy(orig, fname);
 
-    if (path_append(fname, topdir) == 0) {
+    if (gretl_path_prepend(fname, topdir) == 0) {
 	test = gretl_fopen(fname, "r");
 	if (test != NULL) {
 	    fclose(test);
@@ -611,9 +613,12 @@ const char *gretl_plotfile (void)
     return gretl_paths.plotfile;
 }
 
-void set_gretl_plotfile (const char *fname)
+char *set_gretl_plotfile (const char *fname)
 {
-    strcpy(gretl_paths.plotfile, fname);
+    *gretl_paths.plotfile = 0;
+    strncat(gretl_paths.plotfile, fname, MAXLEN - 1);
+
+    return gretl_paths.plotfile;
 }
 
 const char *gretl_png_font (void)
