@@ -23,26 +23,18 @@
 #define INTERACT_H
 
 #define MAXSAVENAME 32
-#define VARDUP     999
 #define CMD_NULL    -1
 #define CMD_COMMENT -2
 
 typedef struct _CMD CMD;
 
-struct _CMD {
-    char word[9];               /* command word */
-    int ci;                     /* command index number */
-    int context;                /* context for subsetted commands */
-    gretlopt opt;               /* option flags */
-    char savename[MAXSAVENAME]; /* name used to save an object from the command */
-    char str[4];                /* used, e.g., in "multiply" command */
-    int nolist;                 /* = 1 if the command does not take a list */
-    int *list;                  /* list of variables by ID number */
-    char *param;                /* general-purpose parameter to command */
-    int errcode;                /* error code */
-};
+typedef enum {
+    CMD_BATCH_MODE     = 1 << 0,
+    CMD_ECHO_TO_STDOUT = 1 << 1,
+    CMD_STACKING       = 1 << 2
+} CmdEchoFlags;
 
-enum option_codes {
+typedef enum {
     OPT_BATCH = 1,
     OPT_HELP,
     OPT_PVALS,
@@ -51,38 +43,57 @@ enum option_codes {
     OPT_DBOPEN,
     OPT_WEBDB,
     OPT_DUMP
-};
+} ProgramOptions;
+
+typedef enum {
+    ENGLISH = 1,
+    BASQUE
+} ForcedLangs;
     
 /* functions follow */
 
 int gretl_cmd_init (CMD *cmd);
- 
-void getcmd (char *line, DATAINFO *pdinfo, CMD *cmd, 
-	     int *ignore, double ***pZ, PRN *cmdprn);
 
-int command_number (const char *cmd);
+void gretl_cmd_free (CMD *cmd);
+
+CMD *gretl_cmd_new (void);
+
+void gretl_cmd_destroy (CMD *cmd);
 
 void gretl_cmd_set_context (CMD *cmd, int ci);
 
 void gretl_cmd_destroy_context (CMD *cmd);
 
-int help (const char *cmd, const char *helpfile, PRN *prn);
+const char *gretl_cmd_get_savename (const CMD *cmd);
 
-int fcast (const char *line, const MODEL *pmod, DATAINFO *pdinfo, 
-	   double ***pZ);
+gretlopt gretl_cmd_get_opt (const CMD *cmd);
 
-int parseopt (const char **argv, int argc, char *fname, int *english);
+void gretl_cmd_set_opt (CMD *cmd, gretlopt opt);
+
+int parse_command_line (char *line, CMD *cmd, double ***pZ, 
+			DATAINFO *pdinfo); 
+
+int get_command_index (const char *line, CMD *cmd);
+
+int command_number (const char *cmd);
+
+int help (const char *cmdword, const char *helpfile, PRN *prn);
+
+int parseopt (const char **argv, int argc, char *fname, 
+	      int *force_lang);
 
 int shell (const char *arg);
 
-void echo_cmd (CMD *pcmd, const DATAINFO *pdinfo, const char *line, 
-	       int batch, int gui, int loopstack, PRN *prn);
+void echo_cmd (const CMD *cmd, const DATAINFO *pdinfo, const char *line, 
+	       unsigned char flags, PRN *prn);
+
+void echo_function_call (const char *line, unsigned char flags, PRN *prn);
 
 int simple_commands (CMD *cmd, const char *line, 
 		     double ***pZ, DATAINFO *datainfo,
 		     PRN *prn);
 
-int call_pca_plugin (CORRMAT *corrmat, double ***pZ,
+int call_pca_plugin (VMatrix *corrmat, double ***pZ,
 		     DATAINFO *pdinfo, gretlopt *pflag,
 		     PRN *prn);
 
