@@ -67,7 +67,7 @@
 # endif /* ENABLE_NLS */
 #endif /* __GNOME_I18N_H__ */
 
-#define MAXLINE 1024  /* maximum length of command line */
+#define MAXLINE 4096  /* maximum length of command line */
 #define MAXLABEL 128  /* maximum length of descriptive labels for variables */
 #define MAXLEN   512  /* max length of regular "long" strings */
 #define ERRLEN   256  /* max length of libgretl error messages */
@@ -128,7 +128,8 @@ enum test_stats {
     GRETL_STAT_LMF,
     GRETL_STAT_HARVEY_COLLIER,
     GRETL_STAT_RESET,
-    GRETL_STAT_LR
+    GRETL_STAT_LR,
+    GRETL_STAT_WALD_CHISQ
 };
 
 enum gretl_opt_flags {
@@ -136,21 +137,23 @@ enum gretl_opt_flags {
     OPT_B = 1 <<  1,
     OPT_C = 1 <<  2,
     OPT_D = 1 <<  3,
-    OPT_F = 1 <<  4,
-    OPT_I = 1 <<  5,
-    OPT_L = 1 <<  6,
-    OPT_M = 1 <<  7,
-    OPT_N = 1 <<  8,
-    OPT_O = 1 <<  9,
-    OPT_P = 1 << 10,
-    OPT_Q = 1 << 11,
-    OPT_R = 1 << 12,
-    OPT_S = 1 << 13,
-    OPT_T = 1 << 14,
-    OPT_V = 1 << 15,
-    OPT_W = 1 << 16,
-    OPT_X = 1 << 17,
-    OPT_Z = 1 << 18
+    OPT_E = 1 <<  4,
+    OPT_F = 1 <<  5,
+    OPT_G = 1 <<  6,
+    OPT_I = 1 <<  7,
+    OPT_L = 1 <<  8,
+    OPT_M = 1 <<  9,
+    OPT_N = 1 << 10,
+    OPT_O = 1 << 11,
+    OPT_P = 1 << 12,
+    OPT_Q = 1 << 13,
+    OPT_R = 1 << 14,
+    OPT_S = 1 << 15,
+    OPT_T = 1 << 16,
+    OPT_V = 1 << 17,
+    OPT_W = 1 << 18,
+    OPT_X = 1 << 19,
+    OPT_Z = 1 << 20
 };
 
 typedef unsigned long gretlopt;
@@ -158,22 +161,19 @@ typedef unsigned long gretlopt;
 typedef struct VARINFO_ VARINFO;
 typedef struct DATAINFO_ DATAINFO;
 typedef struct PATHS_ PATHS;
-typedef struct GRETLTEST_ GRETLTEST;
-typedef struct CORRMAT_ CORRMAT;
+typedef struct VMatrix_ VMatrix;
 typedef struct SAMPLE_ SAMPLE;
 typedef struct ARINFO_ ARINFO;
 typedef struct MODEL_ MODEL;
 typedef struct GRAPHT_ GRAPHT;
 typedef struct PRN_ PRN;
 typedef struct FITRESID_ FITRESID;
-typedef struct CONFINT_ CONFINT;
-typedef struct VCV_ VCV;
 typedef struct DATASET_ DATASET;
 typedef struct GRETL_VAR_ GRETL_VAR;
 
 typedef struct mp_results_ mp_results;
 typedef struct model_data_item_ model_data_item;
-
+typedef struct ModelTest_ ModelTest;
 
 /* information on individual variable */
 struct VARINFO_ {
@@ -181,6 +181,7 @@ struct VARINFO_ {
     char display_name[MAXDISP];
     char compact_method;
     char stack_level;
+    char **sorted_markers;
 };
 
 /* information on data set */
@@ -202,7 +203,7 @@ struct DATAINFO_ {
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     unsigned char *vector; /* hold info on vars: vector versus scalar */
-    char *subdum;       /* subsampling vector */
+    char *submask;      /* subsampling mask */
     void *data;         /* all-purpose pointer */
 };
 
@@ -224,28 +225,20 @@ struct PATHS_ {
     char binbase[MAXLEN];
     char ratsbase[MAXLEN];
     char gnuplot[MAXLEN];
-#ifdef HAVE_X12A
     char x12a[MAXLEN];
     char x12adir[MAXLEN];
-#endif
     char dbhost[32];
     char pngfont[32];
 };
 
-struct GRETLTEST_ {
-    int type;
-    int order;
-    char *param;
-    unsigned char teststat;
-    int dfn, dfd;
-    double value;
-    double pvalue;
-};
-
-struct CORRMAT_ {
-    int n, t1, t2;
+struct VMatrix_ {
+    int ci;
+    int dim;
+    int t1, t2;
+    char **names;
+    double *vec;
     int *list;
-    double *xpx;
+    int missing;
 };
 
 struct SAMPLE_ {
@@ -264,7 +257,7 @@ struct MODEL_ {
     int ID;                      /* ID number for model */
     int t1, t2, nobs;            /* starting observation, ending
                                     observation, and number of obs */
-    char *subdum;                /* keep track of sub-sample in force
+    char *submask;               /* keep track of sub-sample in force
                                     when model was estimated */
     char *missmask;              /* missing observations mask */
     SAMPLE smpl;                 /* numeric start and end of current sample
@@ -297,14 +290,14 @@ struct MODEL_ {
     double criterion[2];         /* array of model selection statistics */
     double dw, rho;              /* Durbin-Watson stat. and estimated 1st
 				    order autocorrelation coefficient */
-    ARINFO *arinfo;              /* struct to hold special info for 
+    ARINFO *arinfo;              /* pointer to struct to hold special info for 
 				    autoregressive model */ 
     int errcode;                 /* Error code in case of failure */
     char *name;
     int nparams;                 /* number of named model parameters */
     char **params;               /* for named model parameters */
     int ntests;
-    GRETLTEST *tests;
+    ModelTest *tests;
     void *data;                  /* pointer for use in re. missing data */
     DATASET *dataset;            /* for handling models estimated on a
 				    sub-sampled portion of the dataset */
@@ -319,14 +312,6 @@ struct GRAPHT_ {
     char fname[MAXLEN];
 }; 
 
-struct PRN_ {
-    FILE *fp;
-    FILE *fpaux;    
-    char *buf;
-    size_t bufsize;
-    int format;
-};
-
 struct mp_results_ {
     int ncoeff;
     int t1, t2, ifc;
@@ -341,50 +326,25 @@ struct mp_results_ {
     double fstt;
 };
 
-struct FITRESID_ {
-    double *actual;
-    double *fitted;
-    double *sderr;
-    double sigma;
-    double tval;
-    int pmax;
-    int df;
-    int t1, t2;
-    int nobs;
-    int real_nobs;
-    int err;
-    char depvar[VNAMELEN];
-};
-
-struct CONFINT_ {
-    int *list;
-    double *coeff;
-    double *maxerr;
-    int df;
-    int ifc;
-};
-
-struct VCV_ {
-    int ci;
-    int *list;
-    double *vec;
-};
-
-#define VARLABEL(p,i)  ((p->varinfo[i])->label)
-#define DISPLAYNAME(p,i)  ((p->varinfo[i])->display_name)
-#define COMPACT_METHOD(p,i) ((p->varinfo[i])->compact_method)
-#define STACK_LEVEL(p,i) ((p->varinfo[i])->stack_level)
+#define VARLABEL(p,i)        ((p->varinfo[i])->label)
+#define DISPLAYNAME(p,i)     ((p->varinfo[i])->display_name)
+#define COMPACT_METHOD(p,i)  ((p->varinfo[i])->compact_method)
+#define STACK_LEVEL(p,i)     ((p->varinfo[i])->stack_level)
+#define SORTED_MARKER(p,i,t) ((p->varinfo[i])->sorted_markers[t])
 
 #include "gretl_commands.h"
 #include "gretl_errors.h"
 #include "interact.h"
+#include "dataset.h"
 #include "estimate.h"
 #include "generate.h"
+#include "compare.h"
+#include "gretl_intl.h"
 #include "gretl_list.h"
 #include "gretl_paths.h"
 #include "gretl_utils.h"
 #include "gretl_model.h"
-#include "compare.h"
+#include "gretl_prn.h"
 #include "pvalues.h"
 #include "dataio.h"
 #include "dbread.h"
@@ -409,7 +369,4 @@ struct VCV_ {
 # include "gretl_win32.h"
 #endif
 
-int parse_set_line (const char *line, int *echo_off, PRN *prn); /* libset.c */
-void set_use_qr (int set);
-
 #endif /* LIBGRETL_H */
