@@ -25,14 +25,17 @@
 #define NADBL DBL_MAX
 #define na(x) (x == NADBL)
 
-#define missing_masked(m,t,t1) (m != NULL && m[t-t1] != 0)
-#define model_missing(m,t)     ((m)->missmask != NULL && \
-                                (m)->missmask[t - (m)->t1] != 0)
-#define has_missing_obs(m)     ((m)->missmask != NULL)
+#define model_missing(m,t) ((m)->missmask != NULL && (m)->missmask[t] == '1')
 
 int model_missval_count (const MODEL *pmod);
 
-void set_miss (const int *list, const char *param, double **Z,
-	       DATAINFO *pdinfo, PRN *prn);
+int varlist_adjust_sample (const int *list, int *t1, int *t2, 
+			   const double **Z);
+
+int check_for_missing_obs (const int *list, int *t1, int *t2,
+			   const double **Z, int *misst);
+
+int set_miss (const int *list, const char *param, double **Z,
+	      DATAINFO *pdinfo, PRN *prn);
 
 #endif /* MISSING_H */
