@@ -20,7 +20,6 @@
 /*  modelprint.c */ 
 
 #include "libgretl.h"
-#include "gretl_private.h"
 #include "libset.h"
 #include "system.h"
 
@@ -35,9 +34,9 @@ struct _cmplx {
 #define NO_RBAR_SQ(a) (a == AUX_SQ || a == AUX_LOG || a == AUX_WHITE || a == AUX_AR)
 
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int c, int longnames, PRN *prn);
+			int i, int longnames, PRN *prn);
 static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			    int c, PRN *prn);
+			    int i, PRN *prn);
 static void depvarstats (const MODEL *pmod, PRN *prn);
 static int essline (const MODEL *pmod, PRN *prn, int wt);
 static void rsqline (const MODEL *pmod, PRN *prn);
@@ -70,8 +69,6 @@ static void noconst (const MODEL *pmod, PRN *prn)
 
 #define RTFTAB "\\par \\ql \\tab "
 
-/* ......................................................... */ 
-
 static void depvarstats (const MODEL *pmod, PRN *prn)
 {
     if (plain_format(prn)) {
@@ -95,7 +92,22 @@ static void depvarstats (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
+static void garch_variance_line (const MODEL *pmod, PRN *prn)
+{
+    const char *varstr = N_("Unconditional error variance");
+    double v = pmod->sigma * pmod->sigma;
+
+    if (plain_format(prn)) {  
+	pprintf(prn, "  %s = %.*g\n", _(varstr), GRETL_DIGITS, v);
+    } else if (rtf_format(prn)) {
+	pprintf(prn, RTFTAB "%s = %g\n", I_(varstr), v);
+    } else if (tex_format(prn)) {
+	char xstr[32];
+
+	tex_dcolumn_double(v, xstr);
+	pprintf(prn, "%s & %s \\\\\n", I_(varstr), xstr);
+    }
+}
 
 static int essline (const MODEL *pmod, PRN *prn, int wt)
 {
@@ -156,8 +168,6 @@ static int essline (const MODEL *pmod, PRN *prn, int wt)
     return 0;
 }
 
-/* ......................................................... */ 
-
 static void rsqline (const MODEL *pmod, PRN *prn)
 {
     if (plain_format(prn)) { 
@@ -191,8 +201,6 @@ static void rsqline (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static const char *aic_str = N_("Akaike information criterion");
 static const char *bic_str = N_("Schwarz Bayesian criterion");
 static const char *aic_abbrev = N_("AIC");
@@ -230,8 +238,6 @@ static void info_stats_lines (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void print_liml_equation_data (const MODEL *pmod, PRN *prn)
 {
     double lmin = gretl_model_get_double(pmod, "lmin");
@@ -260,8 +266,6 @@ static void print_liml_equation_data (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void ladstats (const MODEL *pmod, PRN *prn)
 {
     int utf = plain_format(prn);
@@ -275,9 +279,7 @@ static void ladstats (const MODEL *pmod, PRN *prn)
 		I_("Sum of absolute residuals"), x1str); 
 	pprintf(prn, "%s & %s \\\\\n",
 		I_("Sum of squared residuals"), x2str); 
-    }
-    
-    else {
+    } else {
 	pprintf(prn, "  %s = %.*g\n", 
 		(utf)? _("Sum of absolute residuals") :
 		I_("Sum of absolute residuals"),
@@ -297,8 +299,6 @@ static void ladstats (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void print_f_pval_str (double pval, PRN *prn)
 {
     int utf = plain_format(prn);
@@ -325,18 +325,14 @@ static void print_f_pval_str (double pval, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void Fline (const MODEL *pmod, PRN *prn)
 {
     if (pmod->ifc && pmod->ncoeff <= 2) {
 	if (plain_format(prn)) {
 	    pprintf(prn, "  %s = %d\n", _("Degrees of freedom"), pmod->dfd);
-	}
-	else if (tex_format(prn)) {
+	} else if (tex_format(prn)) {
 	    pprintf(prn, "%s & %d \\\\\n", I_("Degrees of freedom"), pmod->dfd);
-	}
-	else if (rtf_format(prn)) {
+	} else if (rtf_format(prn)) {
 	    pprintf(prn, RTFTAB "%s = %d\n", I_("Degrees of freedom"), pmod->dfd);
 	}
 	return;
@@ -352,9 +348,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, "  %s = %.*g", tmp, GRETL_DIGITS, pmod->fstt);
 	    print_f_pval_str(fdist(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
-    }
-
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	if (na(pmod->fstt)) {
 	    pprintf(prn, "$F(%d, %d)$ & \\multicolumn{1}{c}{\\rm %s} \\\\\n", 
 		    pmod->dfn, pmod->dfd, I_("undefined"));
@@ -365,9 +359,7 @@ static void Fline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, "$F(%d, %d)$ & %s \\\\\n", pmod->dfn, pmod->dfd, x1str);
 	    print_f_pval_str(fdist(pmod->fstt, pmod->dfn, pmod->dfd), prn);
 	}
-    }
-
-    else if (rtf_format(prn)) {
+    } else if (rtf_format(prn)) {
 	char tmp[32];
 
 	sprintf(tmp, "%s (%d, %d)", I_("F-statistic"), pmod->dfn, pmod->dfd);
@@ -380,8 +372,6 @@ static void Fline (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void dwline (const MODEL *pmod, PRN *prn)
 {
     if (plain_format(prn)) {
@@ -391,9 +381,7 @@ static void dwline (const MODEL *pmod, PRN *prn)
 	    pprintf(prn, "  %s = %.*g\n", _("First-order autocorrelation coeff."), 
 		    GRETL_DIGITS, pmod->rho);
 	} 
-    }
-
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	char x1str[32], x2str[32];
 
 	tex_dcolumn_double(pmod->dw, x1str);
@@ -401,9 +389,7 @@ static void dwline (const MODEL *pmod, PRN *prn)
 	pprintf(prn, "%s & %s \\\\\n%s & %s \\\\\n",
 		I_("Durbin--Watson statistic"), x1str, 
 		I_("First-order autocorrelation coeff."), x2str);
-    }
-
-    else if (rtf_format(prn)) {
+    } else if (rtf_format(prn)) {
 	if (!na(pmod->dw)) {
 	    pprintf(prn, RTFTAB "%s = %g\n", I_("Durbin-Watson statistic"), 
 		    pmod->dw);
@@ -413,8 +399,6 @@ static void dwline (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static void dhline (const MODEL *pmod, PRN *prn)
 {
     double sderr, h = 0.0;
@@ -436,9 +420,7 @@ static void dhline (const MODEL *pmod, PRN *prn)
         sprintf(tmp, _("(Using variable %d for h stat, with T' = %d)"), 
                 pmod->list[ldepvar], T);
         pprintf(prn, "  %s\n", tmp);
-    }
-
-    else if (rtf_format(prn)) {
+    } else if (rtf_format(prn)) {
         char tmp[128];
 
         sprintf(tmp, I_("Durbin's h stat. %g"), h);
@@ -447,9 +429,7 @@ static void dhline (const MODEL *pmod, PRN *prn)
         sprintf(tmp, I_("(Using variable %d for h stat, with T' = %d)"), 
                 pmod->list[ldepvar], T);
         pprintf(prn, RTFTAB "%s\n", tmp);
-    }
-
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	char x1str[32];
 
 	tex_dcolumn_double(h, x1str);
@@ -458,8 +438,6 @@ static void dhline (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */ 
-
 static int least_signif_coeff (const MODEL *pmod)
 {
     int i, k = 0;
@@ -480,8 +458,6 @@ static int least_signif_coeff (const MODEL *pmod)
     return 0;
 }
 
-/* ......................................................... */ 
-
 static void pval_max_line (const MODEL *pmod, const DATAINFO *pdinfo, 
 			   PRN *prn)
 {
@@ -498,8 +474,6 @@ static void pval_max_line (const MODEL *pmod, const DATAINFO *pdinfo,
     }
 }
 
-/* ......................................................... */
-
 static const char *aux_string (int aux, PRN *prn)
 {
     if (aux == AUX_SQ) {
@@ -532,9 +506,7 @@ static const char *aux_string (int aux, PRN *prn)
     else return "";
 }
 
-/* ......................................................... */
-
-const char *estimator_string (int ci, int format)
+const char *estimator_string (int ci, PRN *prn)
 {
     if (ci == OLS || ci == VAR) return N_("OLS");
     else if (ci == WLS) return N_("WLS"); 
@@ -552,15 +524,13 @@ const char *estimator_string (int ci, int format)
     else if (ci == LOGISTIC) return N_("Logistic");
     else if (ci == GARCH) return N_("GARCH");
     else if (ci == CORC) {
-	if (is_tex(format)) return N_("Cochrane--Orcutt");
+	if (tex_format(prn)) return N_("Cochrane--Orcutt");
 	else return N_("Cochrane-Orcutt");
-    }
-    else if (ci == HILU) {
-	if (is_tex(format)) return N_("Hildreth--Lu");
+    } else if (ci == HILU) {
+	if (tex_format(prn)) return N_("Hildreth--Lu");
 	else return N_("Hildreth-Lu");
-    }
-    else if (ci == PWE) {
-	if (is_tex(format)) return N_("Prais--Winsten");
+    } else if (ci == PWE) {
+	if (tex_format(prn)) return N_("Prais--Winsten");
 	else return N_("Prais-Winsten");
     }
 
@@ -579,12 +549,10 @@ my_estimator_string (const MODEL *pmod, PRN *prn)
 	    return N_("WLS");
 	}
     } else {
-	return estimator_string(pmod->ci, prn->format);
+	return estimator_string(pmod->ci, prn);
     } 
 }
 
-/* ......................................................... */
-
 static void print_model_tests (const MODEL *pmod, PRN *prn)
 {
     int i;
@@ -599,18 +567,16 @@ static void print_model_tests (const MODEL *pmod, PRN *prn)
 	    if (i > 0) {
 		pputs(prn, "\\vspace{1ex}\n");
 	    }
-	    gretl_model_test_print(&pmod->tests[i], prn);
+	    gretl_model_test_print(pmod, i, prn);
 	}
 	pputs(prn, "\\end{raggedright}\n");
     } else {
 	for (i=0; i<pmod->ntests; i++) {
-	    gretl_model_test_print(&pmod->tests[i], prn);
+	    gretl_model_test_print(pmod, i, prn);
 	}
     }
 }
 
-/* ......................................................... */
-
 static int 
 print_tsls_instruments (const int *list, const DATAINFO *pdinfo, PRN *prn)
 {
@@ -677,8 +643,6 @@ print_tsls_instruments (const int *list, const DATAINFO *pdinfo, PRN *prn)
     return 0;
 }
 
-/* ......................................................... */
-
 static void hac_vcv_line (const MODEL *pmod, PRN *prn)
 {
     int lag;
@@ -759,8 +723,6 @@ static void garch_vcv_line (const MODEL *pmod, PRN *prn)
     }
 }
 
-/* ......................................................... */
-
 static void model_print_newline (PRN *prn)
 {
     if (tex_format(prn)) {
@@ -772,10 +734,9 @@ static void model_print_newline (PRN *prn)
     }
 }
 
-/* ......................................................... */
-
 static void print_model_heading (const MODEL *pmod, 
 				 const DATAINFO *pdinfo, 
+				 gretlopt opt, 
 				 PRN *prn)
 {
     char startdate[OBSLEN], enddate[OBSLEN], vname[16];
@@ -836,7 +797,7 @@ static void print_model_heading (const MODEL *pmod,
 	break;
     case AUX_ADD:
     default:
-	if (pmod->ID < 0) {
+	if (pmod->ID < 0 || (opt & OPT_S)) {
 	    pputc(prn, '\n');
 	} else if (pmod->name) {
 	    pprintf(prn, "\n%s:\n", pmod->name);
@@ -895,24 +856,20 @@ static void print_model_heading (const MODEL *pmod,
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? "$\\hat{u}$" : "uhat");
-    }
-    else if (pmod->aux == AUX_WHITE) {
+    } else if (pmod->aux == AUX_WHITE) {
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? "$\\hat{u}^2$" : "uhat^2");
-    }
-    else if (pmod->aux == AUX_ARCH) {
+    } else if (pmod->aux == AUX_ARCH) {
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? "$u_t^2$" : "ut^2");
-    }
-    else if (pmod->ci == NLS) {
+    } else if (pmod->ci == NLS) {
 	if (tex) tex_escape(vname, pmod->params[0]);
 	pprintf(prn, "%s: %s", 
 		(utf)? _("Dependent variable") : I_("Dependent variable"),
 		(tex)? vname : pmod->params[0]);
-    }
-    else { 
+    } else { 
 	int v = (pmod->ci == ARMA || pmod->ci == GARCH)? 
 	    pmod->list[4] : pmod->list[1];
 
@@ -938,11 +895,9 @@ static void print_model_heading (const MODEL *pmod,
     /* VCV variants */
     if (pmod->aux == AUX_SCR || gretl_model_get_int(pmod, "hac_lag")) {
 	hac_vcv_line(pmod, prn);
-    }
-    else if (gretl_model_get_int(pmod, "hc")) {
+    } else if (gretl_model_get_int(pmod, "hc")) {
 	hc_vcv_line(pmod, prn);
-    }
-    else if (gretl_model_get_int(pmod, "garch_vcv")) {
+    } else if (gretl_model_get_int(pmod, "garch_vcv")) {
 	garch_vcv_line(pmod, prn);
     }
 
@@ -1005,8 +960,10 @@ static void print_model_heading (const MODEL *pmod,
 static void model_format_start (PRN *prn)
 {
     if (tex_format(prn)) {
-	if (doc_format(prn)) {
+	if (tex_doc_format(prn)) {
 	    gretl_tex_preamble(prn, 0);
+	} else {
+	    pputs(prn, "%% You'll need to \\usepackage{dcolumn}\n\n");
 	}
 	pputs(prn, "\\begin{center}\n");
     } else if (rtf_format(prn)) {
@@ -1043,8 +1000,7 @@ static void print_coeff_table_start (const MODEL *pmod, PRN *prn, int discrete)
 			   "       T STAT   2Prob(t > |T|)\n\n"));
 	}
 	return;
-    } 
-    else {
+    } else {
 	char col1[16], col2[16];
 
 	if (pmod->ci == NLS) {
@@ -1124,7 +1080,7 @@ static void model_format_end (PRN *prn)
 {
     if (tex_format(prn)) {
 	pputs(prn, "\n\\end{center}\n");
-	if (doc_format(prn)) {
+	if (tex_doc_format(prn)) {
 	    pputs(prn, "\n\\end{document}\n"); 
 	}
     } else if (rtf_format(prn)) {
@@ -1132,6 +1088,28 @@ static void model_format_end (PRN *prn)
     }
 } 
 
+static int long_coeff_names (const MODEL *pmod, const DATAINFO *pdinfo)
+{
+    int ret = 0;
+
+    if (pmod->aux != AUX_ARCH &&
+	pmod->ci != NLS && 
+	pmod->ci != ARMA && 
+	pmod->ci != GARCH) {
+	int i, len;
+
+	for (i=0; i<pmod->ncoeff; i++) {
+	    len = strlen(pdinfo->varname[pmod->list[i+2]]);
+	    if (len > 8) {
+		ret = 1;
+		break;
+	    }
+	}
+    }
+
+    return ret;
+}
+
 static int 
 print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
@@ -1143,18 +1121,8 @@ print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 	gn = pmod->list[0] - 4;
     }
 
-    if (plain_format(prn) && 
-	pmod->aux != AUX_ARCH &&
-	pmod->ci != NLS && 
-	pmod->ci != ARMA && 
-	pmod->ci != GARCH) {
-	for (i=0; i<pmod->ncoeff; i++) {
-	    int len = strlen(pdinfo->varname[pmod->list[i+2]]);
-
-	    if (len > 8) {
-		longnames = 1;
-	    }
-	}
+    if (plain_format(prn)) {
+	longnames = long_coeff_names(pmod, pdinfo);
     }
 
     for (i=0; i<pmod->ncoeff; i++) {
@@ -1162,18 +1130,19 @@ print_coefficients (const MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 	    if (i == gn) {
 		pputc(prn, '\n');
 	    }
-	    err = print_coeff(pdinfo, pmod, i + 2, longnames, prn);
+	    err = print_coeff(pdinfo, pmod, i, longnames, prn);
 	} else if (tex_format(prn)) {
 	    if (i == gn) {
 		pputs(prn, "\\\\ \n");
 	    }
-	    err = tex_print_coeff(pdinfo, pmod, i + 2, prn);
+	    err = tex_print_coeff(pdinfo, pmod, i, prn);
 	} else if (rtf_format(prn)) {
 	    if (i == gn) {
 		pputc(prn, '\n');
 	    }
-	    err = rtf_print_coeff(pdinfo, pmod, i + 2, prn);
+	    err = rtf_print_coeff(pdinfo, pmod, i, prn);
 	}
+
 	if (err) gotnan = 1;
     }
 
@@ -1214,12 +1183,10 @@ static void weighted_stats_message (PRN *prn)
 {
     if (plain_format(prn)) {
 	pprintf(prn, "%s:\n\n", _("Statistics based on the weighted data"));
-    }
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	pprintf(prn, "\\vspace{1em}%s:\n\n", 
 		I_("Statistics based on the weighted data"));
-    } 
-    else { /* RTF */
+    } else { /* RTF */
 	pprintf(prn, "\\par \\qc\n%s:\n\n", 
 		I_("Statistics based on the weighted data"));	
     }
@@ -1229,12 +1196,10 @@ static void original_stats_message (PRN *prn)
 {
     if (plain_format(prn)) {
 	pprintf(prn, "%s:\n\n", _("Statistics based on the original data"));
-    } 
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	pprintf(prn, "\\vspace{1em}\n%s:\n\n", 
 		I_("Statistics based on the original data"));
-    } 
-    else { /* RTF */
+    } else { /* RTF */
 	pprintf(prn, "\\par \\qc\n%s:\n\n", 
 		I_("Statistics based on the original data"));
     }
@@ -1244,12 +1209,10 @@ static void rho_differenced_stats_message (PRN *prn)
 {
     if (plain_format(prn)) {    
 	pprintf(prn, "%s:\n\n", _("Statistics based on the rho-differenced data"));
-    } 
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	pprintf(prn, "\\vspace{1em}\n%s:\n\n", 
 		I_("Statistics based on the rho-differenced data"));
-    } 
-    else { /* RTF */
+    } else { /* RTF */
 	pprintf(prn, "\\par \\qc\n%s:\n\n", 
 		I_("Statistics based on the rho-differenced data"));
     }	
@@ -1264,18 +1227,14 @@ static void print_whites_results (const MODEL *pmod, PRN *prn)
 		_("with p-value"), _("Chi-square"), 
 		pmod->ncoeff - 1, pmod->rsq * pmod->nobs,
 		chisq(pmod->rsq * pmod->nobs, pmod->ncoeff - 1)); 
-    }
-
-    else if (rtf_format(prn)) { /* FIXME */
+    } else if (rtf_format(prn)) { /* FIXME */
 	pprintf(prn, "\\par \\ql\n%s: TR{\\super 2} = %f,\n", I_("Test statistic"), 
 		pmod->rsq * pmod->nobs);
 	pprintf(prn, "%s = P(%s(%d) > %f) = %f\n\n", 
 		I_("with p-value"), I_("Chi-square"), 
 		pmod->ncoeff - 1, pmod->rsq * pmod->nobs,
 		chisq(pmod->rsq * pmod->nobs, pmod->ncoeff - 1)); 
-    }
-
-    else if (tex_format(prn)) {
+    } else if (tex_format(prn)) {
 	pprintf(prn, "\n%s: $TR^2$ = %f,\n", I_("Test statistic"), 
 		pmod->rsq * pmod->nobs);
 	pprintf(prn, "%s = $P$($\\chi^2(%d)$ > %f) = %f\n\n",
@@ -1334,7 +1293,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 	}
     }
 
-    print_model_heading(pmod, pdinfo, prn);
+    print_model_heading(pmod, pdinfo, opt, prn);
 
     print_coeff_table_start(pmod, prn, is_discrete);
 
@@ -1388,6 +1347,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
     if (pmod->ci == GARCH) {
 	print_middle_table_start(prn);
 	depvarstats(pmod, prn);
+	garch_variance_line(pmod, prn);
 	print_arma_stats(pmod, prn);
 	print_middle_table_end(prn);
 	goto close_format;
@@ -1558,7 +1518,7 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
  close_format:
 
-    if (pmod->ci != LAD && (opt & OPT_O)) {
+    if (opt & OPT_O) {
 	outcovmx(pmod, pdinfo, prn);
     }
 
@@ -1579,30 +1539,6 @@ int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 
 /* ......................................................... */ 
 
-static void make_cname (const char *orig, char *cname)
-{
-    char *p;
-    unsigned char c;
-
-    if (orig == NULL || strlen(orig) == 0) return;
-
-    p = strrchr(orig, '_');
-    if (p == NULL) {
-	strcpy(cname, orig);
-	return;
-    }
-
-    c = (unsigned char) *(p + 1);
-
-    if (isdigit(c)) {
-	int lag = atoi(++p);
-
-	sprintf(cname, "ut^2(-%d)", lag);
-    }
-}
-
-/* ......................................................... */ 
-
 static void print_pval_str (double pval, char *str)
 {
     if (pval < .00001) {
@@ -1612,58 +1548,49 @@ static void print_pval_str (double pval, char *str)
     }
 }
 
-/* ......................................................... */ 
-
 static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			int c, int longnames, PRN *prn)
+			int i, int longnames, PRN *prn)
 {
     double t, pvalue = 999.0;
     int gotnan = 0;
     int do_pval = (pmod->ci != LOGIT && pmod->ci != PROBIT);
     char varname[24];
 
-    /* special treatment for ARCH model coefficients, NLS, ARMA, VAR */
-    if (pmod->aux == AUX_ARCH) {
-	make_cname(pdinfo->varname[pmod->list[c]], varname);
-    } else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
-	strcpy(varname, pmod->params[c-1]);
-    } else {
-	strcpy(varname, pdinfo->varname[pmod->list[c]]);
-    }
+    gretl_model_get_param_name(pmod, pdinfo, i, varname);
 
     if (longnames) {
 	pprintf(prn, " %13s ", varname);
     } else if (pmod->ci == GARCH) {
 	pprintf(prn, "      %8s ", varname);
     } else if (pmod->ci == ARMA) {
-	pprintf(prn, " %3d) %8s ", c - 1, varname);
+	pprintf(prn, " %3d) %8s ", i + 1, varname);
     } else {
-	pprintf(prn, " %3d) %8s ", pmod->list[c], varname);
+	pprintf(prn, " %3d) %8s ", pmod->list[i + 2], varname);
     }
 
     bufspace((strlen(varname) > 12)? 1 : 2, prn);
 
     /* print coeff value if well-defined */
-    if (isnan(pmod->coeff[c-2]) || na(pmod->coeff[c-2])) {
+    if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	pprintf(prn, "%*s", UTF_WIDTH(_("undefined"), 17), _("undefined"));
 	gotnan = 1;
     } else {
-	gretl_print_value(pmod->coeff[c-2], prn);
+	gretl_print_value(pmod->coeff[i], prn);
     }
 
     bufspace(2, prn);
 
     /* get out if std error is undefined */
-    if (isnan(pmod->sderr[c-2]) || na(pmod->sderr[c-2])) {
+    if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
 	pprintf(prn, "%*s\n", UTF_WIDTH(_("undefined"), 16), _("undefined"));
 	return 1;
     }
 
-    gretl_print_value (pmod->sderr[c-2], prn); 
+    gretl_print_value(pmod->sderr[i], prn); 
 
     /* std error is well-defined, but is it positive? */
-    if (pmod->sderr[c-2] > 0.) {
-	t = pmod->coeff[c-2] / pmod->sderr[c-2];
+    if (pmod->sderr[i] > 0.) {
+	t = pmod->coeff[i] / pmod->sderr[i];
 	if (fabs(t) >= 1000.0) { /* || t < .001 ? */
 	    char numstr[9];
 
@@ -1673,7 +1600,7 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	    pprintf(prn, " %7.3f", t);
 	}
 	if (pmod->aux == AUX_ADF || pmod->aux == AUX_DF) {
-	    if (c == gretl_model_get_int(pmod, "dfnum")) {
+	    if (i + 2 == gretl_model_get_int(pmod, "dfnum")) {
 		char pvalstr[16];
 
 		pvalue = gretl_model_get_double(pmod, "dfpval");
@@ -1689,22 +1616,26 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	    print_pval_str(pvalue, pvalstr);
 	    pprintf(prn, "%*s", UTF_WIDTH(pvalstr, 12), pvalstr);
 	}
-    } else if (do_pval) { /* zero standard error */
+    } else if (do_pval) { 
+	/* case of zero standard error */
 	do_pval = 0;
 	pprintf(prn, "     %*s", UTF_WIDTH(_("undefined"), 12), _("undefined"));
     }
 
     if (do_pval) {
-	if (pvalue < 0.01) pputs(prn, " ***");
-	else if (pvalue < 0.05) pputs(prn, " **");
-	else if (pvalue < 0.10) pputs(prn, " *");
-    } 
-    else if (pmod->list[c] != 0 && 
+	if (pvalue < 0.01) {
+	    pputs(prn, " ***");
+	} else if (pvalue < 0.05) {
+	    pputs(prn, " **");
+	} else if (pvalue < 0.10) {
+	    pputs(prn, " *");
+	}
+    } else if (pmod->list[i+2] != 0 && 
 	     (pmod->ci == LOGIT || pmod->ci == PROBIT)) { 
 	double *slopes = gretl_model_get_data(pmod, "slopes");
 
 	if (slopes != NULL) {
-	    gretl_print_value(slopes[c-2], prn);
+	    gretl_print_value(slopes[i], prn);
 	}
     }
 
@@ -1713,8 +1644,6 @@ static int print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     return gotnan;
 }
 
-/* ......................................................... */ 
-
 static void rtf_print_double (double xx, PRN *prn)
 {
     char numstr[32];
@@ -1725,50 +1654,40 @@ static void rtf_print_double (double xx, PRN *prn)
     pprintf(prn, " \\qc %s\\cell", numstr);
 }
 
-/* ......................................................... */ 
-
 static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			    int c, PRN *prn)
+			    int i, PRN *prn)
 {
     double t, pvalue = 999.0;
     int gotnan = 0;
     int do_pval = (pmod->ci != LOGIT && pmod->ci != PROBIT);
-    char varname[12];
+    char varname[24];
 
-
-    /* special treatment for ARCH model coefficients, NLS, ARMA, VAR */
-    if (pmod->aux == AUX_ARCH) {
-	make_cname(pdinfo->varname[pmod->list[c]], varname);
-    } else if (pmod->ci == NLS || pmod->ci == ARMA || pmod->ci == GARCH) {
-	strcpy(varname, pmod->params[c-1]);
-    } else {
-	strcpy(varname, pdinfo->varname[pmod->list[c]]);
-    }
+    gretl_model_get_param_name(pmod, pdinfo, i, varname);
 
     pputs(prn, RTF_COEFF_ROW);
 
     pprintf(prn, " \\qr %d\\cell \\ql %s\\cell", 
-	    ((pmod->ci == ARMA || pmod->ci == GARCH)? c - 1 : pmod->list[c]), 
+	    ((pmod->ci == ARMA || pmod->ci == GARCH)? i+1 : pmod->list[i+2]), 
 	    varname);
 
-    if (isnan(pmod->coeff[c-2]) || na(pmod->coeff[c-2])) {
+    if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
 	gotnan = 1;
     } else {
-	rtf_print_double(pmod->coeff[c-2], prn);
+	rtf_print_double(pmod->coeff[i], prn);
     }
 
-    if (isnan(pmod->sderr[c-2]) || na(pmod->sderr[c-2])) {
+    if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
 	goto rtf_coeff_getout;
     } 
 
-    rtf_print_double(pmod->sderr[c-2], prn); 
+    rtf_print_double(pmod->sderr[i], prn); 
 
-    if (pmod->sderr[c-2] > 0.) {
-	t = pmod->coeff[c-2] / pmod->sderr[c-2];
+    if (pmod->sderr[i] > 0.) {
+	t = pmod->coeff[i] / pmod->sderr[i];
 	pprintf(prn, " \\qc %.4f\\cell", t);
 	if (pmod->aux == AUX_ADF || pmod->aux == AUX_DF) {
 	    do_pval = 0;
@@ -1781,26 +1700,28 @@ static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 	    print_pval_str(pvalue, pvalstr);
 	    pprintf(prn, " \\qc %s\\cell", pvalstr);
 	}	
-    } else if (do_pval) { /* zero standard error */
+    } else if (do_pval) { 
+	/* case of zero standard error */
 	do_pval = 0;
 	pprintf(prn, " \\qc %s\\cell", I_("undefined"));
     }
 
     if (do_pval) {
-	if (pvalue < 0.01) 
+	if (pvalue < 0.01) {
 	    pputs(prn, " \\ql ***\\cell");
-	else if (pvalue < 0.05) 
+	} else if (pvalue < 0.05) { 
 	    pputs(prn, " \\ql **\\cell");
-	else if (pvalue < 0.10) 
+	} else if (pvalue < 0.10) {
 	    pputs(prn, " \\ql *\\cell");
-	else 
+	} else {
 	    pputs(prn, " \\ql \\cell");
-    } else if (pmod->list[c] != 0 && 
+	}
+    } else if (pmod->list[i+2] != 0 && 
 	       (pmod->ci == LOGIT || pmod->ci == PROBIT)) { 
 	double *slopes = gretl_model_get_data(pmod, "slopes");
 
 	if (slopes != NULL) {
-	    rtf_print_double(slopes[c-2], prn);
+	    rtf_print_double(slopes[i], prn);
 	}	
     }
 
@@ -1810,38 +1731,32 @@ static int rtf_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
     return gotnan;
 }
 
-/* ......................................................... */ 
-
 static void print_rho (const ARINFO *arinfo, int c, int dfd, PRN *prn)
 {
     char ustr[32];
     double xx = arinfo->rho[c] / arinfo->sderr[c];
 
     if (plain_format(prn)) {
-	sprintf(ustr, "u_%d", arinfo->arlist[c]);
+	char pvalstr[16];
+	double pval;
+
+	sprintf(ustr, "u_%d", arinfo->arlist[c+1]);
 	pprintf(prn, "%14s", ustr); 
 	bufspace(3, prn);
 	gretl_print_value (arinfo->rho[c], prn);
 	bufspace(2, prn);
 	gretl_print_value (arinfo->sderr[c], prn); 
-	pprintf(prn, " %7.3f ", xx, tprob(xx, dfd));
-	if (1) {
-	    char pvalstr[16];
-	    double pval;
-
-	    pval = tprob(xx, dfd);
-	    print_pval_str(pval, pvalstr);
-	    pprintf(prn, "%*s\n", UTF_WIDTH(pvalstr, 12), pvalstr);
-	}
-    }
-
-    else if (tex_format(prn)) {
+	pval = t_pvalue_2(xx, dfd);
+	pprintf(prn, " %7.3f ", xx, pval);
+	print_pval_str(pval, pvalstr);
+	pprintf(prn, "%*s\n", UTF_WIDTH(pvalstr, 12), pvalstr);
+    } else if (tex_format(prn)) {
 	char coeff[32], sderr[32];
 
 	tex_dcolumn_double(arinfo->rho[c], coeff);
 	tex_dcolumn_double(arinfo->sderr[c], sderr);
 
-	sprintf(ustr, "$\\hat{u}_{t-%d}$", arinfo->arlist[c]);
+	sprintf(ustr, "$\\hat{u}_{t-%d}$", arinfo->arlist[c+1]);
 
 	pprintf(prn, "%s &\n"
 		"  %s &\n"
@@ -1852,12 +1767,10 @@ static void print_rho (const ARINFO *arinfo, int c, int dfd, PRN *prn)
 		coeff,
 		sderr,
 		arinfo->rho[c] / arinfo->sderr[c],
-		tprob(xx, dfd));
+		t_pvalue_2(xx, dfd));
     }
 }
 
-/* ......................................................... */ 
-
 static void print_rho_terms (const MODEL *pmod, PRN *prn)
 {
     int i, dfd;
@@ -1881,8 +1794,8 @@ static void print_rho_terms (const MODEL *pmod, PRN *prn)
     }
 
     for (i=1; i<=pmod->arinfo->arlist[0]; i++) {
-	print_rho(pmod->arinfo, i, dfd, prn);
-	xx += pmod->arinfo->rho[i]; 
+	print_rho(pmod->arinfo, i - 1, dfd, prn);
+	xx += pmod->arinfo->rho[i-1]; 
     }
 
     if (pmod->arinfo->arlist[0] > 1 && plain_format(prn)) {
@@ -2105,21 +2018,63 @@ static void print_arma_stats (const MODEL *pmod, PRN *prn)
     }
 }
 
+static void plain_print_act_pred (const int *ap, PRN *prn)
+{
+    int leftlen;
+    int numwidth = 1;
+    int i, bign = 0;
+
+    for (i=0; i<4; i++) {
+	if (ap[i] > bign) {
+	    bign = ap[i];
+	}
+    }
+
+    while (bign /= 10) {
+	numwidth++;
+    }
+
+    leftlen = strlen(_("Actual")) + 3; /* utflen */
+
+    bufspace(leftlen + 2, prn);
+    pputs(prn, _("Predicted"));
+    pputc(prn, '\n');
+    bufspace(leftlen + 3, prn);
+    pprintf(prn, "%*d   %*d\n\n", numwidth, 0, numwidth, 1);
+    bufspace(leftlen, prn);
+    pprintf(prn, "0  %*d   %*d\n", numwidth, ap[0], numwidth, ap[1]);
+    bufspace(2, prn);
+    pputs(prn, _("Actual"));  
+    pputc(prn, '\n');
+    bufspace(leftlen, prn);
+    pprintf(prn, "1  %*d   %*d\n", numwidth, ap[2], numwidth, ap[3]);
+}
+
 static void print_discrete_statistics (const MODEL *pmod, 
 				       const DATAINFO *pdinfo,
 				       PRN *prn)
 {
-    int i;
-    int correct = gretl_model_get_int(pmod, "correct");
-    double pc_correct = 100 * (double) correct / pmod->nobs;
     double model_chisq = gretl_model_get_double(pmod, "chisq");
     const double *crit = pmod->criterion;
+    const int *act_pred;
+    int correct = -1;
+    double pc_correct = NADBL;
+    int i;
+
+    act_pred = gretl_model_get_data(pmod, "discrete_act_pred");
+    if (act_pred != NULL) {
+	correct = act_pred[0] + act_pred[3];
+	pc_correct = 100 * (double) correct / pmod->nobs;
+    } 
 
     if (plain_format(prn)) {
 	pprintf(prn, "  %s %s = %.3f\n", _("Mean of"), 
 		pdinfo->varname[pmod->list[1]], pmod->ybar);
-	pprintf(prn, "  %s = %d (%.1f%%)\n", _("Number of cases 'correctly predicted'"), 
-		correct, pc_correct);
+	if (correct >= 0) {
+	    pprintf(prn, "  %s = %d (%.1f%%)\n", 
+		    _("Number of cases 'correctly predicted'"), 
+		    correct, pc_correct);
+	}
 	pprintf(prn, "  f(beta'x) %s = %.3f\n", _("at mean of independent vars"), 
 		pmod->sdy);
 	pprintf(prn, "  %s = %g\n", _("Log-likelihood"), pmod->lnL);
@@ -2135,6 +2090,10 @@ static void print_discrete_statistics (const MODEL *pmod,
 		crit[C_BIC]);
 	pprintf(prn, "  %s = %g\n", _("McFadden's pseudo-R-squared"), pmod->rsq);
 	pputc(prn, '\n');
+
+	if (act_pred != NULL) {
+	    plain_print_act_pred(act_pred, prn);
+	}
     }
 
     else if (rtf_format(prn)) {
@@ -2142,8 +2101,11 @@ static void print_discrete_statistics (const MODEL *pmod,
 	pprintf(prn, "\\par {\\super *}%s\n", I_("Evaluated at the mean"));
 	pprintf(prn, "\\par %s %s = %.3f\n", I_("Mean of"), 
 		pdinfo->varname[pmod->list[1]], pmod->ybar);
-	pprintf(prn, "\\par %s = %d (%.1f%%)\n", I_("Number of cases 'correctly predicted'"), 
-		correct, pc_correct);
+	if (correct >= 0) {
+	    pprintf(prn, "\\par %s = %d (%.1f%%)\n", 
+		    I_("Number of cases 'correctly predicted'"), 
+		    correct, pc_correct);
+	}
 	pprintf(prn, "\\par f(beta'x) %s = %.3f\n", I_("at mean of independent vars"), 
 		pmod->sdy);
 	pprintf(prn, "\\par %s = %g\n", I_("Log-likelihood"), pmod->lnL);
@@ -2171,9 +2133,11 @@ static void print_discrete_statistics (const MODEL *pmod,
 	pputs(prn, "\\vspace{1em}\n\\begin{raggedright}\n");
 	pprintf(prn, "%s %s = %.3f\\\\\n", I_("Mean of"), 
 		pdinfo->varname[pmod->list[1]], pmod->ybar);
-	pprintf(prn, "%s = %d (%.1f percent)\\\\\n", 
-		I_("Number of cases `correctly predicted'"), 
-		correct, pc_correct);
+	if (correct >= 0) {
+	    pprintf(prn, "%s = %d (%.1f percent)\\\\\n", 
+		    I_("Number of cases `correctly predicted'"), 
+		    correct, pc_correct);
+	}
 	pprintf(prn, "$f(\\beta'x)$ %s = %.3f\\\\\n", I_("at mean of independent vars"), 
 		pmod->sdy);
 	tex_float_str(pmod->lnL, lnlstr);
@@ -2193,8 +2157,6 @@ static void print_discrete_statistics (const MODEL *pmod,
     }
 }
 
-/* ....................................................... */
-
 static void mp_other_stats (const mp_results *mpvals, PRN *prn)
 {
     char fstr[16];
@@ -2230,8 +2192,6 @@ static void mp_other_stats (const mp_results *mpvals, PRN *prn)
     pputs(prn, "\n\n");
 }
 
-/* ....................................................... */
-
 static void print_mpvals_coeff (const mp_results *mpvals, 
 				int c, PRN *prn)
 {
@@ -2245,8 +2205,6 @@ static void print_mpvals_coeff (const mp_results *mpvals,
     pputc(prn, '\n');
 }
 
-/* ....................................................... */
-
 void print_mpols_results (const mp_results *mpvals, DATAINFO *pdinfo,
 			  PRN *prn)
 {
