@@ -22,7 +22,7 @@
 int printmodel (MODEL *pmod, const DATAINFO *pdinfo, gretlopt opt,
 		PRN *prn);
 
-const char *estimator_string (int ci, int format);
+const char *estimator_string (int ci, PRN *prn);
 
 void print_mpols_results (const mp_results *mpvals, DATAINFO *pdinfo,
 			  PRN *prn);
