@@ -21,10 +21,10 @@
 */  
 
 #include "libgretl.h" 
-#include "gretl_private.h"
 #include "loop_private.h"
 #include "libset.h"
 #include "compat.h"
+#include "cmd_private.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -106,6 +106,7 @@ struct controller_ {
     double val;
     int vnum;
     char vname[VNAMELEN];
+    int vsign;
 };
 
 typedef struct controller_ controller;
@@ -164,7 +165,6 @@ struct LOOPSET_ {
 #define loop_is_quiet(l) (l->flags & LOOP_QUIET)
 #define loop_set_quiet(l) (l->flags |= LOOP_QUIET)
 
-
 static void gretl_loop_init (LOOPSET *loop);
 static int prepare_loop_for_action (LOOPSET *loop);
 static void free_loop_model (LOOP_MODEL *lmod);
@@ -190,7 +190,7 @@ loop_controller_get_value (controller *clr, const double **Z)
     double ret = NADBL;
 
     if (clr->vnum > 0) {
-	ret = Z[clr->vnum][0];
+	ret = Z[clr->vnum][0] * clr->vsign;
     } else {
 	ret = clr->val;
     } 
@@ -205,11 +205,17 @@ loop_controller_get_value (controller *clr, const double **Z)
 
 static double loop_lval (LOOPSET *loop, const double **Z)
 {
+#if LOOP_DEBUG
+    fprintf(stderr, "getting loop_lval...\n");
+#endif
     return loop_controller_get_value(&loop->left, Z);
 }
 
 static double loop_rval (LOOPSET *loop, const double **Z)
 {
+#if LOOP_DEBUG
+    fprintf(stderr, "getting loop_rval...\n");
+#endif
     return loop_controller_get_value(&loop->right, Z);
 }
 
@@ -217,11 +223,30 @@ static double loop_incrval (LOOPSET *loop, const double **Z)
 {
     double x = loop_controller_get_value(&loop->incr, Z);
 
-    if (!na(x)) x *= loop->incrsgn;
+    if (!na(x)) {
+	x *= loop->incrsgn;
+    }
+
+#if LOOP_DEBUG
+    fprintf(stderr, "loop_incrval, returning %g\n", x);
+#endif
 
     return x;
 }
 
+static void set_loop_opts (LOOPSET *loop, gretlopt opt)
+{
+    if (opt & OPT_P) {
+	loop_set_progressive(loop);
+    }
+    if (opt & OPT_V) {
+	loop_set_verbose(loop);
+    }
+    if (opt & OPT_Q) {
+	loop_set_quiet(loop);
+    }
+}
+
 /**
  * ok_in_loop:
  * @ci: command index.
@@ -400,8 +425,15 @@ controller_set_var (controller *clr, LOOPSET *loop, const DATAINFO *pdinfo,
 		    const char *s)
 {
     int v, err = 0;
+    int vsign = 1;
 
-    v = ok_loop_var(loop, pdinfo, s);
+    if (*s == '-' && isalpha((unsigned char) *(s + 1))) {
+	/* negative sign in front of varname? */
+	vsign = -1;
+	v = ok_loop_var(loop, pdinfo, s + 1);
+    } else {
+	v = ok_loop_var(loop, pdinfo, s);
+    }
 
     if (v == LOOP_VAL_BAD) {
 	err = 1;
@@ -411,8 +443,14 @@ controller_set_var (controller *clr, LOOPSET *loop, const DATAINFO *pdinfo,
 	strncat(clr->vname, s, VNAMELEN - 1);
     } else {
 	clr->vnum = v;
+	clr->vsign = vsign;
     }
 
+#if LOOP_DEBUG
+    fprintf(stderr, " controller set var: vnum=%d, vsign=%d\n", 
+	    clr->vnum, clr->vsign);
+#endif
+
     return err;
 }
 
@@ -508,6 +546,10 @@ static int parse_as_indexed_loop (LOOPSET *loop,
     /* starting and ending values: try for dates first, then
        numeric constants, then variables */
 
+#if LOOP_DEBUG
+    fprintf(stderr, "parse_as_indexed_loop: start='%s'\n", start);
+#endif
+
     if (!err) {
 	if (maybe_date(start)) {
 	    nstart = dateton(start, pdinfo);
@@ -523,6 +565,10 @@ static int parse_as_indexed_loop (LOOPSET *loop,
 	}
     }
 
+#if LOOP_DEBUG
+    fprintf(stderr, "parse_as_indexed_loop: end='%s'\n", end);
+#endif
+
     if (!err) {
 	if (dated) {
 	    nend = dateton(end, pdinfo);
@@ -643,7 +689,6 @@ test_forloop_element (const char *s, LOOPSET *loop,
 #if LOOP_DEBUG
 	fprintf(stderr, " lhs: varindex = %d (pdinfo->v = %d)\n", v, pdinfo->v);
 #endif
-
 	/* examine the LHS */
 	if (i == 0) {
 	    if (v == pdinfo->v) {
@@ -733,6 +778,50 @@ static int allocate_each_strings (LOOPSET *loop, int n)
     return err;
 }
 
+static int each_strings_from_named_list (LOOPSET *loop, char *s,
+					 int *nf)
+{
+    int *list;
+    int err = 0;
+
+    chopstr(s);
+    list = get_list_by_name(s);
+    
+    if (list == NULL) {
+	err = 1;
+    } else {
+	err = allocate_each_strings(loop, list[0]);
+    }
+
+    if (!err) {
+	char numstr[16];
+	int i, li;
+
+	for (i=1; i<=list[0] && !err; i++) {
+	    li = list[i];
+	    if (abs(li) > 9999999) {
+		err = 1;
+	    } else {
+		sprintf(numstr, "%d", li);
+		loop->eachstrs[i-1] = gretl_strdup(numstr);
+		if (loop->eachstrs[i-1] == NULL) {
+		    err = 1;
+		}
+	    }
+	}
+    }
+
+    if (err && loop->eachstrs != NULL) {
+	destroy_each_strings(loop, list[0]);
+    }
+
+    if (!err) {
+	*nf = list[0];
+    }
+
+    return err;
+}
+
 static int
 each_strings_from_list_of_vars (LOOPSET *loop, const DATAINFO *pdinfo, 
 				char *s, int *nf)
@@ -791,8 +880,11 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
 	return 1;
     }
 
+    /* we're looking at the string that follows "loop foreach" */
+
     s++;
 
+    /* should be something like "i " */
     if (sscanf(s, "%7s", ivar) != 1) {
 	err = 1;
     } else if (strlen(ivar) > 1) {
@@ -814,10 +906,15 @@ parse_as_each_loop (LOOPSET *loop, const DATAINFO *pdinfo, char *s)
     }
 
     loop->ichar = ichar;
-
+    
     if (nf <= 3 && strstr(s, "..") != NULL) {
+	/* range of values, foo..quux */
 	err = each_strings_from_list_of_vars(loop, pdinfo, s, &nf);
+    } else if (nf == 1) {
+	/* named list? */
+	err = each_strings_from_named_list(loop, s, &nf);
     } else {
+	/* simple list of values */
 	err = allocate_each_strings(loop, nf);
 
 	for (i=0; i<nf && !err; i++) {
@@ -917,27 +1014,27 @@ parse_loopline (char *line, LOOPSET *ploop, int loopstack,
 {
     LOOPSET *loop;
     char lvar[VNAMELEN], rvar[VNAMELEN], op[VNAMELEN];
+    gretlopt opt = OPT_NONE;
     char ichar;
     int err = 0;
 
     *gretl_errmsg = '\0';
 
 #if LOOP_DEBUG
-    fprintf(stderr, "parse_loopline: ploop = %p, loopstack = %d\n",
-	    (void *) ploop, loopstack);
+    fprintf(stderr, "parse_loopline: ploop = %p, loopstack = %d\n"
+	    " line: '%s'\n", (void *) ploop, loopstack, line);
 #endif
 
-    /* skip over the common stuff */
-    while (isspace((unsigned char) *line)) line++;
+    while (isspace((unsigned char) *line)) {
+	line++;
+    }
+
     if (strncmp(line, "loop", 4)) {
 	printf("parse_loopline: line didn't begin with 'loop': '%s'\n", line);
 	strcpy(gretl_errmsg, _("No valid loop condition was given."));
 	return NULL;
     }
 
-    line += 4; /* "loop" */
-    while (isspace((unsigned char) *line)) line++;
-
     if (ploop == NULL) {
 	/* starting from scratch */
 #if LOOP_DEBUG
@@ -957,12 +1054,27 @@ parse_loopline (char *line, LOOPSET *ploop, int loopstack,
 	if (loop == NULL) {
 	    gretl_errmsg_set(_("Out of memory!"));
 	    return NULL;
+	} else {
+	    opt = get_gretl_options(line, &err);
+	    if (err) {
+		gretl_loop_destroy(loop);
+		return NULL;
+	    } 
 	}
     } else {
 	/* shouldn't happen: need error message? */
 	loop = ploop;
     }
 
+    line += 4; /* "loop" */
+    while (isspace((unsigned char) *line)) {
+	line++;
+    }
+
+#if LOOP_DEBUG
+    fprintf(stderr, "ready for parse: '%s'\n", line);
+#endif
+
     /* try parsing the loop line in various ways */
 
     if (sscanf(line, "while %8[^ <>=]%8[ <>=] %8s", lvar, op, rvar) == 3) {
@@ -1022,6 +1134,10 @@ parse_loopline (char *line, LOOPSET *ploop, int loopstack,
 	loop = NULL;
     }
 
+    if (!err && opt != OPT_NONE) {
+	set_loop_opts(loop, opt);
+    }
+
     return loop;
 }
 
@@ -1074,6 +1190,18 @@ eval_numeric_condition (int op, double xl, double xr)
 {
     int cont = 0;
 
+#if LOOP_DEBUG
+    char opstr[4];
+
+    if (op == GT) strcpy(opstr, "GT");
+    else if (op == GTE) strcpy(opstr, "GTE");
+    else if (op == LTE) strcpy(opstr, "LTE");
+    else if (op == LT) strcpy(opstr, "LT");
+    else strcpy(opstr, "??");
+    fprintf(stderr, "** eval_numeric_condition: xl=%g, xr=%g, "
+	    "op=%s\n", xl, xr, opstr);
+#endif
+
     if (op == GT) {
 	cont = (xl > xr);
     } else if (op == GTE) {
@@ -1108,6 +1236,10 @@ loop_condition (LOOPSET *loop, double **Z, DATAINFO *pdinfo)
 	loop->brk = 0;
 	ok = 0;
     } else if (loop->type == COUNT_LOOP || indexed_loop(loop)) {
+#if LOOP_DEBUG
+	fprintf(stderr, "** COUNT_LOOP: iter = %d, itermax = %d\n",
+		loop->iter, loop->itermax);
+#endif
 	/* simple count loop or indexed loop */
 	if (loop->iter < loop->itermax) {
 	    ok = 1;
@@ -1149,6 +1281,7 @@ static void controller_init (controller *clr)
     clr->val = NADBL;
     clr->vnum = 0;
     clr->vname[0] = 0;
+    clr->vsign = 1;
 }
 
 static void gretl_loop_init (LOOPSET *loop)
@@ -1318,7 +1451,7 @@ static int loop_model_init (LOOP_MODEL *lmod, const MODEL *pmod,
     lmod->ssq_sderr = malloc(ncoeff * sizeof *lmod->ssq_sderr);
     if (lmod->ssq_sderr == NULL) goto cleanup;
 
-    lmod->list = copylist(pmod->list);
+    lmod->list = gretl_list_copy(pmod->list);
     if (lmod->list == NULL) goto cleanup;
 
     for (i=0; i<ncoeff; i++) {
@@ -1365,7 +1498,7 @@ static int loop_print_init (LOOP_PRINT *lprn, const int *list, int id)
 {
     int i;
 
-    lprn->list = copylist(list);
+    lprn->list = gretl_list_copy(list);
     if (lprn->list == NULL) return 1;
 
     lprn->sum = malloc(list[0] * sizeof *lprn->sum);
@@ -1768,17 +1901,11 @@ LOOPSET *add_to_loop (char *line, int ci, gretlopt opt,
 	if (lret == NULL) {
 	    if (*gretl_errmsg == '\0') {
 		gretl_errmsg_set(_("No valid loop condition was given."));
+		gretl_errno = E_PARSE;
 	    }
+	    goto bailout;
 	} else {
-	    if (opt & OPT_P) {
-		loop_set_progressive(lret);
-	    }
-	    if (opt & OPT_V) {
-		loop_set_verbose(lret);
-	    }
-	    if (opt & OPT_Q) {
-		loop_set_quiet(lret);
-	    }
+	    set_loop_opts(lret, opt);
 	    *loopstack += 1;
 	}
     } else if (ci == ENDLOOP) {
@@ -1861,7 +1988,7 @@ static void print_loop_model (LOOP_MODEL *lmod, int loopnum,
 
     pputc(prn, '\n');
     pprintf(prn, _("%s estimates using the %d observations %s-%s\n"),
-	    _(estimator_string(lmod->ci, prn->format)), lmod->t2 - lmod->t1 + 1, 
+	    _(estimator_string(lmod->ci, prn)), lmod->t2 - lmod->t1 + 1, 
 	    startdate, enddate);
     pprintf(prn, _("Statistics for %d repetitions\n"), loopnum); 
     pprintf(prn, _("Dependent variable: %s\n\n"), 
@@ -2129,65 +2256,6 @@ static int get_modnum_by_cmdnum (LOOPSET *loop, int cmdnum)
     return -1;
 }
 
-/**
- * get_cmd_ci:
- * @line: command line.
- * @cmd: pointer to gretl command struct.
- *
- * Parse @line and assign to @cmd->ci the index number of
- * the command embedded in @line.
- */
-
-void get_cmd_ci (const char *line, CMD *cmd)
-{
-    static int context;
-
-    /* allow for leading spaces */
-    while (isspace(*line)) {
-	line++;
-    }
-
-    if (*line == '#') {
-	cmd->nolist = 1;
-	cmd->ci = CMD_COMMENT;
-	return;
-    }
-
-#if 0
-    fprintf(stderr, "get_cmd_ci: line = '%s'\n", line);
-#endif
-
-    if (sscanf(line, "%s", cmd->word) != 1 || 
-	*line == '(' || *line == '#') {
-	cmd->nolist = 1;
-	cmd->ci = -1;
-	return;
-    }
-
-    /* subsetted commands (e.g. "deriv" in relation to "nls") */
-    if (!strcmp(cmd->word, "end")) {
-	context = 0;
-	cmd->ci = END;
-    } else if (context && strcmp(cmd->word, "equation")) {
-	/* "equation" occurs in the SYSTEM context, but it is
-	   a command in its own right */
-	cmd->ci = context;
-    } else if ((cmd->ci = gretl_command_number(cmd->word)) == 0) {
-	cmd->errcode = 1;
-	sprintf(gretl_errmsg, _("command \"%s\" not recognized"), 
-		cmd->word);
-	return;
-    }
-
-    if (cmd->ci == NLS) {
-	context = NLS;
-    }
-
-    if (!strcmp(line, "end loop")) {
-	cmd->ci = ENDLOOP;
-    }
-}
-
 static int 
 substitute_dollar_targ (char *str, const LOOPSET *loop,
 			const double **Z, const DATAINFO *pdinfo)
@@ -2323,7 +2391,8 @@ top_of_loop (LOOPSET *loop, double **Z, const DATAINFO *pdinfo)
 
     if (!err) {
 	if (loop->init.vnum > 0) {
-	    loop->init.val = Z[loop->init.vnum][0];
+	    loop->init.val = 
+		loop_controller_get_value(&loop->init, (const double **) Z);
 	} 
 
 	if (loop->type == FOR_LOOP) {
@@ -2337,6 +2406,11 @@ top_of_loop (LOOPSET *loop, double **Z, const DATAINFO *pdinfo)
 		err = 1;
 	    } else {
 		loop->itermax = maxval - loop->init.val + 1;
+#if LOOP_DEBUG
+		fprintf(stderr, "*** itermax = %g - %g + 1 = %d\n",
+			maxval, loop->init.val, loop->itermax);
+#endif
+
 	    }
 	}
     } 
@@ -2397,14 +2471,13 @@ make_dollar_substitutions (char *str, const LOOPSET *loop,
 
 int loop_exec (LOOPSET *loop, char *line,
 	       double ***pZ, DATAINFO **ppdinfo, 
-	       MODEL **models, int *echo_off, 
-	       PRN *prn)
+	       MODEL **models, PRN *prn)
 {
     CMD cmd;
     MODEL *lastmod = models[0];
     char errline[MAXLINE];
     char linecpy[MAXLINE];
-    int m = 0, ignore = 0;
+    int m = 0;
     int modnum = 0;
     int err = 0;
 
@@ -2439,7 +2512,7 @@ int loop_exec (LOOPSET *loop, char *line,
 	fprintf(stderr, "top of loop: iter = %d\n", loop->iter);
 #endif
 
-	if (!(*echo_off) && indexed_loop(loop)) {
+	if (gretl_echo_on() && indexed_loop(loop)) {
 	    print_loop_progress(loop, *ppdinfo, prn);
 	}
 
@@ -2450,9 +2523,6 @@ int loop_exec (LOOPSET *loop, char *line,
 	    strcpy(linecpy, loop->lines[j]);
 	    strcpy(errline, loop->lines[j]);
 
-	    cmd.opt = get_gretl_options(linecpy, &err);
-	    if (err) break;
-
 	    err = make_dollar_substitutions(linecpy, loop, 
 					    (const double **) *pZ,
 					    *ppdinfo);
@@ -2462,20 +2532,19 @@ int loop_exec (LOOPSET *loop, char *line,
 	       will do some checking that hasn't been done earlier.
 	    */
 
-	    getcmd(linecpy, *ppdinfo, &cmd, &ignore, pZ, NULL);
+	    err = parse_command_line(linecpy, &cmd, pZ, *ppdinfo);
 
 	    if (cmd.ci < 0) {
 		continue;
-	    } else if (cmd.errcode) {
-		err = cmd.errcode;
+	    } else if (err) {
 		break;
 	    }
 
-	    if (!(*echo_off) && indexed_loop(loop)) {
+	    if (gretl_echo_on() && indexed_loop(loop)) {
 		if (cmd.ci == ENDLOOP) {
 		    pputc(prn, '\n');
 		} else {
-		    echo_cmd(&cmd, *ppdinfo, linecpy, 0, 1, 0, prn);
+		    echo_cmd(&cmd, *ppdinfo, linecpy, 0, prn);
 		}
 	    }
 
@@ -2510,7 +2579,7 @@ int loop_exec (LOOPSET *loop, char *line,
 
 	    case LOOP:
 		err = loop_exec(loop->children[childnum++], NULL,
-				pZ, ppdinfo, models, echo_off, prn);
+				pZ, ppdinfo, models, prn);
 		break;
 
 	    case BREAK:
@@ -2522,7 +2591,7 @@ int loop_exec (LOOPSET *loop, char *line,
 		break;
 
 	    case GENR:
-		err = generate(pZ, *ppdinfo, linecpy, lastmod);
+		err = generate(linecpy, pZ, *ppdinfo, lastmod, cmd.opt);
 		if (loop_is_verbose(loop) && !err) { 
 		    print_gretl_msg(prn);
 		}
@@ -2562,12 +2631,12 @@ int loop_exec (LOOPSET *loop, char *line,
 		    *models[0] = hsk_func(cmd.list, pZ, *ppdinfo);
 		} else if (cmd.ci == ARMA) {
 		    *models[0] = arma(cmd.list, (const double **) *pZ, *ppdinfo, 
-				      (cmd.opt & OPT_V)? prn : NULL);
+				      cmd.opt, prn);
 		} else if (cmd.ci == GARCH) {
 		    *models[0] = garch(cmd.list, pZ, *ppdinfo, cmd.opt, prn);
 		} else if (cmd.ci == CORC || cmd.ci == HILU || cmd.ci == PWE) {
-		    double rho = estimate_rho(cmd.list, pZ, *ppdinfo, 1, cmd.ci,
-					      &err, prn);
+		    double rho = estimate_rho(cmd.list, pZ, *ppdinfo, cmd.ci,
+					      &err, cmd.opt, prn);
 		    if (err) {
 			break;
 		    }
@@ -2701,7 +2770,7 @@ int loop_exec (LOOPSET *loop, char *line,
 
 		if (err) {
 		    errmsg(err, prn);
-		} else if (1 || !(*echo_off)) {
+		} else if (1 || gretl_echo_on()) {
 		    print_smpl(*ppdinfo, get_full_length_n(), prn);
 		}
 		break;
@@ -2892,13 +2961,16 @@ int if_eval (const char *line, double ***pZ, DATAINFO *pdinfo)
 
     sprintf(formula, "__iftest=%s", line);
 
-    err = generate(pZ, pdinfo, formula, NULL);
+    err = generate(formula, pZ, pdinfo, NULL, OPT_P);
+#if IFDEBUG
+    fprintf(stderr, "if_eval: generate returned %d\n", err);
+#endif
 
     if (err) {
 	strcpy(gretl_errmsg, _("error evaluating 'if'"));
     } else {
 	int v = varindex(pdinfo, "iftest");
-	
+
 	if (v < pdinfo->v) {
 	    double val = (*pZ)[v][0];
 
@@ -2907,7 +2979,7 @@ int if_eval (const char *line, double ***pZ, DATAINFO *pdinfo)
 	    } else {
 		ret = (int) val;
 	    }
-	    dataset_drop_vars(1, pZ, pdinfo);
+	    dataset_drop_last_variables(1, pZ, pdinfo);
 	}
     }
 
