@@ -20,7 +20,6 @@
 /* non-parametric stats for gretl */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 enum {
     RUN_FIRST,
@@ -209,7 +208,7 @@ static int spearman_rho (const double *x, const double *y, int n,
     *sd = xx;
 
     if (nn >= 20 && pval != NULL) {
-	*pval = normal(fabs(yy / xx)); 
+	*pval = normal_pvalue_1(fabs(yy / xx)); 
     } 
 
     /* save the ranks, if wanted */
@@ -232,7 +231,7 @@ static int spearman_rho (const double *x, const double *y, int n,
  * @list: list of (two) variables to process.
  * @Z: data matrix.
  * @pdinfo: information on the data set.
- * @opt: if non-zero, print both the "raw" and the ranked data.
+ * @opt: if includes %OPT_V, print both the "raw" and the ranked data.
  * @prn: gretl printing struct.
  *
  * Calculates and prints Spearman's rank correlation coefficient for the two
@@ -260,8 +259,8 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 		     Z[vy] + pdinfo->t1, 
 		     pdinfo->t2 - pdinfo->t1 + 1, 
 		     &rho, &sd, &pval,
-		     (opt)? &rx : NULL,
-		     (opt)? &ry : NULL,
+		     (opt & OPT_V)? &rx : NULL,
+		     (opt & OPT_V)? &ry : NULL,
 		     &m, RUN_FIRST)) {
 	return E_ALLOC;
     }
@@ -288,7 +287,7 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 		"the normal distribution\n"));
     }
 
-    if (opt) { /* print raw and ranked data */
+    if (opt & OPT_V) { /* print raw and ranked data */
 	int i = 0;
 
 	pprintf(prn, "\n     %s ", _("Obs"));
@@ -297,10 +296,10 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    print_obs_marker(t, pdinfo, prn);
 	    if (!(na(Z[vx][t])) && !(na(Z[vy][t]))) {
-		gretl_printxs(Z[vx][t], 15, PRINT, prn);
-		gretl_printxs(rx[i], 15, PRINT, prn);
-		gretl_printxs(Z[vy][t], 15, PRINT, prn);
-		gretl_printxs(ry[i], 15, PRINT, prn);
+		gretl_printxn(Z[vx][t], 15, prn);
+		gretl_printxn(rx[i], 15, prn);
+		gretl_printxn(Z[vy][t], 15, prn);
+		gretl_printxn(ry[i], 15, prn);
 		i++;
 	    }
 	    pputc(prn, '\n');
@@ -499,7 +498,7 @@ int runs_test (int varno, const double **Z, const DATAINFO *pdinfo,
     mean = (1 + nn / 2.0);
     sd = sqrt((double) n - 1) / 2.0;
     z = fabs((runs - mean) / sd);
-    pval = 2.0 * normal(z);
+    pval = normal_pvalue_2(z);
 
     pprintf(prn, _("\nNumber of runs (R) in the variable '%s' = %d\n"), 
 	    pdinfo->varname[varno], runs);
