@@ -18,20 +18,34 @@
  */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
-#define is_model_ci(c) (c == OLS || c == CORC || c == HILU || \
-                        c == WLS || c == PWE || c == POOLED || c == HCCM || \
-                        c == HSK || c == ADD || c == LAD || \
-                        c == OMIT || c == TSLS || c == LOGIT || \
-                        c == PROBIT || c == TOBIT || c == ARMA || \
-                        c == AR || c == LOGISTIC || c == NLS || \
-                        c == GARCH || c == POISSON)
+/* model commands plus ADD and OMIT */
+#define vcv_opt_ok(c) (c == ADD || \
+                       c == AR || \
+                       c == ARMA || \
+                       c == CORC || \
+                       c == GARCH || \
+                       c == HCCM || \
+                       c == HILU || \
+                       c == HSK || \
+                       c == LAD || \
+                       c == LOGISTIC || \
+                       c == LOGIT || \
+                       c == OLS || \
+                       c == OMIT || \
+                       c == NLS || \
+                       c == POISSON || \
+                       c == POOLED || \
+                       c == PROBIT || \
+                       c == PWE || \
+                       c == TOBIT || \
+                       c == TSLS || \
+                       c == WLS)
 
 struct gretl_option {
-    int ci;              /* command index (context) */
+    int ci;              /* command index (gives context) */
     gretlopt o;          /* index of integer type */
-    const char *longopt; /* -- string representation of option */
+    const char *longopt; /* "--"-style string representation of option */
 };
 
 struct flag_match {
@@ -45,6 +59,7 @@ struct flag_match {
 */
 
 struct gretl_option gretl_opts[] = {
+    { ADD,      OPT_I, "silent" },
     { ADD,      OPT_Q, "quiet" },
     { ADDTO,    OPT_Q, "quiet" },
     { ADF,      OPT_N, "nc" }, 
@@ -54,29 +69,38 @@ struct gretl_option gretl_opts[] = {
     { ADF,      OPT_V, "verbose" },
     { ADF,      OPT_Q, "quiet" },
     { ARMA,     OPT_N, "native" },
+    { ARMA,     OPT_S, "suppress-const" },    
     { ARMA,     OPT_V, "verbose" },
     { ARMA,     OPT_X, "x-12-arima" },
     { BXPLOT,   OPT_O, "notches" },
     { COINT,    OPT_N, "nc" },
-    { COINT2,   OPT_O, "verbose" },
+    { COINT2,   OPT_V, "verbose" },
     { EQNPRINT, OPT_O, "complete" },
     { TABPRINT, OPT_O, "complete" },
     { ESTIMATE, OPT_M, "geomean" },
     { ESTIMATE, OPT_N, "no-df-corr" },
     { ESTIMATE, OPT_T, "iterate" },
+    { FCAST,    OPT_D, "dynamic" },
+    { FCAST,    OPT_S, "static" },
+    { FCASTERR, OPT_D, "dynamic" },
     { FCASTERR, OPT_O, "plot" },
+    { FCASTERR, OPT_S, "static" },
     { FREQ,     OPT_O, "gamma" },
     { FREQ,     OPT_Q, "quiet" },
     { GARCH,    OPT_A, "arma-init" },    
     { GARCH,    OPT_R, "robust" },
     { GARCH,    OPT_V, "verbose" },
+    { GENR,     OPT_L, "local" },
+    { GENR,     OPT_P, "private" },
     { GNUPLOT,  OPT_O, "with-lines" },
     { GNUPLOT,  OPT_M, "with-impulses" },
     { GNUPLOT,  OPT_S, "suppress-fitted" },
     { GNUPLOT,  OPT_Z, "dummy" },
     { GRAPH,    OPT_O, "tall" },
     { HAUSMAN,  OPT_T, "time-effects" },
-    { IMPORT,   OPT_O, "box1" },
+    { HILU,     OPT_B, "no-corc" },
+    { IMPORT,   OPT_B, "box1" },
+    { IMPORT,   OPT_O, "octave" },
     { KPSS,     OPT_T, "trend" },
     { KPSS,     OPT_V, "verbose" },
     { KPSS,     OPT_Q, "quiet" },
@@ -99,6 +123,7 @@ struct gretl_option gretl_opts[] = {
     { OLS,      OPT_R, "robust" },
     { OLS,      OPT_Q, "quiet" },
     { OLS,      OPT_S, "simple-print" },
+    { OMIT,     OPT_I, "silent" },
     { OMIT,     OPT_Q, "quiet" },
     { OMITFROM, OPT_Q, "quiet" },
     { OUTFILE,  OPT_A, "append" },
@@ -115,7 +140,9 @@ struct gretl_option gretl_opts[] = {
     { PERGM,    OPT_O, "bartlett" },
     { PLOT,     OPT_O, "one-scale" },
     { PRINT,    OPT_O, "byobs" },
+    { PRINT,    OPT_N, "no-dates" },
     { PRINT,    OPT_T, "ten" },
+    { QUIT,     OPT_X, "exit" },
     { SETOBS,   OPT_C, "stacked-cross-section" },
     { SETOBS,   OPT_S, "stacked-time-series" },
     { SETOBS,   OPT_T, "time-series" },
@@ -126,10 +153,12 @@ struct gretl_option gretl_opts[] = {
     { SMPL,     OPT_M, "no-missing" },
     { SMPL,     OPT_N, "random" },
     { SMPL,     OPT_R, "restrict" },
-    { SPEARMAN, OPT_O, "verbose" },
+    { SPEARMAN, OPT_V, "verbose" },
     { SQUARE,   OPT_O, "cross" },
     { STORE,    OPT_C, "csv" },
-    { STORE,    OPT_D, "dat" },
+    { STORE,    OPT_D, "database" },
+    { STORE,    OPT_F, "overwrite" },
+    { STORE,    OPT_G, "dat" },
     { STORE,    OPT_M, "gnu-octave" },
     { STORE,    OPT_R, "gnu-R" },
     { STORE,    OPT_T, "traditional" },
@@ -149,7 +178,7 @@ const char **get_opts_for_command (int ci, int *nopt)
     int i, j, n = 0;
     const char **ret = NULL;
 
-    if (is_model_ci(ci) && ci != OLS && ci != LAD) {
+    if (vcv_opt_ok(ci) && ci != OLS) {
 	n++; /* vcv */
     }
 
@@ -172,7 +201,7 @@ const char **get_opts_for_command (int ci, int *nopt)
 	}
     }
 
-    if (is_model_ci(ci) && ci != OLS && ci != LAD) {
+    if (vcv_opt_ok(ci) && ci != OLS) {
 	ret[j++] = "vcv";
     }
 
@@ -186,6 +215,9 @@ struct flag_match flag_matches[] = {
     { OPT_B, 'b' },
     { OPT_C, 'c' },
     { OPT_D, 'd' },
+    { OPT_E, 'e' },
+    { OPT_F, 'f' },
+    { OPT_G, 'g' },
     { OPT_I, 'i' },
     { OPT_L, 'l' },
     { OPT_M, 'm' },
@@ -203,13 +235,9 @@ struct flag_match flag_matches[] = {
     { 0L,   '\0' }
 };
 
-/* note: 'f' is not treated as an option flag for now */
+static const char *ok_flags = "abcdefgilmnopqrstvwxz";
 
-#define isflag(c) (c == 'a' || c == 'b' || c == 'c' || c == 'd' || \
-                   c == 'i' || c == 'l' || c == 'm' || c == 'p' || \
-                   c == 'n' || c == 'o' || c == 'q' || c == 'r' || \
-                   c == 's' || c == 't' || c == 'v' || c == 'w' || \
-                   c == 'x' || c == 'z')
+#define isflag(c) (strchr(ok_flags, c) != NULL)
 
 static gretlopt opt_from_flag (unsigned char c)
 {
@@ -226,7 +254,7 @@ static int opt_is_valid (gretlopt opt, int ci, char c)
 {
     int i;
 
-    if (opt == OPT_O && is_model_ci(ci)) {
+    if (opt == OPT_O && vcv_opt_ok(ci)) {
 	return 1;
     }
 
@@ -266,7 +294,9 @@ static gretlopt get_short_opts (char *line, int ci, int *err)
 	    gretl_delete(p, 0, 2);
 	    match = 1;
 	}
-	if (!match) p++;
+	if (!match) {
+	    p++;
+	}
 	p = strchr(p, '-');
     }
 
@@ -292,8 +322,7 @@ static int valid_long_opt (int ci, const char *lopt)
     int opt = OPT_NONE;
     int i;
 
-    if (is_model_ci(ci) && ci != LAD && !strcmp(lopt, "vcv")) {
-	/* VCV is available for all but LAD models */
+    if (vcv_opt_ok(ci) && !strcmp(lopt, "vcv")) {
 	return OPT_O;
     }
 
@@ -375,7 +404,7 @@ static void tail_strip (char *s)
 /**
  * get_gretl_options:
  * @line: command line to parse.
- * @err: address for error code, which is set to 1 in case any 
+ * @err: location for error code, which is set to 1 in case any 
  * invalid options are found, else set to 0.
  * 
  * Check for option flags in @line: if found, chop them out and set
@@ -399,7 +428,9 @@ gretlopt get_gretl_options (char *line, int *err)
 	*err = 0;
     }
 
-    if (n < 2 || *line == '#') return oflags;
+    if (n < 2 || *line == '#') {
+	return oflags;
+    }
 
     /* to enable reading of trad. esl input files */
     if (line[n-2] == ';' && isspace(line[n-1])) {
@@ -421,7 +452,9 @@ gretlopt get_gretl_options (char *line, int *err)
 	ci = gretl_command_number(cmdword);
     }
 
-    if (ci == 0) return oflags;
+    if (ci == 0) {
+	return oflags;
+    }
 
     /* try for short-form options (e.g. "-o") */
     opt = get_short_opts(line, ci, &myerr);
@@ -465,11 +498,18 @@ const char *print_flags (gretlopt oflags, int ci)
 
     flagstr[0] = '\0';
 
-    if (oflags == 0L) return flagstr;
+    if (oflags == OPT_NONE) {
+	return flagstr;
+    }
+
+    if (ci == QUIT || ci == GENR) {
+	/* any option flags are "hidden" */
+	return flagstr;
+    }
 
     /* special: -o (--vcv) can be used with several model
        commands */
-    if ((oflags & OPT_O) && is_model_ci(ci)) {
+    if ((oflags & OPT_O) && vcv_opt_ok(ci)) {
 	strcat(flagstr, " --vcv");
 	oflags &= ~OPT_O;
     }
@@ -486,14 +526,16 @@ const char *print_flags (gretlopt oflags, int ci)
 
 int check_for_loop_only_options (int ci, gretlopt opt, PRN *prn)
 {
+    int ret = 0;
+
     if (ci == OLS && (opt & OPT_P)) {
 	const char *flagstr = print_flags(OPT_P, OLS);
 
 	pprintf(prn, _("Warning: option%s ignored outside of loop"), 
 		flagstr);
 	pputc(prn, '\n');
-	return 1;
+	ret = 1;
     }
 
-    return 0;
+    return ret;
 }
