@@ -21,9 +21,11 @@
 #include "qr_estimate.h"
 #include "gretl_matrix.h"
 #include "gretl_matrix_private.h"
-#include "gretl_private.h"
 #include "libset.h"
 
+#include "f2c.h"
+#include "clapack_double.h"
+
 #define QR_RCOND_MIN 1e-15 /* experiment with this? */
 #define ESSZERO      1e-22 /* SSR < this counts as zero */
 
@@ -79,7 +81,8 @@ static void qr_compute_r_squared (MODEL *pmod, const double *y, int n)
 	    pmod->rsq = 1.0 - (pmod->ess / pmod->tss);
 	    pmod->adjrsq = 1.0 - (pmod->ess * (n - 1) / den);
 	} else {
-	    double alt = corrrsq(n, y + pmod->t1, pmod->yhat + pmod->t1);
+	    double alt = 
+		gretl_corr_rsq(pmod->t1, pmod->t2, y, pmod->yhat);
 
 	    if (na(alt)) {
 		pmod->rsq = pmod->adjrsq = NADBL;
@@ -93,7 +96,7 @@ static void qr_compute_r_squared (MODEL *pmod, const double *y, int n)
     }
 }
 
-int get_vcv_index (MODEL *pmod, int i, int j, int n)
+static int get_vcv_index (MODEL *pmod, int i, int j, int n)
 {
     int k, vi, vj;
 
@@ -800,7 +803,7 @@ int gretl_qr_regress (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
 	qr_make_regular_vcv(pmod, xpxinv);
     }
 
-    /* get R^2 */
+    /* get R^2, F */
     qr_compute_r_squared(pmod, (*pZ)[pmod->list[1]], T);
     qr_compute_f_stat(pmod, opts);
 
