@@ -26,9 +26,13 @@ void gretl_uniform_dist (double *a, int t1, int t2);
 
 void gretl_normal_dist (double *a, int t1, int t2);
 
-unsigned int gretl_rand_int (void);
-
 unsigned int gretl_rand_int_max (unsigned int max);
 
+void gretl_rand_init (void);
+
+void gretl_rand_free (void);
+
+unsigned int get_gretl_random_seed (void);
+
 #endif /* RANDOM_H */
 
