@@ -20,7 +20,6 @@
 /* subsample.c for gretl */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #undef SUBDEBUG
 
@@ -49,16 +48,16 @@ static const DATAINFO *peerinfo;
 
 /* .......................................................... */
 
-char *copy_subdum (const char *src, int n)
+char *copy_subsample_mask (const char *src, int n)
 {
-    char *ret;
+    char *ret = NULL;
 
-    if (n == 0 || src == NULL) return NULL;
-
-    ret = malloc(n * sizeof *ret);
-    if (ret == NULL) return NULL;
-
-    memcpy(ret, src, n);
+    if (n != 0 && src != NULL) {
+	ret = malloc(n * sizeof *ret);
+	if (ret != NULL) {
+	    memcpy(ret, src, n);
+	}
+    }
 
     return ret;
 }
@@ -99,20 +98,23 @@ static int
 attach_subsample_to_datainfo (DATAINFO *subinfo, const double **Z, 
 			      const DATAINFO *pdinfo)
 {
-    int sv, t, n = pdinfo->n;
+    int s, t;
 
-    subinfo->subdum = malloc(n * sizeof *subinfo->subdum);
-    if (subinfo->subdum == NULL) return E_ALLOC;
+    subinfo->submask = malloc(pdinfo->n * sizeof *subinfo->submask);
+    if (subinfo->submask == NULL) {
+	return E_ALLOC;
+    }
 
-    sv = varindex(pdinfo, "subdum");
-    if (sv == pdinfo->v) { 
+    s = varindex(pdinfo, "subdum");
+
+    if (s == pdinfo->v) { 
 	/* safety measure: should be impossible */
 	fprintf(stderr, "mystery failure in attach_subsample_to_dataset\n");
 	return 1;   
     } 
 
-    for (t=0; t<n; t++) {
-	subinfo->subdum[t] = Z[sv][t];
+    for (t=0; t<pdinfo->n; t++) {
+	subinfo->submask[t] = Z[s][t];
     }
 
     return 0;
@@ -137,8 +139,8 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	/* sync, in case anything has moved */
 	sync_dataset_elements(pdinfo);
 
-	pmod->subdum = copy_subdum(pdinfo->subdum, fullinfo->n);
-	if (pmod->subdum == NULL) {
+	pmod->submask = copy_subsample_mask(pdinfo->submask, fullinfo->n);
+	if (pmod->submask == NULL) {
 	    err = E_ALLOC;
 	}
     }
@@ -151,33 +153,15 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 static double *get_old_mask (double **Z, const DATAINFO *pdinfo)
 {
     double *mask = NULL;
-    int v = varindex(pdinfo, "subdum");
+    int s = varindex(pdinfo, "subdum");
 
-    if (v < pdinfo->v && isdummy(Z[v], 0, pdinfo->n - 1)) {
-	mask = Z[v];
+    if (s < pdinfo->v && gretl_isdummy(0, pdinfo->n - 1, Z[s])) {
+	mask = Z[s];
     }    
 
     return mask;
 }
 
-static int dummy_with_missing (const double *x, int t1, int t2)
-{
-    int t, m = 0;
-    double xx;
-
-    for (t=t1; t<=t2; t++) {
-	xx = x[t];
-	if (!na(xx) && floatneq(xx, 0.0) && floatneq(xx, 1.0)) { 
-	    return 0;
-	}
-	if (floateq(xx, 1.0)) m++;
-    }
-
-    if (m < t2 - t1 + 1) return m;
-
-    return 0;
-} 
-
 static int overlay_masks (char *targ, const double *src, int n)
 {
     int i, sn = 0;
@@ -233,14 +217,7 @@ static void copy_to_mask (char *mask, const double *x, int n)
     int t;
 
     for (t=0; t<n; t++) {
-	mask[t] = (x[t] != 0.0);
-    }
-}
-
-static void check_dummy_missvals (const double *x, int t1, int t2)
-{
-    if (dummy_with_missing(x, t1, t2)) {
-	strcpy(gretl_errmsg, _("Missing values found when applying criterion"));
+	mask[t] = (x[t] != 0.0 && !na(x[t]));
     }
 }
 
@@ -249,26 +226,27 @@ static int sn_from_tmp_dummy (double ***pZ, DATAINFO *pdinfo,
 {
     char formula[MAXLEN];
     int dnum, isdum, dlist[2];
+    int err;
 
     /* + 4 to skip the command word "smpl" */
     sprintf(formula, "__tmpmsk=%s", line + 4);
 
-    if (generate(pZ, pdinfo, formula, NULL)) {
+    err = generate(formula, pZ, pdinfo, NULL, OPT_P);
+    *gretl_msg = '\0';
+
+    if (err) {
 	return -1;
     }
 
     dnum = varindex(pdinfo, "tmpmsk");
 
-    isdum = isdummy((*pZ)[dnum], pdinfo->t1, pdinfo->t2);
-    if (isdum == 0) {
-	check_dummy_missvals((*pZ)[dnum], pdinfo->t1, pdinfo->t2);
-    } 
+    isdum = gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[dnum]);
 
     copy_to_mask(mask, (*pZ)[dnum], pdinfo->n);
 
     dlist[0] = 1;
     dlist[1] = dnum;
-    dataset_drop_listed_vars(dlist, pZ, pdinfo, NULL);
+    dataset_drop_listed_variables(dlist, pZ, pdinfo, NULL);
 
     return isdum;
 }
@@ -284,10 +262,7 @@ static int sn_from_dummy (const double **Z, const DATAINFO *pdinfo,
 	return -1;
     } 
 
-    isdum = isdummy(Z[dnum], pdinfo->t1, pdinfo->t2);
-    if (isdum == 0) {
-	check_dummy_missvals(Z[dnum], pdinfo->t1, pdinfo->t2);
-    } 
+    isdum = gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[dnum]);
 
     copy_to_mask(mask, Z[dnum], pdinfo->n);
 
@@ -347,20 +322,20 @@ static int make_random_mask (double *oldmask, int fulln, int subn,
     return cases;
 }
 
-static int maybe_add_subdum (double ***pZ, DATAINFO *pdinfo, int *subnum)
+static int maybe_add_subdum (double ***pZ, DATAINFO *pdinfo, int *snum)
 {
-    int v = varindex(pdinfo, "subdum");
+    int s = varindex(pdinfo, "subdum");
 
-    if (v == pdinfo->v) {
+    if (s == pdinfo->v) {
 	/* variable doesn't exist: create it */
-	if (dataset_add_vars(1, pZ, pdinfo)) {
+	if (dataset_add_series(1, pZ, pdinfo)) {
 	    return 1;
 	}
-	strcpy(pdinfo->varname[v], "subdum");
-	strcpy(VARLABEL(pdinfo, v), _("automatic sub-sampling dummy"));
+	strcpy(pdinfo->varname[s], "subdum");
+	strcpy(VARLABEL(pdinfo, s), _("automatic sub-sampling dummy"));
     } 
 
-    *subnum = v;
+    *snum = s;
 
     return 0;
 }
@@ -393,34 +368,41 @@ static void relink_full_dataset (double ***pZ, DATAINFO **ppdinfo)
 
 int complex_subsampled (void)
 {
-    if (fullZ == NULL) return 0;
-    else return 1;
+    return (fullZ != NULL);
 }
 
 int get_full_length_n (void)
 {
+    int n = 0;
+
     if (fullinfo != NULL) {
-	return fullinfo->n;
-    } else {
-	return 0;
+	n = fullinfo->n;
     }
+
+    return n;
 }
 
 static int block_retained (const char *s)
 {
     int nrows = 0;
+    int ret = 0;
 
 #if 0
     fprintf(stderr, "block_retained: s = '%s'\n", s);
 #endif
 
     while (*s) {
-	if (*s == '1') nrows++;
-	if (nrows > 1) return 1;
+	if (*s == '1') {
+	    nrows++;
+	}
+	if (nrows > 1) {
+	    ret = 1;
+	    break;
+	}
 	s++;
     }
 
-    return 0;
+    return ret;
 }
 
 static char *make_panel_signature (const DATAINFO *pdinfo,
@@ -634,7 +616,7 @@ int restrict_sample (const char *line,
 		     double ***pZ, DATAINFO **ppdinfo, 
 		     const int *list, gretlopt oflag)
 {
-    char **S = NULL, dname[VNAMELEN] = {0};
+    char dname[VNAMELEN] = {0};
     int subnum = 0;
     int t, sn = 0;
     int opt = SUBSAMPLE_UNKNOWN;
@@ -645,6 +627,7 @@ int restrict_sample (const char *line,
     int err = 0;
 
     *gretl_errmsg = '\0';
+    *gretl_msg = '\0';
 
     if (oflag & OPT_M) {
 	opt = SUBSAMPLE_DROP_MISSING;
@@ -677,16 +660,12 @@ int restrict_sample (const char *line,
 
     if (opt == SUBSAMPLE_DROP_MISSING) {   
 	sn = make_missing_mask((const double **) *pZ, *ppdinfo, list, mask);
-    }  
-
-    else if (opt == SUBSAMPLE_RANDOM) {
+    } else if (opt == SUBSAMPLE_RANDOM) {
 	sn = make_random_mask(oldmask, (*ppdinfo)->n, atoi(line + 4), mask);
 	if (sn == 0) {
 	    err = 1;
 	}
-    }
-
-    else if (opt == SUBSAMPLE_USE_DUMMY || opt == SUBSAMPLE_BOOLEAN) {
+    } else if (opt == SUBSAMPLE_USE_DUMMY || opt == SUBSAMPLE_BOOLEAN) {
 	if (opt == SUBSAMPLE_USE_DUMMY) {
 	    sn = sn_from_dummy((const double **) *pZ, *ppdinfo, dname, mask);
 	} else {
@@ -695,9 +674,7 @@ int restrict_sample (const char *line,
 	if (sn < 0) {
 	    err = 1;
 	}
-    } 
-
-    else {
+    } else {
 	/* impossible */
 	strcpy(gretl_errmsg, _("Sub-sample command failed mysteriously"));
 	err = 1;
@@ -725,7 +702,10 @@ int restrict_sample (const char *line,
 	    if (mask[(*ppdinfo)->t1] == 0) {
 		strcpy(gretl_errmsg, _("No observations would be left!"));
 	    } else {
-		strcpy(gretl_errmsg, _("No observations were dropped!"));
+		/* this is not really an error, just a no-op */
+		strcpy(gretl_msg, _("No observations were dropped!"));
+		free(mask);
+		return 0;
 	    }
 	}
 	err = 1;
@@ -771,8 +751,8 @@ int restrict_sample (const char *line,
 	return E_ALLOC;
     }
 
-    /* link (don't copy) varnames and descriptions 
-       (these are not dependent on the series length) */
+    /* link (don't copy) varnames and descriptions, 
+       these are not dependent on the series length */
     subinfo->varname = (*ppdinfo)->varname;
     subinfo->varinfo = (*ppdinfo)->varinfo;
     subinfo->descrip = (*ppdinfo)->descrip;
@@ -780,14 +760,13 @@ int restrict_sample (const char *line,
 
     /* set up case markers? */
     if ((*ppdinfo)->markers) {
-	S = allocate_case_markers(sn);
-	if (S == NULL) {
+	err = dataset_allocate_obs_markers(subinfo);
+	if (err) {
 	    free_Z(subZ, subinfo);
 	    free(subinfo);
 	    free(mask);
 	    return E_ALLOC;
 	}
-	subinfo->S = S;
 	subinfo->markers = (*ppdinfo)->markers;
     }
 
@@ -824,8 +803,14 @@ int restrict_sample (const char *line,
 
 /* .......................................................... */
 
+enum {
+    SMPL_T1,
+    SMPL_T2
+};
+
 static int 
-get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo)
+get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo,
+		  int code)
 {
     int v, ret = -1;
 
@@ -844,7 +829,11 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo)
 		}
 	    }
 	}
-	ret = pdinfo->t1 + incr;
+	if (code == SMPL_T1) {
+	    ret = pdinfo->t1 + incr;
+	} else {
+	    ret = pdinfo->t2 + incr;
+	}
     } else {
 	/* absolute form */
 	ret = get_t_from_obs_string(s, Z, pdinfo);
@@ -853,14 +842,13 @@ get_sample_limit (char *s, const double **Z, DATAINFO *pdinfo)
     return ret;
 }
 
-/* .......................................................... */
-
 int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 {
     int nf, new_t1 = pdinfo->t1, new_t2 = pdinfo->t2;
     char cmd[5], newstart[OBSLEN], newstop[OBSLEN];
 
     *gretl_errmsg = '\0';
+    *gretl_msg = '\0';
 
     nf = count_fields(line);
 
@@ -885,7 +873,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 	    strcpy(gretl_errmsg, _("error reading smpl line"));
 	    return 1;
 	} else {
-	    new_t1 = get_sample_limit(newstart, Z, pdinfo);
+	    new_t1 = get_sample_limit(newstart, Z, pdinfo, SMPL_T1);
 	    if (new_t1 < 0 || new_t1 >= pdinfo->n) {
 		strcpy(gretl_errmsg, _("error in new starting obs"));
 		return 1;
@@ -895,7 +883,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
 	}
     }
 
-    /* now we're looking at nf = 3 case */
+    /* now we're looking at nf = 3 (3 fields) case */
 
     if (sscanf(line, "%4s %10s %10s", cmd, newstart, newstop) != 3) {
 	strcpy(gretl_errmsg, _("error reading smpl line"));
@@ -903,7 +891,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     }
 
     if (strcmp(newstart, ";")) {
-	new_t1 = get_sample_limit(newstart, Z, pdinfo);
+	new_t1 = get_sample_limit(newstart, Z, pdinfo, SMPL_T1);
 	if (new_t1 < 0 || new_t1 >= pdinfo->n) {
 	    strcpy(gretl_errmsg, _("error in new starting obs"));
 	    return 1;
@@ -911,7 +899,7 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     }
 
     if (strcmp(newstop, ";")) {
-	new_t2 = get_sample_limit(newstop, Z, pdinfo);
+	new_t2 = get_sample_limit(newstop, Z, pdinfo, SMPL_T2);
 	if (new_t2 < 0 || new_t2 >= pdinfo->n) {
 	    strcpy(gretl_errmsg, _("error in new ending obs"));
 	    return 1;
@@ -929,9 +917,54 @@ int set_sample (const char *line, const double **Z, DATAINFO *pdinfo)
     return 0;
 }
 
-/* ........................................................... */
+/* apparatus for updating full dataset when restoring full sample
+   after sub-sampling */
 
-static int datamerge (double ***pZ, DATAINFO *pdinfo)
+static void
+update_full_data_values (const double **subZ, const DATAINFO *pdinfo)
+{
+    int i, t;
+
+    for (i=1; i<fullinfo->v; i++) {
+	int subt = 0;
+
+	if (!pdinfo->vector[i]) {
+	    fullZ[i][0] = subZ[i][0];
+	} else {
+	    for (t=0; t<fullinfo->n; t++) {
+		if (pdinfo->submask[t]) {
+		    fullZ[i][t] = subZ[i][subt++];
+		}
+	    }
+	}
+    }
+}
+
+static int update_case_markers (const DATAINFO *pdinfo)
+{
+    int err = 0;
+
+    if (pdinfo->markers && !fullinfo->markers) {
+	dataset_allocate_obs_markers(fullinfo);
+	if (fullinfo->S == NULL) {
+	    err = 1;
+	} else {
+	    int t, subt = 0;
+
+	    for (t=0; t<fullinfo->n; t++) {
+		if (pdinfo->submask[t]) {
+		    strcpy(fullinfo->S[t], pdinfo->S[subt++]);
+		} else {
+		    sprintf(fullinfo->S[t], "%d", t + 1);
+		}
+	    }
+	}
+    }	
+	
+    return err;
+}
+
+static int add_new_vars_to_full (const double **Z, DATAINFO *pdinfo)
 {
     int i, t, subt;
     int newvars = pdinfo->v - fullinfo->v;
@@ -939,9 +972,11 @@ static int datamerge (double ***pZ, DATAINFO *pdinfo)
     double **newZ = NULL;
     int err = 0;
 
-    if (newvars <= 0) return 0;
+    if (newvars <= 0) {
+	return 0;
+    }
 
-    if (pdinfo->subdum == NULL) {
+    if (pdinfo->submask == NULL) {
 	return E_NOMERGE;
     }
 
@@ -959,24 +994,29 @@ static int datamerge (double ***pZ, DATAINFO *pdinfo)
 		break;
 	    }
 	}
-    } else err = 1;
+    } else {
+	err = 1;
+    }
 
-    if (err) return E_ALLOC;
+    if (err) {
+	return E_ALLOC;
+    }
     
     fullZ = newZ;
 
     for (i=fullinfo->v; i<pdinfo->v; i++) {
 	if (!pdinfo->vector[i]) {
-	   fullZ[i][0] = (*pZ)[i][0]; 
+	   fullZ[i][0] = Z[i][0]; 
 	}
     }
 
     subt = 0;
+
     for (t=0; t<n; t++) {
-	if (pdinfo->subdum[t]) {
+	if (pdinfo->submask[t]) {
 	    for (i=fullinfo->v; i<pdinfo->v; i++) {
 		if (pdinfo->vector[i]) {
-		    fullZ[i][t] = (*pZ)[i][subt];
+		    fullZ[i][t] = Z[i][subt];
 		}
 	    }
 	    subt++;
@@ -994,18 +1034,16 @@ static int datamerge (double ***pZ, DATAINFO *pdinfo)
     return 0;
 }
 
-/* ........................................................... */
-
 static int make_smpl_mask (double ***pZ, DATAINFO *pdinfo)
 {
     int old_vmax = pdinfo->v;
     int v, t;
+    int err = 0;
 
     /* add "subdum" variable if it's not already present */
-    if (maybe_add_subdum(pZ, pdinfo, &v))
-	return 1;
-
-    if (v == old_vmax) {
+    if (maybe_add_subdum(pZ, pdinfo, &v)) {
+	err = 1;
+    } else if (v == old_vmax) {
 	/* no pre-existing mask: write it from scratch */
 	for (t=0; t<pdinfo->n; t++) {
 	    (*pZ)[v][t] = (t < pdinfo->t1 || t > pdinfo->t2)? 0.0 : 1.0;
@@ -1019,7 +1057,7 @@ static int make_smpl_mask (double ***pZ, DATAINFO *pdinfo)
 	}	
     }
 
-    return 0;
+    return err;
 }
 
 /* restore_full_sample is called under various circumstances.
@@ -1038,6 +1076,7 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
     int i, t, err = 0;
 
     *gretl_errmsg = '\0';
+    *gretl_msg = '\0';
 
 #ifdef SUBDEBUG
     fprintf(stderr, "restore_full_sample: pZ=%p, ppdinfo=%p opt=%lu\n",
@@ -1068,9 +1107,20 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
        subsampled, e.g., by some boolean criterion 
     */
 
-    /* in case any new vars were added when subsampled, 
-       try to merge them into the full dataset */
-    err = datamerge(pZ, *ppdinfo);
+    /* reattach malloc'd elements of subsampled dataset,
+       which may have moved */
+    sync_dataset_elements(*ppdinfo);
+
+    /* update values for pre-existing series, which may have been
+       modified via genr */
+    update_full_data_values((const double **) *pZ, *ppdinfo);
+
+    /* if case markers were added when subsampled, carry them back */
+    update_case_markers(*ppdinfo);
+
+    /* in case any new vars were added when subsampled, try to merge
+       them into the full dataset */
+    err = add_new_vars_to_full((const double **) *pZ, *ppdinfo);
     if (err == E_ALLOC) {
         sprintf(gretl_errmsg, _("Out of memory expanding data set\n"));
     } else if (err == E_NOMERGE) {
@@ -1078,17 +1128,6 @@ int restore_full_sample (double ***pZ, DATAINFO **ppdinfo, gretlopt opt)
 		_("Missing sub-sample information; can't merge data\n"));
     }
 
-    /* reattach malloc'd elements of subsampled dataset,
-       which may have moved */
-    sync_dataset_elements(*ppdinfo);
-
-    /* copy any scalars, which may have been modified */
-    for (i=1; i<(*ppdinfo)->v; i++) {
-	if (!(*ppdinfo)->vector[i]) {
-	    fullZ[i][0] = (*pZ)[i][0];
-	}
-    }
-
     /* if we plan to replace existing sample restrictions,
        zero out the "subdum" dummy variable */
     if (opt & OPT_C) {
@@ -1125,7 +1164,7 @@ int count_missing_values (double ***pZ, DATAINFO *pdinfo, PRN *prn)
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	tmiss = 0;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (hidden_var(i, pdinfo) || !pdinfo->vector[i]) continue;
+	    if (is_hidden_variable(i, pdinfo) || !pdinfo->vector[i]) continue;
 	    if (na((*pZ)[i][t])) {
 		if (missvec[i] == 0) {
 		    missvec[0] += 1;
@@ -1199,7 +1238,7 @@ static void copy_series_info (DATAINFO *dest, const DATAINFO *src)
 /* Below: For a model that was estimated using a data sample which
    differs from the current sample, reconstitute the dataset
    appropriate to the model.  If the model used a subsample, we use
-   the special dummy variable (subdum) saved with the model to select
+   the special dummy variable (submask) saved with the model to select
    observations from the full dataset.  Another possibility is that
    the model used the full dataset, which has by now been subsampled.
 
@@ -1234,12 +1273,12 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     /* sync malloced elements that may have moved */
     sync_dataset_elements(pdinfo);
 
-    if (pmod->subdum == NULL) {
+    if (pmod->submask == NULL) {
 	/* no subsample info: model was estimated on full dataset,
 	   so we reconstruct the full dataset */
 	sn = fullinfo->n;
 #ifdef SUBDEBUG
-	fprintf(stderr, "pmod->subdum = NULL, set sn = %d\n", sn);
+	fprintf(stderr, "pmod->submask = NULL, set sn = %d\n", sn);
 #endif
     } else {
 	/* model estimated on subsample, which has to be reconstructed */
@@ -1250,11 +1289,11 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
 	    return E_ALLOC;
 	}
 #ifdef SUBDEBUG
-	fprintf(stderr, "setting mask from pmod->subdum: fullinfo->n = %d\n", 
+	fprintf(stderr, "setting mask from pmod->submask: fullinfo->n = %d\n", 
 		fullinfo->n);
 #endif
 	for (t=0; t<fullinfo->n; t++) {
-	    if (pmod->subdum[t] > 0) {
+	    if (pmod->submask[t] > 0) {
 		mask[t] = 1;
 		sn++;
 	    }
@@ -1307,7 +1346,7 @@ int add_dataset_to_model (MODEL *pmod, const DATAINFO *pdinfo)
        subsampled dataset these features from the full dataset will be
        wrong, and we stay with the simple defaults
     */
-    if (pmod->subdum == NULL) {
+    if (pmod->submask == NULL) {
 	modinfo->pd = fullinfo->pd;
 	modinfo->sd0 = fullinfo->sd0;
 	strcpy(modinfo->stobs, fullinfo->stobs);
