@@ -20,7 +20,6 @@
 /* mechanisms for defining and handling systems of equations */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 #include "system.h"
 #include "system_private.h"
 
@@ -394,7 +393,7 @@ static void sur_rearrange_lists (gretl_equation_system *sys)
 }
 
 int gretl_equation_system_append (gretl_equation_system *sys, 
-				  int *list)
+				  const int *list)
 {
     int i, neq;
 
@@ -987,10 +986,10 @@ const char *gretl_system_get_name (const gretl_equation_system *sys)
 int system_adjust_t1t2 (gretl_equation_system *sys,
 			int *t1, int *t2, const double **Z)
 {
-    int i, misst, err = 0;
+    int i, err = 0;
 
     for (i=0; i<sys->n_equations && !err; i++) {
-	err = adjust_t1t2(NULL, sys->lists[i], t1, t2, Z, &misst);
+	err = check_for_missing_obs(sys->lists[i], t1, t2, Z, NULL);
     }
 
     if (!err) {
