@@ -22,7 +22,7 @@
 
 typedef struct _gretl_equation_system gretl_equation_system;
 
-enum gretl_system_methods {
+typedef enum {
     SYS_SUR = 0,
     SYS_3SLS,
     SYS_FIML,
@@ -31,7 +31,7 @@ enum gretl_system_methods {
     SYS_TSLS,
     SYS_WLS,
     SYS_MAX
-};
+} gretl_system_methods;
 
 gretl_equation_system *system_start (const char *line);
 
@@ -43,7 +43,7 @@ char *get_system_name_from_line (const char *s);
 void gretl_equation_systems_cleanup (void);
 
 int gretl_equation_system_append (gretl_equation_system *sys, 
-				  int *list);
+				  const int *list);
 
 int system_parse_line (gretl_equation_system *sys,
 		       const char *line, 
