@@ -21,8 +21,6 @@
 
 #include "libgretl.h"
 
-/* ......................................................... */
-
 static void tex_modify_exponent (char *numstr)
 {
     char *p = strchr(numstr, 'e');
@@ -34,13 +32,12 @@ static void tex_modify_exponent (char *numstr)
     }
 }
 
-/* ......................................................... */
+/* prints a floating point number as a TeX math string.  if tab != 0,
+   print the sign in front, separated by a tab symbol (for
+   equation-style regression printout).
+*/
 
 static void tex_print_float (double x, PRN *prn)
-     /* prints a floating point number as a TeX math string.
-	if tab != 0, print the sign in front, separated by a
-	tab symbol (for equation-style regression printout).
-     */
 {
     char number[48];
 
@@ -92,6 +89,11 @@ void tex_dcolumn_double (double xx, char *numstr)
 {
     double a;
 
+    if (na(xx)) {
+	strcpy(numstr, "\\multicolumn{1}{c}{}");
+	return;
+    }
+
     xx = screen_zero(xx);
     a = fabs(xx);
 
@@ -116,7 +118,7 @@ static void tex_make_cname (char *cname, const char *src)
     char *p;
     unsigned char c;
 
-    if (src == NULL || strlen(src) == 0) return;
+    if (src == NULL || *src == '\0') return;
 
     p = strrchr(src, '_');
     if (p == NULL) {
@@ -141,20 +143,15 @@ static int tex_greek_param (char *targ, const char *src)
 
     if (!strcmp(src, "alpha")) {
 	strcpy(targ, "$\\alpha$");
-    }
-    else if (!strcmp(src, "beta")) {
+    } else if (!strcmp(src, "beta")) {
 	strcpy(targ, "$\\beta$");
-    }
-    else if (!strcmp(src, "gamma")) {
+    } else if (!strcmp(src, "gamma")) {
 	strcpy(targ, "$\\gamma$");
-    }
-    else if (!strcmp(src, "delta")) {
+    } else if (!strcmp(src, "delta")) {
 	strcpy(targ, "$\\delta$");
-    }
-    else if (!strcmp(src, "pi")) {
+    } else if (!strcmp(src, "pi")) {
 	strcpy(targ, "$\\pi$");
-    }
-    else if (!strcmp(src, "lambda")) {
+    } else if (!strcmp(src, "lambda")) {
 	strcpy(targ, "$\\lambda$");
     }
 
@@ -242,43 +239,42 @@ static void tex_lagname (char *s, const DATAINFO *pdinfo, int v)
 }
 
 int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-		     int c, PRN *prn)
+		     int i, PRN *prn)
 {
     char tmp[24], coeff[64], sderr[64], tratio[64], pval[64];
-    int v = c - 2;
 
-    if (isnan(pmod->coeff[v]) || na(pmod->coeff[v])) {
+    if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	sprintf(coeff, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
     } else {
-	tex_dcolumn_double(pmod->coeff[v], coeff);
+	tex_dcolumn_double(pmod->coeff[i], coeff);
     }
 
-    if (isnan(pmod->sderr[v]) || na(pmod->sderr[v])) {
+    if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
 	sprintf(sderr, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
 	sprintf(tratio, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
 	sprintf(pval, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
     } else {
-	tex_dcolumn_double(pmod->sderr[v], sderr);
-	sprintf(tratio, "%.4f", pmod->coeff[v] / pmod->sderr[v]);
-	sprintf(pval, "%.4f", coeff_pval(pmod, pmod->coeff[v] / pmod->sderr[v], 
+	tex_dcolumn_double(pmod->sderr[i], sderr);
+	sprintf(tratio, "%.4f", pmod->coeff[i] / pmod->sderr[i]);
+	sprintf(pval, "%.4f", coeff_pval(pmod, pmod->coeff[i] / pmod->sderr[i], 
 					 pmod->dfd));
     }    
 
     *tmp = 0;
     if (pmod->aux == AUX_ARCH) {
-	tex_make_cname(tmp, pdinfo->varname[pmod->list[c]]);
+	tex_make_cname(tmp, pdinfo->varname[pmod->list[i+2]]);
     } else if (pmod->ci == NLS) {
-	if (!tex_greek_param(tmp, pmod->params[c-1])) {
-	    tex_escape(tmp, pmod->params[c-1]);
+	if (!tex_greek_param(tmp, pmod->params[i+1])) {
+	    tex_escape(tmp, pmod->params[i+1]);
 	}
     } else if (pmod->ci == ARMA) {
-	tex_arma_coeff_name(tmp, pmod->params[c-1], 0);
+	tex_arma_coeff_name(tmp, pmod->params[i+1], 0);
     } else if (pmod->ci == GARCH) {
-	tex_garch_coeff_name(tmp, pmod->params[c-1], 0);
+	tex_garch_coeff_name(tmp, pmod->params[i+1], 0);
     } else if (pmod->ci == VAR) {
-	tex_lagname(tmp, pdinfo, pmod->list[c]);
+	tex_lagname(tmp, pdinfo, pmod->list[i+2]);
     } else {
-	tex_escape(tmp, pdinfo->varname[pmod->list[c]]);
+	tex_escape(tmp, pdinfo->varname[pmod->list[i+2]]);
     }
 	
     if (pmod->ci != LOGIT && pmod->ci != PROBIT) {
@@ -292,12 +288,13 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 		sderr,
 		tratio,
 		pval);	
-    } else { /* LOGIT, PROBIT */
+    } else { 
+	/* LOGIT, PROBIT */
 	double *slopes = gretl_model_get_data(pmod, "slopes");
 	char slope[32];
 
-	if (pmod->list[c]) {
-	    tex_dcolumn_double(slopes[v], slope);
+	if (pmod->list[i+2]) {
+	    tex_dcolumn_double(slopes[i], slope);
 	}
 	pprintf(prn, "%s &\n"
 		"  %s &\n"
@@ -308,29 +305,35 @@ int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod,
 		coeff,
 		sderr,
 		tratio,
-		(pmod->list[c])? slope : "");
+		(pmod->list[i+2])? slope : "");
     }
 
     return 0;
 }
 
-/* ......................................................... */
-
-static int make_texfile (int ID, int equation, 
-			 char *texfile, PRN *prn)
+static PRN *make_tex_prn (int ID, char *texfile,
+			  int eqn, int doc)
 {
-    FILE *fp;
+    PrnFormat fmt = GRETL_FORMAT_TEX;
+    PRN *prn;
 
-    if (*texfile == 0) {
+    if (*texfile == '\0') {
 	sprintf(texfile, "%s%s_%d.tex", gretl_user_dir(),
-		(equation)? "equation" : "model", ID);
+		(eqn)? "equation" : "model", ID);
     }
 
-    fp = gretl_fopen(texfile, "w");
-    if (fp == NULL) return 1;
+    prn = gretl_print_new_with_filename(texfile);
+    if (prn != NULL) {
+	if (eqn) {
+	    fmt |= GRETL_FORMAT_EQN;
+	}
+	if (doc) {
+	    fmt |= GRETL_FORMAT_DOC;
+	}
+	gretl_print_set_format(prn, fmt);
+    }
 
-    gretl_print_attach_file(prn, fp);
-    return 0;
+    return prn;
 }
 
 /* mechanism for customizing gretl's tex preamble */
@@ -409,7 +412,7 @@ void gretl_tex_preamble (PRN *prn, int ams)
 	if (ams) {
 	    pputs(prn, "\\usepackage{amsmath}\n\n");
 	} else {
-	    pputs(prn, "\\usepackage{dcolumn}\n\n");
+	    pputs(prn, "\\usepackage{dcolumn,longtable}\n\n");
 	}
 
 	pputs(prn, "\\begin{document}\n\n"
@@ -426,7 +429,7 @@ void gretl_tex_preamble (PRN *prn, int ams)
  * @standalone: print as full doc (1) or fragment (0).
  * @prn: gretl printing struct.
  *
- * Prints a gretl model in the form of a LaTeX equation, either as
+ * Prints to @prn a gretl model in the form of a LaTeX equation, either as
  * a stand-alone document or as a fragment of LaTeX source for
  * insertion into a document.
  * 
@@ -527,6 +530,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
     if (split) {
 	pputs(prn, "\\end{split}\n");
     }
+
     pputs(prn, " \\notag \\\\\n");
 
     if (pmod->ci == GARCH) {
@@ -535,6 +539,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	int r = pmod->list[0] - 4;
 
 	tstat = pmod->coeff[r] / pmod->sderr[r];
+
 	pprintf(prn, "\\hat{\\sigma}^2_t = \\underset{(%.3f)}{%g} ", 
 		tstat, pmod->coeff[r]);
 
@@ -546,6 +551,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	    pputs(prn, "}\\,");
 	    pprintf(prn, "\\varepsilon^2_{t-%d}", i);
 	}
+
 	for (i=1; i<=p; i++) {
 	    tstat = pmod->coeff[q+r+i] / pmod->sderr[q+r+i];
 	    pprintf(prn, "%s\\underset{(%.3f)}{", 
@@ -554,6 +560,7 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	    pputs(prn, "}\\,");
 	    pprintf(prn, "\\sigma^2_{t-%d}", i);
 	}
+
 	pputs(prn, "\\notag \\\\\n");
     }	    
 
@@ -571,17 +578,20 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 	    pprintf(prn, "T = %d \\quad \\mbox{ln}L = %.4f ",
 		    pmod->nobs, pmod->lnL);
 	}
+
 	if (pmod->ci != LOGIT && pmod->ci != PROBIT && !na(pmod->fstt)) {
 	    sprintf(tmp, "%.5g", pmod->fstt);
 	    tex_modify_exponent(tmp);
 	    pprintf(prn, "\\quad F(%d,%d) = %s ", 
 		    pmod->dfn, pmod->dfd, tmp);
 	}
+
 	if (!na(pmod->sigma)) {
 	    sprintf(tmp, "%.5g", pmod->sigma);
 	    tex_modify_exponent(tmp);
 	    pprintf(prn, "\\quad \\hat{\\sigma} = %s", tmp);
 	}
+
 	if (!na(gretl_model_get_double(pmod, "rho_in"))) {
 	    double r = gretl_model_get_double(pmod, "rho_in");
 
@@ -606,89 +616,85 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 /**
  * tex_print_model:
  * @pmod:  pointer to gretl MODEL struct.
- * @pdinfo:  information regarding the data set.
- * @standalone: print as full doc (1) or fragment (0).
+ * @pdinfo: information regarding the data set.
  * @prn: gretl printing struct.
  *
- * Prints a gretl model in the form of a LaTeX table, either as
- * a stand-alone document or as a fragment of LaTeX source for
- * insertion into a document.
+ * Prints to @prn a gretl model in the form of either a LaTeX 
+ * table or an equation, and either as a stand-alone document or 
+ * as a fragment of LaTeX source for insertion into a document.
+ *
+ * The options are read from the format field of @prn --
+ * gretl_print_set_format().
  * 
  * Returns: 0 on successful completion.
  */
 
-int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, 
-		     int standalone, PRN *prn)
+int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 {
-    if (standalone) {
-	prn->format = GRETL_PRINT_FORMAT_TEX_DOC;
+    int docopt = tex_doc_format(prn);
+    int ret;
+
+    if (tex_eqn_format(prn)) { 
+	ret = tex_print_equation(pmod, pdinfo, docopt, prn);
     } else {
-	prn->format = GRETL_PRINT_FORMAT_TEX;
+	ret = printmodel(pmod, pdinfo, OPT_NONE, prn);
     }
-    
-    return printmodel (pmod, pdinfo, OPT_NONE, prn);
+
+    return ret;
 }
 
 /**
- * tabprint:
- * @pmod: pointer to gretl MODEL struct.
+ * texprint:
+ * @pmod: pointer to model.
  * @pdinfo: information regarding the data set.
  * @texfile: name of file to save.
- * @oflag: option: complete doc or fragment
+ * @opt: if opt & %OPT_O, complete doc, else fragment;
+ * if opt & %OPT_E print as equation, otherwise use tabular
+ * format.
  *
- * Prints to file a gretl model in the form of a LaTeX table, either as
- * a stand-alone document or as a fragment of LaTeX source for
- * insertion into a document.
+ * Prints to file a gretl model in the form of a LaTeX table or
+ * equation, either as a stand-alone document or as a fragment 
+ * of LaTeX source for insertion into a document.
  * 
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int tabprint (MODEL *pmod, const DATAINFO *pdinfo,
-	      char *texfile, gretlopt oflag)
+int texprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt opt)
 {
-    PRN prn;
+    PRN *prn;
+    int eqn = (opt & OPT_E);
+    int doc = (opt & OPT_O);
+    int err = 0;
 
-    if (make_texfile(pmod->ID, 0, texfile, &prn)) {
-	return 1;
+    prn = make_tex_prn(pmod->ID, texfile, eqn, doc);
+    if (prn == NULL) {
+	err = 1;
+    } else {
+	err = tex_print_model(pmod, pdinfo, prn);
+	gretl_print_destroy(prn);
     }
 
-    tex_print_model(pmod, pdinfo, (oflag & OPT_O), &prn);
-
-    if (prn.fp != NULL) {
-	fclose(prn.fp);
-    }
-
-    return 0;
+    return err;
 }
 
 /**
- * eqnprint:
- * @pmod: pointer to gretl MODEL struct.
- * @pdinfo: information regarding the data set.
- * @texfile: name of file to save.
- * @oflag: if oflag & OPT_O, complete doc, else fragment
+ * tex_print_obs_marker:
+ * @t: observation number.
+ * @pdinfo: data information struct.
+ * @prn: gretl printing struct.
  *
- * Prints to file a gretl model in the form of a LaTeX equation, either as
- * a stand-alone document or as a fragment of LaTeX source for
- * insertion into a document.
- * 
- * Returns: 0 on successful completion, 1 on error.
+ * Print a string (label, date or obs number) representing the given @t.
  */
 
-int eqnprint (MODEL *pmod, const DATAINFO *pdinfo,
-	      char *texfile, gretlopt oflag)
+void tex_print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn)
 {
-    PRN prn;
+    if (pdinfo->markers) { 
+	pprintf(prn, "\\texttt{%s} ", pdinfo->S[t]); 
+    } else {
+	char tmp[OBSLEN]; 
 
-    if (make_texfile(pmod->ID, 1, texfile, &prn)) {
-	return 1;
+	ntodate(tmp, t, pdinfo);
+	pprintf(prn, "%8s ", tmp);
     }
-
-    tex_print_equation(pmod, pdinfo, (oflag & OPT_O), &prn);
-
-    if (prn.fp != NULL) {
-	fclose(prn.fp);
-    }
-
-    return 0;
 }
