@@ -24,17 +24,13 @@
 int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo, 
 			const int standalone, PRN *prn);
 
-int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, 
-		     const int standalone, PRN *prn);
+int tex_print_model (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn);
 
 int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-		      int c, PRN *prn);
+		     int i, PRN *prn);
 
-int tabprint (MODEL *pmod, const DATAINFO *pdinfo,
-	      char *texfile, gretlopt oflag);
-
-int eqnprint (MODEL *pmod, const DATAINFO *pdinfo,
-	      char *texfile, gretlopt oflag);
+int texprint (MODEL *pmod, const DATAINFO *pdinfo,
+	      char *texfile, gretlopt opt);
 
 char *tex_escape (char *targ, const char *src);
 
@@ -44,5 +40,5 @@ void set_gretl_tex_preamble (void);
 
 void gretl_tex_preamble (PRN *prn, int ams);
 
-
+void tex_print_obs_marker (int t, const DATAINFO *pdinfo, PRN *prn);
 
