@@ -18,15 +18,9 @@
  */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #undef TRDEBUG
 
-/* newlag is a library global, used when auto-generating a lag in the
-   context of reading a regression list (interact.c)
-*/
-int newlag;
-
 enum {
     VARS_IDENTICAL,
     X_HAS_MISSING,
@@ -87,7 +81,13 @@ make_transform_varname (char *vname, const char *orig, int cmd,
     } else if (cmd == LAGS) {
 	char ext[6];
 
-	sprintf(ext, "_%d", lag);
+	if (lag >= 0) {
+	    /* an actual lag */
+	    sprintf(ext, "_%d", lag);
+	} else {
+	    /* in fact a lead */
+	    sprintf(ext, "%d", -lag);
+	}
 	strncat(vname, orig, len - strlen(ext));
 	strcat(vname, ext);
     }
@@ -110,7 +110,11 @@ make_transform_label (char *label, const char *parent,
     } else if (cmd == SQUARE) {
 	sprintf(label, _("= %s squared"), parent);
     } else if (cmd == LAGS) {
-	sprintf(label, "= %s(t - %d)", parent, lag);
+	if (lag >= 0) {
+	    sprintf(label, "= %s(t - %d)", parent, lag);
+	} else {
+	    sprintf(label, "= %s(t + %d)", parent, -lag);
+	}
     } else {
 	err = 1;
     }
@@ -131,12 +135,10 @@ static void make_xp_varname (char *vname, const char *v1,
 }
 
 /* Array into which to write a generated variable, prior to testing
-   whether or not the same var already exists.  Declared as non-static
-   so that it can be called in context of library cleanup (with n =
-   0).
+   whether or not the same var already exists.  
 */
 
-double *testvec (int n)
+static double *testvec (int n)
 {
     static double *x;
     static int nx;
@@ -171,6 +173,11 @@ double *testvec (int n)
     return x;
 }
 
+void gretl_transforms_cleanup (void)
+{
+    testvec(0);
+}
+
 /* write lagged values of variable v into lagvec */
 
 static int get_lag (int v, int lag, double *lagvec, 
@@ -362,7 +369,7 @@ check_add_transform (int vnum, const double *x,
 	}
     } else {
 	/* no var of this name, working from scratch */
-	if (dataset_add_vars(1, pZ, pdinfo)) {
+	if (dataset_add_series(1, pZ, pdinfo)) {
 	    ret = VAR_ADD_FAILED;
 	} else {
 	    strcpy(pdinfo->varname[vnum], vname);
@@ -446,52 +453,63 @@ static int get_transform (int ci, int v, int aux,
     return vno;
 }
 
-/* laggenr: create specified lag of variable v if this variable does
-   not already exist.
-
-   Return the ID number of the lag var, or -1 on error.
-*/
+/**
+ * laggenr: 
+ * @v: ID number in dataset of source variable.
+ * @lag: the order of the lag to create.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ *
+ * Creates the specified lag of variable @v if this variable does
+ * not already exist.
+ *
+ * Returns: the ID number of the lagged variable, or -1 on error.
+ */
 
 int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 {
-    int lno, oldv = pdinfo->v;
+    int lno;
 
-    if (!pdinfo->vector[v]) {
-	return -1;
-    }
-
-    /* sanity check on lag length */
-    if (lag > pdinfo->n) {
-	return -1;
-    }
-
-    newlag = 1;
-
-    lno = get_transform(LAGS, v, lag, pZ, pdinfo, 8);
-
-    if (lno < oldv) {
-	newlag = 0;
+    if (!pdinfo->vector[v] || lag > pdinfo->n) {
+	lno = -1;
+    } else if (lag == 0) {
+	lno = v;
+    } else {
+	lno = get_transform(LAGS, v, lag, pZ, pdinfo, 8);
     }
 
     return lno;
 }
 
-/* loggenr: create log of variable v if this variable does not
-   already exist.  
-
-   Return the ID number of the log var, or -1 on error.
-*/
+/**
+ * loggenr: 
+ * @v: ID number in dataset of source variable.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ *
+ * Creates the natural log of variable @v if this variable does
+ * not already exist.
+ *
+ * Returns: the ID number of the log variable, or -1 on error.
+ */
 
 int loggenr (int v, double ***pZ, DATAINFO *pdinfo)
 {
     return get_transform(LOGS, v, 0, pZ, pdinfo, 8);
 }
 
-/* diffgenr: create first difference (or log difference) of variable v,
-   if this variable does not already exist.
-
-   Return the ID number of the differenced var, or -1 on error.
-*/
+/**
+ * diffgenr: 
+ * @v: ID number in dataset of source variable.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ * @ldiff: if non-zero, generate the log-difference.
+ *
+ * Creates the first difference (or log difference) of variable @v 
+ * if this variable does not already exist.
+ *
+ * Returns: the ID number of the differenced variable, or -1 on error.
+ */
 
 int diffgenr (int v, double ***pZ, DATAINFO *pdinfo, int ldiff)
 {
@@ -502,16 +520,24 @@ int diffgenr (int v, double ***pZ, DATAINFO *pdinfo, int ldiff)
     return get_transform((ldiff)? LDIFF : DIFF, v, 0, pZ, pdinfo, 8);
 }
 
-/* xpxgenr: create square or cross-product, if the target variable
-   does not already exist.
-
-   Return the ID number of the squared or cross var, or -1 on error.
-*/
+/**
+ * xpxgenr: 
+ * @vi: ID number in dataset of first source variable.
+ * @vj: ID number in dataset of second source variable.
+ * @pZ: pointer to data array.
+ * @pdinfo: information on dataset.
+ *
+ * Creates the cross product of variables @vi and @vj if this 
+ * variable does not already exist.
+ *
+ * Returns: the ID number of the cross-product variable, 
+ * or -1 on error.
+ */
 
 int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo)
 {
     if (vi == vj) {
-	if (isdummy((*pZ)[vi], pdinfo->t1, pdinfo->t2)) {
+	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[vi])) {
 	    return -1;
 	}
     }
@@ -572,8 +598,9 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
 
 /**
  * list_loggenr:
- * @list: list of variables to process.
- * @pZ: pointer to data matrix.
+ * @list: on entry, list of variables to process; on exit,
+ * holds the ID numbers of the generated variables.
+ * @pZ: pointer to data array.
  * @pdinfo: data information struct.
  *
  * Generates and adds to the data set the natural logs of the
@@ -582,9 +609,9 @@ get_starting_length (const int *list, DATAINFO *pdinfo, int trim)
  * Returns: 0 on success, error code on error.
  */
 
-int list_loggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
+int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo)
 {
-    int lognum, i, v;
+    int tnum, i, v;
     int startlen;
     int n_ok = 0;
 
@@ -596,19 +623,23 @@ int list_loggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
 	if (v == 0 || !pdinfo->vector[v]) {
 	    continue; 
 	}
-	if (isdummy((*pZ)[v], pdinfo->t1, pdinfo->t2)) {
+	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[v])) {
 	    continue;
 	}
 
-	lognum = get_transform(LOGS, v, 0, pZ, pdinfo, startlen);
-	if (lognum > 0) {
+	tnum = get_transform(LOGS, v, 0, pZ, pdinfo, startlen);
+
+	if (tnum > 0) {
 	    n_ok++;
+	    list[i] = tnum;
 	}
     }
 
     return (n_ok > 0)? 0 : E_LOGS;
 }
 
+/* used in var.c */
+
 int 
 real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
 		   int maxlag, int **lagnums)
@@ -649,39 +680,52 @@ real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
     return 0;
 }
 
+/**
+ * default_lag_order:
+ * @pdinfo: data information struct.
+ *
+ * Returns: the default lag order for generating lags, performing
+ * autocorrelation test, and so on.
+ *
+ */
+
+int default_lag_order (const DATAINFO *pdinfo)
+{
+    int order = 1;
+
+    if (!dataset_is_panel(pdinfo)) {
+	order = (pdinfo->pd < 52)? pdinfo->pd : 14;
+    }
+
+    return order;
+}
+
 /**
  * list_laggenr:
+ * @order: number of lags to generate (or 0 for automatic)
  * @list: list of variables to process.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  *
- * Generates and adds to the data set lagged values of the 
- * variables given in @list (up to the frequency of the data).
+ * Generates and adds to the data set @order lagged values of the 
+ * variables given in @list.
  *
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
+int list_laggenr (int order, const int *list, double ***pZ, DATAINFO *pdinfo)
 {
-    int maxlag;
-
-    if (pdinfo->pd < 52) {
-	maxlag = pdinfo->pd;
-    } else {
-	maxlag = 14;
+    if (order == 0) {
+	order = default_lag_order(pdinfo);
     } 
 
-    /* play safe with panel data */
-    if (dataset_is_panel(pdinfo)) {
-	maxlag = 1;
-    }
-
-    return real_list_laggenr(list, pZ, pdinfo, maxlag, NULL);  
+    return real_list_laggenr(list, pZ, pdinfo, order, NULL);  
 }
 
 /**
  * list_diffgenr:
- * @list: list of variables to process.
+ * @list: on entry, list of variables to process; on exit,
+ * ID numbers of the generated variables.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  *
@@ -692,25 +736,30 @@ int list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo)
  *
  */
 
-int list_diffgenr (const int *list, double ***pZ, DATAINFO *pdinfo)
+int list_diffgenr (int *list, double ***pZ, DATAINFO *pdinfo)
 {
     int i, v, startlen;
+    int tnum, err = 0;
 
     startlen = get_starting_length(list, pdinfo, 2);
     
-    for (i=1; i<=list[0]; i++) {
+    for (i=1; i<=list[0] && !err; i++) {
 	v = list[i];
-	if (get_transform(DIFF, v, 0, pZ, pdinfo, startlen) < 0) {
-	    return 1;
+	tnum = get_transform(DIFF, v, 0, pZ, pdinfo, startlen);
+	if (tnum < 0) {
+	    err = 1;
+	} else {
+	    list[i] = tnum;
 	}
     }
 
-    return 0;
+    return err;
 }
 
 /**
  * list_ldiffgenr:
- * @list: list of variables to process.
+ * @list: on entry, list of variables to process; on exit,
+ * ID numbers of the generated variables.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  *
@@ -721,102 +770,100 @@ int list_diffgenr (const int *list, double ***pZ, DATAINFO *pdinfo)
  *
  */
 
-int list_ldiffgenr (const int *list, double ***pZ, DATAINFO *pdinfo)
+int list_ldiffgenr (int *list, double ***pZ, DATAINFO *pdinfo)
 {
     int i, v, startlen;
+    int tnum, err = 0;
 
     startlen = get_starting_length(list, pdinfo, 3);
     
-    for (i=1; i<=list[0]; i++) {
+    for (i=1; i<=list[0] && !err; i++) {
 	v = list[i];
-	if (get_transform(LDIFF, v, 0, pZ, pdinfo, startlen) < 0) {
-	    return 1;
+	tnum = get_transform(LDIFF, v, 0, pZ, pdinfo, startlen);
+	if (tnum < 0) {
+	    err = 1;
+	} else {
+	    list[i] = tnum;
 	}
     }
 
-    return 0;
+    return err;
 }
 
 /**
  * list_xpxgenr:
- * @list: list of variables to process.
+ * @plist: pointer to list of variables to process.  On exit
+ * the list holds the ID numbers of the squares (and possibly 
+ * cross-products).
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
- * @opt: If = 0, only squares are generated, if OPT_O, both
- * squares and cross-products are generated.
+ * @opt: If OPT_O, both squares and cross-products are generated,
+ * otherwise only squares.
  *
- * Generates and adds to the data set squares and (if @opt is non-zero) 
- * cross-products of the variables given in @list.
+ * Generates and adds to the data set squares and (if @opt is %OPT_O) 
+ * cross-products of the variables given in the list pointed to
+ * by @plist.
  *
  * Returns: 0 on success, error code on error.
  */
 
-int list_xpxgenr (const int *list, double ***pZ, DATAINFO *pdinfo, 
+int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt)
 {
-    int xnum, i, j, vi, vj;
+    int *list = *plist;
+    int l0 = list[0];
+    int *xpxlist = NULL;
+    int tnum, i, j, k, vi, vj;
     int startlen;
-    int n_ok = 0;
+
+    if (opt & OPT_O) {
+	int maxterms = (l0 * l0 + l0) / 2;
+    
+	xpxlist = gretl_list_new(maxterms);
+	if (xpxlist == NULL) {
+	    return E_ALLOC;
+	}
+    } else {
+	xpxlist = list;
+    }
 
     startlen = get_starting_length(list, pdinfo, 3);
+    xpxlist[0] = 0;
 
-    for (i=1; i<=list[0]; i++) {
+    k = 1;
+    for (i=1; i<=l0; i++) {
 	vi = list[i];
 
 	if (vi == 0 || !pdinfo->vector[vi]) {
 	    continue; 
 	}
-	if (isdummy((*pZ)[vi], pdinfo->t1, pdinfo->t2)) {
+	if (gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[vi])) {
 	    continue;
 	}
 
-	xnum = get_transform(SQUARE, vi, vi, pZ, pdinfo, startlen);
-	if (xnum > 0) {
-	    n_ok++;
+	tnum = get_transform(SQUARE, vi, vi, pZ, pdinfo, startlen);
+	if (tnum > 0) {
+	    xpxlist[k++] = tnum;
+	    xpxlist[0] += 1;
 	}
 
 	if (opt & OPT_O) {
-	    for (j=i+1; j<=list[0]; j++) {
+	    for (j=i+1; j<=l0; j++) {
 		vj = list[j];
-		xnum = xpxgenr(vi, vj, pZ, pdinfo);
-		if (xnum > 0) {
-		    n_ok++;
+		tnum = xpxgenr(vi, vj, pZ, pdinfo);
+		if (tnum > 0) {
+		    xpxlist[k++] = tnum;
+		    xpxlist[0] += 1;
 		}
 	    }
 	}
     }
 
-    return (n_ok > 0)? 0 : E_SQUARES;
-}
-
-int lagvarnum (int v, int l, const DATAINFO *pdinfo)
-{
-    char label[MAXLABEL];
-    int i;
-
-    make_transform_label(label, pdinfo->varname[v], LAGS, l);
-
-#if TRDEBUG
-    fprintf(stderr, "Looking for lag var with label '%s'...\n", label);
-#endif
-
-    for (i=1; i<pdinfo->v; i++) {
-	if (!strcmp(label, VARLABEL(pdinfo, i))) {
-	    return i;
-	}
+    if (opt & OPT_O) {
+	free(*plist);
+	*plist = xpxlist;
     }
 
-    /* second attempt? */
-    sprintf(label, "= %s(-%d)", pdinfo->varname[v], l);
-    for (i=1; i<pdinfo->v; i++) {
-	if (strstr(VARLABEL(pdinfo, i), label)) {
-	    return i;
-	}
-    }
-
-#if TRDEBUG
-    fputs("*** not found\n", stderr);
-#endif
-
-    return -1;
+    return (xpxlist[0] > 0)? 0 : E_SQUARES;
 }
+
