@@ -20,6 +20,8 @@
 #ifndef TRANSFORMS_H
 #define TRANSFORMS_H
 
+int default_lag_order (const DATAINFO *pdinfo);
+
 int diffgenr (int v, double ***pZ, DATAINFO *pdinfo, int ldiff);
 
 int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo);
@@ -28,15 +30,17 @@ int loggenr (int v, double ***pZ, DATAINFO *pdinfo);
 
 int xpxgenr (int vi, int vj, double ***pZ, DATAINFO *pdinfo);
 
-int list_diffgenr (const int *list, double ***pZ, DATAINFO *pdinfo);
+int list_diffgenr (int *list, double ***pZ, DATAINFO *pdinfo);
 
-int list_ldiffgenr (const int *list, double ***pZ, DATAINFO *pdinfo);
+int list_ldiffgenr (int *list, double ***pZ, DATAINFO *pdinfo);
 
-int list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo);
+int list_laggenr (int order, const int *list, double ***pZ, DATAINFO *pdinfo);
 
-int list_loggenr (const int *list, double ***pZ, DATAINFO *pdinfo);
+int list_loggenr (int *list, double ***pZ, DATAINFO *pdinfo);
 
-int list_xpxgenr (const int *list, double ***pZ, DATAINFO *pdinfo, 
+int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo, 
 		  gretlopt opt);
 
+void gretl_transforms_cleanup (void);
+
 #endif /* TRANSFORMS_H */
