@@ -21,7 +21,15 @@
 
 #include "libgretl.h" 
 #include "var.h"  
-#include "gretl_private.h"
+#include "libset.h"
+
+/* in transforms.c */
+extern int 
+real_list_laggenr (const int *list, double ***pZ, DATAINFO *pdinfo,
+		   int maxlag, int **lagnums);
+
+static gretl_matrix *
+gretl_var_get_fcast_decomp (GRETL_VAR *var, int targ, int periods);
 
 #define VAR_DEBUG 0
 
@@ -33,6 +41,7 @@ struct GRETL_VAR_ {
     gretl_matrix *A;   /* augmented coefficient matrix */
     gretl_matrix *E;   /* residuals matrix */
     gretl_matrix *C;   /* augmented Cholesky-decomposed error matrix */
+    gretl_matrix *F;   /* optional forecast matrix */
     MODEL **models;    /* pointers to individual equation estimates */
     double *Fvals;     /* hold results of F-tests */
     char *name;        /* for use in session management */
@@ -66,8 +75,6 @@ static int real_adf_test (int varno, int order, int niv,
 			  double ***pZ, DATAINFO *pdinfo, 
 			  gretlopt opt, int cointcode, PRN *prn);
 
-/* ...................................................................  */
-
 static void pad_var_coeff_matrix (GRETL_VAR *var)
 {
     int i, j;
@@ -92,6 +99,7 @@ gretl_var_init (GRETL_VAR *var, int neqns, int order, const DATAINFO *pdinfo,
     var->A = NULL;
     var->E = NULL;
     var->C = NULL;
+    var->F = NULL;
     var->models = NULL;
     var->Fvals = NULL;
     var->name = NULL;
@@ -161,6 +169,7 @@ void gretl_var_free (GRETL_VAR *var)
     gretl_matrix_free(var->A);
     gretl_matrix_free(var->E);
     gretl_matrix_free(var->C);
+    gretl_matrix_free(var->F);
 
     free(var->Fvals);
     free(var->name);
@@ -201,7 +210,151 @@ GRETL_VAR *gretl_var_new (int neqns, int order, const DATAINFO *pdinfo,
     return var;
 }
 
-/* ......................................................  */
+static int
+gretl_var_add_forecast (GRETL_VAR *var, int t1, int t2, const double **Z, 
+			const DATAINFO *pdinfo, gretlopt opt)
+{
+    const MODEL *pmod;
+    gretl_matrix *F;
+    double fti, xti;
+    int i, j, k, s, t;
+    int nf, ns, lag, vj, m;
+    int staticfc, fcols;
+
+    pmod = var->models[0];
+
+    nf = t2 - t1 + 1;
+
+    staticfc = (opt & OPT_S);
+    if (staticfc) {
+	fcols = var->neqns;
+    } else {
+	fcols = 2 * var->neqns;
+    }
+
+    /* rows = number of forecast periods; cols = 1 to hold forecast
+       for each variable, plus 1 to hold variance for each variable
+       if forecast is dynamic.
+    */
+    F = gretl_matrix_alloc(nf, fcols);
+    if (F == NULL) {
+	return E_ALLOC;
+    }
+
+    gretl_matrix_zero(F);
+
+    ns = var->order * var->neqns;
+
+    for (t=t1; t<=t2; t++) {
+	int miss = 0;
+
+	s = t - t1;
+	for (i=0; i<var->neqns; i++) {
+	    pmod = var->models[i];
+	    fti = 0.0;
+	    lag = 1;
+	    k = 0;
+	    for (j=0; j<pmod->ncoeff; j++) {
+		vj = pmod->list[j + 2];
+		if (j < ns + pmod->ifc && vj > 0) {
+		    /* stochastic var */
+		    if (staticfc || s - lag < 0) {
+			/* pre-forecast value */
+			m = (j - pmod->ifc) / var->order;
+			vj = var->models[m]->list[1];
+			if (t - lag < 0) {
+			    xti = NADBL;
+			} else {
+			    xti = Z[vj][t-lag];
+			}
+			if (na(xti)) {
+			    miss = 1;
+			}
+		    } else {
+			/* prior forecast value */
+			xti = gretl_matrix_get(F, s - lag, k);
+		    }
+		    lag++;
+		    if (lag > var->order) {
+			lag = 1;
+			k++;
+		    }
+		} else {
+		    /* deterministic var: value from dataset */
+		    xti = Z[vj][t];
+		    if (na(xti)) {
+			miss = 1;
+		    }
+		}
+		if (miss) {
+		    fti = NADBL;
+		} else {
+		    fti += pmod->coeff[j] * xti;
+		}
+	    }
+	    gretl_matrix_set(F, s, i, fti);
+	}
+    }
+
+    /* now get variances, if not static */
+    if (!staticfc) {
+	double vti;
+	int totcol;
+
+	for (i=0; i<var->neqns; i++) {
+	    gretl_matrix *vd;
+	    vd = gretl_var_get_fcast_decomp(var, i, nf);
+	    if (vd != NULL) {
+		totcol = gretl_matrix_cols(vd) - 1;
+		for (s=0; s<nf; s++) {
+		    vti = gretl_matrix_get(vd, s, totcol);
+		    gretl_matrix_set(F, s, var->neqns + i, vti);
+		}
+		gretl_matrix_free(vd);
+	    } else {
+		for (s=0; s<nf; s++) {
+		    gretl_matrix_set(F, s, var->neqns + i, NADBL);
+		}
+	    }
+	}
+    }
+
+#if 0
+    gretl_matrix_print(F, "var->F", NULL);
+#endif
+
+    gretl_matrix_set_int(F, t1);
+
+    var->F = F;
+
+    return 0;
+}
+
+const gretl_matrix *
+gretl_var_get_forecast_matrix (GRETL_VAR *var, int t1, int t2, const double **Z, 
+			       const DATAINFO *pdinfo, gretlopt opt)
+{
+    if (var->F != NULL) {
+	int ncols, nf = t2 - t1 + 1;
+	int ft1 = gretl_matrix_get_int(var->F);
+
+	ncols = (opt & OPT_S)? var->neqns: 2 * var->neqns;
+
+	if (nf == gretl_matrix_rows(var->F) && t1 == ft1 && 
+	    ncols == gretl_matrix_cols(var->F)) {
+	    ; /* already done, fine */
+	} else {
+	    gretl_matrix_free(var->F);
+	    var->F = NULL;
+	}
+    }
+
+    if (var->F == NULL) {
+	gretl_var_add_forecast(var, t1, t2, Z, pdinfo, opt);
+    }
+
+    return var->F;
+}
 
 static int gretl_var_do_error_decomp (GRETL_VAR *var)
 {
@@ -268,8 +421,6 @@ static int gretl_var_do_error_decomp (GRETL_VAR *var)
     return err;
 }
 
-/* ......................................................  */
-
 int gretl_var_get_variable_number (const GRETL_VAR *var, int k)
 {
     return (var->models[k])->list[1];
@@ -280,15 +431,27 @@ int gretl_var_get_n_equations (const GRETL_VAR *var)
     return var->neqns;
 }
 
-/* ......................................................  */
+int gretl_var_get_t1 (const GRETL_VAR *var)
+{
+    return var->models[0]->t1;
+}
+
+int gretl_var_get_t2 (const GRETL_VAR *var)
+{
+    return var->models[0]->t2;
+}
+
+const MODEL *gretl_var_get_model (const GRETL_VAR *var, int i)
+{
+    if (i < var->neqns) {
+	return var->models[i];
+    } else {
+	return NULL;
+    }
+}
 
 #define VARS_IN_ROW 4
 
-#define PLAIN_PRN(p) (p->format == GRETL_PRINT_FORMAT_PLAIN)
-#define RTF_PRN(p)   (p->format == GRETL_PRINT_FORMAT_RTF)
-#define TEX_PRN(p)   (p->format == GRETL_PRINT_FORMAT_TEX || \
-                      p->format == GRETL_PRINT_FORMAT_TEX_DOC)
-
 static void tex_print_double (double x, PRN *prn)
 {
     char number[16];
@@ -297,8 +460,11 @@ static void tex_print_double (double x, PRN *prn)
 
     sprintf(number, "%#.*g", GRETL_DIGITS, x);
 
-    if (x < 0.) pprintf(prn, "$-$%s", number + 1);
-    else pputs(prn, number);
+    if (x < 0.) {
+	pprintf(prn, "$-$%s", number + 1);
+    } else {
+	pputs(prn, number);
+    }
 }
 
 static int periods_from_pd (int pd)
@@ -319,7 +485,18 @@ static int periods_from_pd (int pd)
     return periods;
 }
 
-int 
+int default_VAR_horizon (const DATAINFO *pdinfo)
+{
+    int h = get_VAR_horizon();
+
+    if (h <= 0) {
+	h = periods_from_pd(pdinfo->pd);
+    }
+
+    return h;
+}
+
+static int 
 gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 				  int periods, const DATAINFO *pdinfo, 
 				  int pause, PRN *prn)
@@ -331,19 +508,19 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
     int block, blockmax;
     int err = 0;
 
-    if (prn == NULL) return 0;
+    if (prn == NULL) {
+	return 0;
+    }
 
     if (shock >= var->neqns) {
 	fprintf(stderr, "Shock variable out of bounds\n");
 	return 1;
     }  
 
-    if (periods == 0) {
-	periods = periods_from_pd(pdinfo->pd);
-    }
-
     rtmp = gretl_matrix_alloc(rows, var->neqns);
-    if (rtmp == NULL) return E_ALLOC;
+    if (rtmp == NULL) {
+	return E_ALLOC;
+    }
 
     ctmp = gretl_matrix_alloc(rows, var->neqns);
     if (ctmp == NULL) {
@@ -354,14 +531,16 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
     vsrc = (var->models[shock])->list[1];
 
     blockmax = var->neqns / VARS_IN_ROW;
-    if (var->neqns % VARS_IN_ROW) blockmax++;
+    if (var->neqns % VARS_IN_ROW) {
+	blockmax++;
+    }
 
     for (block=0; block<blockmax && !err; block++) {
 	int vtarg, k;
 	char vname[16];
 	double r;
 
-	if (TEX_PRN(prn)) {
+	if (tex_format(prn)) {
 	    pputs(prn, "\\vspace{1em}\n\n");
 	    pprintf(prn, I_("Responses to a one-standard error shock in %s"), 
 		    tex_escape(vname, pdinfo->varname[vsrc]));
@@ -372,7 +551,7 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 		pprintf(prn, " (%s)\n\n", I_("continued"));
 	    }
 	    pputs(prn, "\\vspace{1em}\n\n"
-		  "\\begin{tabular}{rcccc}\n");
+		  "\\begin{longtable}{rcccc}\n");
 	} else {
 	    pprintf(prn, _("Responses to a one-standard error shock in %s"), 
 		    pdinfo->varname[vsrc]);
@@ -384,7 +563,7 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 	    }
 	}
 
-	if (TEX_PRN(prn)) {
+	if (tex_format(prn)) {
 	    pprintf(prn, "%s & ", I_("period"));
 	} else {
 	    pprintf(prn, "%s ", _("period"));
@@ -392,12 +571,17 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 
 	for (i=0; i<VARS_IN_ROW; i++) {
 	    k = VARS_IN_ROW * block + i;
-	    if (k >= var->neqns) break;
+	    if (k >= var->neqns) {
+		break;
+	    }
 	    vtarg = (var->models[k])->list[1];
-	    if (TEX_PRN(prn)) {
+	    if (tex_format(prn)) {
 		pprintf(prn, " %s ", tex_escape(vname, pdinfo->varname[vtarg]));
-		if (i < VARS_IN_ROW - 1 && k < var->neqns - 1) pputs(prn, "& ");
-		else pputs(prn, "\\\\");
+		if (i < VARS_IN_ROW - 1 && k < var->neqns - 1) {
+		    pputs(prn, "& ");
+		} else {
+		    pputs(prn, "\\\\");
+		}
 	    } else {
 		pprintf(prn, "  %8s  ", pdinfo->varname[vtarg]);
 	    }
@@ -407,7 +591,9 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 
 	for (t=0; t<periods && !err; t++) {
 	    pprintf(prn, " %3d  ", t + 1);
-	    if (TEX_PRN(prn)) pputs(prn, "& ");
+	    if (tex_format(prn)) {
+		pputs(prn, "& ");
+	    }
 	    if (t == 0) {
 		/* calculate initial estimated responses */
 		err = gretl_matrix_copy_values(rtmp, var->C);
@@ -421,9 +607,11 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 
 	    for (i=0; i<VARS_IN_ROW; i++) {
 		k = VARS_IN_ROW * block + i;
-		if (k >= var->neqns) break;
+		if (k >= var->neqns) {
+		    break;
+		}
 		r = gretl_matrix_get(rtmp, k, shock);
-		if (TEX_PRN(prn)) {
+		if (tex_format(prn)) {
 		    tex_print_double(r, prn);
 		    if (i < VARS_IN_ROW - 1 && k < var->neqns - 1) {
 			pputs(prn, " & ");
@@ -432,21 +620,21 @@ gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
 		    pprintf(prn, "%#12.5g ", r);
 		}
 	    }
-	    if (TEX_PRN(prn)) {
+	    if (tex_format(prn)) {
 		pputs(prn, "\\\\\n");
 	    } else {
 		pputc(prn, '\n');
 	    }
 	}
 
-	if (TEX_PRN(prn)) {
-	    pputs(prn, "\\end{tabular}\n\n");
+	if (tex_format(prn)) {
+	    pputs(prn, "\\end{longtable}\n\n");
 	} else {
 	    pputc(prn, '\n');
 	}
 
 	if (pause && block < blockmax - 1) {
-	    takenotes(0);
+	    scroll_pause();
 	}
     }
 
@@ -618,7 +806,7 @@ gretl_var_get_fcast_decomp (GRETL_VAR *var, int targ, int periods)
 
 #define VDROWMAX 5
 
-int 
+static int 
 gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 			      int periods, const DATAINFO *pdinfo, 
 			      int pause, PRN *prn)
@@ -629,24 +817,26 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
     int block, blockmax;
     int err = 0;
 
-    if (prn == NULL) return 0;
+    if (prn == NULL) {
+	return 0;
+    }
 
     if (targ >= var->neqns) {
 	fprintf(stderr, "Target variable out of bounds\n");
 	return 1;
     } 
 
-    if (periods == 0) {
-	periods = periods_from_pd(pdinfo->pd);
-    } 
-
     vd = gretl_var_get_fcast_decomp(var, targ, periods);
-    if (vd == NULL) return E_ALLOC;
+    if (vd == NULL) {
+	return E_ALLOC;
+    }
 
     vtarg = (var->models[targ])->list[1];
 
     blockmax = (var->neqns + 1) / VDROWMAX;
-    if ((var->neqns + 1) % VDROWMAX) blockmax++;
+    if ((var->neqns + 1) % VDROWMAX) {
+	blockmax++;
+    }
 
     for (block=0; block<blockmax; block++) {
 	int k, vsrc;
@@ -654,7 +844,7 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 	double r;
 
 	/* print block header */
-	if (TEX_PRN(prn)) {
+	if (tex_format(prn)) {
 	    pputs(prn, "\\vspace{1em}\n\n");
 	    pprintf(prn, I_("Decomposition of variance for %s"), 
 		    tex_escape(vname, pdinfo->varname[vtarg]));
@@ -665,7 +855,7 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 		pprintf(prn, " (%s)\n\n", I_("continued"));
 	    }
 	    pputs(prn, "\\vspace{1em}\n\n"
-		  "\\begin{tabular}{rccccc}\n");
+		  "\\begin{longtable}{rccccc}\n");
 	} else {
 	    pprintf(prn, _("Decomposition of variance for %s"), 
 		    pdinfo->varname[vtarg]);
@@ -678,7 +868,7 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 	}
 
 	/* first column: print period/step label */
-	if (TEX_PRN(prn)) {
+	if (tex_format(prn)) {
 	    pprintf(prn, "%s & ", I_("period"));
 	} else {
 	    pprintf(prn, "%s ", _("period"));
@@ -688,16 +878,18 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 	for (i=0; i<VDROWMAX; i++) {
 	    k = VDROWMAX * block + i - 1;
 	    if (k < 0) {
-		if (TEX_PRN(prn)) {
+		if (tex_format(prn)) {
 		    pprintf(prn, " %s & ", I_("std. error"));
 		} else {
 		    pprintf(prn, " %12s ", _("std. error"));
 		}
 		continue;
 	    }
-	    if (k >= var->neqns) break;
+	    if (k >= var->neqns) {
+		break;
+	    }
 	    vsrc = (var->models[k])->list[1];
-	    if (TEX_PRN(prn)) {
+	    if (tex_format(prn)) {
 		pprintf(prn, " %s ", tex_escape(vname, pdinfo->varname[vsrc]));
 		if (i < VDROWMAX - 1 && k < var->neqns - 1) pputs(prn, "& ");
 		else pputs(prn, "\\\\");
@@ -711,22 +903,24 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 	/* print block of numbers */
 	for (t=0; t<periods && !err; t++) {
 	    pprintf(prn, " %3d  ", t + 1);
-	    if (TEX_PRN(prn)) pputs(prn, "& ");
+	    if (tex_format(prn)) pputs(prn, "& ");
 
 	    for (i=0; i<VDROWMAX; i++) {
 		k = VDROWMAX * block + i - 1;
 		if (k < 0) {
 		    r = gretl_matrix_get(vd, t, var->neqns);
-		    if (TEX_PRN(prn)) {
+		    if (tex_format(prn)) {
 			pprintf(prn, "%g & ", r);
 		    } else {
 			pprintf(prn, " %14g ", r);
 		    }
 		    continue;
 		}
-		if (k >= var->neqns) break;
+		if (k >= var->neqns) {
+		    break;
+		}
 		r = gretl_matrix_get(vd, t, k);
-		if (TEX_PRN(prn)) {
+		if (tex_format(prn)) {
 		    pprintf(prn, "$%.4f$", r);
 		    if (i < VDROWMAX - 1 && k < var->neqns - 1) {
 			pputs(prn, " & ");
@@ -735,28 +929,31 @@ gretl_var_print_fcast_decomp (GRETL_VAR *var, int targ,
 		    pprintf(prn, "%10.4f ", r);
 		}
 	    }
-	    if (TEX_PRN(prn)) pputs(prn, "\\\\\n");
-	    else pputc(prn, '\n');
+	    if (tex_format(prn)) {
+		pputs(prn, "\\\\\n");
+	    } else {
+		pputc(prn, '\n');
+	    }
 	}
 
-	if (TEX_PRN(prn)) {
-	    pputs(prn, "\\end{tabular}\n\n");
+	if (tex_format(prn)) {
+	    pputs(prn, "\\end{longtable}\n\n");
 	} else {
 	    pputc(prn, '\n');
 	}
 
 	if (pause && block < blockmax - 1) {
-	    takenotes(0);
+	    scroll_pause();
 	}
     }
 
-    if (vd != NULL) gretl_matrix_free(vd);
+    if (vd != NULL) {
+	gretl_matrix_free(vd);
+    }
 
     return err;
 }
 
-/* ......................................................  */
-
 static int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
 {
     int index;
@@ -773,7 +970,7 @@ static int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
 	return index;
     }
     
-    if (dataset_add_vars(1, pZ, pdinfo)) {
+    if (dataset_add_series(1, pZ, pdinfo)) {
 	return TREND_FAILED;
     }
 
@@ -944,7 +1141,7 @@ static int organize_var_lists (const int *list, const double **Z,
 	if (gotsep || 
 	    !strcmp(pdinfo->varname[li], "const") ||	   
 	    !strcmp(pdinfo->varname[li], "time") ||
-	    isdummy(Z[li], pdinfo->t1, pdinfo->t2)) {
+	    gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[li])) {
 	    d[i] = 1;
 	    ndet++;
 	} else {
@@ -1039,41 +1236,47 @@ compose_varlist (struct var_lists *vl, int depvar, int order, int omit,
     return err;
 }
 
-/* ...................................................................  */
-
 static int add_model_data_to_var (GRETL_VAR *var, const MODEL *pmod, int k)
 {
     int i, j;
     int v = 0, lag = 0;
     int start = pmod->ifc;
     int rowmax = var->neqns * var->order + start;
+    int err = 0;
 
     if (k == 0) {
 	/* first equation: set up storage for residuals */
 	var->n = pmod->t2 - pmod->t1 + 1;
 	var->E = gretl_matrix_alloc(var->n, var->neqns);
-	if (var->E == NULL) return 1;
-	var->ifc = pmod->ifc;
+	if (var->E == NULL) {
+	    err = 1;
+	} else {
+	    var->ifc = pmod->ifc;
+	}
     }
 
     /* save residuals */
-    for (i=0; i<var->n; i++) {
-	gretl_matrix_set(var->E, i, k, pmod->uhat[pmod->t1 + i]);
+    if (!err) {
+	for (i=0; i<var->n; i++) {
+	    gretl_matrix_set(var->E, i, k, pmod->uhat[pmod->t1 + i]);
+	}
     }	
 
     /* save coefficients */
-    for (i=start; i<rowmax; i++) {
-	if ((i - start) % var->order == 0) {
-	    v++;
-	    lag = 1;
-	} else {
-	    lag++;
+    if (!err) {
+	for (i=start; i<rowmax; i++) {
+	    if ((i - start) % var->order == 0) {
+		v++;
+		lag = 1;
+	    } else {
+		lag++;
+	    }
+	    j = (lag - 1) * var->neqns + v - 1;
+	    gretl_matrix_set(var->A, k, j, pmod->coeff[i]);
 	}
-	j = (lag - 1) * var->neqns + v - 1;
-	gretl_matrix_set(var->A, k, j, pmod->coeff[i]);
     }
 
-    return 0;
+    return err;
 }
 
 static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
@@ -1091,7 +1294,9 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 
     if (robust) {
 	outlist = malloc(varmod->list[0] * sizeof *outlist);
-	if (outlist == NULL) return E_ALLOC;
+	if (outlist == NULL) {
+	    return E_ALLOC;
+	}
     }
 
     pputs(prn, _("\nF-tests of zero restrictions:\n\n"));
@@ -1103,7 +1308,9 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 	if (robust) {
 	    gretl_list_diff(outlist, varmod->list, vl->reglist);
 	    F = robust_omit_F(outlist, varmod);
-	    if (na(F)) err = 1;
+	    if (na(F)) {
+		err = 1;
+	    }
 	} else {
 	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
 	    err = testmod.errcode;
@@ -1134,7 +1341,9 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
 	if (robust) {
 	    gretl_list_diff(outlist, varmod->list, vl->reglist);
 	    F = robust_omit_F(outlist, varmod);
-	    if (na(F)) err = 1;
+	    if (na(F)) {
+		err = 1;
+	    }
 	} else {
 	    testmod = lsq(vl->reglist, pZ, pdinfo, VAR, OPT_A, 0.0);
 #if VAR_DEBUG
@@ -1162,12 +1371,12 @@ static int var_F_tests (MODEL *varmod, GRETL_VAR *var,
     pputc(prn, '\n');
 
     if (!err && pause) {
-	takenotes(0);
+	scroll_pause();
     }
 
     if (outlist != NULL) {
 	free(outlist);
-    }   
+    }
 
     return err;
 }
@@ -1196,7 +1405,6 @@ static int real_var (int order, const int *inlist,
 {
     int i, k, neqns;
     int t1, t2, oldt1, oldt2;
-    int missv = 0, misst = 0;
     struct var_lists vlists;
     MODEL var_model;
     GRETL_VAR *var = NULL;
@@ -1244,13 +1452,13 @@ static int real_var (int order, const int *inlist,
     /* sort out sample range */
     t1 = pdinfo->t1;
     t2 = pdinfo->t2;
-    
-    if ((missv = adjust_t1t2(NULL, vlists.testlist, &t1, &t2, 
-			     (const double **) *pZ, &misst))) {
-	err = 1;
+
+    if (check_for_missing_obs(vlists.testlist, &t1, &t2,
+			      (const double **) *pZ, NULL)) {
+	err = E_MISSDATA;
 	goto var_bailout;
     }
-
+    
     pdinfo->t1 = t1;
     pdinfo->t2 = t2;
 
@@ -1373,14 +1581,15 @@ static int real_var (int order, const int *inlist,
     }
 
     if (impulses && !err) {
+	int horizon = default_VAR_horizon(pdinfo);
+
 #if VAR_DEBUG
 	gretl_matrix_print(var->C, "var->C", prn);
 #endif
 	for (i=0; i<var->neqns; i++) {
-	    /* FIXME: make horizon configurable */
-	    gretl_var_print_impulse_response(var, i, 0, pdinfo, 
+	    gretl_var_print_impulse_response(var, i, horizon, pdinfo, 
 					     pause, prn);
-	    gretl_var_print_fcast_decomp(var, i, 0, pdinfo, 
+	    gretl_var_print_fcast_decomp(var, i, horizon, pdinfo, 
 					 pause, prn);
 	}
     } 
@@ -1513,7 +1722,7 @@ int coint (int order, const int *list, double ***pZ,
 	   DATAINFO *pdinfo, gretlopt opt, PRN *prn)
 {
     int i, t, n, nv, l0 = list[0];
-    int hasconst = gretl_hasconst(list);
+    int hasconst = gretl_list_has_const(list);
     MODEL cmod;
     int *cointlist = NULL;
     int adfcode = UR_CONST;
@@ -1549,7 +1758,7 @@ int coint (int order, const int *list, double ***pZ,
 	cointlist[l0 + 1] = 0;
 	cointlist[0] += 1;
     } else {
-	cointlist = copylist(list);
+	cointlist = gretl_list_copy(list);
 	if (cointlist == NULL) {
 	    return E_ALLOC;
 	}
@@ -1563,7 +1772,7 @@ int coint (int order, const int *list, double ***pZ,
 
     /* add residuals from cointegrating regression to data set */
     n = pdinfo->n;
-    if (dataset_add_vars(1, pZ, pdinfo)) {
+    if (dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
     }
     nv = pdinfo->v - 1;
@@ -1595,7 +1804,7 @@ int coint (int order, const int *list, double ***pZ,
     /* clean up and get out */
     clear_model(&cmod);
     free(cointlist);
-    dataset_drop_vars(1, pZ, pdinfo);
+    dataset_drop_last_variables(1, pZ, pdinfo);
 
     return 0;
 }
@@ -1683,7 +1892,7 @@ static int auto_adjust_order (int *list, int order_max,
 
 	tstat = kmod.coeff[j] / kmod.sderr[j];
 	clear_model(&kmod);
-	pval = 2.0 * normal(tstat);
+	pval = normal_pvalue_2(tstat);
 
 	if (pval > 0.10) {
 #if ADF_DEBUG
@@ -1720,10 +1929,10 @@ static void copy_list_values (int *targ, const int *src)
     }
 }
 
-static int real_adf_test (int varno, int order, int niv,
-			  double ***pZ, DATAINFO *pdinfo, 
-			  gretlopt opt, int cointcode,
-			  PRN *prn)
+static void 
+print_adf_results (int order, double DFt, double pv, const MODEL *dfmod,
+		   int dfnum, const char *vname, int *blurb_done,
+		   int coint, int i, PRN *prn)
 {
     const char *models[] = {
 	"(1 - L)y = (a-1)*y(-1) + e",
@@ -1744,6 +1953,51 @@ static int real_adf_test (int varno, int order, int niv,
 	N_("with constant and quadratic trend")
     };
 
+    char pvstr[48];
+
+    if (prn == NULL) return;
+
+    if (na(pv)) {
+	sprintf(pvstr, "%s %s", _("p-value"), _("unknown"));
+    } else {
+	sprintf(pvstr, "%s %.4g", 
+		(order > 0)? _("asymptotic p-value") : _("p-value"), 
+		pv);
+    } 
+
+    if (*blurb_done == 0) {
+	if (order > 0) {
+	    pprintf(prn, _("\nAugmented Dickey-Fuller tests, order %d, for %s\n"),
+		    order, vname);
+	} else {
+	    pprintf(prn, _("\nDickey-Fuller tests for %s\n"), vname);
+	}
+	pprintf(prn, _("sample size %d\n"), dfmod->nobs);
+	pputs(prn, _("unit-root null hypothesis: a = 1"));
+	pputs(prn, "\n\n");
+	*blurb_done = 1;
+    }
+
+    pprintf(prn, "   %s\n", _(teststrs[i]));
+
+    if (coint == 0) {
+	pprintf(prn, "   %s: %s\n", _("model"), 
+		(order > 0)? aug_models[i] : models[i]);
+    }
+
+    pprintf(prn, "   %s: %g\n"
+	    "   %s: t = %g\n"
+	    "   %s\n",
+	    _("estimated value of (a - 1)"), dfmod->coeff[dfnum],
+	    _("test statistic"), DFt,
+	    pvstr);	
+}
+
+static int real_adf_test (int varno, int order, int niv,
+			  double ***pZ, DATAINFO *pdinfo, 
+			  gretlopt opt, int cointcode,
+			  PRN *prn)
+{
     MODEL dfmod;
 
     int orig_nvars = pdinfo->v;
@@ -1778,7 +2032,7 @@ static int real_adf_test (int varno, int order, int niv,
 	int tmp = list[0];
 
 	list[0] = order + 5;
-	biglist = copylist(list);
+	biglist = gretl_list_copy(list);
 	if (biglist == NULL) {
 	    free(list);
 	    return E_ALLOC;
@@ -1879,43 +2133,8 @@ static int real_adf_test (int varno, int order, int niv,
 				   niv, itv);
 
 	if (!(opt & OPT_Q)) {
-	    char pvstr[48];
-
-	    if (na(pv)) {
-		sprintf(pvstr, "%s %s", _("p-value"), _("unknown"));
-	    } else {
-		sprintf(pvstr, "%s %.4g", 
-			(order > 0)? _("asymptotic p-value") : _("p-value"), 
-			pv);
-	    } 
-
-	    if (!blurb_done) {
-		if (order > 0) {
-		    pprintf(prn, _("\nAugmented Dickey-Fuller tests, order %d, for %s\n"),
-			    order, pdinfo->varname[varno]);
-		} else {
-		    pprintf(prn, _("\nDickey-Fuller tests for %s\n"),
-			    pdinfo->varname[varno]);
-		}
-		pprintf(prn, _("sample size %d\n"), dfmod.nobs);
-		pputs(prn, _("unit-root null hypothesis: a = 1"));
-		pputs(prn, "\n\n");
-		blurb_done = 1;
-	    }
-
-	    pprintf(prn, "   %s\n", _(teststrs[i]));
-
-	    if (cointcode == 0) {
-		pprintf(prn, "   %s: %s\n", _("model"), 
-			(order > 0)? aug_models[i] : models[i]);
-	    }
-
-	    pprintf(prn, "   %s: %g\n"
-		    "   %s: t = %g\n"
-		    "   %s\n",
-		    _("estimated value of (a - 1)"), dfmod.coeff[dfnum],
-		    _("test statistic"), DFt,
-		    pvstr);	
+	    print_adf_results(order, DFt, pv, &dfmod, dfnum, pdinfo->varname[varno],
+			      &blurb_done, cointcode, i, prn);
 	}
 
 	if (opt & OPT_V) {
@@ -1950,7 +2169,7 @@ static int real_adf_test (int varno, int order, int niv,
 	free(biglist);
     }
 
-    dataset_drop_vars(pdinfo->v - orig_nvars, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - orig_nvars, pZ, pdinfo);
 
     return err;
 }
@@ -2149,7 +2368,7 @@ has_time_trend (const int *varlist, double ***pZ, DATAINFO *pdinfo)
 	}
 
 	tstat = tmod.coeff[0] / tmod.sderr[0];
-	if (tprob(tstat, tmod.dfd) < 0.05) {
+	if (t_pvalue_2(tstat, tmod.dfd) < 0.05) {
 	    trends = 1;
 	}
 
@@ -2158,7 +2377,7 @@ has_time_trend (const int *varlist, double ***pZ, DATAINFO *pdinfo)
 	if (trends) break;
     }
 
-    dataset_drop_vars(pdinfo->v - origv, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
 
     return trends;
 }
@@ -2301,7 +2520,7 @@ int johansen_test (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
     int orig_t1 = pdinfo->t1;
     int orig_v = pdinfo->v;
     int *varlist;
-    int verbose = (opt & OPT_O);
+    int verbose = (opt & OPT_V);
     int hasconst = 0;
     int l0 = list[0];
     int trends = 0;
@@ -2370,7 +2589,7 @@ int johansen_test (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
 	varprn = prn;
     } else {
 	flags = 0;
-	varprn = gretl_print_new(GRETL_PRINT_NULL, NULL);
+	varprn = gretl_print_new(GRETL_PRINT_NULL);
     }
 
     /* FIXME? */
@@ -2466,7 +2685,7 @@ int johansen_test (int order, const int *list, double ***pZ, DATAINFO *pdinfo,
 
     pdinfo->t1 = orig_t1;
 
-    dataset_drop_vars(pdinfo->v - orig_v, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - orig_v, pZ, pdinfo);
 
     return err;
 }
@@ -2475,10 +2694,14 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 {
     int i, j, k, v;
     int dfd = (var->models[0])->dfd;
+    int tex = tex_format(prn);
+    int horizon;
 
-    if (prn == NULL) return 0;
+    if (prn == NULL) {
+	return 0;
+    }
 
-    if (TEX_PRN(prn)) {
+    if (tex) {
 	pputs(prn, "\\noindent");
 	pprintf(prn, I_("\nVAR system, lag order %d\n\n"), var->order);
     } else {
@@ -2492,7 +2715,7 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 
 	if (var->Fvals == NULL) continue;
 
-	if (TEX_PRN(prn)) {
+	if (tex) {
 	    pputs(prn, "\n\\begin{center}\n");
 	    pprintf(prn, "%s\\\\[1em]\n", I_("F-tests of zero restrictions"));
 	    pputs(prn, "\\begin{tabular}{lll}\n");
@@ -2502,7 +2725,7 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 
 	for (j=0; j<var->neqns; j++) {
 	    v = (var->models[j])->list[1];
-	    if (TEX_PRN(prn)) {
+	    if (tex) {
 		pprintf(prn, I_("All lags of %-8s "), pdinfo->varname[v]);
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->order, dfd, var->Fvals[k]);
@@ -2517,7 +2740,7 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 	}
 
 	if (var->order > 1) {
-	    if (TEX_PRN(prn)) {
+	    if (tex) {
 		pprintf(prn, I_("All vars, lag %-6d "), var->order);
 		pputs(prn, "& ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
@@ -2529,7 +2752,7 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 	    } 
 	    k++;
 	}
-	if (TEX_PRN(prn)) {
+	if (tex) {
 	    pputs(prn, "\\end{tabular}\n"
 		  "\\end{center}\n\n"
 		  "\\clearpage\n\n");
@@ -2540,9 +2763,11 @@ int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn)
 
     pputc(prn, '\n');
 
+    horizon = default_VAR_horizon(pdinfo);
+
     for (i=0; i<var->neqns; i++) {
-	gretl_var_print_impulse_response(var, i, 0, pdinfo, 0, prn);
-	gretl_var_print_fcast_decomp(var, i, 0, pdinfo, 0, prn);
+	gretl_var_print_impulse_response(var, i, horizon, pdinfo, 0, prn);
+	gretl_var_print_fcast_decomp(var, i, horizon, pdinfo, 0, prn);
     }
 
     return 0;
@@ -2555,6 +2780,7 @@ void gretl_var_assign_name (GRETL_VAR *var)
     if (var->name != NULL) {
 	free(var->name);
     }
+
     var->name = malloc(8);
     if (var->name != NULL) {
 	sprintf(var->name, "%s %d", _("VAR"), ++n);
@@ -2566,10 +2792,8 @@ void gretl_var_assign_specific_name (GRETL_VAR *var, const char *name)
     if (var->name != NULL) {
 	free(var->name);
     }
-    var->name = malloc(strlen(name) + 1);
-    if (var->name != NULL) {
-	strcpy(var->name, name);
-    }    
+
+    var->name = gretl_strdup(name);
 }
 
 const char *gretl_var_get_name (const GRETL_VAR *var)
@@ -2583,7 +2807,7 @@ int gretl_var_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
     MODEL *pmod = var->models[eqnum];
     int i, t;
 
-    if (dataset_add_vars(1, pZ, pdinfo)) return E_ALLOC;
+    if (dataset_add_series(1, pZ, pdinfo)) return E_ALLOC;
 
     i = pdinfo->v - 1;
 
