@@ -39,6 +39,12 @@ GRETL_VAR *full_var (int order, const int *list,
 		     double ***pZ, DATAINFO *pdinfo,
 		     gretlopt opts, PRN *prn);
 
+const gretl_matrix *
+gretl_var_get_forecast_matrix (GRETL_VAR *var, int t1, int t2,
+			       const double **Z, 
+			       const DATAINFO *pdinfo,
+			       gretlopt opt);
+
 void gretl_var_free (GRETL_VAR *var);
 
 void gretl_var_free_unnamed (GRETL_VAR *var);
@@ -58,15 +64,12 @@ int johansen_test (int order, const int *list,
 		   double ***pZ, DATAINFO *pdinfo,
 		   gretlopt opt, PRN *prn);
 
+int default_VAR_horizon (const DATAINFO *pdinfo);
+
 double *
 gretl_var_get_impulse_responses (GRETL_VAR *var, int targ, int shock,
 				 int periods);
 
-int 
-gretl_var_print_impulse_response (GRETL_VAR *var, int shock,
-				  int periods, const DATAINFO *pdinfo, 
-				  int pause, PRN *prn);
-
 int gretl_var_print (GRETL_VAR *var, const DATAINFO *pdinfo, PRN *prn);
 
 void gretl_var_assign_name (GRETL_VAR *var);
@@ -79,6 +82,12 @@ int gretl_var_get_variable_number (const GRETL_VAR *var, int k);
 
 int gretl_var_get_n_equations (const GRETL_VAR *var);
 
+int gretl_var_get_t1 (const GRETL_VAR *var);
+
+int gretl_var_get_t2 (const GRETL_VAR *var);
+
+const MODEL *gretl_var_get_model (const GRETL_VAR *var, int i);
+
 int gretl_var_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
 				     double ***pZ, DATAINFO *pdinfo);
 
