@@ -23,7 +23,6 @@
 */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 #include "bhhh_max.h"
 
 #include "../cephes/polrt.c"
@@ -33,20 +32,28 @@
 #define MAX_ARMA_ORDER 6
 
 struct arma_info {
-    int p;      /* AR order */
-    int q;      /* MA order */
+    int p;      /* non-seasonal AR order */
+    int q;      /* non-seasonal MA order */
+    int P;      /* seasonal AR order */
+    int Q;      /* seasonal MA order */
     int maxlag; /* longest lag in model */
     int r;      /* number of other regressors (ARMAX) */
     int ifc;    /* 1 for intercept included, otherwise 0 */
+    int nc;     /* total number of coefficients */
     int t1;     /* starting observation */
     int t2;     /* ending observation */
+    int seasonal; /* 1 if any seasonal terms, otherwise 0 */
 };
 
 static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 			       struct arma_info *ainfo)
 {
-    int np = ainfo->p + ainfo->q + ainfo->r + 1 + ainfo->ifc;
-    int i, j;
+    int np = ainfo->nc + 1;
+    int yno, xstart;
+    int i, j, s;
+
+    const char *depvar;
+    size_t n;
 
     pmod->params = malloc(np * sizeof pmod->params);
     if (pmod->params == NULL) {
@@ -70,7 +77,8 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 	}
     }
 
-    strcpy(pmod->params[0], pdinfo->varname[pmod->list[4]]);
+    yno = gretl_model_get_depvar(pmod);
+    strcpy(pmod->params[0], pdinfo->varname[yno]);
 
     if (ainfo->ifc) {
 	strcpy(pmod->params[1], pdinfo->varname[0]);
@@ -79,10 +87,10 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 	j = 1;
     }
 
+    depvar = pmod->params[0];
+    n = strlen(depvar);
+
     for (i=0; i<ainfo->p; i++) {
-	const char *depvar = pmod->params[0];
-	size_t n = strlen(depvar);
-	
 	if (n < VNAMELEN - 4) {
 	    sprintf(pmod->params[j++], "%s(-%d)", depvar, i + 1);
 	} else {
@@ -94,8 +102,28 @@ static void add_arma_varnames (MODEL *pmod, const DATAINFO *pdinfo,
 	sprintf(pmod->params[j++], "e(-%d)", i + 1);
     }
 
+    for (i=0; i<ainfo->P; i++) {
+	s = (i + 1) * pdinfo->pd;
+	if (n < VNAMELEN - 4) {
+	    sprintf(pmod->params[j++], "%s(-%d)", depvar, s);
+	} else {
+	    sprintf(pmod->params[j++], "y(-%d)", s);
+	}
+    }
+
+    for (i=0; i<ainfo->Q; i++) {
+	s = (i + 1) * pdinfo->pd;
+	sprintf(pmod->params[j++], "e(-%d)", s);
+    }    
+
+    if (ainfo->P > 0 || ainfo->Q > 0) {
+	xstart = 8;
+    } else {
+	xstart = 5;
+    }
+
     for (i=0; i<ainfo->r; i++) {
-	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[5+i]]); /* 5? */
+	strcpy(pmod->params[j++], pdinfo->varname[pmod->list[xstart+i]]); 
     }    
 }
 
@@ -168,7 +196,7 @@ static int arma_ll (double *coeff,
     int t1 = model_info_get_t1(arma);
     int t2 = model_info_get_t2(arma);
     int n = t2 - t1 + 1;
-    int p, q, r, ifc;
+    int p, q, r;
 
     const double K = 1.41893853320467274178; /* ln(sqrt(2*pi)) + 0.5 */
     const double *y = bhX[0];
@@ -190,17 +218,16 @@ static int arma_ll (double *coeff,
     p = ainfo->p;
     q = ainfo->q;
     r = ainfo->r;
-    ifc = ainfo->ifc;
 
     /* pointers to blocks of coefficients */
-    ar_coeff = coeff + ifc;
-    ma_coeff = ar_coeff + p;
-    reg_coeff = ma_coeff + q;
+    ar_coeff = coeff + ainfo->ifc;
+    ma_coeff = ar_coeff + ainfo->p;
+    reg_coeff = ma_coeff + ainfo->q;
 
     /* pointers to blocks of derivatives */
-    de_a = de + ifc;
-    de_m = de_a + p;
-    de_r = de_m + q;
+    de_a = de + ainfo->ifc;
+    de_m = de_a + ainfo->p;
+    de_r = de_m + ainfo->q;
 
     if (ma_out_of_bounds(q, ma_coeff)) {
 	fputs("arma: MA estimate(s) out of bounds\n", stderr);
@@ -213,7 +240,7 @@ static int arma_ll (double *coeff,
 
 	e[t] = y[t];
 
-	if (ifc) {
+	if (ainfo->ifc) {
 	    e[t] -= coeff[0];
 	} 
 
@@ -242,13 +269,13 @@ static int arma_ll (double *coeff,
     model_info_set_ll(arma, ll, do_score);
 
     if (do_score) {
-	int lag, nc = p + q + r + ifc;
+	int lag;
 	double x;
 
 	for (t=t1; t<=t2; t++) {
 
 	    /* the constant term (de_0) */
-	    if (ifc) {
+	    if (ainfo->ifc) {
 		de[0][t] = -1.0;
 		for (i=0; i<q; i++) {
 		    de[0][t] -= ma_coeff[i] * de[0][t-i-1];
@@ -287,7 +314,7 @@ static int arma_ll (double *coeff,
 
 	    /* update OPG data set */
 	    x = e[t] / s2; /* sqrt(s2)? does it matter? */
-	    for (i=0; i<nc; i++) {
+	    for (i=0; i<ainfo->nc; i++) {
 		Z[i+1][t] = -de[i][t] * x;
 	    }
 	}
@@ -355,30 +382,30 @@ static cmplx *arma_roots (struct arma_info *ainfo, const double *coeff)
    form of a gretl MODEL struct */
 
 static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
-				      const int *list, int ifc, const double *y, 
-				      const double *theta, int nc)
+				      const int *list, const double *y, 
+				      const double *theta, 
+				      struct arma_info *ainfo)
 {
     int i, t;
-    int p = list[1], q = list[2], r = list[0] - 4;
     double **series = model_info_get_series(arma);
     const double *e = series[0];
     double mean_error;
 
     pmod->ci = ARMA;
-    pmod->ifc = ifc;
+    pmod->ifc = ainfo->ifc;
 
     pmod->lnL = model_info_get_ll(arma);
 
-    pmod->dfn = p + q + r;
+    pmod->dfn = ainfo->nc - ainfo->ifc;
     pmod->dfd = pmod->nobs - pmod->dfn;
-    pmod->ncoeff = nc;
+    pmod->ncoeff = ainfo->nc;
 
     for (i=0; i<pmod->ncoeff; i++) {
 	pmod->coeff[i] = theta[i];
     }
 
     free(pmod->list);
-    pmod->list = copylist(list);
+    pmod->list = gretl_list_copy(list);
 
     pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
@@ -419,16 +446,26 @@ static void rewrite_arma_model_stats (MODEL *pmod, model_info *arma,
     }
 
     mle_aic_bic(pmod, 1);
+
+    if (ainfo->seasonal) {
+	gretl_model_set_int(pmod, "seasonal", 1);
+    }
 }
 
 /* remove the constant or intercept from a list of regressors */
 
-static int remove_const (int *list)
+static int remove_const (int *list, int seasonal)
 {
-    int ret = 0;
+    int xstart, ret = 0;
     int i, j;
 
-    for (i=5; i<=list[0]; i++) {
+    if (seasonal) {
+	xstart = 8;
+    } else {
+	xstart = 5;
+    }
+
+    for (i=xstart; i<=list[0]; i++) {
 	if (list[i] == 0) {
 	    for (j=i; j<list[0]; j++) {
 		list[j] = list[j+1];
@@ -442,31 +479,83 @@ static int remove_const (int *list)
     return ret;
 }
 
-static int check_arma_list (int *list, struct arma_info *ainfo)
+#define has_seasonals(l) (l[0] > 5 && l[3] == LISTSEP && l[6] == LISTSEP)
+
+static int 
+check_arma_list (int *list, gretlopt opt, struct arma_info *ainfo)
 {
+    int armax = 0;
+    int hadconst = 0;
     int err = 0;
 
+    ainfo->seasonal = has_seasonals(list);
+    ainfo->p = ainfo->q = 0;
+    ainfo->P = ainfo->Q = 0;
+    ainfo->r = ainfo->nc = 0;
+
+    if (ainfo->seasonal) {
+	armax = (list[0] > 7);
+    } else {
+	armax = (list[0] > 4);
+    }
+
     if (list[0] < 4) {
 	err = 1;
     } else if (list[1] < 0 || list[1] > MAX_ARMA_ORDER) {
 	err = 1;
     } else if (list[2] < 0 || list[2] > MAX_ARMA_ORDER) {
 	err = 1;
-    } else if (list[1] + list[2] == 0) {
+    } else if (list[1] + list[2] == 0 && !ainfo->seasonal) {
 	err = 1;
     }
 
-    /* signal for adding a constant to the regression list, unless it
-       is deliberately omitted, in an ARMAX model 
+    if (!err) {
+	ainfo->p = list[1];
+	ainfo->q = list[2];
+    }
+
+    if (!err && ainfo->seasonal) {
+	if (list[0] < 7) {
+	    err = 1;
+	} else if (list[4] < 0 || list[4] > MAX_ARMA_ORDER) {
+	    err = 1;
+	} else if (list[5] < 0 || list[5] > MAX_ARMA_ORDER) {
+	    err = 1;
+	} else if (list[4] + list[5] == 0) {
+	    err = 1;
+	}
+    }
+
+    if (!err && ainfo->seasonal) {
+	ainfo->P = list[4];
+	ainfo->Q = list[5];
+    }
+
+    /* If there's an explicit constant in the list here, we'll remove
+       it, since it is added implicitly later.  But if we're supplied
+       with OPT_S (meaning: suppress the intercept) we'll flag this by
+       setting ifc = 0.  Also, if the user gave an armax list
+       (specifying regressors) we'll respect the absence of a constant
+       from that list by setting ifc = 0.
     */
-    if (list[0] == 4 || remove_const(list)) {
-	ainfo->ifc = 1;
-    } else {
-	ainfo->ifc = 0;
+
+    if (!err) {
+	if (armax) {
+	    hadconst = remove_const(list, ainfo->seasonal);
+	}
+	if ((opt & OPT_S) || (armax && !hadconst)) {
+	    ainfo->ifc = 0;
+	} else {
+	    ainfo->ifc = 1;
+	}
     }
 
     if (err) {
 	gretl_errmsg_set(_("Error in arma command"));
+    } else {
+	ainfo->r = list[0] - ((ainfo->seasonal)? 7 : 4);
+	ainfo->nc = ainfo->p + ainfo->q + ainfo->P + ainfo->Q
+	    + ainfo->r + ainfo->ifc;
     }
 
     return err;
@@ -599,16 +688,19 @@ static int ar_init_by_ols (const int *list, double *coeff,
     return err;
 }
 
-static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
-			  struct arma_info *ainfo)
+static int 
+adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
+	       struct arma_info *ainfo)
 {
     int t1 = pdinfo->t1, t2 = pdinfo->t2;
     int an, i, v, t, t1min = 0;
-    int anymiss;
+    int vstart, anymiss;
+
+    vstart = (ainfo->seasonal)? 7 : 4;
 
     for (t=0; t<=pdinfo->t2; t++) {
 	anymiss = 0;
-	for (i=4; i<=list[0]; i++) {
+	for (i=vstart; i<=list[0]; i++) {
 	    v = list[i];
 	    if (na(Z[v][t])) {
 		anymiss = 1;
@@ -630,7 +722,7 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
 
     for (t=pdinfo->t2; t>=t1; t--) {
 	anymiss = 0;
-	for (i=4; i<=list[0]; i++) {
+	for (i=vstart; i<=list[0]; i++) {
 	    v = list[i];
 	    if (na(Z[v][t])) {
 		anymiss = 1;
@@ -645,8 +737,8 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
     }
 
     for (t=t1-ainfo->p; t<t2; t++) {
-	for (i=4; i<=list[0]; i++) {
-	    if (t < t1 && i > 4) {
+	for (i=vstart; i<=list[0]; i++) {
+	    if (t < t1 && i > vstart) {
 		continue;
 	    }
 	    v = list[i];
@@ -662,7 +754,7 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
     }
 
     an = t2 - t1 + 1;
-    if (an <= ainfo->p + ainfo->q + ainfo->r + 1) {
+    if (an <= ainfo->nc) {
 	return 1; 
     }
 
@@ -677,19 +769,14 @@ static int adjust_sample (DATAINFO *pdinfo, const double **Z, const int *list,
 static model_info *
 set_up_arma_model_info (struct arma_info *ainfo)
 {
-    model_info *arma = model_info_new();
-    int m;
+    model_info *arma;
+
+    arma = model_info_new(ainfo->nc, ainfo->t1, ainfo->t2, 1.0e-6);
 
     if (arma == NULL) return NULL;
 
     model_info_set_opts(arma, PRESERVE_OPG_MODEL);
-    model_info_set_tol(arma, 1.0e-6);
-
-    m = ainfo->p + ainfo->q + ainfo->r + ainfo->ifc;
-    model_info_set_k(arma, m);
-
-    model_info_set_n_series(arma, m + 1);
-    model_info_set_t1_t2(arma, ainfo->t1, ainfo->t2);
+    model_info_set_n_series(arma, ainfo->nc + 1);
 
     /* add pointer to ARMA-specific details */
     model_info_set_extra_info(arma, ainfo);
@@ -697,77 +784,81 @@ set_up_arma_model_info (struct arma_info *ainfo)
     return arma;
 }
 
-MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo, 
-		  PRN *prn)
+MODEL arma_model (const int *list, const double **Z, DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn)
 {
-    int nc, yno;
+    int yno;
     double *coeff = NULL;
     const double **X = NULL;
+    int *alist = NULL;
+    PRN *aprn = NULL;
+    model_info *arma = NULL;
     MODEL armod;
-    model_info *arma;
     struct arma_info ainfo;
     int err = 0;
 
+    if (opt & OPT_V) {
+	aprn = prn;
+    } 
+
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
-    if (check_arma_list(list, &ainfo)) {
-	armod.errcode = E_UNSPEC;
-	return armod;
+    alist = gretl_list_copy(list);
+    if (alist == NULL) {
+	armod.errcode = E_ALLOC;
+	goto bailout;
     }
 
-    yno = list[4];         /* dependent variable */
-    ainfo.p = list[1];     /* AR order */
-    ainfo.q = list[2];     /* MA order */
-    ainfo.maxlag = (ainfo.p > ainfo.q)? 
-	ainfo.p : ainfo.q; /* maximum lag in the model */
+    if (check_arma_list(alist, opt, &ainfo)) {
+	armod.errcode = E_UNSPEC;
+	goto bailout;
+    }
 
-    ainfo.r = list[0] - 4; /* number of ordinary regressors */
+    /* dependent variable */
+    yno = (ainfo.seasonal)? alist[7] : alist[4];
+
+    /* FIXME maxlag */
+    ainfo.maxlag = (ainfo.p > ainfo.q)? 
+	ainfo.p : ainfo.q;  /* maximum non-seasonal lag in the model */
 
     /* adjust sample range if need be */
-    if (adjust_sample(pdinfo, Z, list, &ainfo)) {
+    if (adjust_sample(pdinfo, Z, alist, &ainfo)) {
         armod.errcode = E_DATA;
-        return armod;
+	goto bailout;
     }
 
-    /* tally of coefficients */
-    nc = ainfo.p + ainfo.q + ainfo.r + ainfo.ifc;
-
     /* initial coefficient vector */
-    coeff = malloc(nc * sizeof *coeff);
+    coeff = malloc(ainfo.nc * sizeof *coeff);
     if (coeff == NULL) {
 	armod.errcode = E_ALLOC;
-	return armod;
+	goto bailout;
     }
 
     /* create model_info struct to feed to bhhh_max() */
     arma = set_up_arma_model_info(&ainfo);
     if (arma == NULL) {
-	free(coeff);
 	armod.errcode = E_ALLOC;
-	return armod;
+	goto bailout;
     }
 
     /* initialize the coefficients: AR and regression part by OLS, 
        MA at 0 */
-    err = ar_init_by_ols(list, coeff, Z, pdinfo, &ainfo);
+    err = ar_init_by_ols(alist, coeff, Z, pdinfo, &ainfo);
     if (err) {
-	free(coeff);
-	model_info_free(arma);
 	armod.errcode = E_ALLOC;
-	return armod;
+	goto bailout;
     }	
 
     /* construct virtual dataset for dep var, real regressors */
-    X = make_armax_X(list, Z);
+    X = make_armax_X(alist, Z);
     if (X == NULL) {
 	armod.errcode = E_ALLOC;
-	free(coeff);
-	return armod;
+	goto bailout;
     }
 
     /* call BHHH conditional ML function (OPG regression) */
-    err = bhhh_max(arma_ll, X, coeff, arma, prn);
+    err = bhhh_max(arma_ll, X, coeff, arma, aprn);
 
     if (err) {
 	fprintf(stderr, "arma: bhhh_max returned %d\n", err);
@@ -777,8 +868,8 @@ MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo,
 	double *theta = model_info_get_theta(arma);
 	cmplx *roots;
 
-	rewrite_arma_model_stats(pmod, arma, list, ainfo.ifc, Z[yno], 
-				 theta, nc);
+	rewrite_arma_model_stats(pmod, arma, alist, Z[yno], 
+				 theta, &ainfo);
 
 	/* compute and save polynomial roots */
 	roots = arma_roots(&ainfo, theta);
@@ -790,8 +881,12 @@ MODEL arma_model (int *list, const double **Z, DATAINFO *pdinfo,
 	add_arma_varnames(pmod, pdinfo, &ainfo);
 
 	armod = *pmod;
+	free(pmod);
     }
 
+ bailout:
+
+    free(alist);
     free(coeff);
     free(X);
     model_info_free(arma);
