@@ -18,7 +18,6 @@
  */
 
 #include "libgretl.h"
-#include "gretl_private.h"
 
 #include "../cephes/mconf.h"
 
@@ -189,7 +188,7 @@ static void write_arma_model_stats (MODEL *pmod, const int *list,
     pmod->dfd = pmod->nobs - pmod->dfn;
     pmod->ncoeff = p + q + 1;
 
-    pmod->list = copylist(list);
+    pmod->list = gretl_list_copy(list);
 
     pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
     pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
@@ -640,8 +639,9 @@ static void output_series_to_spc (const double *x, int t1, int t2,
     fputs(" )\n", fp);
 }
 
-static int check_for_missing (const double **Z, const DATAINFO *pdinfo,
-			      int v, int *t1, int *t2)
+static int 
+arma_missobs_check (const double **Z, const DATAINFO *pdinfo,
+		    int v, int *t1, int *t2)
 {
     int misst = 0;
     int list[2];
@@ -652,7 +652,7 @@ static int check_for_missing (const double **Z, const DATAINFO *pdinfo,
     *t1 = pdinfo->t1;
     *t2 = pdinfo->t2;
 
-    if (adjust_t1t2(NULL, list, t1, t2, Z, &misst)) {
+    if (check_for_missing_obs(list, t1, t2, Z, &misst)) {
 	gchar *msg;
 
 	msg = g_strdup_printf(_("Missing value encountered for "
@@ -741,14 +741,14 @@ static int check_arma_list (const int *list)
     return err;
 }
 
-MODEL arma_x12_model (int *list, const double **Z, 
-		      const DATAINFO *pdinfo, PRN *prn, 
+MODEL arma_x12_model (const int *list, const double **Z, const DATAINFO *pdinfo, 
 		      const char *prog, const char *workdir,
-		      int gui)
+		      gretlopt opt, int gui, PRN *prn)
 {
     int err = 0;
     int verbose = (prn != NULL);
     char varname[VNAMELEN], path[MAXLEN];
+    PRN *aprn = NULL;
 #ifndef GLIB2
     char cmd[MAXLEN];
 #endif
@@ -756,6 +756,10 @@ MODEL arma_x12_model (int *list, const double **Z,
     int t1, t2;
     MODEL armod;
 
+    if (opt & OPT_V) {
+	aprn = prn;
+    }
+
     gretl_model_init(&armod);  
     gretl_model_smpl_init(&armod, pdinfo);
 
@@ -779,9 +783,8 @@ MODEL arma_x12_model (int *list, const double **Z,
 	return armod;
     }
 
-    /* missing observations check */
-    if (check_for_missing(Z, pdinfo, v, &t1, &t2)) {
-	armod.errcode = E_DATA;
+    if (arma_missobs_check(Z, pdinfo, v, &t1, &t2)) {
+	armod.errcode = E_MISSDATA;
 	return armod;
     }	
 
@@ -812,7 +815,7 @@ MODEL arma_x12_model (int *list, const double **Z,
 	armod.t2 = t2;
 	populate_arma_model(&armod, list, path, y, pdinfo, p + q + 1);
 	if (verbose && !armod.errcode) {
-	    print_iterations(path, prn);
+	    print_iterations(path, aprn);
 	}
 	if (!armod.errcode && gui) {
 	    add_unique_output_file(&armod, path);
