@@ -6,29 +6,32 @@
 #include "gretl_enums.h"
 
 static int audioprint_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
-			     int c, PRN *prn)
+			     int i, PRN *prn)
 {
-    pprintf(prn, "Variable %d, %s.\n", pmod->list[c], 
-	    pdinfo->varname[pmod->list[c]]);
+    char varname[24];
+
+    gretl_model_get_param_name(pmod, pdinfo, i, varname);
+
+    pprintf(prn, "Variable %d, %s.\n", pmod->list[i+2], varname);
 
     /* print coeff value if well-defined */
-    if (isnan(pmod->coeff[c-2]) || na(pmod->coeff[c-2])) {
+    if (isnan(pmod->coeff[i]) || na(pmod->coeff[i])) {
 	pputs(prn, "Coefficient is undefined.\n");
 	return 1;
     } else {
-	pprintf(prn, "Coefficient %g.\n", pmod->coeff[c-2]);
+	pprintf(prn, "Coefficient %g.\n", pmod->coeff[i]);
     }
 
     /* get out if std error is undefined */
-    if (isnan(pmod->sderr[c-2]) || na(pmod->sderr[c-2])) {
+    if (isnan(pmod->sderr[i]) || na(pmod->sderr[i])) {
 	return 1;
     }
 
     /* std error is well-defined, but is it positive? */
-    if (pmod->sderr[c-2] > 0.) {
+    if (pmod->sderr[i] > 0.) {
 	double t, pval;
 
-	t = pmod->coeff[c-2] / pmod->sderr[c-2];
+	t = pmod->coeff[i] / pmod->sderr[i];
 	pval = coeff_pval(pmod, t, pmod->dfd);
 	pprintf(prn, "P-value %.3g.\n", pval);
     } else { /* zero standard error */
@@ -45,7 +48,7 @@ static int audioprint_coefficients (const MODEL *pmod, const DATAINFO *pdinfo,
     int n = pmod->ncoeff;
 
     for (i=0; i<n; i++) {
-	err = audioprint_coeff(pdinfo, pmod, i + 2, prn);
+	err = audioprint_coeff(pdinfo, pmod, i, prn);
 	if (err) gotnan = 1;
     }
 
@@ -85,7 +88,7 @@ audioprint_model (MODEL *pmod, const DATAINFO *pdinfo, PRN *prn)
 }
 
 static void 
-audioprint_summary (GRETLSUMMARY *summ, const DATAINFO *pdinfo,
+audioprint_summary (Summary *summ, const DATAINFO *pdinfo,
 		    PRN *prn)
 {
     char date1[OBSLEN], date2[OBSLEN];
@@ -117,18 +120,18 @@ audioprint_summary (GRETLSUMMARY *summ, const DATAINFO *pdinfo,
 }
 
 static void
-audioprint_matrix (const double *vec, const int *list,
-		   int t1, int t2, int n, int ci,
-		   const DATAINFO *pdinfo, PRN *prn)
+audioprint_matrix (const VMatrix *vmat, const DATAINFO *pdinfo,
+		   PRN *prn)
 {
     int i, j, k;
-    int lo = list[0];
+    int n = vmat->t2 - vmat->t1 + 1;
+    int lo = vmat->dim;
 
-    if (ci == CORR) {
+    if (vmat->ci == CORR) {
 	char date1[OBSLEN], date2[OBSLEN];
 
-	ntodate(date1, t1, pdinfo);
-	ntodate(date2, t2, pdinfo);
+	ntodate(date1, vmat->t1, pdinfo);
+	ntodate(date2, vmat->t2, pdinfo);
 
 	pprintf(prn, "Correlation coefficients, using the observations "
 		"%s to %s.\n", date1, date2);
@@ -142,76 +145,48 @@ audioprint_matrix (const double *vec, const int *list,
 	for (j=i; j<=lo; j++) {
 	    k = ijton(i-1, j-1, lo);
 	    if (i == j) {
-		if (ci == CORR) continue;
-		pprintf(prn, "%s, ", pdinfo->varname[list[i]]);
+		if (vmat->ci == CORR) {
+		    continue;
+		}
+		pprintf(prn, "%s, ", vmat->names[i-1]);
 	    } else {
-		pprintf(prn, "%s and %s, ", 
-			pdinfo->varname[list[i]],
-			pdinfo->varname[list[j]]);
+		pprintf(prn, "%s and %s, ", vmat->names[i-1], 
+			vmat->names[j-1]);
 	    }
-	    if (ci == CORR) {
-		pprintf(prn, "%.3f.\n", vec[k]);
+	    if (vmat->ci == CORR) {
+		pprintf(prn, "%.3f.\n", vmat->vec[k]);
 	    } else {
-		pprintf(prn, "%.4g.\n", vec[k]);
+		pprintf(prn, "%.4g.\n", vmat->vec[k]);
 	    }
 	}
     }
 }
 
-/* ........................................................... */
-
 static void 
-audioprint_corrmat (CORRMAT *corr,
-		    const DATAINFO *pdinfo, 
-		    PRN *prn)
+audioprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
-    audioprint_matrix(corr->xpx, corr->list, corr->t1, corr->t2,
-		      corr->n, CORR, pdinfo, prn);
-}
+    pprintf(prn, "Variable '%s', ", cf->names[i]);
+    pprintf(prn, "point estimate %.4g, 95%% confidence interval, ", cf->coeff[i]);
 
-/* .................................................................. */
-
-static void 
-audioprint_coeff_interval (const CONFINT *cf, const DATAINFO *pdinfo, 
-			   int c, PRN *prn)
-{
-    pprintf(prn, "Variable %d, '%s', ", cf->list[c], pdinfo->varname[cf->list[c]]);
-    pprintf(prn, "point estimate %.4g, 95%% confidence interval, ", cf->coeff[c-2]);
-
-    if (isnan(cf->maxerr[c-2])) {
+    if (isnan(cf->maxerr[i])) {
 	pputs(prn, "undefined.\n");	
     } else {
 	pprintf(prn, "%.4g to %.4g.\n", 
-		cf->coeff[c-2] - cf->maxerr[c-2],
-		cf->coeff[c-2] + cf->maxerr[c-2]);
+		cf->coeff[i] - cf->maxerr[i],
+		cf->coeff[i] + cf->maxerr[i]);
     }
 }
 
-/* .................................................................. */
-
 static void 
-audioprint_confints (const CONFINT *cf, const DATAINFO *pdinfo, 
-		     PRN *prn)
+audioprint_confints (const CoeffIntervals *cf, PRN *prn)
 {
-    int i, ncoeff = cf->list[0];
+    int i;
 
-    for (i=2; i<=ncoeff; i++) {
-	audioprint_coeff_interval(cf, pdinfo, i, prn);
+    for (i=0; i<cf->ncoeff; i++) {
+	audioprint_coeff_interval(cf, i, prn);
     }
 }
 
-/* .................................................................. */
-
-static void 
-audioprint_vcv (const VCV *vcv, const DATAINFO *pdinfo, 
-		PRN *prn)
-{
-    audioprint_matrix(vcv->vec, vcv->list, 0, 0, 0,
-		      COVAR, pdinfo, prn);
-}
-
-/* .................................................................. */
-
 #ifdef HAVE_FLITE
 
 static int speak_buffer (const char *buf, int (*should_stop)())
@@ -337,38 +312,33 @@ static int audio_print_special (int role, void *data, const DATAINFO *pdinfo,
 				int (*should_stop)())
 {
     PRN *prn;
+    const char *buf;
 
-    prn = gretl_print_new(GRETL_PRINT_BUFFER, NULL);
+    prn = gretl_print_new(GRETL_PRINT_BUFFER);
     if (prn == NULL) return 1;
     
     /* descriptive statistics */
     if (role == SUMMARY || role == VAR_SUMMARY) {
-	GRETLSUMMARY *summ = (GRETLSUMMARY *) data;
+	Summary *summ = (Summary *) data;
 
 	audioprint_summary(summ, pdinfo, prn);
-    }
-    else if (role == CORR) {
-	CORRMAT *corr = (CORRMAT *) data;
+    } else if (role == CORR || role == COVAR) {
+	VMatrix *vmat = (VMatrix *) data;
 
-	audioprint_corrmat(corr, pdinfo, prn);
-    }
-    else if (role == COVAR) {
-	VCV *vcv = (VCV *) data;
+	audioprint_matrix(vmat, pdinfo, prn);
+    } else if (role == COEFFINT) {
+	CoeffIntervals *cf = (CoeffIntervals *) data;
 
-	audioprint_vcv(vcv, pdinfo, prn);
-    }
-    else if (role == COEFFINT) {
-	CONFINT *cf = (CONFINT *) data;
-
-	audioprint_confints(cf, pdinfo, prn);
-    }
-    else if (role == VIEW_MODEL) {
+	audioprint_confints(cf, prn);
+    } else if (role == VIEW_MODEL) {
 	MODEL *pmod = (MODEL *) data;
 
 	audioprint_model(pmod, pdinfo, prn);
     }
 
-    speak_buffer(prn->buf, should_stop);
+    buf = gretl_print_get_buffer(prn);
+
+    speak_buffer(buf, should_stop);
     gretl_print_destroy(prn);
 
     return 0;
@@ -480,7 +450,8 @@ static int read_listbox_content (windata_t *vwin, int (*should_stop)())
 	}
 
 	if (!err) {
-	    line = g_strdup_printf("%s. %s. %s.\n", tmpstr[0], tmpstr[1], tmpstr[2]);
+	    line = g_strdup_printf("%s. %s. %s.\n", tmpstr[0], 
+				   tmpstr[1], tmpstr[2]);
 	    err = speak_line(line);
 	    g_free(line);
 	}
