@@ -23,6 +23,7 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "var.h"
 
 #include "fcp.h"
 
@@ -42,7 +43,7 @@ static void add_garch_varnames (MODEL *pmod, const DATAINFO *pdinfo,
     int i, j, np = 3 + p + q + r;
 
     free(pmod->list);
-    pmod->list = copylist(list);
+    pmod->list = gretl_list_copy(list);
 
     pmod->params = malloc(np * sizeof pmod->params);
     if (pmod->params == NULL) {
@@ -129,7 +130,11 @@ static int write_garch_stats (MODEL *pmod, const double **Z,
 {
     int err = 0;
     double *coeff, *sderr, *garch_h;
-    int i, ynum = list[4];
+    double den;
+    int ynum = list[4];
+    int nvp = list[1] + list[2];
+    int xvars = list[0] - 4;
+    int i;
 
     coeff = realloc(pmod->coeff, nparam * sizeof *pmod->coeff);
     sderr = realloc(pmod->sderr, nparam * sizeof *pmod->sderr);
@@ -153,7 +158,13 @@ static int write_garch_stats (MODEL *pmod, const double **Z,
 	pmod->yhat[i] =  Z[ynum][i] * scale - pmod->uhat[i];
     }
 
-    pmod->sigma = NADBL;
+    /* set sigma to its unconditional or steady-state value */
+    den = 1.0;
+    for (i=0; i<nvp; i++) {
+	den -= coeff[i+xvars+1];
+    }
+    pmod->sigma = sqrt(coeff[xvars] / den);
+
     pmod->adjrsq = NADBL; 
     pmod->fstt = NADBL;
 
@@ -433,7 +444,7 @@ add_uhat_squared (const MODEL *pmod, double scale,
 {
     int t, v = pdinfo->v;
 
-    if (dataset_add_vars(1, pZ, pdinfo)) {
+    if (dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
     }
 
@@ -531,7 +542,7 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
 	return E_ALLOC;
     }
 
-    list = copylist(garchlist);
+    list = gretl_list_copy(garchlist);
     if (list == NULL) {
 	err = E_ALLOC;
 	goto bailout;
@@ -542,7 +553,7 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
     /* dep var is squared OLS residual */
     list[4] = v;
 
-    amod = arma(list, (const double **) *pZ, pdinfo, NULL);
+    amod = arma(list, (const double **) *pZ, pdinfo, OPT_NONE, NULL);
     if (amod.errcode) {
 	err = amod.errcode;
 	goto bailout;
@@ -560,7 +571,7 @@ garch_init_by_arma (const MODEL *pmod, const int *garchlist,
 
  bailout:
 
-    dataset_drop_vars(pdinfo->v - v, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - v, pZ, pdinfo);
     free(list);
 
     return err;
@@ -634,11 +645,32 @@ static int *make_ols_list (const int *list)
     return olist;
 }
 
+#if 0
+int garch_pretest (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
+		   PRN *prn)
+{
+    int err;
+
+    err = autocorr_test(pmod, pdinfo->pd, pZ, pdinfo,
+			OPT_S | OPT_Q, prn);
+
+    if (!err) {
+	char teststat[64];
+	double LMF = get_last_test_statistic(teststat);
+	double pval = get_last_pvalue(teststat);
+
+	fprintf(stderr, "LMF = %g, pval = %g\n", LMF, pval);
+    }
+
+    return err;
+}
+#endif
+
 #define GARCH_SCALE_SIGMA 1
 
 /* the driver function for the plugin */
 
-MODEL garch_model (int *cmdlist, double ***pZ, DATAINFO *pdinfo,
+MODEL garch_model (const int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 		   PRN *prn, gretlopt opt) 
 {
     MODEL model;
@@ -668,6 +700,13 @@ MODEL garch_model (int *cmdlist, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
+#if 0
+    /* pretest the residuals for autocorrelation */
+    if (!err) {
+	err = garch_pretest(&model, pZ, pdinfo, prn);
+    }
+#endif
+
 #if GARCH_SCALE_SDY
     if (!err) {
 	yno = ols_list[1];
