@@ -37,8 +37,9 @@ static void time_series_setup (const char *s, DATAINFO *newinfo, int pd,
 			       int *label_strings)
 {
     if (*s == '"' || *s == '\'') s++;
+
     newinfo->pd = pd;
-    newinfo->time_series = TIME_SERIES;
+    newinfo->structure = TIME_SERIES;
 
     strcpy(newinfo->stobs, s);
     colonize_obs(newinfo->stobs);
@@ -46,7 +47,10 @@ static void time_series_setup (const char *s, DATAINFO *newinfo, int pd,
 
     newinfo->sd0 = get_date_x(newinfo->pd, newinfo->stobs);
 
-    if (text_cols != NULL) *text_cols = 1;
+    if (text_cols != NULL) {
+	*text_cols = 1;
+    }
+
     *time_series = 1;
     *label_strings = 0;
 }
@@ -97,7 +101,7 @@ static int label_is_date (char *str)
     return pd;
 }
 
-static int obs_column_heading (char *label)
+static int obs_column_heading (const char *label)
 {
     int ret = 0;
 
@@ -113,12 +117,15 @@ static int obs_column_heading (char *label)
 	if (*label == '\0') {
 	    ret = 1;    
 	} else {
-	    lower(label);
-	    if (strncmp(label, "obs", 3) == 0 ||
-		strcmp(label, "date") == 0 ||
-		strcmp(label, "year") == 0) {
+	    gchar *test = g_strdup(label);
+
+	    lower(test);
+	    if (strncmp(test, "obs", 3) == 0 ||
+		strcmp(test, "date") == 0 ||
+		strcmp(test, "year") == 0) {
 		ret = 1;
 	    }
+	    g_free(test);
 	}
     }
 
@@ -163,6 +170,37 @@ static void wbook_init (wbook *book)
     book->selected = 0;
 }
 
+static const char *column_label (int col)
+{
+    const char *abc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    static char label[5];
+    char a1, a2;
+
+    if (col < 26) {
+	a2 = abc[col];
+	sprintf(label, "(%c)", a2);
+    } else {
+	a1 = abc[col / 26 - 1];
+	a2 = abc[col % 26];
+	sprintf(label, "(%c%c)", a1, a2);
+    }
+
+    return label;
+}
+
+static void colspin_changed (GtkEditable *ed, GtkWidget *w)
+{
+    const gchar *text = gtk_entry_get_text(GTK_ENTRY(ed));
+
+    if (text != NULL && isdigit((unsigned char) *text)) {
+	int col = atoi(text);
+
+	if (col > 0 && col < 257) {
+	    gtk_label_set_text(GTK_LABEL(w), column_label(col - 1));
+	}
+    }
+}
+
 #if GTK_MAJOR_VERSION >= 2
 
 static
@@ -273,53 +311,74 @@ static void wsheet_menu (wbook *book, int multisheet)
 {
     GtkWidget *w, *tmp, *label;
     GtkWidget *vbox, *hbox;
-    GtkObject *adj;
+    GtkObject *c_adj, *r_adj;
 
     w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
     gtk_window_set_title(GTK_WINDOW(w), _("gretl: spreadsheet import"));
+
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
 
     vbox = gtk_vbox_new (FALSE, 5);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+    gtk_container_add(GTK_CONTAINER(w), vbox);
 
-    /* choose starting column and row */
+    /* selection of starting column and row */
     label = gtk_label_new(_("Start import at:"));
     gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 5);
 
     hbox = gtk_hbox_new (FALSE, 5);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
 
+    /* starting column spinner */
     tmp = gtk_label_new(_("column:"));
-    adj = gtk_adjustment_new(1, 1, 25, 1, 1, 1);
-    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-    g_signal_connect (adj, "value_changed",
+    c_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
+    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(c_adj), 1, 0);
+    g_signal_connect (c_adj, "value_changed",
 		      G_CALLBACK (wbook_get_col_offset), book);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->colspin),
+				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
     gtk_box_pack_start (GTK_BOX (hbox), book->colspin, FALSE, FALSE, 5);
 
+    /* starting row spinner */
     tmp = gtk_label_new(_("row:"));
-    adj = gtk_adjustment_new(1, 1, 25, 1, 1, 1);
-    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-    g_signal_connect (adj, "value_changed",
+    r_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
+    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(r_adj), 1, 0);
+    g_signal_connect (r_adj, "value_changed",
 		      G_CALLBACK (wbook_get_row_offset), book);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->rowspin),
+				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
     gtk_box_pack_start (GTK_BOX (hbox), book->rowspin, FALSE, FALSE, 5);
 
+    /* column label feedback */
+    hbox = gtk_hbox_new (FALSE, 5);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    label = gtk_label_new("(A)");
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
+    g_signal_connect (GTK_EDITABLE(book->colspin), "changed",
+		      G_CALLBACK (colspin_changed), label);
+
     /* choose the worksheet (if applicable) */
-    if (multisheet) add_sheets_list(vbox, book);
+    if (multisheet) {
+	add_sheets_list(vbox, book);
+    }
 
     hbox = gtk_hbox_new (TRUE, 5);
+    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
     tmp = gtk_button_new_from_stock(GTK_STOCK_OK);
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, 0);
     g_signal_connect_swapped (G_OBJECT (tmp), "clicked", 
 			      G_CALLBACK (gtk_widget_destroy), 
 			      G_OBJECT (w));
+    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
+    gtk_widget_grab_default(tmp);
 
     tmp = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, 0);
     g_signal_connect (G_OBJECT (tmp), "clicked", 
@@ -328,8 +387,8 @@ static void wsheet_menu (wbook *book, int multisheet)
 			      G_CALLBACK (gtk_widget_destroy), 
 			      G_OBJECT (w));
 
-    gtk_container_add(GTK_CONTAINER(vbox), hbox);
-    gtk_container_add(GTK_CONTAINER(w), vbox);
+    gtk_entry_set_activates_default(GTK_ENTRY(book->colspin), TRUE);
+    gtk_entry_set_activates_default(GTK_ENTRY(book->rowspin), TRUE);
 
     gtk_widget_show_all(w);
 
@@ -382,44 +441,73 @@ void wbook_get_row_offset (GtkWidget *w, wbook *book)
 	(GTK_SPIN_BUTTON(book->rowspin)) - 1;
 }
 
+static gint entry_activate (GtkWidget *w, GdkEventKey *key, gpointer p)
+{
+    GtkWidget *top = gtk_widget_get_toplevel(w);
+
+    gtk_window_activate_default(GTK_WINDOW(top));
+
+    return FALSE;
+}
+
 static void wsheet_menu (wbook *book, int multisheet)
 {
     GtkWidget *w, *tmp, *frame;
     GtkWidget *vbox, *hbox, *list;
-    GtkObject *adj;
+    GtkObject *c_adj, *r_adj;
 
     w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+
     gtk_window_set_title(GTK_WINDOW(w), "gretl: spreadsheet import");
+
     gtk_signal_connect(GTK_OBJECT(w), "destroy",  
 		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
 
     vbox = gtk_vbox_new (FALSE, 5);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
+    gtk_container_add(GTK_CONTAINER(w), vbox);    
 
-    /* choose starting column and row */
-    frame = gtk_frame_new("Start import at");
-    gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 5);
+    /* selection of starting column and row */
+    tmp = gtk_label_new(_("Start import at:"));
+    gtk_box_pack_start (GTK_BOX (vbox), tmp, FALSE, FALSE, 5);
 
     hbox = gtk_hbox_new (FALSE, 5);
-    gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
-    gtk_container_add(GTK_CONTAINER (frame), hbox);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
 
+    /* starting column spinner */
     tmp = gtk_label_new("column:");
-    adj = gtk_adjustment_new(1, 1, 25, 1, 1, 1);
-    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-    gtk_signal_connect (adj, "value_changed",
+    c_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
+    book->colspin = gtk_spin_button_new (GTK_ADJUSTMENT(c_adj), 1, 0);
+    gtk_signal_connect (c_adj, "value_changed",
 			GTK_SIGNAL_FUNC (wbook_get_col_offset), book);
+    gtk_signal_connect(GTK_OBJECT(book->colspin), "activate", 
+		       GTK_SIGNAL_FUNC(entry_activate), NULL);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->colspin),
+				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
     gtk_box_pack_start (GTK_BOX (hbox), book->colspin, FALSE, FALSE, 5);
 
+    /* starting row spinner */
     tmp = gtk_label_new("row:");
-    adj = gtk_adjustment_new(1, 1, 25, 1, 1, 1);
-    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-    gtk_signal_connect (adj, "value_changed",
+    r_adj = gtk_adjustment_new(1, 1, 256, 1, 1, 1);
+    book->rowspin = gtk_spin_button_new (GTK_ADJUSTMENT(r_adj), 1, 0);
+    gtk_signal_connect (r_adj, "value_changed",
 			GTK_SIGNAL_FUNC (wbook_get_row_offset), book);
+    gtk_signal_connect(GTK_OBJECT(book->rowspin), "activate", 
+		       GTK_SIGNAL_FUNC(entry_activate), NULL);
+    gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(book->rowspin),
+				      GTK_UPDATE_IF_VALID);
     gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
     gtk_box_pack_start (GTK_BOX (hbox), book->rowspin, FALSE, FALSE, 5);
 
+    /* column label feedback */
+    hbox = gtk_hbox_new (FALSE, 5);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    tmp = gtk_label_new("(A)");
+    gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, FALSE, 5);
+    gtk_signal_connect (GTK_OBJECT(GTK_EDITABLE(book->colspin)), "changed",
+			GTK_SIGNAL_FUNC (colspin_changed), tmp);
+
     /* choose the worksheet (if applicable) */
     if (multisheet) {
 	frame = gtk_frame_new("Sheet to import");
@@ -434,6 +522,8 @@ static void wsheet_menu (wbook *book, int multisheet)
     }
 
     hbox = gtk_hbox_new (TRUE, 5);
+    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
+
     tmp = gtk_button_new_with_label("OK");
     GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (hbox), 
@@ -444,7 +534,6 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_widget_grab_default (tmp);
 
     tmp = gtk_button_new_with_label("Cancel");
-    GTK_WIDGET_SET_FLAGS (tmp, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (hbox), 
                         tmp, TRUE, TRUE, FALSE);
     gtk_signal_connect (GTK_OBJECT (tmp), "clicked", 
@@ -453,11 +542,8 @@ static void wsheet_menu (wbook *book, int multisheet)
                                GTK_SIGNAL_FUNC (gtk_widget_destroy), 
                                GTK_OBJECT (w));
 
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 5);
-
-    gtk_container_add(GTK_CONTAINER(w), vbox);
-
     gtk_widget_show_all(w);
+
     gtk_main();
 }
 
