@@ -21,6 +21,9 @@
 #include "gretl_matrix.h"
 #include "gretl_matrix_private.h"
 
+#include "f2c.h"
+#include "clapack_double.h"
+
 #include <gtk/gtk.h>
 
 struct flag_info {
@@ -324,7 +327,7 @@ static int studentized_residuals (const MODEL *pmod, double ***pZ,
 	return E_ALLOC;
     }
 
-    if (dataset_add_allocated_var(dum, pZ, pdinfo)) {
+    if (dataset_add_allocated_series(dum, pZ, pdinfo)) {
 	free(dum);
 	free(slist);
 	return E_ALLOC;	
@@ -373,7 +376,7 @@ static int studentized_residuals (const MODEL *pmod, double ***pZ,
 
     free(slist);
 
-    dataset_drop_vars(pdinfo->v - orig_v, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - orig_v, pZ, pdinfo);
 
     return err;
 }
