@@ -24,7 +24,7 @@
 #include "clapack_double.h"
 #include "gretl_model.h"
 
-#define PDEBUG 0
+#undef PDEBUG
 
 enum vcv_ops {
     VCV_INIT,
@@ -172,7 +172,7 @@ static void print_panel_coeff (const MODEL *pmod,
     char pvstr[18];
 
     sprintf(errstr, "(%.5g)", pmod->sderr[i]);
-    sprintf(pvstr, "[%.5f]", tprob(tstat, pmod->dfd));
+    sprintf(pvstr, "[%.5f]", t_pvalue_2(tstat, pmod->dfd));
     pprintf(prn, "%*s: %14.5g %15s %15s\n", VNAMELEN, vname,
 	    pmod->coeff[i], errstr, pvstr);
 }
@@ -209,7 +209,7 @@ within_groups_dataset (const double **Z, double ***wZ,
 
 #if PDEBUG
 	fprintf(stderr, "working on list[%d] (original var %d, var %d in wZ)\n",
-		j, list[j], k);
+		j, diag->vlist[j], k);
 #endif
 
 	for (i=0; i<diag->nunits; i++) { 
@@ -453,6 +453,32 @@ static int bXb (diagnostics_t *diag)
     return info;
 }
 
+static void apply_panel_df_correction (MODEL *pmod, int ndf)
+{
+    double dfcorr = sqrt((double) pmod->dfd / (pmod->dfd - ndf));
+    int i, j, idx;
+
+    pmod->dfd -= ndf;
+    pmod->dfn += ndf; 
+
+    pmod->sigma *= dfcorr;
+
+    for (i=0; i<pmod->ncoeff; i++) {
+	pmod->sderr[i] *= dfcorr;
+    }
+
+    dfcorr *= dfcorr;
+
+    if (pmod->vcv != NULL) {
+	for (i=0; i<pmod->ncoeff; i++) {
+	    for (j=i; j<pmod->ncoeff; j++) {
+		idx = ijton(i, j, pmod->ncoeff);
+		pmod->vcv[idx] *= dfcorr;
+	    }
+	}
+    }
+}
+
 #define MINOBS 1
 
 /* calculate the fixed effects regression, either using dummy variables
@@ -468,7 +494,7 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
     int *felist = NULL;
     MODEL lsdv;
 #if PDEBUG
-    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR);
 #endif
 
     gretl_model_init(&lsdv);
@@ -528,20 +554,11 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
 	       all zero -- this is a symptom of collinearity */
 	    lsdv.errcode = E_SINGULAR;
 	} else {
-	    double sdcorr = sqrt((double) lsdv.dfd / (lsdv.dfd - diag->nunits));
-
 	    /* we estimated a bunch of group means, and have to
 	       subtract that many degrees of freedom */
-	    lsdv.dfd -= diag->nunits;
-	    lsdv.dfn += diag->nunits; 
-
-	    lsdv.sigma *= sdcorr;
-
-	    for (i=0; i<lsdv.ncoeff; i++) {
-		lsdv.sderr[i] *= sdcorr;
-	    }
+	    apply_panel_df_correction(&lsdv, diag->nunits);
 #if PDEBUG
-	    printmodel(&lsdv, winfo, OPT_NONE, prn);
+	    printmodel(&lsdv, winfo, OPT_O, prn);
 #endif
 	}
 
@@ -565,7 +582,7 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
 	    return lsdv;
 	}
 
-	if (dataset_add_vars(ndum, pZ, pdinfo)) {
+	if (dataset_add_series(ndum, pZ, pdinfo)) {
 	    free(felist);
 	    lsdv.errcode = E_ALLOC;
 	    return lsdv;
@@ -616,7 +633,7 @@ fixed_effects_model (diagnostics_t *diag, double ***pZ, DATAINFO *pdinfo,
 	gretl_print_destroy(prn);
 #endif
 
-	dataset_drop_vars(pdinfo->v - oldv, pZ, pdinfo);
+	dataset_drop_last_variables(pdinfo->v - oldv, pZ, pdinfo);
     }
 
     free(felist);
@@ -651,7 +668,7 @@ fixed_effects_variance (diagnostics_t *diag,
 
 #if PDEBUG
 	fprintf(stderr, "f.e. variance, lsdv.ncoeff=%d, var=%g, "
-		"list[0]=%d\n", lsdv.ncoeff, var, diag->list[0]);
+		"list[0]=%d\n", lsdv.ncoeff, diag->fe_var, diag->vlist[0]);
 	fprintf(stderr, "haus->bdiff = %p, haus->sigma = %p\n",
 		diag->bdiff, diag->sigma);
 #endif
@@ -1223,7 +1240,7 @@ write_uvar_to_dataset (double *uvar, int nunits, int T,
 static int
 allocate_weight_var (double ***pZ, DATAINFO *pdinfo)
 {
-    if (dataset_add_vars(1, pZ, pdinfo)) {
+    if (dataset_add_series(1, pZ, pdinfo)) {
 	return E_ALLOC;
     }
 
@@ -1302,9 +1319,10 @@ static int
 ml_hetero_test (MODEL *pmod, double s2, const double *uvar, 
 		int nunits, const int *unit_obs)
 {
-    GRETLTEST test;
+    ModelTest *test;
     double x2, s2h = 0.0;
     int i, Ti, df = 0;
+    int err = 0;
 
     for (i=0; i<nunits; i++) {
 	Ti = unit_obs[i];
@@ -1317,13 +1335,17 @@ ml_hetero_test (MODEL *pmod, double s2, const double *uvar,
     x2 = pmod->nobs * log(s2) - s2h;
     df--;
 
-    gretl_test_init(&test, GRETL_TEST_GROUPWISE);
-    test.teststat = GRETL_STAT_LR;
-    test.dfn = df;
-    test.value = x2;
-    test.pvalue = chisq(x2, df);
+    test = new_test_on_model(pmod, GRETL_TEST_GROUPWISE);
+    if (test != NULL) {
+	model_test_set_teststat(test, GRETL_STAT_LR);
+	model_test_set_dfn(test, df);
+	model_test_set_value(test, x2);
+	model_test_set_pvalue(test, chisq(x2, df));
+    } else {
+	err = 1;
+    }
 
-    return add_test_to_model(pmod, &test);
+    return err;
 }
 
 static double pooled_ll (const MODEL *pmod)
@@ -1368,7 +1390,7 @@ static int singleton_check (const int *unit_obs, int nunits)
 #define SMALLDIFF 0.0001
 #define WLS_MAX   20
 
-MODEL panel_wls_by_unit (int *list, double ***pZ, DATAINFO *pdinfo,
+MODEL panel_wls_by_unit (const int *list, double ***pZ, DATAINFO *pdinfo,
 			 gretlopt opt, PRN *prn)
 {
     MODEL mdl;
@@ -1559,7 +1581,7 @@ MODEL panel_wls_by_unit (int *list, double ***pZ, DATAINFO *pdinfo,
     free(wlist);
     free(bvec);
 
-    dataset_drop_vars(pdinfo->v - orig_v, pZ, pdinfo);
+    dataset_drop_last_variables(pdinfo->v - orig_v, pZ, pdinfo);
     
     return mdl;
 }
@@ -1621,7 +1643,7 @@ static void panel_lag (double **tmpZ, DATAINFO *tmpinfo,
 
 int panel_autocorr_test (MODEL *pmod, int order, 
 			 double **Z, DATAINFO *pdinfo, 
-			 PRN *prn, GRETLTEST *test)
+			 gretlopt opt, PRN *prn)
 {
     int *aclist;
     double **tmpZ;
@@ -1716,31 +1738,40 @@ int panel_autocorr_test (MODEL *pmod, int order,
     } 
 
     if (!err) {
+	int dfd = aux.nobs - pmod->ncoeff - order;
+	double pval;
+
 	aux.aux = AUX_AR;
 	aux.order = order;
 	printmodel(&aux, tmpinfo, OPT_NONE, prn);
 	trsq = aux.rsq * aux.nobs;
-	LMF = (aux.rsq/(1.0 - aux.rsq)) * 
-	    (aux.nobs - pmod->ncoeff - order)/order; 
+	LMF = (aux.rsq / (1.0 - aux.rsq)) * dfd / order; 
+	pval = fdist(LMF, order, dfd);
 
 	pprintf(prn, "\n%s: LMF = %f,\n", _("Test statistic"), LMF);
 	pprintf(prn, "%s = P(F(%d,%d) > %g) = %.3g\n", _("with p-value"), 
-		order, aux.nobs - pmod->ncoeff - order, LMF,
-		fdist(LMF, order, aux.nobs - pmod->ncoeff - order));
+		order, dfd, LMF, pval);
 
 	pprintf(prn, "\n%s: TR^2 = %f,\n", 
 		_("Alternative statistic"), trsq);
 	pprintf(prn, "%s = P(%s(%d) > %g) = %.3g\n\n", 	_("with p-value"), 
 		_("Chi-square"), order, trsq, chisq(trsq, order));
 
-	if (test != NULL) {
-	    gretl_test_init(test, GRETL_TEST_AUTOCORR);
-	    test->order = order;
-	    test->teststat = GRETL_STAT_LMF;
-	    test->dfn = order;
-	    test->dfd = aux.nobs - pmod->ncoeff - order;
-	    test->value = LMF;
-	    test->pvalue = fdist(LMF, test->dfn, test->dfd);
+	if (opt & OPT_S) {
+	    ModelTest *test;
+
+	    test = new_test_on_model(pmod, GRETL_TEST_AUTOCORR);
+
+	    if (test != NULL) {
+		int dfd = aux.nobs - pmod->ncoeff - order;
+
+		model_test_set_teststat(test, GRETL_STAT_LMF);
+		model_test_set_order(test, order);
+		model_test_set_dfn(test, order);
+		model_test_set_dfd(test, dfd);
+		model_test_set_value(test, LMF);
+		model_test_set_pvalue(test, pval);
+	    }	    
 	}
     }
 
