@@ -744,13 +744,13 @@ save_and_print_results (gretl_equation_system *sys,
 
     if (system_save_uhat(sys)) {
 	j = pdinfo->v;
-	err = dataset_add_vars(m, pZ, pdinfo);
+	err = dataset_add_series(m, pZ, pdinfo);
     }
     if (system_save_yhat(sys)) {
 	if (j == 0) {
 	    j = pdinfo->v;
 	}
-	err = dataset_add_vars(m, pZ, pdinfo);
+	err = dataset_add_series(m, pZ, pdinfo);
     }
 
     pputc(prn, '\n');
