@@ -187,19 +187,20 @@ static int make_vcv (MODEL *pmod, gretl_matrix *v, double scale)
 
 static int add_norm_test_to_model (MODEL *pmod, double chi2)
 {
-    pmod->tests = malloc(sizeof *pmod->tests);
-    if (pmod->tests == NULL) return 1;
+    ModelTest *test;
+    int err = 0;
 
-    gretl_test_init(&pmod->tests[0], GRETL_TEST_NORMAL);
-    pmod->tests[0].teststat = GRETL_STAT_NORMAL_CHISQ;
-    pmod->tests[0].value = chi2;
-    pmod->tests[0].dfn = 2;
-    pmod->tests[0].dfd = 0;
-    pmod->tests[0].pvalue = chisq(chi2, 2);
+    test = new_test_on_model(pmod, GRETL_TEST_NORMAL);
+    if (test != NULL) {
+	model_test_set_teststat(test, GRETL_STAT_NORMAL_CHISQ);
+	model_test_set_dfn(test, 2);
+	model_test_set_value(test, chi2);
+	model_test_set_pvalue(test, chisq(chi2, 2));
+    } else {
+	err = 1;
+    }
 
-    pmod->ntests = 1;
-
-    return 0;
+    return err;
 }
 
 static double recompute_tobit_ll (const MODEL *pmod, const double *y)
@@ -276,7 +277,8 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
     }
 
     /* run normality test on the untruncated uhat */
-    moments(pmod->t1, pmod->t2, pmod->uhat, &ubar, &udev, &skew, &kurt, pmod->ncoeff);
+    gretl_moments(pmod->t1, pmod->t2, pmod->uhat, &ubar, &udev, 
+		  &skew, &kurt, pmod->ncoeff);
     chi2 = doornik_chisq(skew, kurt, pmod->nobs); 
     add_norm_test_to_model(pmod, chi2);
 
@@ -308,15 +310,14 @@ static int write_tobit_stats (MODEL *pmod, double *theta, int ncoeff,
 static model_info *
 tobit_model_info_init (int nobs, int k, int n_series)
 {
-    model_info *tobit = model_info_new();
+    model_info *tobit;
 
-    if (tobit == NULL) return NULL;
+    tobit = model_info_new(k, 0, nobs - 1, TOBIT_TOL);
 
-    model_info_set_opts(tobit, FULL_VCV_MATRIX);
-    model_info_set_tol(tobit, TOBIT_TOL);
-    model_info_set_k(tobit, k);
-    model_info_set_n_series(tobit, n_series);
-    model_info_set_t1_t2(tobit, 0, nobs - 1);
+    if (tobit != NULL) {
+	model_info_set_opts(tobit, FULL_VCV_MATRIX);
+	model_info_set_n_series(tobit, n_series);
+    }
 
     return tobit;
 }
@@ -468,7 +469,7 @@ static double tobit_depvar_scale (const MODEL *pmod)
 
 /* the driver function for the plugin */
 
-MODEL tobit_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
+MODEL tobit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
 		      PRN *prn) 
 {
     MODEL model;
