@@ -891,7 +891,9 @@ int main (int argc, char *argv[])
 	strcpy(datadir, argv[2]);
     }
 
-    prn = gretl_print_new(GRETL_PRINT_STDOUT, NULL); 
+    libgretl_init();
+
+    prn = gretl_print_new(GRETL_PRINT_STDOUT); 
 
     for (j=0; j<ntests; j++) {
 	if (read_nist_file(nist_files[j], &Z, &datainfo, &certvals,
@@ -925,6 +927,8 @@ int main (int argc, char *argv[])
 
     gretl_print_destroy(prn);
 
+    libgretl_cleanup();
+
     return (missing || modelerrs || poorvals);
 }
 
@@ -984,7 +988,7 @@ int run_nist_tests (const char *datapath, const char *outfile, int verbosity)
 
     sprintf(datadir, "%snist", datapath);
 
-    prn = gretl_print_new(GRETL_PRINT_FILE, outfile); 
+    prn = gretl_print_new_with_filename(outfile); 
 
     nist_intro(prn);
 
